/*
 *  Produced by f2java.  f2java is part of the Fortran-
 *  -to-Java project at the University of Tennessee Netlib
 *  numerical software repository.
 *
 *  Original authorship for the BLAS and LAPACK numerical
 *  routines may be found in the Fortran source, available at
 *  www.netlib.org.
 *
 *  Fortran input file: jlapack.f
 *
 *  The f2j compiler code was written by
 *  David M. Doolin (doolin@cs.utk.edu) and
 *  Keith  Seymour (seymour@cs.utk.edu)
 */

package org.netlib.blas;
import java.lang.*;
import org.netlib.util.*;



public class Dgemm {

// *     .. Scalar Arguments ..
// *     .. Array Arguments ..
// *     ..
// *
// *  Purpose
// *  =======
// *
// *  DGEMM  performs one of the matrix-matrix operations
// *
// *     C := alpha*op( A )*op( B ) + beta*C,
// *
// *  where  op( X ) is one of
// *
// *     op( X ) = X   or   op( X ) = X',
// *
// *  alpha and beta are scalars, and A, B and C are matrices, with op( A )
// *  an m by k matrix,  op( B )  a  k by n matrix and  C an m by n matrix.
// *
// *  Parameters
// *  ==========
// *
// *  TRANSA - CHARACTER*1.
// *           On entry, TRANSA specifies the form of op( A ) to be used in
// *           the matrix multiplication as follows:
// *
// *              TRANSA = 'N' or 'n',  op( A ) = A.
// *
// *              TRANSA = 'T' or 't',  op( A ) = A'.
// *
// *              TRANSA = 'C' or 'c',  op( A ) = A'.
// *
// *           Unchanged on exit.
// *
// *  TRANSB - CHARACTER*1.
// *           On entry, TRANSB specifies the form of op( B ) to be used in
// *           the matrix multiplication as follows:
// *
// *              TRANSB = 'N' or 'n',  op( B ) = B.
// *
// *              TRANSB = 'T' or 't',  op( B ) = B'.
// *
// *              TRANSB = 'C' or 'c',  op( B ) = B'.
// *
// *           Unchanged on exit.
// *
// *  M      - INTEGER.
// *           On entry,  M  specifies  the number  of rows  of the  matrix
// *           op( A )  and of the  matrix  C.  M  must  be at least  zero.
// *           Unchanged on exit.
// *
// *  N      - INTEGER.
// *           On entry,  N  specifies the number  of columns of the matrix
// *           op( B ) and the number of columns of the matrix C. N must be
// *           at least zero.
// *           Unchanged on exit.
// *
// *  K      - INTEGER.
// *           On entry,  K  specifies  the number of columns of the matrix
// *           op( A ) and the number of rows of the matrix op( B ). K must
// *           be at least  zero.
// *           Unchanged on exit.
// *
// *  ALPHA  - DOUBLE PRECISION.
// *           On entry, ALPHA specifies the scalar alpha.
// *           Unchanged on exit.
// *
// *  A      - DOUBLE PRECISION array of DIMENSION ( LDA, ka ), where ka is
// *           k  when  TRANSA = 'N' or 'n',  and is  m  otherwise.
// *           Before entry with  TRANSA = 'N' or 'n',  the leading  m by k
// *           part of the array  A  must contain the matrix  A,  otherwise
// *           the leading  k by m  part of the array  A  must contain  the
// *           matrix A.
// *           Unchanged on exit.
// *
// *  LDA    - INTEGER.
// *           On entry, LDA specifies the first dimension of A as declared
// *           in the calling (sub) program. When  TRANSA = 'N' or 'n' then
// *           LDA must be at least  max( 1, m ), otherwise  LDA must be at
// *           least  max( 1, k ).
// *           Unchanged on exit.
// *
// *  B      - DOUBLE PRECISION array of DIMENSION ( LDB, kb ), where kb is
// *           n  when  TRANSB = 'N' or 'n',  and is  k  otherwise.
// *           Before entry with  TRANSB = 'N' or 'n',  the leading  k by n
// *           part of the array  B  must contain the matrix  B,  otherwise
// *           the leading  n by k  part of the array  B  must contain  the
// *           matrix B.
// *           Unchanged on exit.
// *
// *  LDB    - INTEGER.
// *           On entry, LDB specifies the first dimension of B as declared
// *           in the calling (sub) program. When  TRANSB = 'N' or 'n' then
// *           LDB must be at least  max( 1, k ), otherwise  LDB must be at
// *           least  max( 1, n ).
// *           Unchanged on exit.
// *
// *  BETA   - DOUBLE PRECISION.
// *           On entry,  BETA  specifies the scalar  beta.  When  BETA  is
// *           supplied as zero then C need not be set on input.
// *           Unchanged on exit.
// *
// *  C      - DOUBLE PRECISION array of DIMENSION ( LDC, n ).
// *           Before entry, the leading  m by n  part of the array  C must
// *           contain the matrix  C,  except when  beta  is zero, in which
// *           case C need not be set on entry.
// *           On exit, the array  C  is overwritten by the  m by n  matrix
// *           ( alpha*op( A )*op( B ) + beta*C ).
// *
// *  LDC    - INTEGER.
// *           On entry, LDC specifies the first dimension of C as declared
// *           in  the  calling  (sub)  program.   LDC  must  be  at  least
// *           max( 1, m ).
// *           Unchanged on exit.
// *
// *
// *  Level 3 Blas routine.
// *
// *  -- Written on 8-February-1989.
// *     Jack Dongarra, Argonne National Laboratory.
// *     Iain Duff, AERE Harwell.
// *     Jeremy Du Croz, Numerical Algorithms Group Ltd.
// *     Sven Hammarling, Numerical Algorithms Group Ltd.
// *
// *
// *     .. External Functions ..
// *     .. External Subroutines ..
// *     .. Intrinsic Functions ..
// *     .. Local Scalars ..
static boolean nota= false;
static boolean notb= false;
static int i= 0;
static int info= 0;
static int j= 0;
static int l= 0;
static int ncola= 0;
static int nrowa= 0;
static int nrowb= 0;
static double temp= 0.0;
// *     .. Parameters ..
static double one= 1.0e+0;
static double zero= 0.0e+0;
// *     ..
// *     .. Executable Statements ..
// *
// *     Set  NOTA  and  NOTB  as  true if  A  and  B  respectively are not
// *     transposed and set  NROWA, NCOLA and  NROWB  as the number of rows
// *     and  columns of  A  and the  number of  rows  of  B  respectively.
// *

public static void dgemm (String transa,
String transb,
int m,
int n,
int k,
double alpha,
double [] a, int _a_offset,
int lda,
double [] b, int _b_offset,
int ldb,
double beta,
double [] c, int _c_offset,
int Ldc)  {

nota = (transa.toLowerCase().charAt(0) == "N".toLowerCase().charAt(0));
notb = (transb.toLowerCase().charAt(0) == "N".toLowerCase().charAt(0));
if (nota)  {
    nrowa = m;
ncola = k;
}              // Close if()
else  {
  nrowa = k;
ncola = m;
}              //  Close else.
if (notb)  {
    nrowb = k;
}              // Close if()
else  {
  nrowb = n;
}              //  Close else.
// *
// *     Test the input parameters.
// *
info = 0;
if ((!nota) && (!(transa.toLowerCase().charAt(0) == "C".toLowerCase().charAt(0))) && (!(transa.toLowerCase().charAt(0) == "T".toLowerCase().charAt(0))))  {
    info = 1;
}              // Close if()
else if ((!notb) && (!(transb.toLowerCase().charAt(0) == "C".toLowerCase().charAt(0))) && (!(transb.toLowerCase().charAt(0) == "T".toLowerCase().charAt(0))))  {
    info = 2;
}              // Close else if()
else if (m < 0)  {
    info = 3;
}              // Close else if()
else if (n < 0)  {
    info = 4;
}              // Close else if()
else if (k < 0)  {
    info = 5;
}              // Close else if()
else if (lda < Math.max(1, nrowa) )  {
    info = 8;
}              // Close else if()
else if (ldb < Math.max(1, nrowb) )  {
    info = 10;
}              // Close else if()
else if (Ldc < Math.max(1, m) )  {
    info = 13;
}              // Close else if()
if (info != 0)  {
    Xerbla.xerbla("DGEMM ",info);
Dummy.go_to("Dgemm",999999);
}              // Close if()
// *
// *     Quick return if possible.
// *
if ((m == 0) || (n == 0) || (((alpha == zero) || (k == 0)) && (beta == one)))  
    Dummy.go_to("Dgemm",999999);
// *
// *     And if  alpha.eq.zero.
// *
if (alpha == zero)  {
    if (beta == zero)  {
    {
forloop20:
for (j = 1; j <= n; j++) {
{
forloop10:
for (i = 1; i <= m; i++) {
c[(i)- 1+(j- 1)*Ldc+ _c_offset] = zero;
Dummy.label("Dgemm",10);
}              //  Close for() loop. 
}
Dummy.label("Dgemm",20);
}              //  Close for() loop. 
}
}              // Close if()
else  {
  {
forloop40:
for (j = 1; j <= n; j++) {
{
forloop30:
for (i = 1; i <= m; i++) {
c[(i)- 1+(j- 1)*Ldc+ _c_offset] = beta*c[(i)- 1+(j- 1)*Ldc+ _c_offset];
Dummy.label("Dgemm",30);
}              //  Close for() loop. 
}
Dummy.label("Dgemm",40);
}              //  Close for() loop. 
}
}              //  Close else.
Dummy.go_to("Dgemm",999999);
}              // Close if()
// *
// *     Start the operations.
// *
if (notb)  {
    if (nota)  {
    // *
// *           Form  C := alpha*A*B + beta*C.
// *
{
forloop90:
for (j = 1; j <= n; j++) {
if (beta == zero)  {
    {
forloop50:
for (i = 1; i <= m; i++) {
c[(i)- 1+(j- 1)*Ldc+ _c_offset] = zero;
Dummy.label("Dgemm",50);
}              //  Close for() loop. 
}
}              // Close if()
else if (beta != one)  {
    {
forloop60:
for (i = 1; i <= m; i++) {
c[(i)- 1+(j- 1)*Ldc+ _c_offset] = beta*c[(i)- 1+(j- 1)*Ldc+ _c_offset];
Dummy.label("Dgemm",60);
}              //  Close for() loop. 
}
}              // Close else if()
{
forloop80:
for (l = 1; l <= k; l++) {
if (b[(l)- 1+(j- 1)*ldb+ _b_offset] != zero)  {
    temp = alpha*b[(l)- 1+(j- 1)*ldb+ _b_offset];
{
forloop70:
for (i = 1; i <= m; i++) {
c[(i)- 1+(j- 1)*Ldc+ _c_offset] = c[(i)- 1+(j- 1)*Ldc+ _c_offset]+temp*a[(i)- 1+(l- 1)*lda+ _a_offset];
Dummy.label("Dgemm",70);
}              //  Close for() loop. 
}
}              // Close if()
Dummy.label("Dgemm",80);
}              //  Close for() loop. 
}
Dummy.label("Dgemm",90);
}              //  Close for() loop. 
}
}              // Close if()
else  {
  // *
// *           Form  C := alpha*A'*B + beta*C
// *
{
forloop120:
for (j = 1; j <= n; j++) {
{
forloop110:
for (i = 1; i <= m; i++) {
temp = zero;
{
forloop100:
for (l = 1; l <= k; l++) {
temp = temp+a[(l)- 1+(i- 1)*lda+ _a_offset]*b[(l)- 1+(j- 1)*ldb+ _b_offset];
Dummy.label("Dgemm",100);
}              //  Close for() loop. 
}
if (beta == zero)  {
    c[(i)- 1+(j- 1)*Ldc+ _c_offset] = alpha*temp;
}              // Close if()
else  {
  c[(i)- 1+(j- 1)*Ldc+ _c_offset] = alpha*temp+beta*c[(i)- 1+(j- 1)*Ldc+ _c_offset];
}              //  Close else.
Dummy.label("Dgemm",110);
}              //  Close for() loop. 
}
Dummy.label("Dgemm",120);
}              //  Close for() loop. 
}
}              //  Close else.
}              // Close if()
else  {
  if (nota)  {
    // *
// *           Form  C := alpha*A*B' + beta*C
// *
{
forloop170:
for (j = 1; j <= n; j++) {
if (beta == zero)  {
    {
forloop130:
for (i = 1; i <= m; i++) {
c[(i)- 1+(j- 1)*Ldc+ _c_offset] = zero;
Dummy.label("Dgemm",130);
}              //  Close for() loop. 
}
}              // Close if()
else if (beta != one)  {
    {
forloop140:
for (i = 1; i <= m; i++) {
c[(i)- 1+(j- 1)*Ldc+ _c_offset] = beta*c[(i)- 1+(j- 1)*Ldc+ _c_offset];
Dummy.label("Dgemm",140);
}              //  Close for() loop. 
}
}              // Close else if()
{
forloop160:
for (l = 1; l <= k; l++) {
if (b[(j)- 1+(l- 1)*ldb+ _b_offset] != zero)  {
    temp = alpha*b[(j)- 1+(l- 1)*ldb+ _b_offset];
{
forloop150:
for (i = 1; i <= m; i++) {
c[(i)- 1+(j- 1)*Ldc+ _c_offset] = c[(i)- 1+(j- 1)*Ldc+ _c_offset]+temp*a[(i)- 1+(l- 1)*lda+ _a_offset];
Dummy.label("Dgemm",150);
}              //  Close for() loop. 
}
}              // Close if()
Dummy.label("Dgemm",160);
}              //  Close for() loop. 
}
Dummy.label("Dgemm",170);
}              //  Close for() loop. 
}
}              // Close if()
else  {
  // *
// *           Form  C := alpha*A'*B' + beta*C
// *
{
forloop200:
for (j = 1; j <= n; j++) {
{
forloop190:
for (i = 1; i <= m; i++) {
temp = zero;
{
forloop180:
for (l = 1; l <= k; l++) {
temp = temp+a[(l)- 1+(i- 1)*lda+ _a_offset]*b[(j)- 1+(l- 1)*ldb+ _b_offset];
Dummy.label("Dgemm",180);
}              //  Close for() loop. 
}
if (beta == zero)  {
    c[(i)- 1+(j- 1)*Ldc+ _c_offset] = alpha*temp;
}              // Close if()
else  {
  c[(i)- 1+(j- 1)*Ldc+ _c_offset] = alpha*temp+beta*c[(i)- 1+(j- 1)*Ldc+ _c_offset];
}              //  Close else.
Dummy.label("Dgemm",190);
}              //  Close for() loop. 
}
Dummy.label("Dgemm",200);
}              //  Close for() loop. 
}
}              //  Close else.
}              //  Close else.
// *
Dummy.go_to("Dgemm",999999);
// *
// *     End of DGEMM .
// *
Dummy.label("Dgemm",999999);
return;
   }
} // End class.
