/*
 *  Produced by f2java.  f2java is part of the Fortran-
 *  -to-Java project at the University of Tennessee Netlib
 *  numerical software repository.
 *
 *  Original authorship for the BLAS and LAPACK numerical
 *  routines may be found in the Fortran source, available at
 *  www.netlib.org.
 *
 *  Fortran input file: jlapack.f
 *
 *  The f2j compiler code was written by
 *  David M. Doolin (doolin@cs.utk.edu) and
 *  Keith  Seymour (seymour@cs.utk.edu)
 */

package org.netlib.blas;
import java.lang.*;
import org.netlib.util.*;



public class Dgemv {

// *     .. Scalar Arguments ..
// *     .. Array Arguments ..
// *     ..
// *
// *  Purpose
// *  =======
// *
// *  DGEMV  performs one of the matrix-vector operations
// *
// *     y := alpha*A*x + beta*y,   or   y := alpha*A'*x + beta*y,
// *
// *  where alpha and beta are scalars, x and y are vectors and A is an
// *  m by n matrix.
// *
// *  Parameters
// *  ==========
// *
// *  TRANS  - CHARACTER*1.
// *           On entry, TRANS specifies the operation to be performed as
// *           follows:
// *
// *              TRANS = 'N' or 'n'   y := alpha*A*x + beta*y.
// *
// *              TRANS = 'T' or 't'   y := alpha*A'*x + beta*y.
// *
// *              TRANS = 'C' or 'c'   y := alpha*A'*x + beta*y.
// *
// *           Unchanged on exit.
// *
// *  M      - INTEGER.
// *           On entry, M specifies the number of rows of the matrix A.
// *           M must be at least zero.
// *           Unchanged on exit.
// *
// *  N      - INTEGER.
// *           On entry, N specifies the number of columns of the matrix A.
// *           N must be at least zero.
// *           Unchanged on exit.
// *
// *  ALPHA  - DOUBLE PRECISION.
// *           On entry, ALPHA specifies the scalar alpha.
// *           Unchanged on exit.
// *
// *  A      - DOUBLE PRECISION array of DIMENSION ( LDA, n ).
// *           Before entry, the leading m by n part of the array A must
// *           contain the matrix of coefficients.
// *           Unchanged on exit.
// *
// *  LDA    - INTEGER.
// *           On entry, LDA specifies the first dimension of A as declared
// *           in the calling (sub) program. LDA must be at least
// *           max( 1, m ).
// *           Unchanged on exit.
// *
// *  X      - DOUBLE PRECISION array of DIMENSION at least
// *           ( 1 + ( n - 1 )*abs( INCX ) ) when TRANS = 'N' or 'n'
// *           and at least
// *           ( 1 + ( m - 1 )*abs( INCX ) ) otherwise.
// *           Before entry, the incremented array X must contain the
// *           vector x.
// *           Unchanged on exit.
// *
// *  INCX   - INTEGER.
// *           On entry, INCX specifies the increment for the elements of
// *           X. INCX must not be zero.
// *           Unchanged on exit.
// *
// *  BETA   - DOUBLE PRECISION.
// *           On entry, BETA specifies the scalar beta. When BETA is
// *           supplied as zero then Y need not be set on input.
// *           Unchanged on exit.
// *
// *  Y      - DOUBLE PRECISION array of DIMENSION at least
// *           ( 1 + ( m - 1 )*abs( INCY ) ) when TRANS = 'N' or 'n'
// *           and at least
// *           ( 1 + ( n - 1 )*abs( INCY ) ) otherwise.
// *           Before entry with BETA non-zero, the incremented array Y
// *           must contain the vector y. On exit, Y is overwritten by the
// *           updated vector y.
// *
// *  INCY   - INTEGER.
// *           On entry, INCY specifies the increment for the elements of
// *           Y. INCY must not be zero.
// *           Unchanged on exit.
// *
// *
// *  Level 2 Blas routine.
// *
// *  -- Written on 22-October-1986.
// *     Jack Dongarra, Argonne National Lab.
// *     Jeremy Du Croz, Nag Central Office.
// *     Sven Hammarling, Nag Central Office.
// *     Richard Hanson, Sandia National Labs.
// *
// *
// *     .. Parameters ..
static double one= 1.0e+0;
static double zero= 0.0e+0;
// *     .. Local Scalars ..
static double temp= 0.0;
static int i= 0;
static int info= 0;
static int ix= 0;
static int iy= 0;
static int j= 0;
static int jx= 0;
static int jy= 0;
static int kx= 0;
static int ky= 0;
static int lenx= 0;
static int leny= 0;
// *     .. External Functions ..
// *     .. External Subroutines ..
// *     .. Intrinsic Functions ..
// *     ..
// *     .. Executable Statements ..
// *
// *     Test the input parameters.
// *

public static void dgemv (String trans,
int m,
int n,
double alpha,
double [] a, int _a_offset,
int lda,
double [] x, int _x_offset,
int incx,
double beta,
double [] y, int _y_offset,
int incy)  {

info = 0;
if (!(trans.toLowerCase().charAt(0) == "N".toLowerCase().charAt(0)) && !(trans.toLowerCase().charAt(0) == "T".toLowerCase().charAt(0)) && !(trans.toLowerCase().charAt(0) == "C".toLowerCase().charAt(0)))  {
    info = 1;
}              // Close if()
else if (m < 0)  {
    info = 2;
}              // Close else if()
else if (n < 0)  {
    info = 3;
}              // Close else if()
else if (lda < Math.max(1, m) )  {
    info = 6;
}              // Close else if()
else if (incx == 0)  {
    info = 8;
}              // Close else if()
else if (incy == 0)  {
    info = 11;
}              // Close else if()
if (info != 0)  {
    Xerbla.xerbla("DGEMV ",info);
Dummy.go_to("Dgemv",999999);
}              // Close if()
// *
// *     Quick return if possible.
// *
if ((m == 0) || (n == 0) || ((alpha == zero) && (beta == one)))  
    Dummy.go_to("Dgemv",999999);
// *
// *     Set  LENX  and  LENY, the lengths of the vectors x and y, and set
// *     up the start points in  X  and  Y.
// *
if ((trans.toLowerCase().charAt(0) == "N".toLowerCase().charAt(0)))  {
    lenx = n;
leny = m;
}              // Close if()
else  {
  lenx = m;
leny = n;
}              //  Close else.
if (incx > 0)  {
    kx = 1;
}              // Close if()
else  {
  kx = 1-(lenx-1)*incx;
}              //  Close else.
if (incy > 0)  {
    ky = 1;
}              // Close if()
else  {
  ky = 1-(leny-1)*incy;
}              //  Close else.
// *
// *     Start the operations. In this version the elements of A are
// *     accessed sequentially with one pass through A.
// *
// *     First form  y := beta*y.
// *
if (beta != one)  {
    if (incy == 1)  {
    if (beta == zero)  {
    {
forloop10:
for (i = 1; i <= leny; i++) {
y[(i)- 1+ _y_offset] = zero;
Dummy.label("Dgemv",10);
}              //  Close for() loop. 
}
}              // Close if()
else  {
  {
forloop20:
for (i = 1; i <= leny; i++) {
y[(i)- 1+ _y_offset] = beta*y[(i)- 1+ _y_offset];
Dummy.label("Dgemv",20);
}              //  Close for() loop. 
}
}              //  Close else.
}              // Close if()
else  {
  iy = ky;
if (beta == zero)  {
    {
forloop30:
for (i = 1; i <= leny; i++) {
y[(iy)- 1+ _y_offset] = zero;
iy = iy+incy;
Dummy.label("Dgemv",30);
}              //  Close for() loop. 
}
}              // Close if()
else  {
  {
forloop40:
for (i = 1; i <= leny; i++) {
y[(iy)- 1+ _y_offset] = beta*y[(iy)- 1+ _y_offset];
iy = iy+incy;
Dummy.label("Dgemv",40);
}              //  Close for() loop. 
}
}              //  Close else.
}              //  Close else.
}              // Close if()
if (alpha == zero)  
    Dummy.go_to("Dgemv",999999);
if ((trans.toLowerCase().charAt(0) == "N".toLowerCase().charAt(0)))  {
    // *
// *        Form  y := alpha*A*x + y.
// *
jx = kx;
if (incy == 1)  {
    {
forloop60:
for (j = 1; j <= n; j++) {
if (x[(jx)- 1+ _x_offset] != zero)  {
    temp = alpha*x[(jx)- 1+ _x_offset];
{
forloop50:
for (i = 1; i <= m; i++) {
y[(i)- 1+ _y_offset] = y[(i)- 1+ _y_offset]+temp*a[(i)- 1+(j- 1)*lda+ _a_offset];
Dummy.label("Dgemv",50);
}              //  Close for() loop. 
}
}              // Close if()
jx = jx+incx;
Dummy.label("Dgemv",60);
}              //  Close for() loop. 
}
}              // Close if()
else  {
  {
forloop80:
for (j = 1; j <= n; j++) {
if (x[(jx)- 1+ _x_offset] != zero)  {
    temp = alpha*x[(jx)- 1+ _x_offset];
iy = ky;
{
forloop70:
for (i = 1; i <= m; i++) {
y[(iy)- 1+ _y_offset] = y[(iy)- 1+ _y_offset]+temp*a[(i)- 1+(j- 1)*lda+ _a_offset];
iy = iy+incy;
Dummy.label("Dgemv",70);
}              //  Close for() loop. 
}
}              // Close if()
jx = jx+incx;
Dummy.label("Dgemv",80);
}              //  Close for() loop. 
}
}              //  Close else.
}              // Close if()
else  {
  // *
// *        Form  y := alpha*A'*x + y.
// *
jy = ky;
if (incx == 1)  {
    {
forloop100:
for (j = 1; j <= n; j++) {
temp = zero;
{
forloop90:
for (i = 1; i <= m; i++) {
temp = temp+a[(i)- 1+(j- 1)*lda+ _a_offset]*x[(i)- 1+ _x_offset];
Dummy.label("Dgemv",90);
}              //  Close for() loop. 
}
y[(jy)- 1+ _y_offset] = y[(jy)- 1+ _y_offset]+alpha*temp;
jy = jy+incy;
Dummy.label("Dgemv",100);
}              //  Close for() loop. 
}
}              // Close if()
else  {
  {
forloop120:
for (j = 1; j <= n; j++) {
temp = zero;
ix = kx;
{
forloop110:
for (i = 1; i <= m; i++) {
temp = temp+a[(i)- 1+(j- 1)*lda+ _a_offset]*x[(ix)- 1+ _x_offset];
ix = ix+incx;
Dummy.label("Dgemv",110);
}              //  Close for() loop. 
}
y[(jy)- 1+ _y_offset] = y[(jy)- 1+ _y_offset]+alpha*temp;
jy = jy+incy;
Dummy.label("Dgemv",120);
}              //  Close for() loop. 
}
}              //  Close else.
}              //  Close else.
// *
Dummy.go_to("Dgemv",999999);
// *
// *     End of DGEMV .
// *
Dummy.label("Dgemv",999999);
return;
   }
} // End class.
