/*
 *  Produced by f2java.  f2java is part of the Fortran-
 *  -to-Java project at the University of Tennessee Netlib
 *  numerical software repository.
 *
 *  Original authorship for the BLAS and LAPACK numerical
 *  routines may be found in the Fortran source, available at
 *  www.netlib.org.
 *
 *  Fortran input file: jlapack.f
 *
 *  The f2j compiler code was written by
 *  David M. Doolin (doolin@cs.utk.edu) and
 *  Keith  Seymour (seymour@cs.utk.edu)
 */

package org.netlib.blas;
import java.lang.*;
import org.netlib.util.*;



public class Dger {

// *     .. Scalar Arguments ..
// *     .. Array Arguments ..
// *     ..
// *
// *  Purpose
// *  =======
// *
// *  DGER   performs the rank 1 operation
// *
// *     A := alpha*x*y' + A,
// *
// *  where alpha is a scalar, x is an m element vector, y is an n element
// *  vector and A is an m by n matrix.
// *
// *  Parameters
// *  ==========
// *
// *  M      - INTEGER.
// *           On entry, M specifies the number of rows of the matrix A.
// *           M must be at least zero.
// *           Unchanged on exit.
// *
// *  N      - INTEGER.
// *           On entry, N specifies the number of columns of the matrix A.
// *           N must be at least zero.
// *           Unchanged on exit.
// *
// *  ALPHA  - DOUBLE PRECISION.
// *           On entry, ALPHA specifies the scalar alpha.
// *           Unchanged on exit.
// *
// *  X      - DOUBLE PRECISION array of dimension at least
// *           ( 1 + ( m - 1 )*abs( INCX ) ).
// *           Before entry, the incremented array X must contain the m
// *           element vector x.
// *           Unchanged on exit.
// *
// *  INCX   - INTEGER.
// *           On entry, INCX specifies the increment for the elements of
// *           X. INCX must not be zero.
// *           Unchanged on exit.
// *
// *  Y      - DOUBLE PRECISION array of dimension at least
// *           ( 1 + ( n - 1 )*abs( INCY ) ).
// *           Before entry, the incremented array Y must contain the n
// *           element vector y.
// *           Unchanged on exit.
// *
// *  INCY   - INTEGER.
// *           On entry, INCY specifies the increment for the elements of
// *           Y. INCY must not be zero.
// *           Unchanged on exit.
// *
// *  A      - DOUBLE PRECISION array of DIMENSION ( LDA, n ).
// *           Before entry, the leading m by n part of the array A must
// *           contain the matrix of coefficients. On exit, A is
// *           overwritten by the updated matrix.
// *
// *  LDA    - INTEGER.
// *           On entry, LDA specifies the first dimension of A as declared
// *           in the calling (sub) program. LDA must be at least
// *           max( 1, m ).
// *           Unchanged on exit.
// *
// *
// *  Level 2 Blas routine.
// *
// *  -- Written on 22-October-1986.
// *     Jack Dongarra, Argonne National Lab.
// *     Jeremy Du Croz, Nag Central Office.
// *     Sven Hammarling, Nag Central Office.
// *     Richard Hanson, Sandia National Labs.
// *
// *
// *     .. Parameters ..
static double zero= 0.0e+0;
// *     .. Local Scalars ..
static double temp= 0.0;
static int i= 0;
static int info= 0;
static int ix= 0;
static int j= 0;
static int jy= 0;
static int kx= 0;
// *     .. External Subroutines ..
// *     .. Intrinsic Functions ..
// *     ..
// *     .. Executable Statements ..
// *
// *     Test the input parameters.
// *

public static void dger (int m,
int n,
double alpha,
double [] x, int _x_offset,
int incx,
double [] y, int _y_offset,
int incy,
double [] a, int _a_offset,
int lda)  {

info = 0;
if (m < 0)  {
    info = 1;
}              // Close if()
else if (n < 0)  {
    info = 2;
}              // Close else if()
else if (incx == 0)  {
    info = 5;
}              // Close else if()
else if (incy == 0)  {
    info = 7;
}              // Close else if()
else if (lda < Math.max(1, m) )  {
    info = 9;
}              // Close else if()
if (info != 0)  {
    Xerbla.xerbla("DGER  ",info);
Dummy.go_to("Dger",999999);
}              // Close if()
// *
// *     Quick return if possible.
// *
if ((m == 0) || (n == 0) || (alpha == zero))  
    Dummy.go_to("Dger",999999);
// *
// *     Start the operations. In this version the elements of A are
// *     accessed sequentially with one pass through A.
// *
if (incy > 0)  {
    jy = 1;
}              // Close if()
else  {
  jy = 1-(n-1)*incy;
}              //  Close else.
if (incx == 1)  {
    {
forloop20:
for (j = 1; j <= n; j++) {
if (y[(jy)- 1+ _y_offset] != zero)  {
    temp = alpha*y[(jy)- 1+ _y_offset];
{
forloop10:
for (i = 1; i <= m; i++) {
a[(i)- 1+(j- 1)*lda+ _a_offset] = a[(i)- 1+(j- 1)*lda+ _a_offset]+x[(i)- 1+ _x_offset]*temp;
Dummy.label("Dger",10);
}              //  Close for() loop. 
}
}              // Close if()
jy = jy+incy;
Dummy.label("Dger",20);
}              //  Close for() loop. 
}
}              // Close if()
else  {
  if (incx > 0)  {
    kx = 1;
}              // Close if()
else  {
  kx = 1-(m-1)*incx;
}              //  Close else.
{
forloop40:
for (j = 1; j <= n; j++) {
if (y[(jy)- 1+ _y_offset] != zero)  {
    temp = alpha*y[(jy)- 1+ _y_offset];
ix = kx;
{
forloop30:
for (i = 1; i <= m; i++) {
a[(i)- 1+(j- 1)*lda+ _a_offset] = a[(i)- 1+(j- 1)*lda+ _a_offset]+x[(ix)- 1+ _x_offset]*temp;
ix = ix+incx;
Dummy.label("Dger",30);
}              //  Close for() loop. 
}
}              // Close if()
jy = jy+incy;
Dummy.label("Dger",40);
}              //  Close for() loop. 
}
}              //  Close else.
// *
Dummy.go_to("Dger",999999);
// *
// *     End of DGER  .
// *
Dummy.label("Dger",999999);
return;
   }
} // End class.
