/*
 *  Produced by f2java.  f2java is part of the Fortran-
 *  -to-Java project at the University of Tennessee Netlib
 *  numerical software repository.
 *
 *  Original authorship for the BLAS and LAPACK numerical
 *  routines may be found in the Fortran source, available at
 *  www.netlib.org.
 *
 *  Fortran input file: jlapack.f
 *
 *  The f2j compiler code was written by
 *  David M. Doolin (doolin@cs.utk.edu) and
 *  Keith  Seymour (seymour@cs.utk.edu)
 */

package org.netlib.blas;
import java.lang.*;
import org.netlib.util.*;



public class Dspr {

// *     .. Scalar Arguments ..
// *     .. Array Arguments ..
// *     ..
// *
// *  Purpose
// *  =======
// *
// *  DSPR    performs the symmetric rank 1 operation
// *
// *     A := alpha*x*x' + A,
// *
// *  where alpha is a real scalar, x is an n element vector and A is an
// *  n by n symmetric matrix, supplied in packed form.
// *
// *  Parameters
// *  ==========
// *
// *  UPLO   - CHARACTER*1.
// *           On entry, UPLO specifies whether the upper or lower
// *           triangular part of the matrix A is supplied in the packed
// *           array AP as follows:
// *
// *              UPLO = 'U' or 'u'   The upper triangular part of A is
// *                                  supplied in AP.
// *
// *              UPLO = 'L' or 'l'   The lower triangular part of A is
// *                                  supplied in AP.
// *
// *           Unchanged on exit.
// *
// *  N      - INTEGER.
// *           On entry, N specifies the order of the matrix A.
// *           N must be at least zero.
// *           Unchanged on exit.
// *
// *  ALPHA  - DOUBLE PRECISION.
// *           On entry, ALPHA specifies the scalar alpha.
// *           Unchanged on exit.
// *
// *  X      - DOUBLE PRECISION array of dimension at least
// *           ( 1 + ( n - 1 )*abs( INCX ) ).
// *           Before entry, the incremented array X must contain the n
// *           element vector x.
// *           Unchanged on exit.
// *
// *  INCX   - INTEGER.
// *           On entry, INCX specifies the increment for the elements of
// *           X. INCX must not be zero.
// *           Unchanged on exit.
// *
// *  AP     - DOUBLE PRECISION array of DIMENSION at least
// *           ( ( n*( n + 1 ) )/2 ).
// *           Before entry with  UPLO = 'U' or 'u', the array AP must
// *           contain the upper triangular part of the symmetric matrix
// *           packed sequentially, column by column, so that AP( 1 )
// *           contains a( 1, 1 ), AP( 2 ) and AP( 3 ) contain a( 1, 2 )
// *           and a( 2, 2 ) respectively, and so on. On exit, the array
// *           AP is overwritten by the upper triangular part of the
// *           updated matrix.
// *           Before entry with UPLO = 'L' or 'l', the array AP must
// *           contain the lower triangular part of the symmetric matrix
// *           packed sequentially, column by column, so that AP( 1 )
// *           contains a( 1, 1 ), AP( 2 ) and AP( 3 ) contain a( 2, 1 )
// *           and a( 3, 1 ) respectively, and so on. On exit, the array
// *           AP is overwritten by the lower triangular part of the
// *           updated matrix.
// *
// *
// *  Level 2 Blas routine.
// *
// *  -- Written on 22-October-1986.
// *     Jack Dongarra, Argonne National Lab.
// *     Jeremy Du Croz, Nag Central Office.
// *     Sven Hammarling, Nag Central Office.
// *     Richard Hanson, Sandia National Labs.
// *
// *
// *     .. Parameters ..
static double zero= 0.0e+0;
// *     .. Local Scalars ..
static double temp= 0.0;
static int i= 0;
static int info= 0;
static int ix= 0;
static int j= 0;
static int jx= 0;
static int k= 0;
static int kk= 0;
static int kx= 0;
// *     .. External Functions ..
// *     .. External Subroutines ..
// *     ..
// *     .. Executable Statements ..
// *
// *     Test the input parameters.
// *

public static void dspr (String uplo,
int n,
double alpha,
double [] x, int _x_offset,
int incx,
double [] ap, int _ap_offset)  {

info = 0;
if (!(uplo.toLowerCase().charAt(0) == "U".toLowerCase().charAt(0)) && !(uplo.toLowerCase().charAt(0) == "L".toLowerCase().charAt(0)))  {
    info = 1;
}              // Close if()
else if (n < 0)  {
    info = 2;
}              // Close else if()
else if (incx == 0)  {
    info = 5;
}              // Close else if()
if (info != 0)  {
    Xerbla.xerbla("DSPR  ",info);
Dummy.go_to("Dspr",999999);
}              // Close if()
// *
// *     Quick return if possible.
// *
if ((n == 0) || (alpha == zero))  
    Dummy.go_to("Dspr",999999);
// *
// *     Set the start point in X if the increment is not unity.
// *
if (incx <= 0)  {
    kx = 1-(n-1)*incx;
}              // Close if()
else if (incx != 1)  {
    kx = 1;
}              // Close else if()
// *
// *     Start the operations. In this version the elements of the array AP
// *     are accessed sequentially with one pass through AP.
// *
kk = 1;
if ((uplo.toLowerCase().charAt(0) == "U".toLowerCase().charAt(0)))  {
    // *
// *        Form  A  when upper triangle is stored in AP.
// *
if (incx == 1)  {
    {
forloop20:
for (j = 1; j <= n; j++) {
if (x[(j)- 1+ _x_offset] != zero)  {
    temp = alpha*x[(j)- 1+ _x_offset];
k = kk;
{
forloop10:
for (i = 1; i <= j; i++) {
ap[(k)- 1+ _ap_offset] = ap[(k)- 1+ _ap_offset]+x[(i)- 1+ _x_offset]*temp;
k = k+1;
Dummy.label("Dspr",10);
}              //  Close for() loop. 
}
}              // Close if()
kk = kk+j;
Dummy.label("Dspr",20);
}              //  Close for() loop. 
}
}              // Close if()
else  {
  jx = kx;
{
forloop40:
for (j = 1; j <= n; j++) {
if (x[(jx)- 1+ _x_offset] != zero)  {
    temp = alpha*x[(jx)- 1+ _x_offset];
ix = kx;
{
forloop30:
for (k = kk; k <= kk+j-1; k++) {
ap[(k)- 1+ _ap_offset] = ap[(k)- 1+ _ap_offset]+x[(ix)- 1+ _x_offset]*temp;
ix = ix+incx;
Dummy.label("Dspr",30);
}              //  Close for() loop. 
}
}              // Close if()
jx = jx+incx;
kk = kk+j;
Dummy.label("Dspr",40);
}              //  Close for() loop. 
}
}              //  Close else.
}              // Close if()
else  {
  // *
// *        Form  A  when lower triangle is stored in AP.
// *
if (incx == 1)  {
    {
forloop60:
for (j = 1; j <= n; j++) {
if (x[(j)- 1+ _x_offset] != zero)  {
    temp = alpha*x[(j)- 1+ _x_offset];
k = kk;
{
forloop50:
for (i = j; i <= n; i++) {
ap[(k)- 1+ _ap_offset] = ap[(k)- 1+ _ap_offset]+x[(i)- 1+ _x_offset]*temp;
k = k+1;
Dummy.label("Dspr",50);
}              //  Close for() loop. 
}
}              // Close if()
kk = kk+n-j+1;
Dummy.label("Dspr",60);
}              //  Close for() loop. 
}
}              // Close if()
else  {
  jx = kx;
{
forloop80:
for (j = 1; j <= n; j++) {
if (x[(jx)- 1+ _x_offset] != zero)  {
    temp = alpha*x[(jx)- 1+ _x_offset];
ix = jx;
{
forloop70:
for (k = kk; k <= kk+n-j; k++) {
ap[(k)- 1+ _ap_offset] = ap[(k)- 1+ _ap_offset]+x[(ix)- 1+ _x_offset]*temp;
ix = ix+incx;
Dummy.label("Dspr",70);
}              //  Close for() loop. 
}
}              // Close if()
jx = jx+incx;
kk = kk+n-j+1;
Dummy.label("Dspr",80);
}              //  Close for() loop. 
}
}              //  Close else.
}              //  Close else.
// *
Dummy.go_to("Dspr",999999);
// *
// *     End of DSPR  .
// *
Dummy.label("Dspr",999999);
return;
   }
} // End class.
