/*
 *  Produced by f2java.  f2java is part of the Fortran-
 *  -to-Java project at the University of Tennessee Netlib
 *  numerical software repository.
 *
 *  Original authorship for the BLAS and LAPACK numerical
 *  routines may be found in the Fortran source, available at
 *  www.netlib.org.
 *
 *  Fortran input file: jlapack.f
 *
 *  The f2j compiler code was written by
 *  David M. Doolin (doolin@cs.utk.edu) and
 *  Keith  Seymour (seymour@cs.utk.edu)
 */

package org.netlib.blas;
import java.lang.*;
import org.netlib.util.*;



public class Dspr2 {

// *     .. Scalar Arguments ..
// *     .. Array Arguments ..
// *     ..
// *
// *  Purpose
// *  =======
// *
// *  DSPR2  performs the symmetric rank 2 operation
// *
// *     A := alpha*x*y' + alpha*y*x' + A,
// *
// *  where alpha is a scalar, x and y are n element vectors and A is an
// *  n by n symmetric matrix, supplied in packed form.
// *
// *  Parameters
// *  ==========
// *
// *  UPLO   - CHARACTER*1.
// *           On entry, UPLO specifies whether the upper or lower
// *           triangular part of the matrix A is supplied in the packed
// *           array AP as follows:
// *
// *              UPLO = 'U' or 'u'   The upper triangular part of A is
// *                                  supplied in AP.
// *
// *              UPLO = 'L' or 'l'   The lower triangular part of A is
// *                                  supplied in AP.
// *
// *           Unchanged on exit.
// *
// *  N      - INTEGER.
// *           On entry, N specifies the order of the matrix A.
// *           N must be at least zero.
// *           Unchanged on exit.
// *
// *  ALPHA  - DOUBLE PRECISION.
// *           On entry, ALPHA specifies the scalar alpha.
// *           Unchanged on exit.
// *
// *  X      - DOUBLE PRECISION array of dimension at least
// *           ( 1 + ( n - 1 )*abs( INCX ) ).
// *           Before entry, the incremented array X must contain the n
// *           element vector x.
// *           Unchanged on exit.
// *
// *  INCX   - INTEGER.
// *           On entry, INCX specifies the increment for the elements of
// *           X. INCX must not be zero.
// *           Unchanged on exit.
// *
// *  Y      - DOUBLE PRECISION array of dimension at least
// *           ( 1 + ( n - 1 )*abs( INCY ) ).
// *           Before entry, the incremented array Y must contain the n
// *           element vector y.
// *           Unchanged on exit.
// *
// *  INCY   - INTEGER.
// *           On entry, INCY specifies the increment for the elements of
// *           Y. INCY must not be zero.
// *           Unchanged on exit.
// *
// *  AP     - DOUBLE PRECISION array of DIMENSION at least
// *           ( ( n*( n + 1 ) )/2 ).
// *           Before entry with  UPLO = 'U' or 'u', the array AP must
// *           contain the upper triangular part of the symmetric matrix
// *           packed sequentially, column by column, so that AP( 1 )
// *           contains a( 1, 1 ), AP( 2 ) and AP( 3 ) contain a( 1, 2 )
// *           and a( 2, 2 ) respectively, and so on. On exit, the array
// *           AP is overwritten by the upper triangular part of the
// *           updated matrix.
// *           Before entry with UPLO = 'L' or 'l', the array AP must
// *           contain the lower triangular part of the symmetric matrix
// *           packed sequentially, column by column, so that AP( 1 )
// *           contains a( 1, 1 ), AP( 2 ) and AP( 3 ) contain a( 2, 1 )
// *           and a( 3, 1 ) respectively, and so on. On exit, the array
// *           AP is overwritten by the lower triangular part of the
// *           updated matrix.
// *
// *
// *  Level 2 Blas routine.
// *
// *  -- Written on 22-October-1986.
// *     Jack Dongarra, Argonne National Lab.
// *     Jeremy Du Croz, Nag Central Office.
// *     Sven Hammarling, Nag Central Office.
// *     Richard Hanson, Sandia National Labs.
// *
// *
// *     .. Parameters ..
static double zero= 0.0e+0;
// *     .. Local Scalars ..
static double temp1= 0.0;
static double temp2= 0.0;
static int i= 0;
static int info= 0;
static int ix= 0;
static int iy= 0;
static int j= 0;
static int jx= 0;
static int jy= 0;
static int k= 0;
static int kk= 0;
static int kx= 0;
static int ky= 0;
// *     .. External Functions ..
// *     .. External Subroutines ..
// *     ..
// *     .. Executable Statements ..
// *
// *     Test the input parameters.
// *

public static void dspr2 (String uplo,
int n,
double alpha,
double [] x, int _x_offset,
int incx,
double [] y, int _y_offset,
int incy,
double [] ap, int _ap_offset)  {

info = 0;
if (!(uplo.toLowerCase().charAt(0) == "U".toLowerCase().charAt(0)) && !(uplo.toLowerCase().charAt(0) == "L".toLowerCase().charAt(0)))  {
    info = 1;
}              // Close if()
else if (n < 0)  {
    info = 2;
}              // Close else if()
else if (incx == 0)  {
    info = 5;
}              // Close else if()
else if (incy == 0)  {
    info = 7;
}              // Close else if()
if (info != 0)  {
    Xerbla.xerbla("DSPR2 ",info);
Dummy.go_to("Dspr2",999999);
}              // Close if()
// *
// *     Quick return if possible.
// *
if ((n == 0) || (alpha == zero))  
    Dummy.go_to("Dspr2",999999);
// *
// *     Set up the start points in X and Y if the increments are not both
// *     unity.
// *
if ((incx != 1) || (incy != 1))  {
    if (incx > 0)  {
    kx = 1;
}              // Close if()
else  {
  kx = 1-(n-1)*incx;
}              //  Close else.
if (incy > 0)  {
    ky = 1;
}              // Close if()
else  {
  ky = 1-(n-1)*incy;
}              //  Close else.
jx = kx;
jy = ky;
}              // Close if()
// *
// *     Start the operations. In this version the elements of the array AP
// *     are accessed sequentially with one pass through AP.
// *
kk = 1;
if ((uplo.toLowerCase().charAt(0) == "U".toLowerCase().charAt(0)))  {
    // *
// *        Form  A  when upper triangle is stored in AP.
// *
if ((incx == 1) && (incy == 1))  {
    {
forloop20:
for (j = 1; j <= n; j++) {
if ((x[(j)- 1+ _x_offset] != zero) || (y[(j)- 1+ _y_offset] != zero))  {
    temp1 = alpha*y[(j)- 1+ _y_offset];
temp2 = alpha*x[(j)- 1+ _x_offset];
k = kk;
{
forloop10:
for (i = 1; i <= j; i++) {
ap[(k)- 1+ _ap_offset] = ap[(k)- 1+ _ap_offset]+x[(i)- 1+ _x_offset]*temp1+y[(i)- 1+ _y_offset]*temp2;
k = k+1;
Dummy.label("Dspr2",10);
}              //  Close for() loop. 
}
}              // Close if()
kk = kk+j;
Dummy.label("Dspr2",20);
}              //  Close for() loop. 
}
}              // Close if()
else  {
  {
forloop40:
for (j = 1; j <= n; j++) {
if ((x[(jx)- 1+ _x_offset] != zero) || (y[(jy)- 1+ _y_offset] != zero))  {
    temp1 = alpha*y[(jy)- 1+ _y_offset];
temp2 = alpha*x[(jx)- 1+ _x_offset];
ix = kx;
iy = ky;
{
forloop30:
for (k = kk; k <= kk+j-1; k++) {
ap[(k)- 1+ _ap_offset] = ap[(k)- 1+ _ap_offset]+x[(ix)- 1+ _x_offset]*temp1+y[(iy)- 1+ _y_offset]*temp2;
ix = ix+incx;
iy = iy+incy;
Dummy.label("Dspr2",30);
}              //  Close for() loop. 
}
}              // Close if()
jx = jx+incx;
jy = jy+incy;
kk = kk+j;
Dummy.label("Dspr2",40);
}              //  Close for() loop. 
}
}              //  Close else.
}              // Close if()
else  {
  // *
// *        Form  A  when lower triangle is stored in AP.
// *
if ((incx == 1) && (incy == 1))  {
    {
forloop60:
for (j = 1; j <= n; j++) {
if ((x[(j)- 1+ _x_offset] != zero) || (y[(j)- 1+ _y_offset] != zero))  {
    temp1 = alpha*y[(j)- 1+ _y_offset];
temp2 = alpha*x[(j)- 1+ _x_offset];
k = kk;
{
forloop50:
for (i = j; i <= n; i++) {
ap[(k)- 1+ _ap_offset] = ap[(k)- 1+ _ap_offset]+x[(i)- 1+ _x_offset]*temp1+y[(i)- 1+ _y_offset]*temp2;
k = k+1;
Dummy.label("Dspr2",50);
}              //  Close for() loop. 
}
}              // Close if()
kk = kk+n-j+1;
Dummy.label("Dspr2",60);
}              //  Close for() loop. 
}
}              // Close if()
else  {
  {
forloop80:
for (j = 1; j <= n; j++) {
if ((x[(jx)- 1+ _x_offset] != zero) || (y[(jy)- 1+ _y_offset] != zero))  {
    temp1 = alpha*y[(jy)- 1+ _y_offset];
temp2 = alpha*x[(jx)- 1+ _x_offset];
ix = jx;
iy = jy;
{
forloop70:
for (k = kk; k <= kk+n-j; k++) {
ap[(k)- 1+ _ap_offset] = ap[(k)- 1+ _ap_offset]+x[(ix)- 1+ _x_offset]*temp1+y[(iy)- 1+ _y_offset]*temp2;
ix = ix+incx;
iy = iy+incy;
Dummy.label("Dspr2",70);
}              //  Close for() loop. 
}
}              // Close if()
jx = jx+incx;
jy = jy+incy;
kk = kk+n-j+1;
Dummy.label("Dspr2",80);
}              //  Close for() loop. 
}
}              //  Close else.
}              //  Close else.
// *
Dummy.go_to("Dspr2",999999);
// *
// *     End of DSPR2 .
// *
Dummy.label("Dspr2",999999);
return;
   }
} // End class.
