/*
 *  Produced by f2java.  f2java is part of the Fortran-
 *  -to-Java project at the University of Tennessee Netlib
 *  numerical software repository.
 *
 *  Original authorship for the BLAS and LAPACK numerical
 *  routines may be found in the Fortran source, available at
 *  www.netlib.org.
 *
 *  Fortran input file: jlapack.f
 *
 *  The f2j compiler code was written by
 *  David M. Doolin (doolin@cs.utk.edu) and
 *  Keith  Seymour (seymour@cs.utk.edu)
 */

package org.netlib.blas;
import java.lang.*;
import org.netlib.util.*;



public class Dsymv {

// *     .. Scalar Arguments ..
// *     .. Array Arguments ..
// *     ..
// *
// *  Purpose
// *  =======
// *
// *  DSYMV  performs the matrix-vector  operation
// *
// *     y := alpha*A*x + beta*y,
// *
// *  where alpha and beta are scalars, x and y are n element vectors and
// *  A is an n by n symmetric matrix.
// *
// *  Parameters
// *  ==========
// *
// *  UPLO   - CHARACTER*1.
// *           On entry, UPLO specifies whether the upper or lower
// *           triangular part of the array A is to be referenced as
// *           follows:
// *
// *              UPLO = 'U' or 'u'   Only the upper triangular part of A
// *                                  is to be referenced.
// *
// *              UPLO = 'L' or 'l'   Only the lower triangular part of A
// *                                  is to be referenced.
// *
// *           Unchanged on exit.
// *
// *  N      - INTEGER.
// *           On entry, N specifies the order of the matrix A.
// *           N must be at least zero.
// *           Unchanged on exit.
// *
// *  ALPHA  - DOUBLE PRECISION.
// *           On entry, ALPHA specifies the scalar alpha.
// *           Unchanged on exit.
// *
// *  A      - DOUBLE PRECISION array of DIMENSION ( LDA, n ).
// *           Before entry with  UPLO = 'U' or 'u', the leading n by n
// *           upper triangular part of the array A must contain the upper
// *           triangular part of the symmetric matrix and the strictly
// *           lower triangular part of A is not referenced.
// *           Before entry with UPLO = 'L' or 'l', the leading n by n
// *           lower triangular part of the array A must contain the lower
// *           triangular part of the symmetric matrix and the strictly
// *           upper triangular part of A is not referenced.
// *           Unchanged on exit.
// *
// *  LDA    - INTEGER.
// *           On entry, LDA specifies the first dimension of A as declared
// *           in the calling (sub) program. LDA must be at least
// *           max( 1, n ).
// *           Unchanged on exit.
// *
// *  X      - DOUBLE PRECISION array of dimension at least
// *           ( 1 + ( n - 1 )*abs( INCX ) ).
// *           Before entry, the incremented array X must contain the n
// *           element vector x.
// *           Unchanged on exit.
// *
// *  INCX   - INTEGER.
// *           On entry, INCX specifies the increment for the elements of
// *           X. INCX must not be zero.
// *           Unchanged on exit.
// *
// *  BETA   - DOUBLE PRECISION.
// *           On entry, BETA specifies the scalar beta. When BETA is
// *           supplied as zero then Y need not be set on input.
// *           Unchanged on exit.
// *
// *  Y      - DOUBLE PRECISION array of dimension at least
// *           ( 1 + ( n - 1 )*abs( INCY ) ).
// *           Before entry, the incremented array Y must contain the n
// *           element vector y. On exit, Y is overwritten by the updated
// *           vector y.
// *
// *  INCY   - INTEGER.
// *           On entry, INCY specifies the increment for the elements of
// *           Y. INCY must not be zero.
// *           Unchanged on exit.
// *
// *
// *  Level 2 Blas routine.
// *
// *  -- Written on 22-October-1986.
// *     Jack Dongarra, Argonne National Lab.
// *     Jeremy Du Croz, Nag Central Office.
// *     Sven Hammarling, Nag Central Office.
// *     Richard Hanson, Sandia National Labs.
// *
// *
// *     .. Parameters ..
static double one= 1.0e+0;
static double zero= 0.0e+0;
// *     .. Local Scalars ..
static double temp1= 0.0;
static double temp2= 0.0;
static int i= 0;
static int info= 0;
static int ix= 0;
static int iy= 0;
static int j= 0;
static int jx= 0;
static int jy= 0;
static int kx= 0;
static int ky= 0;
// *     .. External Functions ..
// *     .. External Subroutines ..
// *     .. Intrinsic Functions ..
// *     ..
// *     .. Executable Statements ..
// *
// *     Test the input parameters.
// *

public static void dsymv (String uplo,
int n,
double alpha,
double [] a, int _a_offset,
int lda,
double [] x, int _x_offset,
int incx,
double beta,
double [] y, int _y_offset,
int incy)  {

info = 0;
if (!(uplo.toLowerCase().charAt(0) == "U".toLowerCase().charAt(0)) && !(uplo.toLowerCase().charAt(0) == "L".toLowerCase().charAt(0)))  {
    info = 1;
}              // Close if()
else if (n < 0)  {
    info = 2;
}              // Close else if()
else if (lda < Math.max(1, n) )  {
    info = 5;
}              // Close else if()
else if (incx == 0)  {
    info = 7;
}              // Close else if()
else if (incy == 0)  {
    info = 10;
}              // Close else if()
if (info != 0)  {
    Xerbla.xerbla("DSYMV ",info);
Dummy.go_to("Dsymv",999999);
}              // Close if()
// *
// *     Quick return if possible.
// *
if ((n == 0) || ((alpha == zero) && (beta == one)))  
    Dummy.go_to("Dsymv",999999);
// *
// *     Set up the start points in  X  and  Y.
// *
if (incx > 0)  {
    kx = 1;
}              // Close if()
else  {
  kx = 1-(n-1)*incx;
}              //  Close else.
if (incy > 0)  {
    ky = 1;
}              // Close if()
else  {
  ky = 1-(n-1)*incy;
}              //  Close else.
// *
// *     Start the operations. In this version the elements of A are
// *     accessed sequentially with one pass through the triangular part
// *     of A.
// *
// *     First form  y := beta*y.
// *
if (beta != one)  {
    if (incy == 1)  {
    if (beta == zero)  {
    {
forloop10:
for (i = 1; i <= n; i++) {
y[(i)- 1+ _y_offset] = zero;
Dummy.label("Dsymv",10);
}              //  Close for() loop. 
}
}              // Close if()
else  {
  {
forloop20:
for (i = 1; i <= n; i++) {
y[(i)- 1+ _y_offset] = beta*y[(i)- 1+ _y_offset];
Dummy.label("Dsymv",20);
}              //  Close for() loop. 
}
}              //  Close else.
}              // Close if()
else  {
  iy = ky;
if (beta == zero)  {
    {
forloop30:
for (i = 1; i <= n; i++) {
y[(iy)- 1+ _y_offset] = zero;
iy = iy+incy;
Dummy.label("Dsymv",30);
}              //  Close for() loop. 
}
}              // Close if()
else  {
  {
forloop40:
for (i = 1; i <= n; i++) {
y[(iy)- 1+ _y_offset] = beta*y[(iy)- 1+ _y_offset];
iy = iy+incy;
Dummy.label("Dsymv",40);
}              //  Close for() loop. 
}
}              //  Close else.
}              //  Close else.
}              // Close if()
if (alpha == zero)  
    Dummy.go_to("Dsymv",999999);
if ((uplo.toLowerCase().charAt(0) == "U".toLowerCase().charAt(0)))  {
    // *
// *        Form  y  when A is stored in upper triangle.
// *
if ((incx == 1) && (incy == 1))  {
    {
forloop60:
for (j = 1; j <= n; j++) {
temp1 = alpha*x[(j)- 1+ _x_offset];
temp2 = zero;
{
forloop50:
for (i = 1; i <= j-1; i++) {
y[(i)- 1+ _y_offset] = y[(i)- 1+ _y_offset]+temp1*a[(i)- 1+(j- 1)*lda+ _a_offset];
temp2 = temp2+a[(i)- 1+(j- 1)*lda+ _a_offset]*x[(i)- 1+ _x_offset];
Dummy.label("Dsymv",50);
}              //  Close for() loop. 
}
y[(j)- 1+ _y_offset] = y[(j)- 1+ _y_offset]+temp1*a[(j)- 1+(j- 1)*lda+ _a_offset]+alpha*temp2;
Dummy.label("Dsymv",60);
}              //  Close for() loop. 
}
}              // Close if()
else  {
  jx = kx;
jy = ky;
{
forloop80:
for (j = 1; j <= n; j++) {
temp1 = alpha*x[(jx)- 1+ _x_offset];
temp2 = zero;
ix = kx;
iy = ky;
{
forloop70:
for (i = 1; i <= j-1; i++) {
y[(iy)- 1+ _y_offset] = y[(iy)- 1+ _y_offset]+temp1*a[(i)- 1+(j- 1)*lda+ _a_offset];
temp2 = temp2+a[(i)- 1+(j- 1)*lda+ _a_offset]*x[(ix)- 1+ _x_offset];
ix = ix+incx;
iy = iy+incy;
Dummy.label("Dsymv",70);
}              //  Close for() loop. 
}
y[(jy)- 1+ _y_offset] = y[(jy)- 1+ _y_offset]+temp1*a[(j)- 1+(j- 1)*lda+ _a_offset]+alpha*temp2;
jx = jx+incx;
jy = jy+incy;
Dummy.label("Dsymv",80);
}              //  Close for() loop. 
}
}              //  Close else.
}              // Close if()
else  {
  // *
// *        Form  y  when A is stored in lower triangle.
// *
if ((incx == 1) && (incy == 1))  {
    {
forloop100:
for (j = 1; j <= n; j++) {
temp1 = alpha*x[(j)- 1+ _x_offset];
temp2 = zero;
y[(j)- 1+ _y_offset] = y[(j)- 1+ _y_offset]+temp1*a[(j)- 1+(j- 1)*lda+ _a_offset];
{
forloop90:
for (i = j+1; i <= n; i++) {
y[(i)- 1+ _y_offset] = y[(i)- 1+ _y_offset]+temp1*a[(i)- 1+(j- 1)*lda+ _a_offset];
temp2 = temp2+a[(i)- 1+(j- 1)*lda+ _a_offset]*x[(i)- 1+ _x_offset];
Dummy.label("Dsymv",90);
}              //  Close for() loop. 
}
y[(j)- 1+ _y_offset] = y[(j)- 1+ _y_offset]+alpha*temp2;
Dummy.label("Dsymv",100);
}              //  Close for() loop. 
}
}              // Close if()
else  {
  jx = kx;
jy = ky;
{
forloop120:
for (j = 1; j <= n; j++) {
temp1 = alpha*x[(jx)- 1+ _x_offset];
temp2 = zero;
y[(jy)- 1+ _y_offset] = y[(jy)- 1+ _y_offset]+temp1*a[(j)- 1+(j- 1)*lda+ _a_offset];
ix = jx;
iy = jy;
{
forloop110:
for (i = j+1; i <= n; i++) {
ix = ix+incx;
iy = iy+incy;
y[(iy)- 1+ _y_offset] = y[(iy)- 1+ _y_offset]+temp1*a[(i)- 1+(j- 1)*lda+ _a_offset];
temp2 = temp2+a[(i)- 1+(j- 1)*lda+ _a_offset]*x[(ix)- 1+ _x_offset];
Dummy.label("Dsymv",110);
}              //  Close for() loop. 
}
y[(jy)- 1+ _y_offset] = y[(jy)- 1+ _y_offset]+alpha*temp2;
jx = jx+incx;
jy = jy+incy;
Dummy.label("Dsymv",120);
}              //  Close for() loop. 
}
}              //  Close else.
}              //  Close else.
// *
Dummy.go_to("Dsymv",999999);
// *
// *     End of DSYMV .
// *
Dummy.label("Dsymv",999999);
return;
   }
} // End class.
