/*
 *  Produced by f2java.  f2java is part of the Fortran-
 *  -to-Java project at the University of Tennessee Netlib
 *  numerical software repository.
 *
 *  Original authorship for the BLAS and LAPACK numerical
 *  routines may be found in the Fortran source, available at
 *  www.netlib.org.
 *
 *  Fortran input file: jlapack.f
 *
 *  The f2j compiler code was written by
 *  David M. Doolin (doolin@cs.utk.edu) and
 *  Keith  Seymour (seymour@cs.utk.edu)
 */

package org.netlib.blas;
import java.lang.*;
import org.netlib.util.*;



public class Dsyr {

// *     .. Scalar Arguments ..
// *     .. Array Arguments ..
// *     ..
// *
// *  Purpose
// *  =======
// *
// *  DSYR   performs the symmetric rank 1 operation
// *
// *     A := alpha*x*x' + A,
// *
// *  where alpha is a real scalar, x is an n element vector and A is an
// *  n by n symmetric matrix.
// *
// *  Parameters
// *  ==========
// *
// *  UPLO   - CHARACTER*1.
// *           On entry, UPLO specifies whether the upper or lower
// *           triangular part of the array A is to be referenced as
// *           follows:
// *
// *              UPLO = 'U' or 'u'   Only the upper triangular part of A
// *                                  is to be referenced.
// *
// *              UPLO = 'L' or 'l'   Only the lower triangular part of A
// *                                  is to be referenced.
// *
// *           Unchanged on exit.
// *
// *  N      - INTEGER.
// *           On entry, N specifies the order of the matrix A.
// *           N must be at least zero.
// *           Unchanged on exit.
// *
// *  ALPHA  - DOUBLE PRECISION.
// *           On entry, ALPHA specifies the scalar alpha.
// *           Unchanged on exit.
// *
// *  X      - DOUBLE PRECISION array of dimension at least
// *           ( 1 + ( n - 1 )*abs( INCX ) ).
// *           Before entry, the incremented array X must contain the n
// *           element vector x.
// *           Unchanged on exit.
// *
// *  INCX   - INTEGER.
// *           On entry, INCX specifies the increment for the elements of
// *           X. INCX must not be zero.
// *           Unchanged on exit.
// *
// *  A      - DOUBLE PRECISION array of DIMENSION ( LDA, n ).
// *           Before entry with  UPLO = 'U' or 'u', the leading n by n
// *           upper triangular part of the array A must contain the upper
// *           triangular part of the symmetric matrix and the strictly
// *           lower triangular part of A is not referenced. On exit, the
// *           upper triangular part of the array A is overwritten by the
// *           upper triangular part of the updated matrix.
// *           Before entry with UPLO = 'L' or 'l', the leading n by n
// *           lower triangular part of the array A must contain the lower
// *           triangular part of the symmetric matrix and the strictly
// *           upper triangular part of A is not referenced. On exit, the
// *           lower triangular part of the array A is overwritten by the
// *           lower triangular part of the updated matrix.
// *
// *  LDA    - INTEGER.
// *           On entry, LDA specifies the first dimension of A as declared
// *           in the calling (sub) program. LDA must be at least
// *           max( 1, n ).
// *           Unchanged on exit.
// *
// *
// *  Level 2 Blas routine.
// *
// *  -- Written on 22-October-1986.
// *     Jack Dongarra, Argonne National Lab.
// *     Jeremy Du Croz, Nag Central Office.
// *     Sven Hammarling, Nag Central Office.
// *     Richard Hanson, Sandia National Labs.
// *
// *
// *     .. Parameters ..
static double zero= 0.0e+0;
// *     .. Local Scalars ..
static double temp= 0.0;
static int i= 0;
static int info= 0;
static int ix= 0;
static int j= 0;
static int jx= 0;
static int kx= 0;
// *     .. External Functions ..
// *     .. External Subroutines ..
// *     .. Intrinsic Functions ..
// *     ..
// *     .. Executable Statements ..
// *
// *     Test the input parameters.
// *

public static void dsyr (String uplo,
int n,
double alpha,
double [] x, int _x_offset,
int incx,
double [] a, int _a_offset,
int lda)  {

info = 0;
if (!(uplo.toLowerCase().charAt(0) == "U".toLowerCase().charAt(0)) && !(uplo.toLowerCase().charAt(0) == "L".toLowerCase().charAt(0)))  {
    info = 1;
}              // Close if()
else if (n < 0)  {
    info = 2;
}              // Close else if()
else if (incx == 0)  {
    info = 5;
}              // Close else if()
else if (lda < Math.max(1, n) )  {
    info = 7;
}              // Close else if()
if (info != 0)  {
    Xerbla.xerbla("DSYR  ",info);
Dummy.go_to("Dsyr",999999);
}              // Close if()
// *
// *     Quick return if possible.
// *
if ((n == 0) || (alpha == zero))  
    Dummy.go_to("Dsyr",999999);
// *
// *     Set the start point in X if the increment is not unity.
// *
if (incx <= 0)  {
    kx = 1-(n-1)*incx;
}              // Close if()
else if (incx != 1)  {
    kx = 1;
}              // Close else if()
// *
// *     Start the operations. In this version the elements of A are
// *     accessed sequentially with one pass through the triangular part
// *     of A.
// *
if ((uplo.toLowerCase().charAt(0) == "U".toLowerCase().charAt(0)))  {
    // *
// *        Form  A  when A is stored in upper triangle.
// *
if (incx == 1)  {
    {
forloop20:
for (j = 1; j <= n; j++) {
if (x[(j)- 1+ _x_offset] != zero)  {
    temp = alpha*x[(j)- 1+ _x_offset];
{
forloop10:
for (i = 1; i <= j; i++) {
a[(i)- 1+(j- 1)*lda+ _a_offset] = a[(i)- 1+(j- 1)*lda+ _a_offset]+x[(i)- 1+ _x_offset]*temp;
Dummy.label("Dsyr",10);
}              //  Close for() loop. 
}
}              // Close if()
Dummy.label("Dsyr",20);
}              //  Close for() loop. 
}
}              // Close if()
else  {
  jx = kx;
{
forloop40:
for (j = 1; j <= n; j++) {
if (x[(jx)- 1+ _x_offset] != zero)  {
    temp = alpha*x[(jx)- 1+ _x_offset];
ix = kx;
{
forloop30:
for (i = 1; i <= j; i++) {
a[(i)- 1+(j- 1)*lda+ _a_offset] = a[(i)- 1+(j- 1)*lda+ _a_offset]+x[(ix)- 1+ _x_offset]*temp;
ix = ix+incx;
Dummy.label("Dsyr",30);
}              //  Close for() loop. 
}
}              // Close if()
jx = jx+incx;
Dummy.label("Dsyr",40);
}              //  Close for() loop. 
}
}              //  Close else.
}              // Close if()
else  {
  // *
// *        Form  A  when A is stored in lower triangle.
// *
if (incx == 1)  {
    {
forloop60:
for (j = 1; j <= n; j++) {
if (x[(j)- 1+ _x_offset] != zero)  {
    temp = alpha*x[(j)- 1+ _x_offset];
{
forloop50:
for (i = j; i <= n; i++) {
a[(i)- 1+(j- 1)*lda+ _a_offset] = a[(i)- 1+(j- 1)*lda+ _a_offset]+x[(i)- 1+ _x_offset]*temp;
Dummy.label("Dsyr",50);
}              //  Close for() loop. 
}
}              // Close if()
Dummy.label("Dsyr",60);
}              //  Close for() loop. 
}
}              // Close if()
else  {
  jx = kx;
{
forloop80:
for (j = 1; j <= n; j++) {
if (x[(jx)- 1+ _x_offset] != zero)  {
    temp = alpha*x[(jx)- 1+ _x_offset];
ix = jx;
{
forloop70:
for (i = j; i <= n; i++) {
a[(i)- 1+(j- 1)*lda+ _a_offset] = a[(i)- 1+(j- 1)*lda+ _a_offset]+x[(ix)- 1+ _x_offset]*temp;
ix = ix+incx;
Dummy.label("Dsyr",70);
}              //  Close for() loop. 
}
}              // Close if()
jx = jx+incx;
Dummy.label("Dsyr",80);
}              //  Close for() loop. 
}
}              //  Close else.
}              //  Close else.
// *
Dummy.go_to("Dsyr",999999);
// *
// *     End of DSYR  .
// *
Dummy.label("Dsyr",999999);
return;
   }
} // End class.
