/*
 *  Produced by f2java.  f2java is part of the Fortran-
 *  -to-Java project at the University of Tennessee Netlib
 *  numerical software repository.
 *
 *  Original authorship for the BLAS and LAPACK numerical
 *  routines may be found in the Fortran source, available at
 *  www.netlib.org.
 *
 *  Fortran input file: jlapack.f
 *
 *  The f2j compiler code was written by
 *  David M. Doolin (doolin@cs.utk.edu) and
 *  Keith  Seymour (seymour@cs.utk.edu)
 */

package org.netlib.blas;
import java.lang.*;
import org.netlib.util.*;



public class Dtbmv {

// *     .. Scalar Arguments ..
// *     .. Array Arguments ..
// *     ..
// *
// *  Purpose
// *  =======
// *
// *  DTBMV  performs one of the matrix-vector operations
// *
// *     x := A*x,   or   x := A'*x,
// *
// *  where x is an n element vector and  A is an n by n unit, or non-unit,
// *  upper or lower triangular band matrix, with ( k + 1 ) diagonals.
// *
// *  Parameters
// *  ==========
// *
// *  UPLO   - CHARACTER*1.
// *           On entry, UPLO specifies whether the matrix is an upper or
// *           lower triangular matrix as follows:
// *
// *              UPLO = 'U' or 'u'   A is an upper triangular matrix.
// *
// *              UPLO = 'L' or 'l'   A is a lower triangular matrix.
// *
// *           Unchanged on exit.
// *
// *  TRANS  - CHARACTER*1.
// *           On entry, TRANS specifies the operation to be performed as
// *           follows:
// *
// *              TRANS = 'N' or 'n'   x := A*x.
// *
// *              TRANS = 'T' or 't'   x := A'*x.
// *
// *              TRANS = 'C' or 'c'   x := A'*x.
// *
// *           Unchanged on exit.
// *
// *  DIAG   - CHARACTER*1.
// *           On entry, DIAG specifies whether or not A is unit
// *           triangular as follows:
// *
// *              DIAG = 'U' or 'u'   A is assumed to be unit triangular.
// *
// *              DIAG = 'N' or 'n'   A is not assumed to be unit
// *                                  triangular.
// *
// *           Unchanged on exit.
// *
// *  N      - INTEGER.
// *           On entry, N specifies the order of the matrix A.
// *           N must be at least zero.
// *           Unchanged on exit.
// *
// *  K      - INTEGER.
// *           On entry with UPLO = 'U' or 'u', K specifies the number of
// *           super-diagonals of the matrix A.
// *           On entry with UPLO = 'L' or 'l', K specifies the number of
// *           sub-diagonals of the matrix A.
// *           K must satisfy  0 .le. K.
// *           Unchanged on exit.
// *
// *  A      - DOUBLE PRECISION array of DIMENSION ( LDA, n ).
// *           Before entry with UPLO = 'U' or 'u', the leading ( k + 1 )
// *           by n part of the array A must contain the upper triangular
// *           band part of the matrix of coefficients, supplied column by
// *           column, with the leading diagonal of the matrix in row
// *           ( k + 1 ) of the array, the first super-diagonal starting at
// *           position 2 in row k, and so on. The top left k by k triangle
// *           of the array A is not referenced.
// *           The following program segment will transfer an upper
// *           triangular band matrix from conventional full matrix storage
// *           to band storage:
// *
// *                 DO 20, J = 1, N
// *                    M = K + 1 - J
// *                    DO 10, I = MAX( 1, J - K ), J
// *                       A( M + I, J ) = matrix( I, J )
// *              10    CONTINUE
// *              20 CONTINUE
// *
// *           Before entry with UPLO = 'L' or 'l', the leading ( k + 1 )
// *           by n part of the array A must contain the lower triangular
// *           band part of the matrix of coefficients, supplied column by
// *           column, with the leading diagonal of the matrix in row 1 of
// *           the array, the first sub-diagonal starting at position 1 in
// *           row 2, and so on. The bottom right k by k triangle of the
// *           array A is not referenced.
// *           The following program segment will transfer a lower
// *           triangular band matrix from conventional full matrix storage
// *           to band storage:
// *
// *                 DO 20, J = 1, N
// *                    M = 1 - J
// *                    DO 10, I = J, MIN( N, J + K )
// *                       A( M + I, J ) = matrix( I, J )
// *              10    CONTINUE
// *              20 CONTINUE
// *
// *           Note that when DIAG = 'U' or 'u' the elements of the array A
// *           corresponding to the diagonal elements of the matrix are not
// *           referenced, but are assumed to be unity.
// *           Unchanged on exit.
// *
// *  LDA    - INTEGER.
// *           On entry, LDA specifies the first dimension of A as declared
// *           in the calling (sub) program. LDA must be at least
// *           ( k + 1 ).
// *           Unchanged on exit.
// *
// *  X      - DOUBLE PRECISION array of dimension at least
// *           ( 1 + ( n - 1 )*abs( INCX ) ).
// *           Before entry, the incremented array X must contain the n
// *           element vector x. On exit, X is overwritten with the
// *           tranformed vector x.
// *
// *  INCX   - INTEGER.
// *           On entry, INCX specifies the increment for the elements of
// *           X. INCX must not be zero.
// *           Unchanged on exit.
// *
// *
// *  Level 2 Blas routine.
// *
// *  -- Written on 22-October-1986.
// *     Jack Dongarra, Argonne National Lab.
// *     Jeremy Du Croz, Nag Central Office.
// *     Sven Hammarling, Nag Central Office.
// *     Richard Hanson, Sandia National Labs.
// *
// *
// *     .. Parameters ..
static double zero= 0.0e+0;
// *     .. Local Scalars ..
static double temp= 0.0;
static int i= 0;
static int info= 0;
static int ix= 0;
static int j= 0;
static int jx= 0;
static int kplus1= 0;
static int kx= 0;
static int l= 0;
static boolean nounit= false;
// *     .. External Functions ..
// *     .. External Subroutines ..
// *     .. Intrinsic Functions ..
// *     ..
// *     .. Executable Statements ..
// *
// *     Test the input parameters.
// *

public static void dtbmv (String uplo,
String trans,
String diag,
int n,
int k,
double [] a, int _a_offset,
int lda,
double [] x, int _x_offset,
int incx)  {

info = 0;
if (!(uplo.toLowerCase().charAt(0) == "U".toLowerCase().charAt(0)) && !(uplo.toLowerCase().charAt(0) == "L".toLowerCase().charAt(0)))  {
    info = 1;
}              // Close if()
else if (!(trans.toLowerCase().charAt(0) == "N".toLowerCase().charAt(0)) && !(trans.toLowerCase().charAt(0) == "T".toLowerCase().charAt(0)) && !(trans.toLowerCase().charAt(0) == "C".toLowerCase().charAt(0)))  {
    info = 2;
}              // Close else if()
else if (!(diag.toLowerCase().charAt(0) == "U".toLowerCase().charAt(0)) && !(diag.toLowerCase().charAt(0) == "N".toLowerCase().charAt(0)))  {
    info = 3;
}              // Close else if()
else if (n < 0)  {
    info = 4;
}              // Close else if()
else if (k < 0)  {
    info = 5;
}              // Close else if()
else if (lda < (k+1))  {
    info = 7;
}              // Close else if()
else if (incx == 0)  {
    info = 9;
}              // Close else if()
if (info != 0)  {
    Xerbla.xerbla("DTBMV ",info);
Dummy.go_to("Dtbmv",999999);
}              // Close if()
// *
// *     Quick return if possible.
// *
if (n == 0)  
    Dummy.go_to("Dtbmv",999999);
// *
nounit = (diag.toLowerCase().charAt(0) == "N".toLowerCase().charAt(0));
// *
// *     Set up the start point in X if the increment is not unity. This
// *     will be  ( N - 1 )*INCX   too small for descending loops.
// *
if (incx <= 0)  {
    kx = 1-(n-1)*incx;
}              // Close if()
else if (incx != 1)  {
    kx = 1;
}              // Close else if()
// *
// *     Start the operations. In this version the elements of A are
// *     accessed sequentially with one pass through A.
// *
if ((trans.toLowerCase().charAt(0) == "N".toLowerCase().charAt(0)))  {
    // *
// *         Form  x := A*x.
// *
if ((uplo.toLowerCase().charAt(0) == "U".toLowerCase().charAt(0)))  {
    kplus1 = k+1;
if (incx == 1)  {
    {
forloop20:
for (j = 1; j <= n; j++) {
if (x[(j)- 1+ _x_offset] != zero)  {
    temp = x[(j)- 1+ _x_offset];
l = kplus1-j;
{
forloop10:
for (i = (int)(Math.max(1, j-k) ); i <= j-1; i++) {
x[(i)- 1+ _x_offset] = x[(i)- 1+ _x_offset]+temp*a[(l+i)- 1+(j- 1)*lda+ _a_offset];
Dummy.label("Dtbmv",10);
}              //  Close for() loop. 
}
if (nounit)  
    x[(j)- 1+ _x_offset] = x[(j)- 1+ _x_offset]*a[(kplus1)- 1+(j- 1)*lda+ _a_offset];
}              // Close if()
Dummy.label("Dtbmv",20);
}              //  Close for() loop. 
}
}              // Close if()
else  {
  jx = kx;
{
forloop40:
for (j = 1; j <= n; j++) {
if (x[(jx)- 1+ _x_offset] != zero)  {
    temp = x[(jx)- 1+ _x_offset];
ix = kx;
l = kplus1-j;
{
forloop30:
for (i = (int)(Math.max(1, j-k) ); i <= j-1; i++) {
x[(ix)- 1+ _x_offset] = x[(ix)- 1+ _x_offset]+temp*a[(l+i)- 1+(j- 1)*lda+ _a_offset];
ix = ix+incx;
Dummy.label("Dtbmv",30);
}              //  Close for() loop. 
}
if (nounit)  
    x[(jx)- 1+ _x_offset] = x[(jx)- 1+ _x_offset]*a[(kplus1)- 1+(j- 1)*lda+ _a_offset];
}              // Close if()
jx = jx+incx;
if (j > k)  
    kx = kx+incx;
Dummy.label("Dtbmv",40);
}              //  Close for() loop. 
}
}              //  Close else.
}              // Close if()
else  {
  if (incx == 1)  {
    {
int _j_inc = -1;
forloop60:
for (j = n; j >= 1; j += _j_inc) {
if (x[(j)- 1+ _x_offset] != zero)  {
    temp = x[(j)- 1+ _x_offset];
l = 1-j;
{
int _i_inc = -1;
forloop50:
for (i = (int)(Math.min(n, j+k) ); i >= j+1; i += _i_inc) {
x[(i)- 1+ _x_offset] = x[(i)- 1+ _x_offset]+temp*a[(l+i)- 1+(j- 1)*lda+ _a_offset];
Dummy.label("Dtbmv",50);
}              //  Close for() loop. 
}
if (nounit)  
    x[(j)- 1+ _x_offset] = x[(j)- 1+ _x_offset]*a[(1)- 1+(j- 1)*lda+ _a_offset];
}              // Close if()
Dummy.label("Dtbmv",60);
}              //  Close for() loop. 
}
}              // Close if()
else  {
  kx = kx+(n-1)*incx;
jx = kx;
{
int _j_inc = -1;
forloop80:
for (j = n; j >= 1; j += _j_inc) {
if (x[(jx)- 1+ _x_offset] != zero)  {
    temp = x[(jx)- 1+ _x_offset];
ix = kx;
l = 1-j;
{
int _i_inc = -1;
forloop70:
for (i = (int)(Math.min(n, j+k) ); i >= j+1; i += _i_inc) {
x[(ix)- 1+ _x_offset] = x[(ix)- 1+ _x_offset]+temp*a[(l+i)- 1+(j- 1)*lda+ _a_offset];
ix = ix-incx;
Dummy.label("Dtbmv",70);
}              //  Close for() loop. 
}
if (nounit)  
    x[(jx)- 1+ _x_offset] = x[(jx)- 1+ _x_offset]*a[(1)- 1+(j- 1)*lda+ _a_offset];
}              // Close if()
jx = jx-incx;
if ((n-j) >= k)  
    kx = kx-incx;
Dummy.label("Dtbmv",80);
}              //  Close for() loop. 
}
}              //  Close else.
}              //  Close else.
}              // Close if()
else  {
  // *
// *        Form  x := A'*x.
// *
if ((uplo.toLowerCase().charAt(0) == "U".toLowerCase().charAt(0)))  {
    kplus1 = k+1;
if (incx == 1)  {
    {
int _j_inc = -1;
forloop100:
for (j = n; j >= 1; j += _j_inc) {
temp = x[(j)- 1+ _x_offset];
l = kplus1-j;
if (nounit)  
    temp = temp*a[(kplus1)- 1+(j- 1)*lda+ _a_offset];
{
int _i_inc = -1;
forloop90:
for (i = j-1; i >= Math.max(1, j-k) ; i += _i_inc) {
temp = temp+a[(l+i)- 1+(j- 1)*lda+ _a_offset]*x[(i)- 1+ _x_offset];
Dummy.label("Dtbmv",90);
}              //  Close for() loop. 
}
x[(j)- 1+ _x_offset] = temp;
Dummy.label("Dtbmv",100);
}              //  Close for() loop. 
}
}              // Close if()
else  {
  kx = kx+(n-1)*incx;
jx = kx;
{
int _j_inc = -1;
forloop120:
for (j = n; j >= 1; j += _j_inc) {
temp = x[(jx)- 1+ _x_offset];
kx = kx-incx;
ix = kx;
l = kplus1-j;
if (nounit)  
    temp = temp*a[(kplus1)- 1+(j- 1)*lda+ _a_offset];
{
int _i_inc = -1;
forloop110:
for (i = j-1; i >= Math.max(1, j-k) ; i += _i_inc) {
temp = temp+a[(l+i)- 1+(j- 1)*lda+ _a_offset]*x[(ix)- 1+ _x_offset];
ix = ix-incx;
Dummy.label("Dtbmv",110);
}              //  Close for() loop. 
}
x[(jx)- 1+ _x_offset] = temp;
jx = jx-incx;
Dummy.label("Dtbmv",120);
}              //  Close for() loop. 
}
}              //  Close else.
}              // Close if()
else  {
  if (incx == 1)  {
    {
forloop140:
for (j = 1; j <= n; j++) {
temp = x[(j)- 1+ _x_offset];
l = 1-j;
if (nounit)  
    temp = temp*a[(1)- 1+(j- 1)*lda+ _a_offset];
{
forloop130:
for (i = j+1; i <= Math.min(n, j+k) ; i++) {
temp = temp+a[(l+i)- 1+(j- 1)*lda+ _a_offset]*x[(i)- 1+ _x_offset];
Dummy.label("Dtbmv",130);
}              //  Close for() loop. 
}
x[(j)- 1+ _x_offset] = temp;
Dummy.label("Dtbmv",140);
}              //  Close for() loop. 
}
}              // Close if()
else  {
  jx = kx;
{
forloop160:
for (j = 1; j <= n; j++) {
temp = x[(jx)- 1+ _x_offset];
kx = kx+incx;
ix = kx;
l = 1-j;
if (nounit)  
    temp = temp*a[(1)- 1+(j- 1)*lda+ _a_offset];
{
forloop150:
for (i = j+1; i <= Math.min(n, j+k) ; i++) {
temp = temp+a[(l+i)- 1+(j- 1)*lda+ _a_offset]*x[(ix)- 1+ _x_offset];
ix = ix+incx;
Dummy.label("Dtbmv",150);
}              //  Close for() loop. 
}
x[(jx)- 1+ _x_offset] = temp;
jx = jx+incx;
Dummy.label("Dtbmv",160);
}              //  Close for() loop. 
}
}              //  Close else.
}              //  Close else.
}              //  Close else.
// *
Dummy.go_to("Dtbmv",999999);
// *
// *     End of DTBMV .
// *
Dummy.label("Dtbmv",999999);
return;
   }
} // End class.
