/*
 *  Produced by f2java.  f2java is part of the Fortran-
 *  -to-Java project at the University of Tennessee Netlib
 *  numerical software repository.
 *
 *  Original authorship for the BLAS and LAPACK numerical
 *  routines may be found in the Fortran source, available at
 *  www.netlib.org.
 *
 *  Fortran input file: jlapack.f
 *
 *  The f2j compiler code was written by
 *  David M. Doolin (doolin@cs.utk.edu) and
 *  Keith  Seymour (seymour@cs.utk.edu)
 */

package org.netlib.blas;
import java.lang.*;
import org.netlib.util.*;



public class Dtpsv {

// *     .. Scalar Arguments ..
// *     .. Array Arguments ..
// *     ..
// *
// *  Purpose
// *  =======
// *
// *  DTPSV  solves one of the systems of equations
// *
// *     A*x = b,   or   A'*x = b,
// *
// *  where b and x are n element vectors and A is an n by n unit, or
// *  non-unit, upper or lower triangular matrix, supplied in packed form.
// *
// *  No test for singularity or near-singularity is included in this
// *  routine. Such tests must be performed before calling this routine.
// *
// *  Parameters
// *  ==========
// *
// *  UPLO   - CHARACTER*1.
// *           On entry, UPLO specifies whether the matrix is an upper or
// *           lower triangular matrix as follows:
// *
// *              UPLO = 'U' or 'u'   A is an upper triangular matrix.
// *
// *              UPLO = 'L' or 'l'   A is a lower triangular matrix.
// *
// *           Unchanged on exit.
// *
// *  TRANS  - CHARACTER*1.
// *           On entry, TRANS specifies the equations to be solved as
// *           follows:
// *
// *              TRANS = 'N' or 'n'   A*x = b.
// *
// *              TRANS = 'T' or 't'   A'*x = b.
// *
// *              TRANS = 'C' or 'c'   A'*x = b.
// *
// *           Unchanged on exit.
// *
// *  DIAG   - CHARACTER*1.
// *           On entry, DIAG specifies whether or not A is unit
// *           triangular as follows:
// *
// *              DIAG = 'U' or 'u'   A is assumed to be unit triangular.
// *
// *              DIAG = 'N' or 'n'   A is not assumed to be unit
// *                                  triangular.
// *
// *           Unchanged on exit.
// *
// *  N      - INTEGER.
// *           On entry, N specifies the order of the matrix A.
// *           N must be at least zero.
// *           Unchanged on exit.
// *
// *  AP     - DOUBLE PRECISION array of DIMENSION at least
// *           ( ( n*( n + 1 ) )/2 ).
// *           Before entry with  UPLO = 'U' or 'u', the array AP must
// *           contain the upper triangular matrix packed sequentially,
// *           column by column, so that AP( 1 ) contains a( 1, 1 ),
// *           AP( 2 ) and AP( 3 ) contain a( 1, 2 ) and a( 2, 2 )
// *           respectively, and so on.
// *           Before entry with UPLO = 'L' or 'l', the array AP must
// *           contain the lower triangular matrix packed sequentially,
// *           column by column, so that AP( 1 ) contains a( 1, 1 ),
// *           AP( 2 ) and AP( 3 ) contain a( 2, 1 ) and a( 3, 1 )
// *           respectively, and so on.
// *           Note that when  DIAG = 'U' or 'u', the diagonal elements of
// *           A are not referenced, but are assumed to be unity.
// *           Unchanged on exit.
// *
// *  X      - DOUBLE PRECISION array of dimension at least
// *           ( 1 + ( n - 1 )*abs( INCX ) ).
// *           Before entry, the incremented array X must contain the n
// *           element right-hand side vector b. On exit, X is overwritten
// *           with the solution vector x.
// *
// *  INCX   - INTEGER.
// *           On entry, INCX specifies the increment for the elements of
// *           X. INCX must not be zero.
// *           Unchanged on exit.
// *
// *
// *  Level 2 Blas routine.
// *
// *  -- Written on 22-October-1986.
// *     Jack Dongarra, Argonne National Lab.
// *     Jeremy Du Croz, Nag Central Office.
// *     Sven Hammarling, Nag Central Office.
// *     Richard Hanson, Sandia National Labs.
// *
// *
// *     .. Parameters ..
static double zero= 0.0e+0;
// *     .. Local Scalars ..
static double temp= 0.0;
static int i= 0;
static int info= 0;
static int ix= 0;
static int j= 0;
static int jx= 0;
static int k= 0;
static int kk= 0;
static int kx= 0;
static boolean nounit= false;
// *     .. External Functions ..
// *     .. External Subroutines ..
// *     ..
// *     .. Executable Statements ..
// *
// *     Test the input parameters.
// *

public static void dtpsv (String uplo,
String trans,
String diag,
int n,
double [] ap, int _ap_offset,
double [] x, int _x_offset,
int incx)  {

info = 0;
if (!(uplo.toLowerCase().charAt(0) == "U".toLowerCase().charAt(0)) && !(uplo.toLowerCase().charAt(0) == "L".toLowerCase().charAt(0)))  {
    info = 1;
}              // Close if()
else if (!(trans.toLowerCase().charAt(0) == "N".toLowerCase().charAt(0)) && !(trans.toLowerCase().charAt(0) == "T".toLowerCase().charAt(0)) && !(trans.toLowerCase().charAt(0) == "C".toLowerCase().charAt(0)))  {
    info = 2;
}              // Close else if()
else if (!(diag.toLowerCase().charAt(0) == "U".toLowerCase().charAt(0)) && !(diag.toLowerCase().charAt(0) == "N".toLowerCase().charAt(0)))  {
    info = 3;
}              // Close else if()
else if (n < 0)  {
    info = 4;
}              // Close else if()
else if (incx == 0)  {
    info = 7;
}              // Close else if()
if (info != 0)  {
    Xerbla.xerbla("DTPSV ",info);
Dummy.go_to("Dtpsv",999999);
}              // Close if()
// *
// *     Quick return if possible.
// *
if (n == 0)  
    Dummy.go_to("Dtpsv",999999);
// *
nounit = (diag.toLowerCase().charAt(0) == "N".toLowerCase().charAt(0));
// *
// *     Set up the start point in X if the increment is not unity. This
// *     will be  ( N - 1 )*INCX  too small for descending loops.
// *
if (incx <= 0)  {
    kx = 1-(n-1)*incx;
}              // Close if()
else if (incx != 1)  {
    kx = 1;
}              // Close else if()
// *
// *     Start the operations. In this version the elements of AP are
// *     accessed sequentially with one pass through AP.
// *
if ((trans.toLowerCase().charAt(0) == "N".toLowerCase().charAt(0)))  {
    // *
// *        Form  x := inv( A )*x.
// *
if ((uplo.toLowerCase().charAt(0) == "U".toLowerCase().charAt(0)))  {
    kk = (n*(n+1))/2;
if (incx == 1)  {
    {
int _j_inc = -1;
forloop20:
for (j = n; j >= 1; j += _j_inc) {
if (x[(j)- 1+ _x_offset] != zero)  {
    if (nounit)  
    x[(j)- 1+ _x_offset] = x[(j)- 1+ _x_offset]/ap[(kk)- 1+ _ap_offset];
temp = x[(j)- 1+ _x_offset];
k = kk-1;
{
int _i_inc = -1;
forloop10:
for (i = j-1; i >= 1; i += _i_inc) {
x[(i)- 1+ _x_offset] = x[(i)- 1+ _x_offset]-temp*ap[(k)- 1+ _ap_offset];
k = k-1;
Dummy.label("Dtpsv",10);
}              //  Close for() loop. 
}
}              // Close if()
kk = kk-j;
Dummy.label("Dtpsv",20);
}              //  Close for() loop. 
}
}              // Close if()
else  {
  jx = kx+(n-1)*incx;
{
int _j_inc = -1;
forloop40:
for (j = n; j >= 1; j += _j_inc) {
if (x[(jx)- 1+ _x_offset] != zero)  {
    if (nounit)  
    x[(jx)- 1+ _x_offset] = x[(jx)- 1+ _x_offset]/ap[(kk)- 1+ _ap_offset];
temp = x[(jx)- 1+ _x_offset];
ix = jx;
{
int _k_inc = -1;
forloop30:
for (k = kk-1; k >= kk-j+1; k += _k_inc) {
ix = ix-incx;
x[(ix)- 1+ _x_offset] = x[(ix)- 1+ _x_offset]-temp*ap[(k)- 1+ _ap_offset];
Dummy.label("Dtpsv",30);
}              //  Close for() loop. 
}
}              // Close if()
jx = jx-incx;
kk = kk-j;
Dummy.label("Dtpsv",40);
}              //  Close for() loop. 
}
}              //  Close else.
}              // Close if()
else  {
  kk = 1;
if (incx == 1)  {
    {
forloop60:
for (j = 1; j <= n; j++) {
if (x[(j)- 1+ _x_offset] != zero)  {
    if (nounit)  
    x[(j)- 1+ _x_offset] = x[(j)- 1+ _x_offset]/ap[(kk)- 1+ _ap_offset];
temp = x[(j)- 1+ _x_offset];
k = kk+1;
{
forloop50:
for (i = j+1; i <= n; i++) {
x[(i)- 1+ _x_offset] = x[(i)- 1+ _x_offset]-temp*ap[(k)- 1+ _ap_offset];
k = k+1;
Dummy.label("Dtpsv",50);
}              //  Close for() loop. 
}
}              // Close if()
kk = kk+(n-j+1);
Dummy.label("Dtpsv",60);
}              //  Close for() loop. 
}
}              // Close if()
else  {
  jx = kx;
{
forloop80:
for (j = 1; j <= n; j++) {
if (x[(jx)- 1+ _x_offset] != zero)  {
    if (nounit)  
    x[(jx)- 1+ _x_offset] = x[(jx)- 1+ _x_offset]/ap[(kk)- 1+ _ap_offset];
temp = x[(jx)- 1+ _x_offset];
ix = jx;
{
forloop70:
for (k = kk+1; k <= kk+n-j; k++) {
ix = ix+incx;
x[(ix)- 1+ _x_offset] = x[(ix)- 1+ _x_offset]-temp*ap[(k)- 1+ _ap_offset];
Dummy.label("Dtpsv",70);
}              //  Close for() loop. 
}
}              // Close if()
jx = jx+incx;
kk = kk+(n-j+1);
Dummy.label("Dtpsv",80);
}              //  Close for() loop. 
}
}              //  Close else.
}              //  Close else.
}              // Close if()
else  {
  // *
// *        Form  x := inv( A' )*x.
// *
if ((uplo.toLowerCase().charAt(0) == "U".toLowerCase().charAt(0)))  {
    kk = 1;
if (incx == 1)  {
    {
forloop100:
for (j = 1; j <= n; j++) {
temp = x[(j)- 1+ _x_offset];
k = kk;
{
forloop90:
for (i = 1; i <= j-1; i++) {
temp = temp-ap[(k)- 1+ _ap_offset]*x[(i)- 1+ _x_offset];
k = k+1;
Dummy.label("Dtpsv",90);
}              //  Close for() loop. 
}
if (nounit)  
    temp = temp/ap[(kk+j-1)- 1+ _ap_offset];
x[(j)- 1+ _x_offset] = temp;
kk = kk+j;
Dummy.label("Dtpsv",100);
}              //  Close for() loop. 
}
}              // Close if()
else  {
  jx = kx;
{
forloop120:
for (j = 1; j <= n; j++) {
temp = x[(jx)- 1+ _x_offset];
ix = kx;
{
forloop110:
for (k = kk; k <= kk+j-2; k++) {
temp = temp-ap[(k)- 1+ _ap_offset]*x[(ix)- 1+ _x_offset];
ix = ix+incx;
Dummy.label("Dtpsv",110);
}              //  Close for() loop. 
}
if (nounit)  
    temp = temp/ap[(kk+j-1)- 1+ _ap_offset];
x[(jx)- 1+ _x_offset] = temp;
jx = jx+incx;
kk = kk+j;
Dummy.label("Dtpsv",120);
}              //  Close for() loop. 
}
}              //  Close else.
}              // Close if()
else  {
  kk = (n*(n+1))/2;
if (incx == 1)  {
    {
int _j_inc = -1;
forloop140:
for (j = n; j >= 1; j += _j_inc) {
temp = x[(j)- 1+ _x_offset];
k = kk;
{
int _i_inc = -1;
forloop130:
for (i = n; i >= j+1; i += _i_inc) {
temp = temp-ap[(k)- 1+ _ap_offset]*x[(i)- 1+ _x_offset];
k = k-1;
Dummy.label("Dtpsv",130);
}              //  Close for() loop. 
}
if (nounit)  
    temp = temp/ap[(kk-n+j)- 1+ _ap_offset];
x[(j)- 1+ _x_offset] = temp;
kk = kk-(n-j+1);
Dummy.label("Dtpsv",140);
}              //  Close for() loop. 
}
}              // Close if()
else  {
  kx = kx+(n-1)*incx;
jx = kx;
{
int _j_inc = -1;
forloop160:
for (j = n; j >= 1; j += _j_inc) {
temp = x[(jx)- 1+ _x_offset];
ix = kx;
{
int _k_inc = -1;
forloop150:
for (k = kk; k >= kk-(n-(j+1)); k += _k_inc) {
temp = temp-ap[(k)- 1+ _ap_offset]*x[(ix)- 1+ _x_offset];
ix = ix-incx;
Dummy.label("Dtpsv",150);
}              //  Close for() loop. 
}
if (nounit)  
    temp = temp/ap[(kk-n+j)- 1+ _ap_offset];
x[(jx)- 1+ _x_offset] = temp;
jx = jx-incx;
kk = kk-(n-j+1);
Dummy.label("Dtpsv",160);
}              //  Close for() loop. 
}
}              //  Close else.
}              //  Close else.
}              //  Close else.
// *
Dummy.go_to("Dtpsv",999999);
// *
// *     End of DTPSV .
// *
Dummy.label("Dtpsv",999999);
return;
   }
} // End class.
