/*
 *  Produced by f2java.  f2java is part of the Fortran-
 *  -to-Java project at the University of Tennessee Netlib
 *  numerical software repository.
 *
 *  Original authorship for the BLAS and LAPACK numerical
 *  routines may be found in the Fortran source, available at
 *  www.netlib.org.
 *
 *  Fortran input file: jlapack.f
 *
 *  The f2j compiler code was written by
 *  David M. Doolin (doolin@cs.utk.edu) and
 *  Keith  Seymour (seymour@cs.utk.edu)
 */

package org.netlib.blas;
import java.lang.*;
import org.netlib.util.*;



public class Dtrmm {

// *     .. Scalar Arguments ..
// *     .. Array Arguments ..
// *     ..
// *
// *  Purpose
// *  =======
// *
// *  DTRMM  performs one of the matrix-matrix operations
// *
// *     B := alpha*op( A )*B,   or   B := alpha*B*op( A ),
// *
// *  where  alpha  is a scalar,  B  is an m by n matrix,  A  is a unit, or
// *  non-unit,  upper or lower triangular matrix  and  op( A )  is one  of
// *
// *     op( A ) = A   or   op( A ) = A'.
// *
// *  Parameters
// *  ==========
// *
// *  SIDE   - CHARACTER*1.
// *           On entry,  SIDE specifies whether  op( A ) multiplies B from
// *           the left or right as follows:
// *
// *              SIDE = 'L' or 'l'   B := alpha*op( A )*B.
// *
// *              SIDE = 'R' or 'r'   B := alpha*B*op( A ).
// *
// *           Unchanged on exit.
// *
// *  UPLO   - CHARACTER*1.
// *           On entry, UPLO specifies whether the matrix A is an upper or
// *           lower triangular matrix as follows:
// *
// *              UPLO = 'U' or 'u'   A is an upper triangular matrix.
// *
// *              UPLO = 'L' or 'l'   A is a lower triangular matrix.
// *
// *           Unchanged on exit.
// *
// *  TRANSA - CHARACTER*1.
// *           On entry, TRANSA specifies the form of op( A ) to be used in
// *           the matrix multiplication as follows:
// *
// *              TRANSA = 'N' or 'n'   op( A ) = A.
// *
// *              TRANSA = 'T' or 't'   op( A ) = A'.
// *
// *              TRANSA = 'C' or 'c'   op( A ) = A'.
// *
// *           Unchanged on exit.
// *
// *  DIAG   - CHARACTER*1.
// *           On entry, DIAG specifies whether or not A is unit triangular
// *           as follows:
// *
// *              DIAG = 'U' or 'u'   A is assumed to be unit triangular.
// *
// *              DIAG = 'N' or 'n'   A is not assumed to be unit
// *                                  triangular.
// *
// *           Unchanged on exit.
// *
// *  M      - INTEGER.
// *           On entry, M specifies the number of rows of B. M must be at
// *           least zero.
// *           Unchanged on exit.
// *
// *  N      - INTEGER.
// *           On entry, N specifies the number of columns of B.  N must be
// *           at least zero.
// *           Unchanged on exit.
// *
// *  ALPHA  - DOUBLE PRECISION.
// *           On entry,  ALPHA specifies the scalar  alpha. When  alpha is
// *           zero then  A is not referenced and  B need not be set before
// *           entry.
// *           Unchanged on exit.
// *
// *  A      - DOUBLE PRECISION array of DIMENSION ( LDA, k ), where k is m
// *           when  SIDE = 'L' or 'l'  and is  n  when  SIDE = 'R' or 'r'.
// *           Before entry  with  UPLO = 'U' or 'u',  the  leading  k by k
// *           upper triangular part of the array  A must contain the upper
// *           triangular matrix  and the strictly lower triangular part of
// *           A is not referenced.
// *           Before entry  with  UPLO = 'L' or 'l',  the  leading  k by k
// *           lower triangular part of the array  A must contain the lower
// *           triangular matrix  and the strictly upper triangular part of
// *           A is not referenced.
// *           Note that when  DIAG = 'U' or 'u',  the diagonal elements of
// *           A  are not referenced either,  but are assumed to be  unity.
// *           Unchanged on exit.
// *
// *  LDA    - INTEGER.
// *           On entry, LDA specifies the first dimension of A as declared
// *           in the calling (sub) program.  When  SIDE = 'L' or 'l'  then
// *           LDA  must be at least  max( 1, m ),  when  SIDE = 'R' or 'r'
// *           then LDA must be at least max( 1, n ).
// *           Unchanged on exit.
// *
// *  B      - DOUBLE PRECISION array of DIMENSION ( LDB, n ).
// *           Before entry,  the leading  m by n part of the array  B must
// *           contain the matrix  B,  and  on exit  is overwritten  by the
// *           transformed matrix.
// *
// *  LDB    - INTEGER.
// *           On entry, LDB specifies the first dimension of B as declared
// *           in  the  calling  (sub)  program.   LDB  must  be  at  least
// *           max( 1, m ).
// *           Unchanged on exit.
// *
// *
// *  Level 3 Blas routine.
// *
// *  -- Written on 8-February-1989.
// *     Jack Dongarra, Argonne National Laboratory.
// *     Iain Duff, AERE Harwell.
// *     Jeremy Du Croz, Numerical Algorithms Group Ltd.
// *     Sven Hammarling, Numerical Algorithms Group Ltd.
// *
// *
// *     .. External Functions ..
// *     .. External Subroutines ..
// *     .. Intrinsic Functions ..
// *     .. Local Scalars ..
static boolean lside= false;
static boolean nounit= false;
static boolean upper= false;
static int i= 0;
static int info= 0;
static int j= 0;
static int k= 0;
static int nrowa= 0;
static double temp= 0.0;
// *     .. Parameters ..
static double one= 1.0e+0;
static double zero= 0.0e+0;
// *     ..
// *     .. Executable Statements ..
// *
// *     Test the input parameters.
// *

public static void dtrmm (String side,
String uplo,
String transa,
String diag,
int m,
int n,
double alpha,
double [] a, int _a_offset,
int lda,
double [] b, int _b_offset,
int ldb)  {

lside = (side.toLowerCase().charAt(0) == "L".toLowerCase().charAt(0));
if (lside)  {
    nrowa = m;
}              // Close if()
else  {
  nrowa = n;
}              //  Close else.
nounit = (diag.toLowerCase().charAt(0) == "N".toLowerCase().charAt(0));
upper = (uplo.toLowerCase().charAt(0) == "U".toLowerCase().charAt(0));
// *
info = 0;
if ((!lside) && (!(side.toLowerCase().charAt(0) == "R".toLowerCase().charAt(0))))  {
    info = 1;
}              // Close if()
else if ((!upper) && (!(uplo.toLowerCase().charAt(0) == "L".toLowerCase().charAt(0))))  {
    info = 2;
}              // Close else if()
else if ((!(transa.toLowerCase().charAt(0) == "N".toLowerCase().charAt(0))) && (!(transa.toLowerCase().charAt(0) == "T".toLowerCase().charAt(0))) && (!(transa.toLowerCase().charAt(0) == "C".toLowerCase().charAt(0))))  {
    info = 3;
}              // Close else if()
else if ((!(diag.toLowerCase().charAt(0) == "U".toLowerCase().charAt(0))) && (!(diag.toLowerCase().charAt(0) == "N".toLowerCase().charAt(0))))  {
    info = 4;
}              // Close else if()
else if (m < 0)  {
    info = 5;
}              // Close else if()
else if (n < 0)  {
    info = 6;
}              // Close else if()
else if (lda < Math.max(1, nrowa) )  {
    info = 9;
}              // Close else if()
else if (ldb < Math.max(1, m) )  {
    info = 11;
}              // Close else if()
if (info != 0)  {
    Xerbla.xerbla("DTRMM ",info);
Dummy.go_to("Dtrmm",999999);
}              // Close if()
// *
// *     Quick return if possible.
// *
if (n == 0)  
    Dummy.go_to("Dtrmm",999999);
// *
// *     And when  alpha.eq.zero.
// *
if (alpha == zero)  {
    {
forloop20:
for (j = 1; j <= n; j++) {
{
forloop10:
for (i = 1; i <= m; i++) {
b[(i)- 1+(j- 1)*ldb+ _b_offset] = zero;
Dummy.label("Dtrmm",10);
}              //  Close for() loop. 
}
Dummy.label("Dtrmm",20);
}              //  Close for() loop. 
}
Dummy.go_to("Dtrmm",999999);
}              // Close if()
// *
// *     Start the operations.
// *
if (lside)  {
    if ((transa.toLowerCase().charAt(0) == "N".toLowerCase().charAt(0)))  {
    // *
// *           Form  B := alpha*A*B.
// *
if (upper)  {
    {
forloop50:
for (j = 1; j <= n; j++) {
{
forloop40:
for (k = 1; k <= m; k++) {
if (b[(k)- 1+(j- 1)*ldb+ _b_offset] != zero)  {
    temp = alpha*b[(k)- 1+(j- 1)*ldb+ _b_offset];
{
forloop30:
for (i = 1; i <= k-1; i++) {
b[(i)- 1+(j- 1)*ldb+ _b_offset] = b[(i)- 1+(j- 1)*ldb+ _b_offset]+temp*a[(i)- 1+(k- 1)*lda+ _a_offset];
Dummy.label("Dtrmm",30);
}              //  Close for() loop. 
}
if (nounit)  
    temp = temp*a[(k)- 1+(k- 1)*lda+ _a_offset];
b[(k)- 1+(j- 1)*ldb+ _b_offset] = temp;
}              // Close if()
Dummy.label("Dtrmm",40);
}              //  Close for() loop. 
}
Dummy.label("Dtrmm",50);
}              //  Close for() loop. 
}
}              // Close if()
else  {
  {
forloop80:
for (j = 1; j <= n; j++) {
{
int _k_inc = -1;
forloop70:
for (k = m; k >= 1; k += _k_inc) {
if (b[(k)- 1+(j- 1)*ldb+ _b_offset] != zero)  {
    temp = alpha*b[(k)- 1+(j- 1)*ldb+ _b_offset];
b[(k)- 1+(j- 1)*ldb+ _b_offset] = temp;
if (nounit)  
    b[(k)- 1+(j- 1)*ldb+ _b_offset] = b[(k)- 1+(j- 1)*ldb+ _b_offset]*a[(k)- 1+(k- 1)*lda+ _a_offset];
{
forloop60:
for (i = k+1; i <= m; i++) {
b[(i)- 1+(j- 1)*ldb+ _b_offset] = b[(i)- 1+(j- 1)*ldb+ _b_offset]+temp*a[(i)- 1+(k- 1)*lda+ _a_offset];
Dummy.label("Dtrmm",60);
}              //  Close for() loop. 
}
}              // Close if()
Dummy.label("Dtrmm",70);
}              //  Close for() loop. 
}
Dummy.label("Dtrmm",80);
}              //  Close for() loop. 
}
}              //  Close else.
}              // Close if()
else  {
  // *
// *           Form  B := alpha*B*A'.
// *
if (upper)  {
    {
forloop110:
for (j = 1; j <= n; j++) {
{
int _i_inc = -1;
forloop100:
for (i = m; i >= 1; i += _i_inc) {
temp = b[(i)- 1+(j- 1)*ldb+ _b_offset];
if (nounit)  
    temp = temp*a[(i)- 1+(i- 1)*lda+ _a_offset];
{
forloop90:
for (k = 1; k <= i-1; k++) {
temp = temp+a[(k)- 1+(i- 1)*lda+ _a_offset]*b[(k)- 1+(j- 1)*ldb+ _b_offset];
Dummy.label("Dtrmm",90);
}              //  Close for() loop. 
}
b[(i)- 1+(j- 1)*ldb+ _b_offset] = alpha*temp;
Dummy.label("Dtrmm",100);
}              //  Close for() loop. 
}
Dummy.label("Dtrmm",110);
}              //  Close for() loop. 
}
}              // Close if()
else  {
  {
forloop140:
for (j = 1; j <= n; j++) {
{
forloop130:
for (i = 1; i <= m; i++) {
temp = b[(i)- 1+(j- 1)*ldb+ _b_offset];
if (nounit)  
    temp = temp*a[(i)- 1+(i- 1)*lda+ _a_offset];
{
forloop120:
for (k = i+1; k <= m; k++) {
temp = temp+a[(k)- 1+(i- 1)*lda+ _a_offset]*b[(k)- 1+(j- 1)*ldb+ _b_offset];
Dummy.label("Dtrmm",120);
}              //  Close for() loop. 
}
b[(i)- 1+(j- 1)*ldb+ _b_offset] = alpha*temp;
Dummy.label("Dtrmm",130);
}              //  Close for() loop. 
}
Dummy.label("Dtrmm",140);
}              //  Close for() loop. 
}
}              //  Close else.
}              //  Close else.
}              // Close if()
else  {
  if ((transa.toLowerCase().charAt(0) == "N".toLowerCase().charAt(0)))  {
    // *
// *           Form  B := alpha*B*A.
// *
if (upper)  {
    {
int _j_inc = -1;
forloop180:
for (j = n; j >= 1; j += _j_inc) {
temp = alpha;
if (nounit)  
    temp = temp*a[(j)- 1+(j- 1)*lda+ _a_offset];
{
forloop150:
for (i = 1; i <= m; i++) {
b[(i)- 1+(j- 1)*ldb+ _b_offset] = temp*b[(i)- 1+(j- 1)*ldb+ _b_offset];
Dummy.label("Dtrmm",150);
}              //  Close for() loop. 
}
{
forloop170:
for (k = 1; k <= j-1; k++) {
if (a[(k)- 1+(j- 1)*lda+ _a_offset] != zero)  {
    temp = alpha*a[(k)- 1+(j- 1)*lda+ _a_offset];
{
forloop160:
for (i = 1; i <= m; i++) {
b[(i)- 1+(j- 1)*ldb+ _b_offset] = b[(i)- 1+(j- 1)*ldb+ _b_offset]+temp*b[(i)- 1+(k- 1)*ldb+ _b_offset];
Dummy.label("Dtrmm",160);
}              //  Close for() loop. 
}
}              // Close if()
Dummy.label("Dtrmm",170);
}              //  Close for() loop. 
}
Dummy.label("Dtrmm",180);
}              //  Close for() loop. 
}
}              // Close if()
else  {
  {
forloop220:
for (j = 1; j <= n; j++) {
temp = alpha;
if (nounit)  
    temp = temp*a[(j)- 1+(j- 1)*lda+ _a_offset];
{
forloop190:
for (i = 1; i <= m; i++) {
b[(i)- 1+(j- 1)*ldb+ _b_offset] = temp*b[(i)- 1+(j- 1)*ldb+ _b_offset];
Dummy.label("Dtrmm",190);
}              //  Close for() loop. 
}
{
forloop210:
for (k = j+1; k <= n; k++) {
if (a[(k)- 1+(j- 1)*lda+ _a_offset] != zero)  {
    temp = alpha*a[(k)- 1+(j- 1)*lda+ _a_offset];
{
forloop200:
for (i = 1; i <= m; i++) {
b[(i)- 1+(j- 1)*ldb+ _b_offset] = b[(i)- 1+(j- 1)*ldb+ _b_offset]+temp*b[(i)- 1+(k- 1)*ldb+ _b_offset];
Dummy.label("Dtrmm",200);
}              //  Close for() loop. 
}
}              // Close if()
Dummy.label("Dtrmm",210);
}              //  Close for() loop. 
}
Dummy.label("Dtrmm",220);
}              //  Close for() loop. 
}
}              //  Close else.
}              // Close if()
else  {
  // *
// *           Form  B := alpha*B*A'.
// *
if (upper)  {
    {
forloop260:
for (k = 1; k <= n; k++) {
{
forloop240:
for (j = 1; j <= k-1; j++) {
if (a[(j)- 1+(k- 1)*lda+ _a_offset] != zero)  {
    temp = alpha*a[(j)- 1+(k- 1)*lda+ _a_offset];
{
forloop230:
for (i = 1; i <= m; i++) {
b[(i)- 1+(j- 1)*ldb+ _b_offset] = b[(i)- 1+(j- 1)*ldb+ _b_offset]+temp*b[(i)- 1+(k- 1)*ldb+ _b_offset];
Dummy.label("Dtrmm",230);
}              //  Close for() loop. 
}
}              // Close if()
Dummy.label("Dtrmm",240);
}              //  Close for() loop. 
}
temp = alpha;
if (nounit)  
    temp = temp*a[(k)- 1+(k- 1)*lda+ _a_offset];
if (temp != one)  {
    {
forloop250:
for (i = 1; i <= m; i++) {
b[(i)- 1+(k- 1)*ldb+ _b_offset] = temp*b[(i)- 1+(k- 1)*ldb+ _b_offset];
Dummy.label("Dtrmm",250);
}              //  Close for() loop. 
}
}              // Close if()
Dummy.label("Dtrmm",260);
}              //  Close for() loop. 
}
}              // Close if()
else  {
  {
int _k_inc = -1;
forloop300:
for (k = n; k >= 1; k += _k_inc) {
{
forloop280:
for (j = k+1; j <= n; j++) {
if (a[(j)- 1+(k- 1)*lda+ _a_offset] != zero)  {
    temp = alpha*a[(j)- 1+(k- 1)*lda+ _a_offset];
{
forloop270:
for (i = 1; i <= m; i++) {
b[(i)- 1+(j- 1)*ldb+ _b_offset] = b[(i)- 1+(j- 1)*ldb+ _b_offset]+temp*b[(i)- 1+(k- 1)*ldb+ _b_offset];
Dummy.label("Dtrmm",270);
}              //  Close for() loop. 
}
}              // Close if()
Dummy.label("Dtrmm",280);
}              //  Close for() loop. 
}
temp = alpha;
if (nounit)  
    temp = temp*a[(k)- 1+(k- 1)*lda+ _a_offset];
if (temp != one)  {
    {
forloop290:
for (i = 1; i <= m; i++) {
b[(i)- 1+(k- 1)*ldb+ _b_offset] = temp*b[(i)- 1+(k- 1)*ldb+ _b_offset];
Dummy.label("Dtrmm",290);
}              //  Close for() loop. 
}
}              // Close if()
Dummy.label("Dtrmm",300);
}              //  Close for() loop. 
}
}              //  Close else.
}              //  Close else.
}              //  Close else.
// *
Dummy.go_to("Dtrmm",999999);
// *
// *     End of DTRMM .
// *
Dummy.label("Dtrmm",999999);
return;
   }
} // End class.
