/*
 *  Produced by f2java.  f2java is part of the Fortran-
 *  -to-Java project at the University of Tennessee Netlib
 *  numerical software repository.
 *
 *  Original authorship for the BLAS and LAPACK numerical
 *  routines may be found in the Fortran source, available at
 *  www.netlib.org.
 *
 *  Fortran input file: jlapack.f
 *
 *  The f2j compiler code was written by
 *  David M. Doolin (doolin@cs.utk.edu) and
 *  Keith  Seymour (seymour@cs.utk.edu)
 */

package org.netlib.blas;
import java.lang.*;
import org.netlib.util.*;



public class Dtrmv {

// *     .. Scalar Arguments ..
// *     .. Array Arguments ..
// *     ..
// *
// *  Purpose
// *  =======
// *
// *  DTRMV  performs one of the matrix-vector operations
// *
// *     x := A*x,   or   x := A'*x,
// *
// *  where x is an n element vector and  A is an n by n unit, or non-unit,
// *  upper or lower triangular matrix.
// *
// *  Parameters
// *  ==========
// *
// *  UPLO   - CHARACTER*1.
// *           On entry, UPLO specifies whether the matrix is an upper or
// *           lower triangular matrix as follows:
// *
// *              UPLO = 'U' or 'u'   A is an upper triangular matrix.
// *
// *              UPLO = 'L' or 'l'   A is a lower triangular matrix.
// *
// *           Unchanged on exit.
// *
// *  TRANS  - CHARACTER*1.
// *           On entry, TRANS specifies the operation to be performed as
// *           follows:
// *
// *              TRANS = 'N' or 'n'   x := A*x.
// *
// *              TRANS = 'T' or 't'   x := A'*x.
// *
// *              TRANS = 'C' or 'c'   x := A'*x.
// *
// *           Unchanged on exit.
// *
// *  DIAG   - CHARACTER*1.
// *           On entry, DIAG specifies whether or not A is unit
// *           triangular as follows:
// *
// *              DIAG = 'U' or 'u'   A is assumed to be unit triangular.
// *
// *              DIAG = 'N' or 'n'   A is not assumed to be unit
// *                                  triangular.
// *
// *           Unchanged on exit.
// *
// *  N      - INTEGER.
// *           On entry, N specifies the order of the matrix A.
// *           N must be at least zero.
// *           Unchanged on exit.
// *
// *  A      - DOUBLE PRECISION array of DIMENSION ( LDA, n ).
// *           Before entry with  UPLO = 'U' or 'u', the leading n by n
// *           upper triangular part of the array A must contain the upper
// *           triangular matrix and the strictly lower triangular part of
// *           A is not referenced.
// *           Before entry with UPLO = 'L' or 'l', the leading n by n
// *           lower triangular part of the array A must contain the lower
// *           triangular matrix and the strictly upper triangular part of
// *           A is not referenced.
// *           Note that when  DIAG = 'U' or 'u', the diagonal elements of
// *           A are not referenced either, but are assumed to be unity.
// *           Unchanged on exit.
// *
// *  LDA    - INTEGER.
// *           On entry, LDA specifies the first dimension of A as declared
// *           in the calling (sub) program. LDA must be at least
// *           max( 1, n ).
// *           Unchanged on exit.
// *
// *  X      - DOUBLE PRECISION array of dimension at least
// *           ( 1 + ( n - 1 )*abs( INCX ) ).
// *           Before entry, the incremented array X must contain the n
// *           element vector x. On exit, X is overwritten with the
// *           tranformed vector x.
// *
// *  INCX   - INTEGER.
// *           On entry, INCX specifies the increment for the elements of
// *           X. INCX must not be zero.
// *           Unchanged on exit.
// *
// *
// *  Level 2 Blas routine.
// *
// *  -- Written on 22-October-1986.
// *     Jack Dongarra, Argonne National Lab.
// *     Jeremy Du Croz, Nag Central Office.
// *     Sven Hammarling, Nag Central Office.
// *     Richard Hanson, Sandia National Labs.
// *
// *
// *     .. Parameters ..
static double zero= 0.0e+0;
// *     .. Local Scalars ..
static double temp= 0.0;
static int i= 0;
static int info= 0;
static int ix= 0;
static int j= 0;
static int jx= 0;
static int kx= 0;
static boolean nounit= false;
// *     .. External Functions ..
// *     .. External Subroutines ..
// *     .. Intrinsic Functions ..
// *     ..
// *     .. Executable Statements ..
// *
// *     Test the input parameters.
// *

public static void dtrmv (String uplo,
String trans,
String diag,
int n,
double [] a, int _a_offset,
int lda,
double [] x, int _x_offset,
int incx)  {

info = 0;
if (!(uplo.toLowerCase().charAt(0) == "U".toLowerCase().charAt(0)) && !(uplo.toLowerCase().charAt(0) == "L".toLowerCase().charAt(0)))  {
    info = 1;
}              // Close if()
else if (!(trans.toLowerCase().charAt(0) == "N".toLowerCase().charAt(0)) && !(trans.toLowerCase().charAt(0) == "T".toLowerCase().charAt(0)) && !(trans.toLowerCase().charAt(0) == "C".toLowerCase().charAt(0)))  {
    info = 2;
}              // Close else if()
else if (!(diag.toLowerCase().charAt(0) == "U".toLowerCase().charAt(0)) && !(diag.toLowerCase().charAt(0) == "N".toLowerCase().charAt(0)))  {
    info = 3;
}              // Close else if()
else if (n < 0)  {
    info = 4;
}              // Close else if()
else if (lda < Math.max(1, n) )  {
    info = 6;
}              // Close else if()
else if (incx == 0)  {
    info = 8;
}              // Close else if()
if (info != 0)  {
    Xerbla.xerbla("DTRMV ",info);
Dummy.go_to("Dtrmv",999999);
}              // Close if()
// *
// *     Quick return if possible.
// *
if (n == 0)  
    Dummy.go_to("Dtrmv",999999);
// *
nounit = (diag.toLowerCase().charAt(0) == "N".toLowerCase().charAt(0));
// *
// *     Set up the start point in X if the increment is not unity. This
// *     will be  ( N - 1 )*INCX  too small for descending loops.
// *
if (incx <= 0)  {
    kx = 1-(n-1)*incx;
}              // Close if()
else if (incx != 1)  {
    kx = 1;
}              // Close else if()
// *
// *     Start the operations. In this version the elements of A are
// *     accessed sequentially with one pass through A.
// *
if ((trans.toLowerCase().charAt(0) == "N".toLowerCase().charAt(0)))  {
    // *
// *        Form  x := A*x.
// *
if ((uplo.toLowerCase().charAt(0) == "U".toLowerCase().charAt(0)))  {
    if (incx == 1)  {
    {
forloop20:
for (j = 1; j <= n; j++) {
if (x[(j)- 1+ _x_offset] != zero)  {
    temp = x[(j)- 1+ _x_offset];
{
forloop10:
for (i = 1; i <= j-1; i++) {
x[(i)- 1+ _x_offset] = x[(i)- 1+ _x_offset]+temp*a[(i)- 1+(j- 1)*lda+ _a_offset];
Dummy.label("Dtrmv",10);
}              //  Close for() loop. 
}
if (nounit)  
    x[(j)- 1+ _x_offset] = x[(j)- 1+ _x_offset]*a[(j)- 1+(j- 1)*lda+ _a_offset];
}              // Close if()
Dummy.label("Dtrmv",20);
}              //  Close for() loop. 
}
}              // Close if()
else  {
  jx = kx;
{
forloop40:
for (j = 1; j <= n; j++) {
if (x[(jx)- 1+ _x_offset] != zero)  {
    temp = x[(jx)- 1+ _x_offset];
ix = kx;
{
forloop30:
for (i = 1; i <= j-1; i++) {
x[(ix)- 1+ _x_offset] = x[(ix)- 1+ _x_offset]+temp*a[(i)- 1+(j- 1)*lda+ _a_offset];
ix = ix+incx;
Dummy.label("Dtrmv",30);
}              //  Close for() loop. 
}
if (nounit)  
    x[(jx)- 1+ _x_offset] = x[(jx)- 1+ _x_offset]*a[(j)- 1+(j- 1)*lda+ _a_offset];
}              // Close if()
jx = jx+incx;
Dummy.label("Dtrmv",40);
}              //  Close for() loop. 
}
}              //  Close else.
}              // Close if()
else  {
  if (incx == 1)  {
    {
int _j_inc = -1;
forloop60:
for (j = n; j >= 1; j += _j_inc) {
if (x[(j)- 1+ _x_offset] != zero)  {
    temp = x[(j)- 1+ _x_offset];
{
int _i_inc = -1;
forloop50:
for (i = n; i >= j+1; i += _i_inc) {
x[(i)- 1+ _x_offset] = x[(i)- 1+ _x_offset]+temp*a[(i)- 1+(j- 1)*lda+ _a_offset];
Dummy.label("Dtrmv",50);
}              //  Close for() loop. 
}
if (nounit)  
    x[(j)- 1+ _x_offset] = x[(j)- 1+ _x_offset]*a[(j)- 1+(j- 1)*lda+ _a_offset];
}              // Close if()
Dummy.label("Dtrmv",60);
}              //  Close for() loop. 
}
}              // Close if()
else  {
  kx = kx+(n-1)*incx;
jx = kx;
{
int _j_inc = -1;
forloop80:
for (j = n; j >= 1; j += _j_inc) {
if (x[(jx)- 1+ _x_offset] != zero)  {
    temp = x[(jx)- 1+ _x_offset];
ix = kx;
{
int _i_inc = -1;
forloop70:
for (i = n; i >= j+1; i += _i_inc) {
x[(ix)- 1+ _x_offset] = x[(ix)- 1+ _x_offset]+temp*a[(i)- 1+(j- 1)*lda+ _a_offset];
ix = ix-incx;
Dummy.label("Dtrmv",70);
}              //  Close for() loop. 
}
if (nounit)  
    x[(jx)- 1+ _x_offset] = x[(jx)- 1+ _x_offset]*a[(j)- 1+(j- 1)*lda+ _a_offset];
}              // Close if()
jx = jx-incx;
Dummy.label("Dtrmv",80);
}              //  Close for() loop. 
}
}              //  Close else.
}              //  Close else.
}              // Close if()
else  {
  // *
// *        Form  x := A'*x.
// *
if ((uplo.toLowerCase().charAt(0) == "U".toLowerCase().charAt(0)))  {
    if (incx == 1)  {
    {
int _j_inc = -1;
forloop100:
for (j = n; j >= 1; j += _j_inc) {
temp = x[(j)- 1+ _x_offset];
if (nounit)  
    temp = temp*a[(j)- 1+(j- 1)*lda+ _a_offset];
{
int _i_inc = -1;
forloop90:
for (i = j-1; i >= 1; i += _i_inc) {
temp = temp+a[(i)- 1+(j- 1)*lda+ _a_offset]*x[(i)- 1+ _x_offset];
Dummy.label("Dtrmv",90);
}              //  Close for() loop. 
}
x[(j)- 1+ _x_offset] = temp;
Dummy.label("Dtrmv",100);
}              //  Close for() loop. 
}
}              // Close if()
else  {
  jx = kx+(n-1)*incx;
{
int _j_inc = -1;
forloop120:
for (j = n; j >= 1; j += _j_inc) {
temp = x[(jx)- 1+ _x_offset];
ix = jx;
if (nounit)  
    temp = temp*a[(j)- 1+(j- 1)*lda+ _a_offset];
{
int _i_inc = -1;
forloop110:
for (i = j-1; i >= 1; i += _i_inc) {
ix = ix-incx;
temp = temp+a[(i)- 1+(j- 1)*lda+ _a_offset]*x[(ix)- 1+ _x_offset];
Dummy.label("Dtrmv",110);
}              //  Close for() loop. 
}
x[(jx)- 1+ _x_offset] = temp;
jx = jx-incx;
Dummy.label("Dtrmv",120);
}              //  Close for() loop. 
}
}              //  Close else.
}              // Close if()
else  {
  if (incx == 1)  {
    {
forloop140:
for (j = 1; j <= n; j++) {
temp = x[(j)- 1+ _x_offset];
if (nounit)  
    temp = temp*a[(j)- 1+(j- 1)*lda+ _a_offset];
{
forloop130:
for (i = j+1; i <= n; i++) {
temp = temp+a[(i)- 1+(j- 1)*lda+ _a_offset]*x[(i)- 1+ _x_offset];
Dummy.label("Dtrmv",130);
}              //  Close for() loop. 
}
x[(j)- 1+ _x_offset] = temp;
Dummy.label("Dtrmv",140);
}              //  Close for() loop. 
}
}              // Close if()
else  {
  jx = kx;
{
forloop160:
for (j = 1; j <= n; j++) {
temp = x[(jx)- 1+ _x_offset];
ix = jx;
if (nounit)  
    temp = temp*a[(j)- 1+(j- 1)*lda+ _a_offset];
{
forloop150:
for (i = j+1; i <= n; i++) {
ix = ix+incx;
temp = temp+a[(i)- 1+(j- 1)*lda+ _a_offset]*x[(ix)- 1+ _x_offset];
Dummy.label("Dtrmv",150);
}              //  Close for() loop. 
}
x[(jx)- 1+ _x_offset] = temp;
jx = jx+incx;
Dummy.label("Dtrmv",160);
}              //  Close for() loop. 
}
}              //  Close else.
}              //  Close else.
}              //  Close else.
// *
Dummy.go_to("Dtrmv",999999);
// *
// *     End of DTRMV .
// *
Dummy.label("Dtrmv",999999);
return;
   }
} // End class.
