/*
 *  Produced by f2java.  f2java is part of the Fortran-
 *  -to-Java project at the University of Tennessee Netlib
 *  numerical software repository.
 *
 *  Original authorship for the BLAS and LAPACK numerical
 *  routines may be found in the Fortran source, available at
 *  www.netlib.org.
 *
 *  Fortran input file: jlapack.f
 *
 *  The f2j compiler code was written by
 *  David M. Doolin (doolin@cs.utk.edu) and
 *  Keith  Seymour (seymour@cs.utk.edu)
 */

package org.netlib.lapack;
import java.lang.*;
import org.netlib.util.*;



public class Dgehd2 {

// *
// *  -- LAPACK routine (version 2.0) --
// *     Univ. of Tennessee, Univ. of California Berkeley, NAG Ltd.,
// *     Courant Institute, Argonne National Lab, and Rice University
// *     October 31, 1992
// *
// *     .. Scalar Arguments ..
// *     ..
// *     .. Array Arguments ..
// *     ..
// *
// *  Purpose
// *  =======
// *
// *  DGEHD2 reduces a real general matrix A to upper Hessenberg form H by
// *  an orthogonal similarity transformation:  Q' * A * Q = H .
// *
// *  Arguments
// *  =========
// *
// *  N       (input) INTEGER
// *          The order of the matrix A.  N >= 0.
// *
// *  ILO     (input) INTEGER
// *  IHI     (input) INTEGER
// *          It is assumed that A is already upper triangular in rows
// *          and columns 1:ILO-1 and IHI+1:N. ILO and IHI are normally
// *          set by a previous call to DGEBAL; otherwise they should be
// *          set to 1 and N respectively. See Further Details.
// *          1 <= ILO <= IHI <= max(1,N).
// *
// *  A       (input/output) DOUBLE PRECISION array, dimension (LDA,N)
// *          On entry, the n by n general matrix to be reduced.
// *          On exit, the upper triangle and the first subdiagonal of A
// *          are overwritten with the upper Hessenberg matrix H, and the
// *          elements below the first subdiagonal, with the array TAU,
// *          represent the orthogonal matrix Q as a product of elementary
// *          reflectors. See Further Details.
// *
// *  LDA     (input) INTEGER
// *          The leading dimension of the array A.  LDA >= max(1,N).
// *
// *  TAU     (output) DOUBLE PRECISION array, dimension (N-1)
// *          The scalar factors of the elementary reflectors (see Further
// *          Details).
// *
// *  WORK    (workspace) DOUBLE PRECISION array, dimension (N)
// *
// *  INFO    (output) INTEGER
// *          = 0:  successful exit.
// *          < 0:  if INFO = -i, the i-th argument had an illegal value.
// *
// *  Further Details
// *  ===============
// *
// *  The matrix Q is represented as a product of (ihi-ilo) elementary
// *  reflectors
// *
// *     Q = H(ilo) H(ilo+1) . . . H(ihi-1).
// *
// *  Each H(i) has the form
// *
// *     H(i) = I - tau * v * v'
// *
// *  where tau is a real scalar, and v is a real vector with
// *  v(1:i) = 0, v(i+1) = 1 and v(ihi+1:n) = 0; v(i+2:ihi) is stored on
// *  exit in A(i+2:ihi,i), and tau in TAU(i).
// *
// *  The contents of A are illustrated by the following example, with
// *  n = 7, ilo = 2 and ihi = 6:
// *
// *  on entry,                        on exit,
// *
// *  ( a   a   a   a   a   a   a )    (  a   a   h   h   h   h   a )
// *  (     a   a   a   a   a   a )    (      a   h   h   h   h   a )
// *  (     a   a   a   a   a   a )    (      h   h   h   h   h   h )
// *  (     a   a   a   a   a   a )    (      v2  h   h   h   h   h )
// *  (     a   a   a   a   a   a )    (      v2  v3  h   h   h   h )
// *  (     a   a   a   a   a   a )    (      v2  v3  v4  h   h   h )
// *  (                         a )    (                          a )
// *
// *  where a denotes an element of the original matrix A, h denotes a
// *  modified element of the upper Hessenberg matrix H, and vi denotes an
// *  element of the vector defining H(i).
// *
// *  =====================================================================
// *
// *     .. Parameters ..
static double one= 1.0e+0;
// *     ..
// *     .. Local Scalars ..
static int i= 0;
static double aii= 0.0;
// *     ..
// *     .. External Subroutines ..
// *     ..
// *     .. Intrinsic Functions ..
// *     ..
// *     .. Executable Statements ..
// *
// *     Test the input parameters
// *

public static void dgehd2 (int n,
int ilo,
int ihi,
double [] a, int _a_offset,
int lda,
double [] tau, int _tau_offset,
double [] work, int _work_offset,
intW info)  {

info.val = 0;
if (n < 0)  {
    info.val = -1;
}              // Close if()
else if (ilo < 1 || ilo > Math.max(1, n) )  {
    info.val = -2;
}              // Close else if()
else if (ihi < Math.min(ilo, n)  || ihi > n)  {
    info.val = -3;
}              // Close else if()
else if (lda < Math.max(1, n) )  {
    info.val = -5;
}              // Close else if()
if (info.val != 0)  {
    Xerbla.xerbla("DGEHD2",-info.val);
Dummy.go_to("Dgehd2",999999);
}              // Close if()
// *
{
forloop10:
for (i = ilo; i <= ihi-1; i++) {
// *
// *        Compute elementary reflector H(i) to annihilate A(i+2:ihi,i)
// *
dlarfg_adapter(ihi-i,a,(i+1)- 1+(i- 1)*lda+ _a_offset,a,(int)((Math.min(i+2, n) )- 1+(i- 1)*lda+ _a_offset),1,tau,(i)- 1+ _tau_offset);
aii = a[(i+1)- 1+(i- 1)*lda+ _a_offset];
a[(i+1)- 1+(i- 1)*lda+ _a_offset] = one;
// *
// *        Apply H(i) to A(1:ihi,i+1:ihi) from the right
// *
Dlarf.dlarf("Right",ihi,ihi-i,a,(i+1)- 1+(i- 1)*lda+ _a_offset,1,tau[(i)- 1+ _tau_offset],a,(1)- 1+(i+1- 1)*lda+ _a_offset,lda,work,_work_offset);
// *
// *        Apply H(i) to A(i+1:ihi,i+1:n) from the left
// *
Dlarf.dlarf("Left",ihi-i,n-i,a,(i+1)- 1+(i- 1)*lda+ _a_offset,1,tau[(i)- 1+ _tau_offset],a,(i+1)- 1+(i+1- 1)*lda+ _a_offset,lda,work,_work_offset);
// *
a[(i+1)- 1+(i- 1)*lda+ _a_offset] = aii;
Dummy.label("Dgehd2",10);
}              //  Close for() loop. 
}
// *
Dummy.go_to("Dgehd2",999999);
// *
// *     End of DGEHD2
// *
Dummy.label("Dgehd2",999999);
return;
   }
// adapter for dlarfg
private static void dlarfg_adapter(int arg0 ,double [] arg1 , int arg1_offset ,double [] arg2 , int arg2_offset ,int arg3 ,double [] arg4 , int arg4_offset )
{
doubleW _f2j_tmp1 = new doubleW(arg1[arg1_offset]);
doubleW _f2j_tmp4 = new doubleW(arg4[arg4_offset]);

Dlarfg.dlarfg(arg0,_f2j_tmp1,arg2, arg2_offset,arg3,_f2j_tmp4);

arg1[arg1_offset] = _f2j_tmp1.val;
arg4[arg4_offset] = _f2j_tmp4.val;
}

} // End class.
