/*
 *  Produced by f2java.  f2java is part of the Fortran-
 *  -to-Java project at the University of Tennessee Netlib
 *  numerical software repository.
 *
 *  Original authorship for the BLAS and LAPACK numerical
 *  routines may be found in the Fortran source, available at
 *  www.netlib.org.
 *
 *  Fortran input file: jlapack.f
 *
 *  The f2j compiler code was written by
 *  David M. Doolin (doolin@cs.utk.edu) and
 *  Keith  Seymour (seymour@cs.utk.edu)
 */

package org.netlib.lapack;
import java.lang.*;
import org.netlib.util.*;



public class Dgeqlf {

// *
// *  -- LAPACK routine (version 2.0) --
// *     Univ. of Tennessee, Univ. of California Berkeley, NAG Ltd.,
// *     Courant Institute, Argonne National Lab, and Rice University
// *     September 30, 1994
// *
// *     .. Scalar Arguments ..
// *     ..
// *     .. Array Arguments ..
// *     ..
// *
// *  Purpose
// *  =======
// *
// *  DGEQLF computes a QL factorization of a real M-by-N matrix A:
// *  A = Q * L.
// *
// *  Arguments
// *  =========
// *
// *  M       (input) INTEGER
// *          The number of rows of the matrix A.  M >= 0.
// *
// *  N       (input) INTEGER
// *          The number of columns of the matrix A.  N >= 0.
// *
// *  A       (input/output) DOUBLE PRECISION array, dimension (LDA,N)
// *          On entry, the M-by-N matrix A.
// *          On exit,
// *          if m >= n, the lower triangle of the subarray
// *          A(m-n+1:m,1:n) contains the N-by-N lower triangular matrix L;
// *          if m <= n, the elements on and below the (n-m)-th
// *          superdiagonal contain the M-by-N lower trapezoidal matrix L;
// *          the remaining elements, with the array TAU, represent the
// *          orthogonal matrix Q as a product of elementary reflectors
// *          (see Further Details).
// *
// *  LDA     (input) INTEGER
// *          The leading dimension of the array A.  LDA >= max(1,M).
// *
// *  TAU     (output) DOUBLE PRECISION array, dimension (min(M,N))
// *          The scalar factors of the elementary reflectors (see Further
// *          Details).
// *
// *  WORK    (workspace/output) DOUBLE PRECISION array, dimension (LWORK)
// *          On exit, if INFO = 0, WORK(1) returns the optimal LWORK.
// *
// *  LWORK   (input) INTEGER
// *          The dimension of the array WORK.  LWORK >= max(1,N).
// *          For optimum performance LWORK >= N*NB, where NB is the
// *          optimal blocksize.
// *
// *  INFO    (output) INTEGER
// *          = 0:  successful exit
// *          < 0:  if INFO = -i, the i-th argument had an illegal value
// *
// *  Further Details
// *  ===============
// *
// *  The matrix Q is represented as a product of elementary reflectors
// *
// *     Q = H(k) . . . H(2) H(1), where k = min(m,n).
// *
// *  Each H(i) has the form
// *
// *     H(i) = I - tau * v * v'
// *
// *  where tau is a real scalar, and v is a real vector with
// *  v(m-k+i+1:m) = 0 and v(m-k+i) = 1; v(1:m-k+i-1) is stored on exit in
// *  A(1:m-k+i-1,n-k+i), and tau in TAU(i).
// *
// *  =====================================================================
// *
// *     .. Local Scalars ..
static int i= 0;
static int ib= 0;
static intW iinfo= new intW(0);
static int iws= 0;
static int k= 0;
static int ki= 0;
static int kk= 0;
static int ldwork= 0;
static int mu= 0;
static int nb= 0;
static int nbmin= 0;
static int nu= 0;
static int nx= 0;
// *     ..
// *     .. External Subroutines ..
// *     ..
// *     .. Intrinsic Functions ..
// *     ..
// *     .. External Functions ..
// *     ..
// *     .. Executable Statements ..
// *
// *     Test the input arguments
// *

public static void dgeqlf (int m,
int n,
double [] a, int _a_offset,
int lda,
double [] tau, int _tau_offset,
double [] work, int _work_offset,
int lwork,
intW info)  {

info.val = 0;
if (m < 0)  {
    info.val = -1;
}              // Close if()
else if (n < 0)  {
    info.val = -2;
}              // Close else if()
else if (lda < Math.max(1, m) )  {
    info.val = -4;
}              // Close else if()
else if (lwork < Math.max(1, n) )  {
    info.val = -7;
}              // Close else if()
if (info.val != 0)  {
    Xerbla.xerbla("DGEQLF",-info.val);
Dummy.go_to("Dgeqlf",999999);
}              // Close if()
// *
// *     Quick return if possible
// *
k = (int)(Math.min(m, n) );
if (k == 0)  {
    work[(1)- 1+ _work_offset] = (double)(1);
Dummy.go_to("Dgeqlf",999999);
}              // Close if()
// *
// *     Determine the block size.
// *
nb = Ilaenv.ilaenv(1,"DGEQLF"," ",m,n,-1,-1);
nbmin = 2;
nx = 1;
iws = n;
if (nb > 1 && nb < k)  {
    // *
// *        Determine when to cross over from blocked to unblocked code.
// *
nx = (int)(Math.max(0, Ilaenv.ilaenv(3,"DGEQLF"," ",m,n,-1,-1)) );
if (nx < k)  {
    // *
// *           Determine if workspace is large enough for blocked code.
// *
ldwork = n;
iws = ldwork*nb;
if (lwork < iws)  {
    // *
// *              Not enough workspace to use optimal NB:  reduce NB and
// *              determine the minimum value of NB.
// *
nb = lwork/ldwork;
nbmin = (int)(Math.max(2, Ilaenv.ilaenv(2,"DGEQLF"," ",m,n,-1,-1)) );
}              // Close if()
}              // Close if()
}              // Close if()
// *
if (nb >= nbmin && nb < k && nx < k)  {
    // *
// *        Use blocked code initially.
// *        The last kk columns are handled by the block method.
// *
ki = ((k-nx-1)/nb)*nb;
kk = (int)(Math.min(k, ki+nb) );
// *
{
int _i_inc = -nb;
forloop10:
for (i = k-kk+ki+1; (_i_inc < 0) ? i >= k-kk+1 : i <= k-kk+1; i += _i_inc) {
ib = (int)(Math.min(k-i+1, nb) );
// *
// *           Compute the QL factorization of the current block
// *           A(1:m-k+i+ib-1,n-k+i:n-k+i+ib-1)
// *
Dgeql2.dgeql2(m-k+i+ib-1,ib,a,(1)- 1+(n-k+i- 1)*lda+ _a_offset,lda,tau,(i)- 1+ _tau_offset,work,_work_offset,iinfo);
if (n-k+i > 1)  {
    // *
// *              Form the triangular factor of the block reflector
// *              H = H(i+ib-1) . . . H(i+1) H(i)
// *
Dlarft.dlarft("Backward","Columnwise",m-k+i+ib-1,ib,a,(1)- 1+(n-k+i- 1)*lda+ _a_offset,lda,tau,(i)- 1+ _tau_offset,work,_work_offset,ldwork);
// *
// *              Apply H' to A(1:m-k+i+ib-1,1:n-k+i-1) from the left
// *
Dlarfb.dlarfb("Left","Transpose","Backward","Columnwise",m-k+i+ib-1,n-k+i-1,ib,a,(1)- 1+(n-k+i- 1)*lda+ _a_offset,lda,work,_work_offset,ldwork,a,_a_offset,lda,work,(ib+1)- 1+ _work_offset,ldwork);
}              // Close if()
Dummy.label("Dgeqlf",10);
}              //  Close for() loop. 
}
mu = m-k+i+nb-1;
nu = n-k+i+nb-1;
}              // Close if()
else  {
  mu = m;
nu = n;
}              //  Close else.
// *
// *     Use unblocked code to factor the last or only block
// *
if (mu > 0 && nu > 0)  
    Dgeql2.dgeql2(mu,nu,a,_a_offset,lda,tau,_tau_offset,work,_work_offset,iinfo);
// *
work[(1)- 1+ _work_offset] = (double)(iws);
Dummy.go_to("Dgeqlf",999999);
// *
// *     End of DGEQLF
// *
Dummy.label("Dgeqlf",999999);
return;
   }
} // End class.
