/*
 *  Produced by f2java.  f2java is part of the Fortran-
 *  -to-Java project at the University of Tennessee Netlib
 *  numerical software repository.
 *
 *  Original authorship for the BLAS and LAPACK numerical
 *  routines may be found in the Fortran source, available at
 *  www.netlib.org.
 *
 *  Fortran input file: jlapack.f
 *
 *  The f2j compiler code was written by
 *  David M. Doolin (doolin@cs.utk.edu) and
 *  Keith  Seymour (seymour@cs.utk.edu)
 */

package org.netlib.lapack;
import java.lang.*;
import org.netlib.util.*;



public class Dggrqf {

// *
// *  -- LAPACK routine (version 2.0) --
// *     Univ. of Tennessee, Univ. of California Berkeley, NAG Ltd.,
// *     Courant Institute, Argonne National Lab, and Rice University
// *     September 30, 1994
// *
// *     .. Scalar Arguments ..
// *     ..
// *     .. Array Arguments ..
// *     ..
// *
// *  Purpose
// *  =======
// *
// *  DGGRQF computes a generalized RQ factorization of an M-by-N matrix A
// *  and a P-by-N matrix B:
// *
// *              A = R*Q,        B = Z*T*Q,
// *
// *  where Q is an N-by-N orthogonal matrix, Z is a P-by-P orthogonal
// *  matrix, and R and T assume one of the forms:
// *
// *  if M <= N,  R = ( 0  R12 ) M,   or if M > N,  R = ( R11 ) M-N,
// *                   N-M  M                           ( R21 ) N
// *                                                       N
// *
// *  where R12 or R21 is upper triangular, and
// *
// *  if P >= N,  T = ( T11 ) N  ,   or if P < N,  T = ( T11  T12 ) P,
// *                  (  0  ) P-N                         P   N-P
// *                     N
// *
// *  where T11 is upper triangular.
// *
// *  In particular, if B is square and nonsingular, the GRQ factorization
// *  of A and B implicitly gives the RQ factorization of A*inv(B):
// *
// *               A*inv(B) = (R*inv(T))*Z'
// *
// *  where inv(B) denotes the inverse of the matrix B, and Z' denotes the
// *  transpose of the matrix Z.
// *
// *  Arguments
// *  =========
// *
// *  M       (input) INTEGER
// *          The number of rows of the matrix A.  M >= 0.
// *
// *  P       (input) INTEGER
// *          The number of rows of the matrix B.  P >= 0.
// *
// *  N       (input) INTEGER
// *          The number of columns of the matrices A and B. N >= 0.
// *
// *  A       (input/output) DOUBLE PRECISION array, dimension (LDA,N)
// *          On entry, the M-by-N matrix A.
// *          On exit, if M <= N, the upper triangle of the subarray
// *          A(1:M,N-M+1:N) contains the M-by-M upper triangular matrix R;
// *          if M > N, the elements on and above the (M-N)-th subdiagonal
// *          contain the M-by-N upper trapezoidal matrix R; the remaining
// *          elements, with the array TAUA, represent the orthogonal
// *          matrix Q as a product of elementary reflectors (see Further
// *          Details).
// *
// *  LDA     (input) INTEGER
// *          The leading dimension of the array A. LDA >= max(1,M).
// *
// *  TAUA    (output) DOUBLE PRECISION array, dimension (min(M,N))
// *          The scalar factors of the elementary reflectors which
// *          represent the orthogonal matrix Q (see Further Details).
// *
// *  B       (input/output) DOUBLE PRECISION array, dimension (LDB,N)
// *          On entry, the P-by-N matrix B.
// *          On exit, the elements on and above the diagonal of the array
// *          contain the min(P,N)-by-N upper trapezoidal matrix T (T is
// *          upper triangular if P >= N); the elements below the diagonal,
// *          with the array TAUB, represent the orthogonal matrix Z as a
// *          product of elementary reflectors (see Further Details).
// *
// *  LDB     (input) INTEGER
// *          The leading dimension of the array B. LDB >= max(1,P).
// *
// *  TAUB    (output) DOUBLE PRECISION array, dimension (min(P,N))
// *          The scalar factors of the elementary reflectors which
// *          represent the orthogonal matrix Z (see Further Details).
// *
// *  WORK    (workspace/output) DOUBLE PRECISION array, dimension (LWORK)
// *          On exit, if INFO = 0, WORK(1) returns the optimal LWORK.
// *
// *  LWORK   (input) INTEGER
// *          The dimension of the array WORK. LWORK >= max(1,N,M,P).
// *          For optimum performance LWORK >= max(N,M,P)*max(NB1,NB2,NB3),
// *          where NB1 is the optimal blocksize for the RQ factorization
// *          of an M-by-N matrix, NB2 is the optimal blocksize for the
// *          QR factorization of a P-by-N matrix, and NB3 is the optimal
// *          blocksize for a call of DORMRQ.
// *
// *  INFO    (output) INTEGER
// *          = 0:  successful exit
// *          < 0:  if INF0= -i, the i-th argument had an illegal value.
// *
// *  Further Details
// *  ===============
// *
// *  The matrix Q is represented as a product of elementary reflectors
// *
// *     Q = H(1) H(2) . . . H(k), where k = min(m,n).
// *
// *  Each H(i) has the form
// *
// *     H(i) = I - taua * v * v'
// *
// *  where taua is a real scalar, and v is a real vector with
// *  v(n-k+i+1:n) = 0 and v(n-k+i) = 1; v(1:n-k+i-1) is stored on exit in
// *  A(m-k+i,1:n-k+i-1), and taua in TAUA(i).
// *  To form Q explicitly, use LAPACK subroutine DORGRQ.
// *  To use Q to update another matrix, use LAPACK subroutine DORMRQ.
// *
// *  The matrix Z is represented as a product of elementary reflectors
// *
// *     Z = H(1) H(2) . . . H(k), where k = min(p,n).
// *
// *  Each H(i) has the form
// *
// *     H(i) = I - taub * v * v'
// *
// *  where taub is a real scalar, and v is a real vector with
// *  v(1:i-1) = 0 and v(i) = 1; v(i+1:p) is stored on exit in B(i+1:p,i),
// *  and taub in TAUB(i).
// *  To form Z explicitly, use LAPACK subroutine DORGQR.
// *  To use Z to update another matrix, use LAPACK subroutine DORMQR.
// *
// *  =====================================================================
// *
// *     .. Local Scalars ..
static int lopt= 0;
// *     ..
// *     .. External Subroutines ..
// *     ..
// *     .. Intrinsic Functions ..
// *     ..
// *     .. Executable Statements ..
// *
// *     Test the input parameters
// *

public static void dggrqf (int m,
int p,
int n,
double [] a, int _a_offset,
int lda,
double [] taua, int _taua_offset,
double [] b, int _b_offset,
int ldb,
double [] taub, int _taub_offset,
double [] work, int _work_offset,
int lwork,
intW info)  {

info.val = 0;
if (m < 0)  {
    info.val = -1;
}              // Close if()
else if (p < 0)  {
    info.val = -2;
}              // Close else if()
else if (n < 0)  {
    info.val = -3;
}              // Close else if()
else if (lda < Math.max(1, m) )  {
    info.val = -5;
}              // Close else if()
else if (ldb < Math.max(1, p) )  {
    info.val = -8;
}              // Close else if()
else if (lwork < Math.max(Math.max(Math.max(1, m), p), n) )  {
    info.val = -11;
}              // Close else if()
if (info.val != 0)  {
    Xerbla.xerbla("DGGRQF",-info.val);
Dummy.go_to("Dggrqf",999999);
}              // Close if()
// *
// *     RQ factorization of M-by-N matrix A: A = R*Q
// *
Dgerqf.dgerqf(m,n,a,_a_offset,lda,taua,_taua_offset,work,_work_offset,lwork,info);
lopt = (int)(work[(1)- 1+ _work_offset]);
// *
// *     Update B := B*Q'
// *
Dormrq.dormrq("Right","Transpose",p,n,(int) ( Math.min(m, n) ),a,(int)((Math.max(1, m-n+1) )- 1+(1- 1)*lda+ _a_offset),lda,taua,_taua_offset,b,_b_offset,ldb,work,_work_offset,lwork,info);
lopt = (int)(Math.max(lopt, (int)(work[(1)- 1+ _work_offset])) );
// *
// *     QR factorization of P-by-N matrix B: B = Z*T
// *
Dgeqrf.dgeqrf(p,n,b,_b_offset,ldb,taub,_taub_offset,work,_work_offset,lwork,info);
work[(1)- 1+ _work_offset] = Math.max(lopt, (int)(work[(1)- 1+ _work_offset])) ;
// *
Dummy.go_to("Dggrqf",999999);
// *
// *     End of DGGRQF
// *
Dummy.label("Dggrqf",999999);
return;
   }
} // End class.
