/*
 *  Produced by f2java.  f2java is part of the Fortran-
 *  -to-Java project at the University of Tennessee Netlib
 *  numerical software repository.
 *
 *  Original authorship for the BLAS and LAPACK numerical
 *  routines may be found in the Fortran source, available at
 *  www.netlib.org.
 *
 *  Fortran input file: jlapack.f
 *
 *  The f2j compiler code was written by
 *  David M. Doolin (doolin@cs.utk.edu) and
 *  Keith  Seymour (seymour@cs.utk.edu)
 */

package org.netlib.lapack;
import java.lang.*;
import org.netlib.util.*;

import org.netlib.blas.*;


public class Dhseqr {

// *
// *  -- LAPACK routine (version 2.0) --
// *     Univ. of Tennessee, Univ. of California Berkeley, NAG Ltd.,
// *     Courant Institute, Argonne National Lab, and Rice University
// *     September 30, 1994
// *
// *     .. Scalar Arguments ..
// *     ..
// *     .. Array Arguments ..
// *     ..
// *
// *  Purpose
// *  =======
// *
// *  DHSEQR computes the eigenvalues of a real upper Hessenberg matrix H
// *  and, optionally, the matrices T and Z from the Schur decomposition
// *  H = Z T Z**T, where T is an upper quasi-triangular matrix (the Schur
// *  form), and Z is the orthogonal matrix of Schur vectors.
// *
// *  Optionally Z may be postmultiplied into an input orthogonal matrix Q,
// *  so that this routine can give the Schur factorization of a matrix A
// *  which has been reduced to the Hessenberg form H by the orthogonal
// *  matrix Q:  A = Q*H*Q**T = (QZ)*T*(QZ)**T.
// *
// *  Arguments
// *  =========
// *
// *  JOB     (input) CHARACTER*1
// *          = 'E':  compute eigenvalues only;
// *          = 'S':  compute eigenvalues and the Schur form T.
// *
// *  COMPZ   (input) CHARACTER*1
// *          = 'N':  no Schur vectors are computed;
// *          = 'I':  Z is initialized to the unit matrix and the matrix Z
// *                  of Schur vectors of H is returned;
// *          = 'V':  Z must contain an orthogonal matrix Q on entry, and
// *                  the product Q*Z is returned.
// *
// *  N       (input) INTEGER
// *          The order of the matrix H.  N >= 0.
// *
// *  ILO     (input) INTEGER
// *  IHI     (input) INTEGER
// *          It is assumed that H is already upper triangular in rows
// *          and columns 1:ILO-1 and IHI+1:N. ILO and IHI are normally
// *          set by a previous call to DGEBAL, and then passed to SGEHRD
// *          when the matrix output by DGEBAL is reduced to Hessenberg
// *          form. Otherwise ILO and IHI should be set to 1 and N
// *          respectively.
// *          1 <= ILO <= IHI <= N, if N > 0; ILO=1 and IHI=0, if N=0.
// *
// *  H       (input/output) DOUBLE PRECISION array, dimension (LDH,N)
// *          On entry, the upper Hessenberg matrix H.
// *          On exit, if JOB = 'S', H contains the upper quasi-triangular
// *          matrix T from the Schur decomposition (the Schur form);
// *          2-by-2 diagonal blocks (corresponding to complex conjugate
// *          pairs of eigenvalues) are returned in standard form, with
// *          H(i,i) = H(i+1,i+1) and H(i+1,i)*H(i,i+1) < 0. If JOB = 'E',
// *          the contents of H are unspecified on exit.
// *
// *  LDH     (input) INTEGER
// *          The leading dimension of the array H. LDH >= max(1,N).
// *
// *  WR      (output) DOUBLE PRECISION array, dimension (N)
// *  WI      (output) DOUBLE PRECISION array, dimension (N)
// *          The real and imaginary parts, respectively, of the computed
// *          eigenvalues. If two eigenvalues are computed as a complex
// *          conjugate pair, they are stored in consecutive elements of
// *          WR and WI, say the i-th and (i+1)th, with WI(i) > 0 and
// *          WI(i+1) < 0. If JOB = 'S', the eigenvalues are stored in the
// *          same order as on the diagonal of the Schur form returned in
// *          H, with WR(i) = H(i,i) and, if H(i:i+1,i:i+1) is a 2-by-2
// *          diagonal block, WI(i) = sqrt(H(i+1,i)*H(i,i+1)) and
// *          WI(i+1) = -WI(i).
// *
// *  Z       (input/output) DOUBLE PRECISION array, dimension (LDZ,N)
// *          If COMPZ = 'N': Z is not referenced.
// *          If COMPZ = 'I': on entry, Z need not be set, and on exit, Z
// *          contains the orthogonal matrix Z of the Schur vectors of H.
// *          If COMPZ = 'V': on entry Z must contain an N-by-N matrix Q,
// *          which is assumed to be equal to the unit matrix except for
// *          the submatrix Z(ILO:IHI,ILO:IHI); on exit Z contains Q*Z.
// *          Normally Q is the orthogonal matrix generated by DORGHR after
// *          the call to DGEHRD which formed the Hessenberg matrix H.
// *
// *  LDZ     (input) INTEGER
// *          The leading dimension of the array Z.
// *          LDZ >= max(1,N) if COMPZ = 'I' or 'V'; LDZ >= 1 otherwise.
// *
// *  WORK    (workspace) DOUBLE PRECISION array, dimension (N)
// *
// *  LWORK   (input) INTEGER
// *          This argument is currently redundant.
// *
// *  INFO    (output) INTEGER
// *          = 0:  successful exit
// *          < 0:  if INFO = -i, the i-th argument had an illegal value
// *          > 0:  if INFO = i, DHSEQR failed to compute all of the
// *                eigenvalues in a total of 30*(IHI-ILO+1) iterations;
// *                elements 1:ilo-1 and i+1:n of WR and WI contain those
// *                eigenvalues which have been successfully computed.
// *
// *  =====================================================================
// *
// *     .. Parameters ..
static double zero= 0.0e+0;
static double one= 1.0e+0;
static double two= 2.0e+0;
static double Const= 1.5e+0;
static int nsmax= 15;
static int lds= nsmax;
// *     ..
// *     .. Local Scalars ..
static boolean initz= false;
static boolean wantt= false;
static boolean wantz= false;
static int i= 0;
static int i1= 0;
static int i2= 0;
static intW ierr= new intW(0);
static int ii= 0;
static int itemp= 0;
static int itn= 0;
static int its= 0;
static int j= 0;
static int k= 0;
static int l= 0;
static int maxb= 0;
static int nh= 0;
static int nr= 0;
static int ns= 0;
static int nv= 0;
static double absw= 0.0;
static doubleW ovfl= new doubleW(0.0);
static double smlnum= 0.0;
static doubleW tau= new doubleW(0.0);
static double temp= 0.0;
static double tst1= 0.0;
static double ulp= 0.0;
static doubleW unfl= new doubleW(0.0);
// *     ..
// *     .. Local Arrays ..
static double [] s= new double[(lds) * (nsmax)];
static double [] v= new double[(nsmax+1)];
static double [] vv= new double[(nsmax+1)];
// *     ..
// *     .. External Functions ..
// *     ..
// *     .. External Subroutines ..
// *     ..
// *     .. Intrinsic Functions ..
// *     ..
// *     .. Executable Statements ..
// *
// *     Decode and test the input parameters
// *

public static void dhseqr (String job,
String compz,
int n,
int ilo,
int ihi,
double [] h, int _h_offset,
int ldh,
double [] wr, int _wr_offset,
double [] wi, int _wi_offset,
double [] z, int _z_offset,
int ldz,
double [] work, int _work_offset,
int lwork,
intW info)  {

wantt = (job.toLowerCase().charAt(0) == "S".toLowerCase().charAt(0));
initz = (compz.toLowerCase().charAt(0) == "I".toLowerCase().charAt(0));
wantz = initz || (compz.toLowerCase().charAt(0) == "V".toLowerCase().charAt(0));
// *
info.val = 0;
if (!(job.toLowerCase().charAt(0) == "E".toLowerCase().charAt(0)) && !wantt)  {
    info.val = -1;
}              // Close if()
else if (!(compz.toLowerCase().charAt(0) == "N".toLowerCase().charAt(0)) && !wantz)  {
    info.val = -2;
}              // Close else if()
else if (n < 0)  {
    info.val = -3;
}              // Close else if()
else if (ilo < 1 || ilo > Math.max(1, n) )  {
    info.val = -4;
}              // Close else if()
else if (ihi < Math.min(ilo, n)  || ihi > n)  {
    info.val = -5;
}              // Close else if()
else if (ldh < Math.max(1, n) )  {
    info.val = -7;
}              // Close else if()
else if (ldz < 1 || wantz && ldz < Math.max(1, n) )  {
    info.val = -11;
}              // Close else if()
if (info.val != 0)  {
    Xerbla.xerbla("DHSEQR",-info.val);
Dummy.go_to("Dhseqr",999999);
}              // Close if()
// *
// *     Initialize Z, if necessary
// *
if (initz)  
    Dlaset.dlaset("Full",n,n,zero,one,z,_z_offset,ldz);
// *
// *     Store the eigenvalues isolated by DGEBAL.
// *
{
forloop10:
for (i = 1; i <= ilo-1; i++) {
wr[(i)- 1+ _wr_offset] = h[(i)- 1+(i- 1)*ldh+ _h_offset];
wi[(i)- 1+ _wi_offset] = zero;
Dummy.label("Dhseqr",10);
}              //  Close for() loop. 
}
{
forloop20:
for (i = ihi+1; i <= n; i++) {
wr[(i)- 1+ _wr_offset] = h[(i)- 1+(i- 1)*ldh+ _h_offset];
wi[(i)- 1+ _wi_offset] = zero;
Dummy.label("Dhseqr",20);
}              //  Close for() loop. 
}
// *
// *     Quick return if possible.
// *
if (n == 0)  
    Dummy.go_to("Dhseqr",999999);
if (ilo == ihi)  {
    wr[(ilo)- 1+ _wr_offset] = h[(ilo)- 1+(ilo- 1)*ldh+ _h_offset];
wi[(ilo)- 1+ _wi_offset] = zero;
Dummy.go_to("Dhseqr",999999);
}              // Close if()
// *
// *     Set rows and columns ILO to IHI to zero below the first
// *     subdiagonal.
// *
{
forloop40:
for (j = ilo; j <= ihi-2; j++) {
{
forloop30:
for (i = j+2; i <= n; i++) {
h[(i)- 1+(j- 1)*ldh+ _h_offset] = zero;
Dummy.label("Dhseqr",30);
}              //  Close for() loop. 
}
Dummy.label("Dhseqr",40);
}              //  Close for() loop. 
}
nh = ihi-ilo+1;
// *
// *     Determine the order of the multi-shift QR algorithm to be used.
// *
ns = Ilaenv.ilaenv(4,"DHSEQR",job+compz,n,ilo,ihi,-1);
maxb = Ilaenv.ilaenv(8,"DHSEQR",job+compz,n,ilo,ihi,-1);
if (ns <= 2 || ns > nh || maxb >= nh)  {
    // *
// *        Use the standard double-shift algorithm
// *
Dlahqr.dlahqr(wantt,wantz,n,ilo,ihi,h,_h_offset,ldh,wr,_wr_offset,wi,_wi_offset,ilo,ihi,z,_z_offset,ldz,info);
Dummy.go_to("Dhseqr",999999);
}              // Close if()
maxb = (int)(Math.max(3, maxb) );
ns = (int)(Math.min((ns) < (maxb) ? (ns) : (maxb), nsmax));
// *
// *     Now 2 < NS <= MAXB < NH.
// *
// *     Set machine-dependent constants for the stopping criterion.
// *     If norm(H) <= sqrt(OVFL), overflow should not occur.
// *
unfl.val = Dlamch.dlamch("Safe minimum");
ovfl.val = one/unfl.val;
Dlabad.dlabad(unfl,ovfl);
ulp = Dlamch.dlamch("Precision");
smlnum = unfl.val*(nh/ulp);
// *
// *     I1 and I2 are the indices of the first row and last column of H
// *     to which transformations must be applied. If eigenvalues only are
// *     being computed, I1 and I2 are set inside the main loop.
// *
if (wantt)  {
    i1 = 1;
i2 = n;
}              // Close if()
// *
// *     ITN is the total number of multiple-shift QR iterations allowed.
// *
itn = 30*nh;
// *
// *     The main loop begins here. I is the loop index and decreases from
// *     IHI to ILO in steps of at most MAXB. Each iteration of the loop
// *     works with the active submatrix in rows and columns L to I.
// *     Eigenvalues I+1 to IHI have already converged. Either L = ILO or
// *     H(L,L-1) is negligible so that the matrix splits.
// *
i = ihi;
label50:
   Dummy.label("Dhseqr",50);
l = ilo;
if (i < ilo)  
    Dummy.go_to("Dhseqr",170);
// *
// *     Perform multiple-shift QR iterations on rows and columns ILO to I
// *     until a submatrix of order at most MAXB splits off at the bottom
// *     because a subdiagonal element has become negligible.
// *
{
forloop150:
for (its = 0; its <= itn; its++) {
// *
// *        Look for a single small subdiagonal element.
// *
{
int _k_inc = -1;
forloop60:
for (k = i; k >= l+1; k += _k_inc) {
tst1 = Math.abs(h[(k-1)- 1+(k-1- 1)*ldh+ _h_offset])+Math.abs(h[(k)- 1+(k- 1)*ldh+ _h_offset]);
if (tst1 == zero)  
    tst1 = Dlanhs.dlanhs("1",i-l+1,h,(l)- 1+(l- 1)*ldh+ _h_offset,ldh,work,_work_offset);
if (Math.abs(h[(k)- 1+(k-1- 1)*ldh+ _h_offset]) <= Math.max(ulp*tst1, smlnum) )  
    Dummy.go_to("Dhseqr",70);
Dummy.label("Dhseqr",60);
}              //  Close for() loop. 
}
label70:
   Dummy.label("Dhseqr",70);
l = k;
if (l > ilo)  {
    // *
// *           H(L,L-1) is negligible.
// *
h[(l)- 1+(l-1- 1)*ldh+ _h_offset] = zero;
}              // Close if()
// *
// *        Exit from loop if a submatrix of order <= MAXB has split off.
// *
if (l >= i-maxb+1)  
    Dummy.go_to("Dhseqr",160);
// *
// *        Now the active submatrix is in rows and columns L to I. If
// *        eigenvalues only are being computed, only the active submatrix
// *        need be transformed.
// *
if (!wantt)  {
    i1 = l;
i2 = i;
}              // Close if()
// *
if (its == 20 || its == 30)  {
    // *
// *           Exceptional shifts.
// *
{
forloop80:
for (ii = i-ns+1; ii <= i; ii++) {
wr[(ii)- 1+ _wr_offset] = Const*(Math.abs(h[(ii)- 1+(ii-1- 1)*ldh+ _h_offset])+Math.abs(h[(ii)- 1+(ii- 1)*ldh+ _h_offset]));
wi[(ii)- 1+ _wi_offset] = zero;
Dummy.label("Dhseqr",80);
}              //  Close for() loop. 
}
}              // Close if()
else  {
  // *
// *           Use eigenvalues of trailing submatrix of order NS as shifts.
// *
Dlacpy.dlacpy("Full",ns,ns,h,(i-ns+1)- 1+(i-ns+1- 1)*ldh+ _h_offset,ldh,s,0,lds);
Dlahqr.dlahqr(false,false,ns,1,ns,s,0,lds,wr,(i-ns+1)- 1+ _wr_offset,wi,(i-ns+1)- 1+ _wi_offset,1,ns,z,_z_offset,ldz,ierr);
if (ierr.val > 0)  {
    // *
// *              If DLAHQR failed to compute all NS eigenvalues, use the
// *              unconverged diagonal elements as the remaining shifts.
// *
{
forloop90:
for (ii = 1; ii <= ierr.val; ii++) {
wr[(i-ns+ii)- 1+ _wr_offset] = s[(ii)- 1+(ii- 1)*lds];
wi[(i-ns+ii)- 1+ _wi_offset] = zero;
Dummy.label("Dhseqr",90);
}              //  Close for() loop. 
}
}              // Close if()
}              //  Close else.
// *
// *        Form the first column of (G-w(1)) (G-w(2)) . . . (G-w(ns))
// *        where G is the Hessenberg submatrix H(L:I,L:I) and w is
// *        the vector of shifts (stored in WR and WI). The result is
// *        stored in the local array V.
// *
v[(1)- 1] = one;
{
forloop100:
for (ii = 2; ii <= ns+1; ii++) {
v[(ii)- 1] = zero;
Dummy.label("Dhseqr",100);
}              //  Close for() loop. 
}
nv = 1;
{
forloop120:
for (j = i-ns+1; j <= i; j++) {
if (wi[(j)- 1+ _wi_offset] >= zero)  {
    if (wi[(j)- 1+ _wi_offset] == zero)  {
    // *
// *                 real shift
// *
Dcopy.dcopy(nv+1,v,0,1,vv,0,1);
Dgemv.dgemv("No transpose",nv+1,nv,one,h,(l)- 1+(l- 1)*ldh+ _h_offset,ldh,vv,0,1,-wr[(j)- 1+ _wr_offset],v,0,1);
nv = nv+1;
}              // Close if()
else if (wi[(j)- 1+ _wi_offset] > zero)  {
    // *
// *                 complex conjugate pair of shifts
// *
Dcopy.dcopy(nv+1,v,0,1,vv,0,1);
Dgemv.dgemv("No transpose",nv+1,nv,one,h,(l)- 1+(l- 1)*ldh+ _h_offset,ldh,v,0,1,-two*wr[(j)- 1+ _wr_offset],vv,0,1);
itemp = Idamax.idamax(nv+1,vv,0,1);
temp = one/Math.max(Math.abs(vv[(itemp)- 1]), smlnum) ;
Dscal.dscal(nv+1,temp,vv,0,1);
absw = Dlapy2.dlapy2(wr[(j)- 1+ _wr_offset],wi[(j)- 1+ _wi_offset]);
temp = (temp*absw)*absw;
Dgemv.dgemv("No transpose",nv+2,nv+1,one,h,(l)- 1+(l- 1)*ldh+ _h_offset,ldh,vv,0,1,temp,v,0,1);
nv = nv+2;
}              // Close else if()
// *
// *              Scale V(1:NV) so that max(abs(V(i))) = 1. If V is zero,
// *              reset it to the unit vector.
// *
itemp = Idamax.idamax(nv,v,0,1);
temp = Math.abs(v[(itemp)- 1]);
if (temp == zero)  {
    v[(1)- 1] = one;
{
forloop110:
for (ii = 2; ii <= nv; ii++) {
v[(ii)- 1] = zero;
Dummy.label("Dhseqr",110);
}              //  Close for() loop. 
}
}              // Close if()
else  {
  temp = Math.max(temp, smlnum) ;
Dscal.dscal(nv,one/temp,v,0,1);
}              //  Close else.
}              // Close if()
Dummy.label("Dhseqr",120);
}              //  Close for() loop. 
}
// *
// *        Multiple-shift QR step
// *
{
forloop140:
for (k = l; k <= i-1; k++) {
// *
// *           The first iteration of this loop determines a reflection G
// *           from the vector V and applies it from left and right to H,
// *           thus creating a nonzero bulge below the subdiagonal.
// *
// *           Each subsequent iteration determines a reflection G to
// *           restore the Hessenberg form in the (K-1)th column, and thus
// *           chases the bulge one step toward the bottom of the active
// *           submatrix. NR is the order of G.
// *
nr = (int)(Math.min(ns+1, i-k+1) );
if (k > l)  
    Dcopy.dcopy(nr,h,(k)- 1+(k-1- 1)*ldh+ _h_offset,1,v,0,1);
dlarfg_adapter(nr,v,(1)- 1,v,(2)- 1,1,tau);
if (k > l)  {
    h[(k)- 1+(k-1- 1)*ldh+ _h_offset] = v[(1)- 1];
{
forloop130:
for (ii = k+1; ii <= i; ii++) {
h[(ii)- 1+(k-1- 1)*ldh+ _h_offset] = zero;
Dummy.label("Dhseqr",130);
}              //  Close for() loop. 
}
}              // Close if()
v[(1)- 1] = one;
// *
// *           Apply G from the left to transform the rows of the matrix in
// *           columns K to I2.
// *
Dlarfx.dlarfx("Left",nr,i2-k+1,v,0,tau.val,h,(k)- 1+(k- 1)*ldh+ _h_offset,ldh,work,_work_offset);
// *
// *           Apply G from the right to transform the columns of the
// *           matrix in rows I1 to min(K+NR,I).
// *
Dlarfx.dlarfx("Right",(int) ( Math.min(k+nr, i) -i1+1),nr,v,0,tau.val,h,(i1)- 1+(k- 1)*ldh+ _h_offset,ldh,work,_work_offset);
// *
if (wantz)  {
    // *
// *              Accumulate transformations in the matrix Z
// *
Dlarfx.dlarfx("Right",nh,nr,v,0,tau.val,z,(ilo)- 1+(k- 1)*ldz+ _z_offset,ldz,work,_work_offset);
}              // Close if()
Dummy.label("Dhseqr",140);
}              //  Close for() loop. 
}
// *
Dummy.label("Dhseqr",150);
}              //  Close for() loop. 
}
// *
// *     Failure to converge in remaining number of iterations
// *
info.val = i;
Dummy.go_to("Dhseqr",999999);
// *
label160:
   Dummy.label("Dhseqr",160);
// *
// *     A submatrix of order <= MAXB in rows and columns L to I has split
// *     off. Use the double-shift QR algorithm to handle it.
// *
Dlahqr.dlahqr(wantt,wantz,n,l,i,h,_h_offset,ldh,wr,_wr_offset,wi,_wi_offset,ilo,ihi,z,_z_offset,ldz,info);
if (info.val > 0)  
    Dummy.go_to("Dhseqr",999999);
// *
// *     Decrement number of remaining iterations, and return to start of
// *     the main loop with a new value of I.
// *
itn = itn-its;
i = l-1;
Dummy.go_to("Dhseqr",50);
// *
label170:
   Dummy.label("Dhseqr",170);
Dummy.go_to("Dhseqr",999999);
// *
// *     End of DHSEQR
// *
Dummy.label("Dhseqr",999999);
return;
   }
// adapter for dlarfg
private static void dlarfg_adapter(int arg0 ,double [] arg1 , int arg1_offset ,double [] arg2 , int arg2_offset ,int arg3 ,doubleW arg4 )
{
doubleW _f2j_tmp1 = new doubleW(arg1[arg1_offset]);

Dlarfg.dlarfg(arg0,_f2j_tmp1,arg2, arg2_offset,arg3,arg4);

arg1[arg1_offset] = _f2j_tmp1.val;
}

} // End class.
