/*
 *  Produced by f2java.  f2java is part of the Fortran-
 *  -to-Java project at the University of Tennessee Netlib
 *  numerical software repository.
 *
 *  Original authorship for the BLAS and LAPACK numerical
 *  routines may be found in the Fortran source, available at
 *  www.netlib.org.
 *
 *  Fortran input file: jlapack.f
 *
 *  The f2j compiler code was written by
 *  David M. Doolin (doolin@cs.utk.edu) and
 *  Keith  Seymour (seymour@cs.utk.edu)
 */

package org.netlib.lapack;
import java.lang.*;
import org.netlib.util.*;



public class Dlae2 {

// *
// *  -- LAPACK auxiliary routine (version 2.0) --
// *     Univ. of Tennessee, Univ. of California Berkeley, NAG Ltd.,
// *     Courant Institute, Argonne National Lab, and Rice University
// *     October 31, 1992
// *
// *     .. Scalar Arguments ..
// *     ..
// *
// *  Purpose
// *  =======
// *
// *  DLAE2  computes the eigenvalues of a 2-by-2 symmetric matrix
// *     [  A   B  ]
// *     [  B   C  ].
// *  On return, RT1 is the eigenvalue of larger absolute value, and RT2
// *  is the eigenvalue of smaller absolute value.
// *
// *  Arguments
// *  =========
// *
// *  A       (input) DOUBLE PRECISION
// *          The (1,1) element of the 2-by-2 matrix.
// *
// *  B       (input) DOUBLE PRECISION
// *          The (1,2) and (2,1) elements of the 2-by-2 matrix.
// *
// *  C       (input) DOUBLE PRECISION
// *          The (2,2) element of the 2-by-2 matrix.
// *
// *  RT1     (output) DOUBLE PRECISION
// *          The eigenvalue of larger absolute value.
// *
// *  RT2     (output) DOUBLE PRECISION
// *          The eigenvalue of smaller absolute value.
// *
// *  Further Details
// *  ===============
// *
// *  RT1 is accurate to a few ulps barring over/underflow.
// *
// *  RT2 may be inaccurate if there is massive cancellation in the
// *  determinant A*C-B*B; higher precision or correctly rounded or
// *  correctly truncated arithmetic would be needed to compute RT2
// *  accurately in all cases.
// *
// *  Overflow is possible only if RT1 is within a factor of 5 of overflow.
// *  Underflow is harmless if the input data is 0 or exceeds
// *     underflow_threshold / macheps.
// *
// * =====================================================================
// *
// *     .. Parameters ..
static double one= 1.0e0;
static double two= 2.0e0;
static double zero= 0.0e0;
static double half= 0.5e0;
// *     ..
// *     .. Local Scalars ..
static double ab= 0.0;
static double acmn= 0.0;
static double acmx= 0.0;
static double adf= 0.0;
static double df= 0.0;
static double rt= 0.0;
static double sm= 0.0;
static double tb= 0.0;
// *     ..
// *     .. Intrinsic Functions ..
// *     ..
// *     .. Executable Statements ..
// *
// *     Compute the eigenvalues
// *

public static void dlae2 (double a,
double b,
double c,
doubleW rt1,
doubleW rt2)  {

sm = a+c;
df = a-c;
adf = Math.abs(df);
tb = b+b;
ab = Math.abs(tb);
if (Math.abs(a) > Math.abs(c))  {
    acmx = a;
acmn = c;
}              // Close if()
else  {
  acmx = c;
acmn = a;
}              //  Close else.
if (adf > ab)  {
    rt = adf*Math.sqrt(one+Math.pow((ab/adf), 2));
}              // Close if()
else if (adf < ab)  {
    rt = ab*Math.sqrt(one+Math.pow((adf/ab), 2));
}              // Close else if()
else  {
  // *
// *        Includes case AB=ADF=0
// *
rt = ab*Math.sqrt(two);
}              //  Close else.
if (sm < zero)  {
    rt1.val = half*(sm-rt);
// *
// *        Order of execution important.
// *        To get fully accurate smaller eigenvalue,
// *        next line needs to be executed in higher precision.
// *
rt2.val = (acmx/rt1.val)*acmn-(b/rt1.val)*b;
}              // Close if()
else if (sm > zero)  {
    rt1.val = half*(sm+rt);
// *
// *        Order of execution important.
// *        To get fully accurate smaller eigenvalue,
// *        next line needs to be executed in higher precision.
// *
rt2.val = (acmx/rt1.val)*acmn-(b/rt1.val)*b;
}              // Close else if()
else  {
  // *
// *        Includes case RT1 = RT2 = 0
// *
rt1.val = half*rt;
rt2.val = -half*rt;
}              //  Close else.
Dummy.go_to("Dlae2",999999);
// *
// *     End of DLAE2
// *
Dummy.label("Dlae2",999999);
return;
   }
} // End class.
