/*
 *  Produced by f2java.  f2java is part of the Fortran-
 *  -to-Java project at the University of Tennessee Netlib
 *  numerical software repository.
 *
 *  Original authorship for the BLAS and LAPACK numerical
 *  routines may be found in the Fortran source, available at
 *  www.netlib.org.
 *
 *  Fortran input file: jlapack.f
 *
 *  The f2j compiler code was written by
 *  David M. Doolin (doolin@cs.utk.edu) and
 *  Keith  Seymour (seymour@cs.utk.edu)
 */

package org.netlib.lapack;
import java.lang.*;
import org.netlib.util.*;



public class Dlamc5 {

// *
// ************************************************************************
// *
// *
// *  -- LAPACK auxiliary routine (version 2.0) --
// *     Univ. of Tennessee, Univ. of California Berkeley, NAG Ltd.,
// *     Courant Institute, Argonne National Lab, and Rice University
// *     October 31, 1992
// *
// *     .. Scalar Arguments ..
// *     ..
// *
// *  Purpose
// *  =======
// *
// *  DLAMC5 attempts to compute RMAX, the largest machine floating-point
// *  number, without overflow.  It assumes that EMAX + abs(EMIN) sum
// *  approximately to a power of 2.  It will fail on machines where this
// *  assumption does not hold, for example, the Cyber 205 (EMIN = -28625,
// *  EMAX = 28718).  It will also fail if the value supplied for EMIN is
// *  too large (i.e. too close to zero), probably with overflow.
// *
// *  Arguments
// *  =========
// *
// *  BETA    (input) INTEGER
// *          The base of floating-point arithmetic.
// *
// *  P       (input) INTEGER
// *          The number of base BETA digits in the mantissa of a
// *          floating-point value.
// *
// *  EMIN    (input) INTEGER
// *          The minimum exponent before (gradual) underflow.
// *
// *  IEEE    (input) LOGICAL
// *          A logical flag specifying whether or not the arithmetic
// *          system is thought to comply with the IEEE standard.
// *
// *  EMAX    (output) INTEGER
// *          The largest exponent before overflow
// *
// *  RMAX    (output) DOUBLE PRECISION
// *          The largest machine floating-point number.
// *
// * =====================================================================
// *
// *     .. Parameters ..
static double zero= 0.0e0;
static double one= 1.0e0;
// *     ..
// *     .. Local Scalars ..
static int exbits= 0;
static int expsum= 0;
static int i= 0;
static int lexp= 0;
static int nbits= 0;
static int Try= 0;
static int uexp= 0;
static double oldy= 0.0;
static double recbas= 0.0;
static double y= 0.0;
static double z= 0.0;
// *     ..
// *     .. External Functions ..
// *     ..
// *     .. Intrinsic Functions ..
// *     ..
// *     .. Executable Statements ..
// *
// *     First compute LEXP and UEXP, two powers of 2 that bound
// *     abs(EMIN). We then assume that EMAX + abs(EMIN) will sum
// *     approximately to the bound that is closest to abs(EMIN).
// *     (EMAX is the exponent of the required number RMAX).
// *

public static void dlamc5 (int beta,
int p,
int emin,
boolean ieee,
intW emax,
doubleW rmax)  {

lexp = 1;
exbits = 1;
label10:
   Dummy.label("Dlamc5",10);
Try = lexp*2;
if (Try <= (-emin))  {
    lexp = Try;
exbits = exbits+1;
Dummy.go_to("Dlamc5",10);
}              // Close if()
if (lexp == -emin)  {
    uexp = lexp;
}              // Close if()
else  {
  uexp = Try;
exbits = exbits+1;
}              //  Close else.
// *
// *     Now -LEXP is less than or equal to EMIN, and -UEXP is greater
// *     than or equal to EMIN. EXBITS is the number of bits needed to
// *     store the exponent.
// *
if ((uexp+emin) > (-lexp-emin))  {
    expsum = 2*lexp;
}              // Close if()
else  {
  expsum = 2*uexp;
}              //  Close else.
// *
// *     EXPSUM is the exponent range, approximately equal to
// *     EMAX - EMIN + 1 .
// *
emax.val = expsum+emin-1;
nbits = 1+exbits+p;
// *
// *     NBITS is the total number of bits needed to store a
// *     floating-point number.
// *
if (((nbits)%(2)  == 1) && (beta == 2))  {
    // *
// *        Either there are an odd number of bits used to store a
// *        floating-point number, which is unlikely, or some bits are
// *        not used in the representation of numbers, which is possible,
// *        (e.g. Cray machines) or the mantissa has an implicit bit,
// *        (e.g. IEEE machines, Dec Vax machines), which is perhaps the
// *        most likely. We have to assume the last alternative.
// *        If this is true, then we need to reduce EMAX by one because
// *        there must be some way of representing zero in an implicit-bit
// *        system. On machines like Cray, we are reducing EMAX by one
// *        unnecessarily.
// *
emax.val = emax.val-1;
}              // Close if()
// *
if (ieee)  {
    // *
// *        Assume we are on an IEEE machine which reserves one exponent
// *        for infinity and NaN.
// *
emax.val = emax.val-1;
}              // Close if()
// *
// *     Now create RMAX, the largest machine number, which should
// *     be equal to (1.0 - BETA**(-P)) * BETA**EMAX .
// *
// *     First compute 1.0 - BETA**(-P), being careful that the
// *     result is less than 1.0 .
// *
recbas = one/beta;
z = beta-one;
y = zero;
{
forloop20:
for (i = 1; i <= p; i++) {
z = z*recbas;
if (y < one)  
    oldy = y;
y = Dlamc3.dlamc3(y,z);
Dummy.label("Dlamc5",20);
}              //  Close for() loop. 
}
if (y >= one)  
    y = oldy;
// *
// *     Now multiply by BETA**EMAX to get RMAX.
// *
{
forloop30:
for (i = 1; i <= emax.val; i++) {
y = Dlamc3.dlamc3(y*beta,zero);
Dummy.label("Dlamc5",30);
}              //  Close for() loop. 
}
// *
rmax.val = y;
Dummy.go_to("Dlamc5",999999);
// *
// *     End of DLAMC5
// *
Dummy.label("Dlamc5",999999);
return;
   }
} // End class.
