/*
 *  Produced by f2java.  f2java is part of the Fortran-
 *  -to-Java project at the University of Tennessee Netlib
 *  numerical software repository.
 *
 *  Original authorship for the BLAS and LAPACK numerical
 *  routines may be found in the Fortran source, available at
 *  www.netlib.org.
 *
 *  Fortran input file: jlapack.f
 *
 *  The f2j compiler code was written by
 *  David M. Doolin (doolin@cs.utk.edu) and
 *  Keith  Seymour (seymour@cs.utk.edu)
 */

package org.netlib.lapack;
import java.lang.*;
import org.netlib.util.*;



public class Dlaqsb {

// *
// *  -- LAPACK auxiliary routine (version 2.0) --
// *     Univ. of Tennessee, Univ. of California Berkeley, NAG Ltd.,
// *     Courant Institute, Argonne National Lab, and Rice University
// *     February 29, 1992
// *
// *     .. Scalar Arguments ..
// *     ..
// *     .. Array Arguments ..
// *     ..
// *
// *  Purpose
// *  =======
// *
// *  DLAQSB equilibrates a symmetric band matrix A using the scaling
// *  factors in the vector S.
// *
// *  Arguments
// *  =========
// *
// *  UPLO    (input) CHARACTER*1
// *          Specifies whether the upper or lower triangular part of the
// *          symmetric matrix A is stored.
// *          = 'U':  Upper triangular
// *          = 'L':  Lower triangular
// *
// *  N       (input) INTEGER
// *          The order of the matrix A.  N >= 0.
// *
// *  KD      (input) INTEGER
// *          The number of super-diagonals of the matrix A if UPLO = 'U',
// *          or the number of sub-diagonals if UPLO = 'L'.  KD >= 0.
// *
// *  AB      (input/output) DOUBLE PRECISION array, dimension (LDAB,N)
// *          On entry, the upper or lower triangle of the symmetric band
// *          matrix A, stored in the first KD+1 rows of the array.  The
// *          j-th column of A is stored in the j-th column of the array AB
// *          as follows:
// *          if UPLO = 'U', AB(kd+1+i-j,j) = A(i,j) for max(1,j-kd)<=i<=j;
// *          if UPLO = 'L', AB(1+i-j,j)    = A(i,j) for j<=i<=min(n,j+kd).
// *
// *          On exit, if INFO = 0, the triangular factor U or L from the
// *          Cholesky factorization A = U'*U or A = L*L' of the band
// *          matrix A, in the same storage format as A.
// *
// *  LDAB    (input) INTEGER
// *          The leading dimension of the array AB.  LDAB >= KD+1.
// *
// *  S       (output) DOUBLE PRECISION array, dimension (N)
// *          The scale factors for A.
// *
// *  SCOND   (input) DOUBLE PRECISION
// *          Ratio of the smallest S(i) to the largest S(i).
// *
// *  AMAX    (input) DOUBLE PRECISION
// *          Absolute value of largest matrix entry.
// *
// *  EQUED   (output) CHARACTER*1
// *          Specifies whether or not equilibration was done.
// *          = 'N':  No equilibration.
// *          = 'Y':  Equilibration was done, i.e., A has been replaced by
// *                  diag(S) * A * diag(S).
// *
// *  Internal Parameters
// *  ===================
// *
// *  THRESH is a threshold value used to decide if scaling should be done
// *  based on the ratio of the scaling factors.  If SCOND < THRESH,
// *  scaling is done.
// *
// *  LARGE and SMALL are threshold values used to decide if scaling should
// *  be done based on the absolute size of the largest matrix element.
// *  If AMAX > LARGE or AMAX < SMALL, scaling is done.
// *
// *  =====================================================================
// *
// *     .. Parameters ..
static double one= 1.0e+0;
static double thresh= 0.1e+0;
// *     ..
// *     .. Local Scalars ..
static int i= 0;
static int j= 0;
static double cj= 0.0;
static double large= 0.0;
static double small= 0.0;
// *     ..
// *     .. External Functions ..
// *     ..
// *     .. Intrinsic Functions ..
// *     ..
// *     .. Executable Statements ..
// *
// *     Quick return if possible
// *

public static void dlaqsb (String uplo,
int n,
int kd,
double [] ab, int _ab_offset,
int ldab,
double [] s, int _s_offset,
double scond,
double amax,
StringW equed)  {

if (n <= 0)  {
    equed.val = "N";
Dummy.go_to("Dlaqsb",999999);
}              // Close if()
// *
// *     Initialize LARGE and SMALL.
// *
small = Dlamch.dlamch("Safe minimum")/Dlamch.dlamch("Precision");
large = one/small;
// *
if (scond >= thresh && amax >= small && amax <= large)  {
    // *
// *        No equilibration
// *
equed.val = "N";
}              // Close if()
else  {
  // *
// *        Replace A by diag(S) * A * diag(S).
// *
if ((uplo.toLowerCase().charAt(0) == "U".toLowerCase().charAt(0)))  {
    // *
// *           Upper triangle of A is stored in band format.
// *
{
forloop20:
for (j = 1; j <= n; j++) {
cj = s[(j)- 1+ _s_offset];
{
forloop10:
for (i = (int)(Math.max(1, j-kd) ); i <= j; i++) {
ab[(kd+1+i-j)- 1+(j- 1)*ldab+ _ab_offset] = cj*s[(i)- 1+ _s_offset]*ab[(kd+1+i-j)- 1+(j- 1)*ldab+ _ab_offset];
Dummy.label("Dlaqsb",10);
}              //  Close for() loop. 
}
Dummy.label("Dlaqsb",20);
}              //  Close for() loop. 
}
}              // Close if()
else  {
  // *
// *           Lower triangle of A is stored.
// *
{
forloop40:
for (j = 1; j <= n; j++) {
cj = s[(j)- 1+ _s_offset];
{
forloop30:
for (i = j; i <= Math.min(n, j+kd) ; i++) {
ab[(1+i-j)- 1+(j- 1)*ldab+ _ab_offset] = cj*s[(i)- 1+ _s_offset]*ab[(1+i-j)- 1+(j- 1)*ldab+ _ab_offset];
Dummy.label("Dlaqsb",30);
}              //  Close for() loop. 
}
Dummy.label("Dlaqsb",40);
}              //  Close for() loop. 
}
}              //  Close else.
equed.val = "Y";
}              //  Close else.
// *
Dummy.go_to("Dlaqsb",999999);
// *
// *     End of DLAQSB
// *
Dummy.label("Dlaqsb",999999);
return;
   }
} // End class.
