/*
 *  Produced by f2java.  f2java is part of the Fortran-
 *  -to-Java project at the University of Tennessee Netlib
 *  numerical software repository.
 *
 *  Original authorship for the BLAS and LAPACK numerical
 *  routines may be found in the Fortran source, available at
 *  www.netlib.org.
 *
 *  Fortran input file: jlapack.f
 *
 *  The f2j compiler code was written by
 *  David M. Doolin (doolin@cs.utk.edu) and
 *  Keith  Seymour (seymour@cs.utk.edu)
 */

package org.netlib.lapack;
import java.lang.*;
import org.netlib.util.*;



public class Dlaqsp {

// *
// *  -- LAPACK auxiliary routine (version 2.0) --
// *     Univ. of Tennessee, Univ. of California Berkeley, NAG Ltd.,
// *     Courant Institute, Argonne National Lab, and Rice University
// *     February 29, 1992
// *
// *     .. Scalar Arguments ..
// *     ..
// *     .. Array Arguments ..
// *     ..
// *
// *  Purpose
// *  =======
// *
// *  DLAQSP equilibrates a symmetric matrix A using the scaling factors
// *  in the vector S.
// *
// *  Arguments
// *  =========
// *
// *  UPLO    (input) CHARACTER*1
// *          Specifies whether the upper or lower triangular part of the
// *          symmetric matrix A is stored.
// *          = 'U':  Upper triangular
// *          = 'L':  Lower triangular
// *
// *  N       (input) INTEGER
// *          The order of the matrix A.  N >= 0.
// *
// *  AP      (input/output) DOUBLE PRECISION array, dimension (N*(N+1)/2)
// *          On entry, the upper or lower triangle of the symmetric matrix
// *          A, packed columnwise in a linear array.  The j-th column of A
// *          is stored in the array AP as follows:
// *          if UPLO = 'U', AP(i + (j-1)*j/2) = A(i,j) for 1<=i<=j;
// *          if UPLO = 'L', AP(i + (j-1)*(2n-j)/2) = A(i,j) for j<=i<=n.
// *
// *          On exit, the equilibrated matrix:  diag(S) * A * diag(S), in
// *          the same storage format as A.
// *
// *  S       (input) DOUBLE PRECISION array, dimension (N)
// *          The scale factors for A.
// *
// *  SCOND   (input) DOUBLE PRECISION
// *          Ratio of the smallest S(i) to the largest S(i).
// *
// *  AMAX    (input) DOUBLE PRECISION
// *          Absolute value of largest matrix entry.
// *
// *  EQUED   (output) CHARACTER*1
// *          Specifies whether or not equilibration was done.
// *          = 'N':  No equilibration.
// *          = 'Y':  Equilibration was done, i.e., A has been replaced by
// *                  diag(S) * A * diag(S).
// *
// *  Internal Parameters
// *  ===================
// *
// *  THRESH is a threshold value used to decide if scaling should be done
// *  based on the ratio of the scaling factors.  If SCOND < THRESH,
// *  scaling is done.
// *
// *  LARGE and SMALL are threshold values used to decide if scaling should
// *  be done based on the absolute size of the largest matrix element.
// *  If AMAX > LARGE or AMAX < SMALL, scaling is done.
// *
// *  =====================================================================
// *
// *     .. Parameters ..
static double one= 1.0e+0;
static double thresh= 0.1e+0;
// *     ..
// *     .. Local Scalars ..
static int i= 0;
static int j= 0;
static int jc= 0;
static double cj= 0.0;
static double large= 0.0;
static double small= 0.0;
// *     ..
// *     .. External Functions ..
// *     ..
// *     .. Executable Statements ..
// *
// *     Quick return if possible
// *

public static void dlaqsp (String uplo,
int n,
double [] ap, int _ap_offset,
double [] s, int _s_offset,
double scond,
double amax,
StringW equed)  {

if (n <= 0)  {
    equed.val = "N";
Dummy.go_to("Dlaqsp",999999);
}              // Close if()
// *
// *     Initialize LARGE and SMALL.
// *
small = Dlamch.dlamch("Safe minimum")/Dlamch.dlamch("Precision");
large = one/small;
// *
if (scond >= thresh && amax >= small && amax <= large)  {
    // *
// *        No equilibration
// *
equed.val = "N";
}              // Close if()
else  {
  // *
// *        Replace A by diag(S) * A * diag(S).
// *
if ((uplo.toLowerCase().charAt(0) == "U".toLowerCase().charAt(0)))  {
    // *
// *           Upper triangle of A is stored.
// *
jc = 1;
{
forloop20:
for (j = 1; j <= n; j++) {
cj = s[(j)- 1+ _s_offset];
{
forloop10:
for (i = 1; i <= j; i++) {
ap[(jc+i-1)- 1+ _ap_offset] = cj*s[(i)- 1+ _s_offset]*ap[(jc+i-1)- 1+ _ap_offset];
Dummy.label("Dlaqsp",10);
}              //  Close for() loop. 
}
jc = jc+j;
Dummy.label("Dlaqsp",20);
}              //  Close for() loop. 
}
}              // Close if()
else  {
  // *
// *           Lower triangle of A is stored.
// *
jc = 1;
{
forloop40:
for (j = 1; j <= n; j++) {
cj = s[(j)- 1+ _s_offset];
{
forloop30:
for (i = j; i <= n; i++) {
ap[(jc+i-j)- 1+ _ap_offset] = cj*s[(i)- 1+ _s_offset]*ap[(jc+i-j)- 1+ _ap_offset];
Dummy.label("Dlaqsp",30);
}              //  Close for() loop. 
}
jc = jc+n-j+1;
Dummy.label("Dlaqsp",40);
}              //  Close for() loop. 
}
}              //  Close else.
equed.val = "Y";
}              //  Close else.
// *
Dummy.go_to("Dlaqsp",999999);
// *
// *     End of DLAQSP
// *
Dummy.label("Dlaqsp",999999);
return;
   }
} // End class.
