/*
 *  Produced by f2java.  f2java is part of the Fortran-
 *  -to-Java project at the University of Tennessee Netlib
 *  numerical software repository.
 *
 *  Original authorship for the BLAS and LAPACK numerical
 *  routines may be found in the Fortran source, available at
 *  www.netlib.org.
 *
 *  Fortran input file: jlapack.f
 *
 *  The f2j compiler code was written by
 *  David M. Doolin (doolin@cs.utk.edu) and
 *  Keith  Seymour (seymour@cs.utk.edu)
 */

package org.netlib.lapack;
import java.lang.*;
import org.netlib.util.*;

import org.netlib.blas.*;


public class Dlasyf {

// *
// *  -- LAPACK routine (version 2.0) --
// *     Univ. of Tennessee, Univ. of California Berkeley, NAG Ltd.,
// *     Courant Institute, Argonne National Lab, and Rice University
// *     February 29, 1992
// *
// *     .. Scalar Arguments ..
// *     ..
// *     .. Array Arguments ..
// *     ..
// *
// *  Purpose
// *  =======
// *
// *  DLASYF computes a partial factorization of a real symmetric matrix A
// *  using the Bunch-Kaufman diagonal pivoting method. The partial
// *  factorization has the form:
// *
// *  A  =  ( I  U12 ) ( A11  0  ) (  I    0   )  if UPLO = 'U', or:
// *        ( 0  U22 ) (  0   D  ) ( U12' U22' )
// *
// *  A  =  ( L11  0 ) (  D   0  ) ( L11' L21' )  if UPLO = 'L'
// *        ( L21  I ) (  0  A22 ) (  0    I   )
// *
// *  where the order of D is at most NB. The actual order is returned in
// *  the argument KB, and is either NB or NB-1, or N if N <= NB.
// *
// *  DLASYF is an auxiliary routine called by DSYTRF. It uses blocked code
// *  (calling Level 3 BLAS) to update the submatrix A11 (if UPLO = 'U') or
// *  A22 (if UPLO = 'L').
// *
// *  Arguments
// *  =========
// *
// *  UPLO    (input) CHARACTER*1
// *          Specifies whether the upper or lower triangular part of the
// *          symmetric matrix A is stored:
// *          = 'U':  Upper triangular
// *          = 'L':  Lower triangular
// *
// *  N       (input) INTEGER
// *          The order of the matrix A.  N >= 0.
// *
// *  NB      (input) INTEGER
// *          The maximum number of columns of the matrix A that should be
// *          factored.  NB should be at least 2 to allow for 2-by-2 pivot
// *          blocks.
// *
// *  KB      (output) INTEGER
// *          The number of columns of A that were actually factored.
// *          KB is either NB-1 or NB, or N if N <= NB.
// *
// *  A       (input/output) DOUBLE PRECISION array, dimension (LDA,N)
// *          On entry, the symmetric matrix A.  If UPLO = 'U', the leading
// *          n-by-n upper triangular part of A contains the upper
// *          triangular part of the matrix A, and the strictly lower
// *          triangular part of A is not referenced.  If UPLO = 'L', the
// *          leading n-by-n lower triangular part of A contains the lower
// *          triangular part of the matrix A, and the strictly upper
// *          triangular part of A is not referenced.
// *          On exit, A contains details of the partial factorization.
// *
// *  LDA     (input) INTEGER
// *          The leading dimension of the array A.  LDA >= max(1,N).
// *
// *  IPIV    (output) INTEGER array, dimension (N)
// *          Details of the interchanges and the block structure of D.
// *          If UPLO = 'U', only the last KB elements of IPIV are set;
// *          if UPLO = 'L', only the first KB elements are set.
// *
// *          If IPIV(k) > 0, then rows and columns k and IPIV(k) were
// *          interchanged and D(k,k) is a 1-by-1 diagonal block.
// *          If UPLO = 'U' and IPIV(k) = IPIV(k-1) < 0, then rows and
// *          columns k-1 and -IPIV(k) were interchanged and D(k-1:k,k-1:k)
// *          is a 2-by-2 diagonal block.  If UPLO = 'L' and IPIV(k) =
// *          IPIV(k+1) < 0, then rows and columns k+1 and -IPIV(k) were
// *          interchanged and D(k:k+1,k:k+1) is a 2-by-2 diagonal block.
// *
// *  W       (workspace) DOUBLE PRECISION array, dimension (LDW,NB)
// *
// *  LDW     (input) INTEGER
// *          The leading dimension of the array W.  LDW >= max(1,N).
// *
// *  INFO    (output) INTEGER
// *          = 0: successful exit
// *          > 0: if INFO = k, D(k,k) is exactly zero.  The factorization
// *               has been completed, but the block diagonal matrix D is
// *               exactly singular.
// *
// *  =====================================================================
// *
// *     .. Parameters ..
static double zero= 0.0e+0;
static double one= 1.0e+0;
static double eight= 8.0e+0;
static double sevten= 17.0e+0;
// *     ..
// *     .. Local Scalars ..
static int imax= 0;
static int j= 0;
static int jb= 0;
static int jj= 0;
static int jmax= 0;
static int jp= 0;
static int k= 0;
static int kk= 0;
static int kkw= 0;
static int kp= 0;
static int kstep= 0;
static int kw= 0;
static double absakk= 0.0;
static double alpha= 0.0;
static double colmax= 0.0;
static double d11= 0.0;
static double d21= 0.0;
static double d22= 0.0;
static double r1= 0.0;
static double rowmax= 0.0;
static double t= 0.0;
// *     ..
// *     .. External Functions ..
// *     ..
// *     .. External Subroutines ..
// *     ..
// *     .. Intrinsic Functions ..
// *     ..
// *     .. Executable Statements ..
// *

public static void dlasyf (String uplo,
int n,
int nb,
intW kb,
double [] a, int _a_offset,
int lda,
int [] ipiv, int _ipiv_offset,
double [] w, int _w_offset,
int ldw,
intW info)  {

info.val = 0;
// *
// *     Initialize ALPHA for use in choosing pivot block size.
// *
alpha = (one+Math.sqrt(sevten))/eight;
// *
if ((uplo.toLowerCase().charAt(0) == "U".toLowerCase().charAt(0)))  {
    // *
// *        Factorize the trailing columns of A using the upper triangle
// *        of A and working backwards, and compute the matrix W = U12*D
// *        for use in updating A11
// *
// *        K is the main loop index, decreasing from N in steps of 1 or 2
// *
// *        KW is the column of W which corresponds to column K of A
// *
k = n;
label10:
   Dummy.label("Dlasyf",10);
kw = nb+k-n;
// *
// *        Exit from loop
// *
if ((k <= n-nb+1 && nb < n) || k < 1)  
    Dummy.go_to("Dlasyf",30);
// *
// *        Copy column K of A to column KW of W and update it
// *
Dcopy.dcopy(k,a,(1)- 1+(k- 1)*lda+ _a_offset,1,w,(1)- 1+(kw- 1)*ldw+ _w_offset,1);
if (k < n)  
    Dgemv.dgemv("No transpose",k,n-k,-one,a,(1)- 1+(k+1- 1)*lda+ _a_offset,lda,w,(k)- 1+(kw+1- 1)*ldw+ _w_offset,ldw,one,w,(1)- 1+(kw- 1)*ldw+ _w_offset,1);
// *
kstep = 1;
// *
// *        Determine rows and columns to be interchanged and whether
// *        a 1-by-1 or 2-by-2 pivot block will be used
// *
absakk = Math.abs(w[(k)- 1+(kw- 1)*ldw+ _w_offset]);
// *
// *        IMAX is the row-index of the largest off-diagonal element in
// *        column K, and COLMAX is its absolute value
// *
if (k > 1)  {
    imax = Idamax.idamax(k-1,w,(1)- 1+(kw- 1)*ldw+ _w_offset,1);
colmax = Math.abs(w[(imax)- 1+(kw- 1)*ldw+ _w_offset]);
}              // Close if()
else  {
  colmax = zero;
}              //  Close else.
// *
if (Math.max(absakk, colmax)  == zero)  {
    // *
// *           Column K is zero: set INFO and continue
// *
if (info.val == 0)  
    info.val = k;
kp = k;
}              // Close if()
else  {
  if (absakk >= alpha*colmax)  {
    // *
// *              no interchange, use 1-by-1 pivot block
// *
kp = k;
}              // Close if()
else  {
  // *
// *              Copy column IMAX to column KW-1 of W and update it
// *
Dcopy.dcopy(imax,a,(1)- 1+(imax- 1)*lda+ _a_offset,1,w,(1)- 1+(kw-1- 1)*ldw+ _w_offset,1);
Dcopy.dcopy(k-imax,a,(imax)- 1+(imax+1- 1)*lda+ _a_offset,lda,w,(imax+1)- 1+(kw-1- 1)*ldw+ _w_offset,1);
if (k < n)  
    Dgemv.dgemv("No transpose",k,n-k,-one,a,(1)- 1+(k+1- 1)*lda+ _a_offset,lda,w,(imax)- 1+(kw+1- 1)*ldw+ _w_offset,ldw,one,w,(1)- 1+(kw-1- 1)*ldw+ _w_offset,1);
// *
// *              JMAX is the column-index of the largest off-diagonal
// *              element in row IMAX, and ROWMAX is its absolute value
// *
jmax = imax+Idamax.idamax(k-imax,w,(imax+1)- 1+(kw-1- 1)*ldw+ _w_offset,1);
rowmax = Math.abs(w[(jmax)- 1+(kw-1- 1)*ldw+ _w_offset]);
if (imax > 1)  {
    jmax = Idamax.idamax(imax-1,w,(1)- 1+(kw-1- 1)*ldw+ _w_offset,1);
rowmax = Math.max(rowmax, Math.abs(w[(jmax)- 1+(kw-1- 1)*ldw+ _w_offset])) ;
}              // Close if()
// *
if (absakk >= alpha*colmax*(colmax/rowmax))  {
    // *
// *                 no interchange, use 1-by-1 pivot block
// *
kp = k;
}              // Close if()
else if (Math.abs(w[(imax)- 1+(kw-1- 1)*ldw+ _w_offset]) >= alpha*rowmax)  {
    // *
// *                 interchange rows and columns K and IMAX, use 1-by-1
// *                 pivot block
// *
kp = imax;
// *
// *                 copy column KW-1 of W to column KW
// *
Dcopy.dcopy(k,w,(1)- 1+(kw-1- 1)*ldw+ _w_offset,1,w,(1)- 1+(kw- 1)*ldw+ _w_offset,1);
}              // Close else if()
else  {
  // *
// *                 interchange rows and columns K-1 and IMAX, use 2-by-2
// *                 pivot block
// *
kp = imax;
kstep = 2;
}              //  Close else.
}              //  Close else.
// *
kk = k-kstep+1;
kkw = nb+kk-n;
// *
// *           Updated column KP is already stored in column KKW of W
// *
if (kp != kk)  {
    // *
// *              Copy non-updated column KK to column KP
// *
a[(kp)- 1+(k- 1)*lda+ _a_offset] = a[(kk)- 1+(k- 1)*lda+ _a_offset];
Dcopy.dcopy(k-1-kp,a,(kp+1)- 1+(kk- 1)*lda+ _a_offset,1,a,(kp)- 1+(kp+1- 1)*lda+ _a_offset,lda);
Dcopy.dcopy(kp,a,(1)- 1+(kk- 1)*lda+ _a_offset,1,a,(1)- 1+(kp- 1)*lda+ _a_offset,1);
// *
// *              Interchange rows KK and KP in last KK columns of A and W
// *
Dswap.dswap(n-kk+1,a,(kk)- 1+(kk- 1)*lda+ _a_offset,lda,a,(kp)- 1+(kk- 1)*lda+ _a_offset,lda);
Dswap.dswap(n-kk+1,w,(kk)- 1+(kkw- 1)*ldw+ _w_offset,ldw,w,(kp)- 1+(kkw- 1)*ldw+ _w_offset,ldw);
}              // Close if()
// *
if (kstep == 1)  {
    // *
// *              1-by-1 pivot block D(k): column KW of W now holds
// *
// *              W(k) = U(k)*D(k)
// *
// *              where U(k) is the k-th column of U
// *
// *              Store U(k) in column k of A
// *
Dcopy.dcopy(k,w,(1)- 1+(kw- 1)*ldw+ _w_offset,1,a,(1)- 1+(k- 1)*lda+ _a_offset,1);
r1 = one/a[(k)- 1+(k- 1)*lda+ _a_offset];
Dscal.dscal(k-1,r1,a,(1)- 1+(k- 1)*lda+ _a_offset,1);
}              // Close if()
else  {
  // *
// *              2-by-2 pivot block D(k): columns KW and KW-1 of W now
// *              hold
// *
// *              ( W(k-1) W(k) ) = ( U(k-1) U(k) )*D(k)
// *
// *              where U(k) and U(k-1) are the k-th and (k-1)-th columns
// *              of U
// *
if (k > 2)  {
    // *
// *                 Store U(k) and U(k-1) in columns k and k-1 of A
// *
d21 = w[(k-1)- 1+(kw- 1)*ldw+ _w_offset];
d11 = w[(k)- 1+(kw- 1)*ldw+ _w_offset]/d21;
d22 = w[(k-1)- 1+(kw-1- 1)*ldw+ _w_offset]/d21;
t = one/(d11*d22-one);
d21 = t/d21;
{
forloop20:
for (j = 1; j <= k-2; j++) {
a[(j)- 1+(k-1- 1)*lda+ _a_offset] = d21*(d11*w[(j)- 1+(kw-1- 1)*ldw+ _w_offset]-w[(j)- 1+(kw- 1)*ldw+ _w_offset]);
a[(j)- 1+(k- 1)*lda+ _a_offset] = d21*(d22*w[(j)- 1+(kw- 1)*ldw+ _w_offset]-w[(j)- 1+(kw-1- 1)*ldw+ _w_offset]);
Dummy.label("Dlasyf",20);
}              //  Close for() loop. 
}
}              // Close if()
// *
// *              Copy D(k) to A
// *
a[(k-1)- 1+(k-1- 1)*lda+ _a_offset] = w[(k-1)- 1+(kw-1- 1)*ldw+ _w_offset];
a[(k-1)- 1+(k- 1)*lda+ _a_offset] = w[(k-1)- 1+(kw- 1)*ldw+ _w_offset];
a[(k)- 1+(k- 1)*lda+ _a_offset] = w[(k)- 1+(kw- 1)*ldw+ _w_offset];
}              //  Close else.
}              //  Close else.
// *
// *        Store details of the interchanges in IPIV
// *
if (kstep == 1)  {
    ipiv[(k)- 1+ _ipiv_offset] = kp;
}              // Close if()
else  {
  ipiv[(k)- 1+ _ipiv_offset] = -kp;
ipiv[(k-1)- 1+ _ipiv_offset] = -kp;
}              //  Close else.
// *
// *        Decrease K and return to the start of the main loop
// *
k = k-kstep;
Dummy.go_to("Dlasyf",10);
// *
label30:
   Dummy.label("Dlasyf",30);
// *
// *        Update the upper triangle of A11 (= A(1:k,1:k)) as
// *
// *        A11 := A11 - U12*D*U12' = A11 - U12*W'
// *
// *        computing blocks of NB columns at a time
// *
{
int _j_inc = -nb;
forloop50:
for (j = ((k-1)/nb)*nb+1; (_j_inc < 0) ? j >= 1 : j <= 1; j += _j_inc) {
jb = (int)(Math.min(nb, k-j+1) );
// *
// *           Update the upper triangle of the diagonal block
// *
{
forloop40:
for (jj = j; jj <= j+jb-1; jj++) {
Dgemv.dgemv("No transpose",jj-j+1,n-k,-one,a,(j)- 1+(k+1- 1)*lda+ _a_offset,lda,w,(jj)- 1+(kw+1- 1)*ldw+ _w_offset,ldw,one,a,(j)- 1+(jj- 1)*lda+ _a_offset,1);
Dummy.label("Dlasyf",40);
}              //  Close for() loop. 
}
// *
// *           Update the rectangular superdiagonal block
// *
Dgemm.dgemm("No transpose","Transpose",j-1,jb,n-k,-one,a,(1)- 1+(k+1- 1)*lda+ _a_offset,lda,w,(j)- 1+(kw+1- 1)*ldw+ _w_offset,ldw,one,a,(1)- 1+(j- 1)*lda+ _a_offset,lda);
Dummy.label("Dlasyf",50);
}              //  Close for() loop. 
}
// *
// *        Put U12 in standard form by partially undoing the interchanges
// *        in columns k+1:n
// *
j = k+1;
label60:
   Dummy.label("Dlasyf",60);
jj = j;
jp = ipiv[(j)- 1+ _ipiv_offset];
if (jp < 0)  {
    jp = -jp;
j = j+1;
}              // Close if()
j = j+1;
if (jp != jj && j <= n)  
    Dswap.dswap(n-j+1,a,(jp)- 1+(j- 1)*lda+ _a_offset,lda,a,(jj)- 1+(j- 1)*lda+ _a_offset,lda);
if (j <= n)  
    Dummy.go_to("Dlasyf",60);
// *
// *        Set KB to the number of columns factorized
// *
kb.val = n-k;
// *
}              // Close if()
else  {
  // *
// *        Factorize the leading columns of A using the lower triangle
// *        of A and working forwards, and compute the matrix W = L21*D
// *        for use in updating A22
// *
// *        K is the main loop index, increasing from 1 in steps of 1 or 2
// *
k = 1;
label70:
   Dummy.label("Dlasyf",70);
// *
// *        Exit from loop
// *
if ((k >= nb && nb < n) || k > n)  
    Dummy.go_to("Dlasyf",90);
// *
// *        Copy column K of A to column K of W and update it
// *
Dcopy.dcopy(n-k+1,a,(k)- 1+(k- 1)*lda+ _a_offset,1,w,(k)- 1+(k- 1)*ldw+ _w_offset,1);
Dgemv.dgemv("No transpose",n-k+1,k-1,-one,a,(k)- 1+(1- 1)*lda+ _a_offset,lda,w,(k)- 1+(1- 1)*ldw+ _w_offset,ldw,one,w,(k)- 1+(k- 1)*ldw+ _w_offset,1);
// *
kstep = 1;
// *
// *        Determine rows and columns to be interchanged and whether
// *        a 1-by-1 or 2-by-2 pivot block will be used
// *
absakk = Math.abs(w[(k)- 1+(k- 1)*ldw+ _w_offset]);
// *
// *        IMAX is the row-index of the largest off-diagonal element in
// *        column K, and COLMAX is its absolute value
// *
if (k < n)  {
    imax = k+Idamax.idamax(n-k,w,(k+1)- 1+(k- 1)*ldw+ _w_offset,1);
colmax = Math.abs(w[(imax)- 1+(k- 1)*ldw+ _w_offset]);
}              // Close if()
else  {
  colmax = zero;
}              //  Close else.
// *
if (Math.max(absakk, colmax)  == zero)  {
    // *
// *           Column K is zero: set INFO and continue
// *
if (info.val == 0)  
    info.val = k;
kp = k;
}              // Close if()
else  {
  if (absakk >= alpha*colmax)  {
    // *
// *              no interchange, use 1-by-1 pivot block
// *
kp = k;
}              // Close if()
else  {
  // *
// *              Copy column IMAX to column K+1 of W and update it
// *
Dcopy.dcopy(imax-k,a,(imax)- 1+(k- 1)*lda+ _a_offset,lda,w,(k)- 1+(k+1- 1)*ldw+ _w_offset,1);
Dcopy.dcopy(n-imax+1,a,(imax)- 1+(imax- 1)*lda+ _a_offset,1,w,(imax)- 1+(k+1- 1)*ldw+ _w_offset,1);
Dgemv.dgemv("No transpose",n-k+1,k-1,-one,a,(k)- 1+(1- 1)*lda+ _a_offset,lda,w,(imax)- 1+(1- 1)*ldw+ _w_offset,ldw,one,w,(k)- 1+(k+1- 1)*ldw+ _w_offset,1);
// *
// *              JMAX is the column-index of the largest off-diagonal
// *              element in row IMAX, and ROWMAX is its absolute value
// *
jmax = k-1+Idamax.idamax(imax-k,w,(k)- 1+(k+1- 1)*ldw+ _w_offset,1);
rowmax = Math.abs(w[(jmax)- 1+(k+1- 1)*ldw+ _w_offset]);
if (imax < n)  {
    jmax = imax+Idamax.idamax(n-imax,w,(imax+1)- 1+(k+1- 1)*ldw+ _w_offset,1);
rowmax = Math.max(rowmax, Math.abs(w[(jmax)- 1+(k+1- 1)*ldw+ _w_offset])) ;
}              // Close if()
// *
if (absakk >= alpha*colmax*(colmax/rowmax))  {
    // *
// *                 no interchange, use 1-by-1 pivot block
// *
kp = k;
}              // Close if()
else if (Math.abs(w[(imax)- 1+(k+1- 1)*ldw+ _w_offset]) >= alpha*rowmax)  {
    // *
// *                 interchange rows and columns K and IMAX, use 1-by-1
// *                 pivot block
// *
kp = imax;
// *
// *                 copy column K+1 of W to column K
// *
Dcopy.dcopy(n-k+1,w,(k)- 1+(k+1- 1)*ldw+ _w_offset,1,w,(k)- 1+(k- 1)*ldw+ _w_offset,1);
}              // Close else if()
else  {
  // *
// *                 interchange rows and columns K+1 and IMAX, use 2-by-2
// *                 pivot block
// *
kp = imax;
kstep = 2;
}              //  Close else.
}              //  Close else.
// *
kk = k+kstep-1;
// *
// *           Updated column KP is already stored in column KK of W
// *
if (kp != kk)  {
    // *
// *              Copy non-updated column KK to column KP
// *
a[(kp)- 1+(k- 1)*lda+ _a_offset] = a[(kk)- 1+(k- 1)*lda+ _a_offset];
Dcopy.dcopy(kp-k-1,a,(k+1)- 1+(kk- 1)*lda+ _a_offset,1,a,(kp)- 1+(k+1- 1)*lda+ _a_offset,lda);
Dcopy.dcopy(n-kp+1,a,(kp)- 1+(kk- 1)*lda+ _a_offset,1,a,(kp)- 1+(kp- 1)*lda+ _a_offset,1);
// *
// *              Interchange rows KK and KP in first KK columns of A and W
// *
Dswap.dswap(kk,a,(kk)- 1+(1- 1)*lda+ _a_offset,lda,a,(kp)- 1+(1- 1)*lda+ _a_offset,lda);
Dswap.dswap(kk,w,(kk)- 1+(1- 1)*ldw+ _w_offset,ldw,w,(kp)- 1+(1- 1)*ldw+ _w_offset,ldw);
}              // Close if()
// *
if (kstep == 1)  {
    // *
// *              1-by-1 pivot block D(k): column k of W now holds
// *
// *              W(k) = L(k)*D(k)
// *
// *              where L(k) is the k-th column of L
// *
// *              Store L(k) in column k of A
// *
Dcopy.dcopy(n-k+1,w,(k)- 1+(k- 1)*ldw+ _w_offset,1,a,(k)- 1+(k- 1)*lda+ _a_offset,1);
if (k < n)  {
    r1 = one/a[(k)- 1+(k- 1)*lda+ _a_offset];
Dscal.dscal(n-k,r1,a,(k+1)- 1+(k- 1)*lda+ _a_offset,1);
}              // Close if()
}              // Close if()
else  {
  // *
// *              2-by-2 pivot block D(k): columns k and k+1 of W now hold
// *
// *              ( W(k) W(k+1) ) = ( L(k) L(k+1) )*D(k)
// *
// *              where L(k) and L(k+1) are the k-th and (k+1)-th columns
// *              of L
// *
if (k < n-1)  {
    // *
// *                 Store L(k) and L(k+1) in columns k and k+1 of A
// *
d21 = w[(k+1)- 1+(k- 1)*ldw+ _w_offset];
d11 = w[(k+1)- 1+(k+1- 1)*ldw+ _w_offset]/d21;
d22 = w[(k)- 1+(k- 1)*ldw+ _w_offset]/d21;
t = one/(d11*d22-one);
d21 = t/d21;
{
forloop80:
for (j = k+2; j <= n; j++) {
a[(j)- 1+(k- 1)*lda+ _a_offset] = d21*(d11*w[(j)- 1+(k- 1)*ldw+ _w_offset]-w[(j)- 1+(k+1- 1)*ldw+ _w_offset]);
a[(j)- 1+(k+1- 1)*lda+ _a_offset] = d21*(d22*w[(j)- 1+(k+1- 1)*ldw+ _w_offset]-w[(j)- 1+(k- 1)*ldw+ _w_offset]);
Dummy.label("Dlasyf",80);
}              //  Close for() loop. 
}
}              // Close if()
// *
// *              Copy D(k) to A
// *
a[(k)- 1+(k- 1)*lda+ _a_offset] = w[(k)- 1+(k- 1)*ldw+ _w_offset];
a[(k+1)- 1+(k- 1)*lda+ _a_offset] = w[(k+1)- 1+(k- 1)*ldw+ _w_offset];
a[(k+1)- 1+(k+1- 1)*lda+ _a_offset] = w[(k+1)- 1+(k+1- 1)*ldw+ _w_offset];
}              //  Close else.
}              //  Close else.
// *
// *        Store details of the interchanges in IPIV
// *
if (kstep == 1)  {
    ipiv[(k)- 1+ _ipiv_offset] = kp;
}              // Close if()
else  {
  ipiv[(k)- 1+ _ipiv_offset] = -kp;
ipiv[(k+1)- 1+ _ipiv_offset] = -kp;
}              //  Close else.
// *
// *        Increase K and return to the start of the main loop
// *
k = k+kstep;
Dummy.go_to("Dlasyf",70);
// *
label90:
   Dummy.label("Dlasyf",90);
// *
// *        Update the lower triangle of A22 (= A(k:n,k:n)) as
// *
// *        A22 := A22 - L21*D*L21' = A22 - L21*W'
// *
// *        computing blocks of NB columns at a time
// *
{
int _j_inc = nb;
forloop110:
for (j = k; (_j_inc < 0) ? j >= n : j <= n; j += _j_inc) {
jb = (int)(Math.min(nb, n-j+1) );
// *
// *           Update the lower triangle of the diagonal block
// *
{
forloop100:
for (jj = j; jj <= j+jb-1; jj++) {
Dgemv.dgemv("No transpose",j+jb-jj,k-1,-one,a,(jj)- 1+(1- 1)*lda+ _a_offset,lda,w,(jj)- 1+(1- 1)*ldw+ _w_offset,ldw,one,a,(jj)- 1+(jj- 1)*lda+ _a_offset,1);
Dummy.label("Dlasyf",100);
}              //  Close for() loop. 
}
// *
// *           Update the rectangular subdiagonal block
// *
if (j+jb <= n)  
    Dgemm.dgemm("No transpose","Transpose",n-j-jb+1,jb,k-1,-one,a,(j+jb)- 1+(1- 1)*lda+ _a_offset,lda,w,(j)- 1+(1- 1)*ldw+ _w_offset,ldw,one,a,(j+jb)- 1+(j- 1)*lda+ _a_offset,lda);
Dummy.label("Dlasyf",110);
}              //  Close for() loop. 
}
// *
// *        Put L21 in standard form by partially undoing the interchanges
// *        in columns 1:k-1
// *
j = k-1;
label120:
   Dummy.label("Dlasyf",120);
jj = j;
jp = ipiv[(j)- 1+ _ipiv_offset];
if (jp < 0)  {
    jp = -jp;
j = j-1;
}              // Close if()
j = j-1;
if (jp != jj && j >= 1)  
    Dswap.dswap(j,a,(jp)- 1+(1- 1)*lda+ _a_offset,lda,a,(jj)- 1+(1- 1)*lda+ _a_offset,lda);
if (j >= 1)  
    Dummy.go_to("Dlasyf",120);
// *
// *        Set KB to the number of columns factorized
// *
kb.val = k-1;
// *
}              //  Close else.
Dummy.go_to("Dlasyf",999999);
// *
// *     End of DLASYF
// *
Dummy.label("Dlasyf",999999);
return;
   }
} // End class.
