/*
 *  Produced by f2java.  f2java is part of the Fortran-
 *  -to-Java project at the University of Tennessee Netlib
 *  numerical software repository.
 *
 *  Original authorship for the BLAS and LAPACK numerical
 *  routines may be found in the Fortran source, available at
 *  www.netlib.org.
 *
 *  Fortran input file: jlapack.f
 *
 *  The f2j compiler code was written by
 *  David M. Doolin (doolin@cs.utk.edu) and
 *  Keith  Seymour (seymour@cs.utk.edu)
 */

package org.netlib.lapack;
import java.lang.*;
import org.netlib.util.*;

import org.netlib.blas.*;


public class Dlatzm {

// *
// *  -- LAPACK routine (version 2.0) --
// *     Univ. of Tennessee, Univ. of California Berkeley, NAG Ltd.,
// *     Courant Institute, Argonne National Lab, and Rice University
// *     February 29, 1992
// *
// *     .. Scalar Arguments ..
// *     ..
// *     .. Array Arguments ..
// *     ..
// *
// *  Purpose
// *  =======
// *
// *  DLATZM applies a Householder matrix generated by DTZRQF to a matrix.
// *
// *  Let P = I - tau*u*u',   u = ( 1 ),
// *                              ( v )
// *  where v is an (m-1) vector if SIDE = 'L', or a (n-1) vector if
// *  SIDE = 'R'.
// *
// *  If SIDE equals 'L', let
// *         C = [ C1 ] 1
// *             [ C2 ] m-1
// *               n
// *  Then C is overwritten by P*C.
// *
// *  If SIDE equals 'R', let
// *         C = [ C1, C2 ] m
// *                1  n-1
// *  Then C is overwritten by C*P.
// *
// *  Arguments
// *  =========
// *
// *  SIDE    (input) CHARACTER*1
// *          = 'L': form P * C
// *          = 'R': form C * P
// *
// *  M       (input) INTEGER
// *          The number of rows of the matrix C.
// *
// *  N       (input) INTEGER
// *          The number of columns of the matrix C.
// *
// *  V       (input) DOUBLE PRECISION array, dimension
// *                  (1 + (M-1)*abs(INCV)) if SIDE = 'L'
// *                  (1 + (N-1)*abs(INCV)) if SIDE = 'R'
// *          The vector v in the representation of P. V is not used
// *          if TAU = 0.
// *
// *  INCV    (input) INTEGER
// *          The increment between elements of v. INCV <> 0
// *
// *  TAU     (input) DOUBLE PRECISION
// *          The value tau in the representation of P.
// *
// *  C1      (input/output) DOUBLE PRECISION array, dimension
// *                         (LDC,N) if SIDE = 'L'
// *                         (M,1)   if SIDE = 'R'
// *          On entry, the n-vector C1 if SIDE = 'L', or the m-vector C1
// *          if SIDE = 'R'.
// *
// *          On exit, the first row of P*C if SIDE = 'L', or the first
// *          column of C*P if SIDE = 'R'.
// *
// *  C2      (input/output) DOUBLE PRECISION array, dimension
// *                         (LDC, N)   if SIDE = 'L'
// *                         (LDC, N-1) if SIDE = 'R'
// *          On entry, the (m - 1) x n matrix C2 if SIDE = 'L', or the
// *          m x (n - 1) matrix C2 if SIDE = 'R'.
// *
// *          On exit, rows 2:m of P*C if SIDE = 'L', or columns 2:m of C*P
// *          if SIDE = 'R'.
// *
// *  LDC     (input) INTEGER
// *          The leading dimension of the arrays C1 and C2. LDC >= (1,M).
// *
// *  WORK    (workspace) DOUBLE PRECISION array, dimension
// *                      (N) if SIDE = 'L'
// *                      (M) if SIDE = 'R'
// *
// *  =====================================================================
// *
// *     .. Parameters ..
static double one= 1.0e+0;
static double zero= 0.0e+0;
// *     ..
// *     .. External Subroutines ..
// *     ..
// *     .. External Functions ..
// *     ..
// *     .. Intrinsic Functions ..
// *     ..
// *     .. Executable Statements ..
// *

public static void dlatzm (String side,
int m,
int n,
double [] v, int _v_offset,
int incv,
double tau,
double [] c1, int _c1_offset,
double [] c2, int _c2_offset,
int Ldc,
double [] work, int _work_offset)  {

if ((Math.min(m, n)  == 0) || (tau == zero))  
    Dummy.go_to("Dlatzm",999999);
// *
if ((side.toLowerCase().charAt(0) == "L".toLowerCase().charAt(0)))  {
    // *
// *        w := C1 + v' * C2
// *
Dcopy.dcopy(n,c1,_c1_offset,Ldc,work,_work_offset,1);
Dgemv.dgemv("Transpose",m-1,n,one,c2,_c2_offset,Ldc,v,_v_offset,incv,one,work,_work_offset,1);
// *
// *        [ C1 ] := [ C1 ] - tau* [ 1 ] * w'
// *        [ C2 ]    [ C2 ]        [ v ]
// *
Daxpy.daxpy(n,-tau,work,_work_offset,1,c1,_c1_offset,Ldc);
Dger.dger(m-1,n,-tau,v,_v_offset,incv,work,_work_offset,1,c2,_c2_offset,Ldc);
// *
}              // Close if()
else if ((side.toLowerCase().charAt(0) == "R".toLowerCase().charAt(0)))  {
    // *
// *        w := C1 + C2 * v
// *
Dcopy.dcopy(m,c1,_c1_offset,1,work,_work_offset,1);
Dgemv.dgemv("No transpose",m,n-1,one,c2,_c2_offset,Ldc,v,_v_offset,incv,one,work,_work_offset,1);
// *
// *        [ C1, C2 ] := [ C1, C2 ] - tau* w * [ 1 , v']
// *
Daxpy.daxpy(m,-tau,work,_work_offset,1,c1,_c1_offset,1);
Dger.dger(m,n-1,-tau,work,_work_offset,1,v,_v_offset,incv,c2,_c2_offset,Ldc);
}              // Close else if()
// *
Dummy.go_to("Dlatzm",999999);
// *
// *     End of DLATZM
// *
Dummy.label("Dlatzm",999999);
return;
   }
} // End class.
