/*
 *  Produced by f2java.  f2java is part of the Fortran-
 *  -to-Java project at the University of Tennessee Netlib
 *  numerical software repository.
 *
 *  Original authorship for the BLAS and LAPACK numerical
 *  routines may be found in the Fortran source, available at
 *  www.netlib.org.
 *
 *  Fortran input file: jlapack.f
 *
 *  The f2j compiler code was written by
 *  David M. Doolin (doolin@cs.utk.edu) and
 *  Keith  Seymour (seymour@cs.utk.edu)
 */

package org.netlib.lapack;
import java.lang.*;
import org.netlib.util.*;



public class Dorglq {

// *
// *  -- LAPACK routine (version 2.0) --
// *     Univ. of Tennessee, Univ. of California Berkeley, NAG Ltd.,
// *     Courant Institute, Argonne National Lab, and Rice University
// *     September 30, 1994
// *
// *     .. Scalar Arguments ..
// *     ..
// *     .. Array Arguments ..
// *     ..
// *
// *  Purpose
// *  =======
// *
// *  DORGLQ generates an M-by-N real matrix Q with orthonormal rows,
// *  which is defined as the first M rows of a product of K elementary
// *  reflectors of order N
// *
// *        Q  =  H(k) . . . H(2) H(1)
// *
// *  as returned by DGELQF.
// *
// *  Arguments
// *  =========
// *
// *  M       (input) INTEGER
// *          The number of rows of the matrix Q. M >= 0.
// *
// *  N       (input) INTEGER
// *          The number of columns of the matrix Q. N >= M.
// *
// *  K       (input) INTEGER
// *          The number of elementary reflectors whose product defines the
// *          matrix Q. M >= K >= 0.
// *
// *  A       (input/output) DOUBLE PRECISION array, dimension (LDA,N)
// *          On entry, the i-th row must contain the vector which defines
// *          the elementary reflector H(i), for i = 1,2,...,k, as returned
// *          by DGELQF in the first k rows of its array argument A.
// *          On exit, the M-by-N matrix Q.
// *
// *  LDA     (input) INTEGER
// *          The first dimension of the array A. LDA >= max(1,M).
// *
// *  TAU     (input) DOUBLE PRECISION array, dimension (K)
// *          TAU(i) must contain the scalar factor of the elementary
// *          reflector H(i), as returned by DGELQF.
// *
// *  WORK    (workspace/output) DOUBLE PRECISION array, dimension (LWORK)
// *          On exit, if INFO = 0, WORK(1) returns the optimal LWORK.
// *
// *  LWORK   (input) INTEGER
// *          The dimension of the array WORK. LWORK >= max(1,M).
// *          For optimum performance LWORK >= M*NB, where NB is
// *          the optimal blocksize.
// *
// *  INFO    (output) INTEGER
// *          = 0:  successful exit
// *          < 0:  if INFO = -i, the i-th argument has an illegal value
// *
// *  =====================================================================
// *
// *     .. Parameters ..
static double zero= 0.0e+0;
// *     ..
// *     .. Local Scalars ..
static int i= 0;
static int ib= 0;
static intW iinfo= new intW(0);
static int iws= 0;
static int j= 0;
static int ki= 0;
static int kk= 0;
static int l= 0;
static int ldwork= 0;
static int nb= 0;
static int nbmin= 0;
static int nx= 0;
// *     ..
// *     .. External Subroutines ..
// *     ..
// *     .. Intrinsic Functions ..
// *     ..
// *     .. External Functions ..
// *     ..
// *     .. Executable Statements ..
// *
// *     Test the input arguments
// *

public static void dorglq (int m,
int n,
int k,
double [] a, int _a_offset,
int lda,
double [] tau, int _tau_offset,
double [] work, int _work_offset,
int lwork,
intW info)  {

info.val = 0;
if (m < 0)  {
    info.val = -1;
}              // Close if()
else if (n < m)  {
    info.val = -2;
}              // Close else if()
else if (k < 0 || k > m)  {
    info.val = -3;
}              // Close else if()
else if (lda < Math.max(1, m) )  {
    info.val = -5;
}              // Close else if()
else if (lwork < Math.max(1, m) )  {
    info.val = -8;
}              // Close else if()
if (info.val != 0)  {
    Xerbla.xerbla("DORGLQ",-info.val);
Dummy.go_to("Dorglq",999999);
}              // Close if()
// *
// *     Quick return if possible
// *
if (m <= 0)  {
    work[(1)- 1+ _work_offset] = (double)(1);
Dummy.go_to("Dorglq",999999);
}              // Close if()
// *
// *     Determine the block size.
// *
nb = Ilaenv.ilaenv(1,"DORGLQ"," ",m,n,k,-1);
nbmin = 2;
nx = 0;
iws = m;
if (nb > 1 && nb < k)  {
    // *
// *        Determine when to cross over from blocked to unblocked code.
// *
nx = (int)(Math.max(0, Ilaenv.ilaenv(3,"DORGLQ"," ",m,n,k,-1)) );
if (nx < k)  {
    // *
// *           Determine if workspace is large enough for blocked code.
// *
ldwork = m;
iws = ldwork*nb;
if (lwork < iws)  {
    // *
// *              Not enough workspace to use optimal NB:  reduce NB and
// *              determine the minimum value of NB.
// *
nb = lwork/ldwork;
nbmin = (int)(Math.max(2, Ilaenv.ilaenv(2,"DORGLQ"," ",m,n,k,-1)) );
}              // Close if()
}              // Close if()
}              // Close if()
// *
if (nb >= nbmin && nb < k && nx < k)  {
    // *
// *        Use blocked code after the last block.
// *        The first kk rows are handled by the block method.
// *
ki = ((k-nx-1)/nb)*nb;
kk = (int)(Math.min(k, ki+nb) );
// *
// *        Set A(kk+1:m,1:kk) to zero.
// *
{
forloop20:
for (j = 1; j <= kk; j++) {
{
forloop10:
for (i = kk+1; i <= m; i++) {
a[(i)- 1+(j- 1)*lda+ _a_offset] = zero;
Dummy.label("Dorglq",10);
}              //  Close for() loop. 
}
Dummy.label("Dorglq",20);
}              //  Close for() loop. 
}
}              // Close if()
else  {
  kk = 0;
}              //  Close else.
// *
// *     Use unblocked code for the last or only block.
// *
if (kk < m)  
    Dorgl2.dorgl2(m-kk,n-kk,k-kk,a,(kk+1)- 1+(kk+1- 1)*lda+ _a_offset,lda,tau,(kk+1)- 1+ _tau_offset,work,_work_offset,iinfo);
// *
if (kk > 0)  {
    // *
// *        Use blocked code
// *
{
int _i_inc = -nb;
forloop50:
for (i = ki+1; (_i_inc < 0) ? i >= 1 : i <= 1; i += _i_inc) {
ib = (int)(Math.min(nb, k-i+1) );
if (i+ib <= m)  {
    // *
// *              Form the triangular factor of the block reflector
// *              H = H(i) H(i+1) . . . H(i+ib-1)
// *
Dlarft.dlarft("Forward","Rowwise",n-i+1,ib,a,(i)- 1+(i- 1)*lda+ _a_offset,lda,tau,(i)- 1+ _tau_offset,work,_work_offset,ldwork);
// *
// *              Apply H' to A(i+ib:m,i:n) from the right
// *
Dlarfb.dlarfb("Right","Transpose","Forward","Rowwise",m-i-ib+1,n-i+1,ib,a,(i)- 1+(i- 1)*lda+ _a_offset,lda,work,_work_offset,ldwork,a,(i+ib)- 1+(i- 1)*lda+ _a_offset,lda,work,(ib+1)- 1+ _work_offset,ldwork);
}              // Close if()
// *
// *           Apply H' to columns i:n of current block
// *
Dorgl2.dorgl2(ib,n-i+1,ib,a,(i)- 1+(i- 1)*lda+ _a_offset,lda,tau,(i)- 1+ _tau_offset,work,_work_offset,iinfo);
// *
// *           Set columns 1:i-1 of current block to zero
// *
{
forloop40:
for (j = 1; j <= i-1; j++) {
{
forloop30:
for (l = i; l <= i+ib-1; l++) {
a[(l)- 1+(j- 1)*lda+ _a_offset] = zero;
Dummy.label("Dorglq",30);
}              //  Close for() loop. 
}
Dummy.label("Dorglq",40);
}              //  Close for() loop. 
}
Dummy.label("Dorglq",50);
}              //  Close for() loop. 
}
}              // Close if()
// *
work[(1)- 1+ _work_offset] = (double)(iws);
Dummy.go_to("Dorglq",999999);
// *
// *     End of DORGLQ
// *
Dummy.label("Dorglq",999999);
return;
   }
} // End class.
