/*
 *  Produced by f2java.  f2java is part of the Fortran-
 *  -to-Java project at the University of Tennessee Netlib
 *  numerical software repository.
 *
 *  Original authorship for the BLAS and LAPACK numerical
 *  routines may be found in the Fortran source, available at
 *  www.netlib.org.
 *
 *  Fortran input file: jlapack.f
 *
 *  The f2j compiler code was written by
 *  David M. Doolin (doolin@cs.utk.edu) and
 *  Keith  Seymour (seymour@cs.utk.edu)
 */

package org.netlib.lapack;
import java.lang.*;
import org.netlib.util.*;



public class Dormhr {

// *
// *  -- LAPACK routine (version 2.0) --
// *     Univ. of Tennessee, Univ. of California Berkeley, NAG Ltd.,
// *     Courant Institute, Argonne National Lab, and Rice University
// *     September 30, 1994
// *
// *     .. Scalar Arguments ..
// *     ..
// *     .. Array Arguments ..
// *     ..
// *
// *  Purpose
// *  =======
// *
// *  DORMHR overwrites the general real M-by-N matrix C with
// *
// *                  SIDE = 'L'     SIDE = 'R'
// *  TRANS = 'N':      Q * C          C * Q
// *  TRANS = 'T':      Q**T * C       C * Q**T
// *
// *  where Q is a real orthogonal matrix of order nq, with nq = m if
// *  SIDE = 'L' and nq = n if SIDE = 'R'. Q is defined as the product of
// *  IHI-ILO elementary reflectors, as returned by DGEHRD:
// *
// *  Q = H(ilo) H(ilo+1) . . . H(ihi-1).
// *
// *  Arguments
// *  =========
// *
// *  SIDE    (input) CHARACTER*1
// *          = 'L': apply Q or Q**T from the Left;
// *          = 'R': apply Q or Q**T from the Right.
// *
// *  TRANS   (input) CHARACTER*1
// *          = 'N':  No transpose, apply Q;
// *          = 'T':  Transpose, apply Q**T.
// *
// *  M       (input) INTEGER
// *          The number of rows of the matrix C. M >= 0.
// *
// *  N       (input) INTEGER
// *          The number of columns of the matrix C. N >= 0.
// *
// *  ILO     (input) INTEGER
// *  IHI     (input) INTEGER
// *          ILO and IHI must have the same values as in the previous call
// *          of DGEHRD. Q is equal to the unit matrix except in the
// *          submatrix Q(ilo+1:ihi,ilo+1:ihi).
// *          If SIDE = 'L', then 1 <= ILO <= IHI <= M, if M > 0, and
// *          ILO = 1 and IHI = 0, if M = 0;
// *          if SIDE = 'R', then 1 <= ILO <= IHI <= N, if N > 0, and
// *          ILO = 1 and IHI = 0, if N = 0.
// *
// *  A       (input) DOUBLE PRECISION array, dimension
// *                               (LDA,M) if SIDE = 'L'
// *                               (LDA,N) if SIDE = 'R'
// *          The vectors which define the elementary reflectors, as
// *          returned by DGEHRD.
// *
// *  LDA     (input) INTEGER
// *          The leading dimension of the array A.
// *          LDA >= max(1,M) if SIDE = 'L'; LDA >= max(1,N) if SIDE = 'R'.
// *
// *  TAU     (input) DOUBLE PRECISION array, dimension
// *                               (M-1) if SIDE = 'L'
// *                               (N-1) if SIDE = 'R'
// *          TAU(i) must contain the scalar factor of the elementary
// *          reflector H(i), as returned by DGEHRD.
// *
// *  C       (input/output) DOUBLE PRECISION array, dimension (LDC,N)
// *          On entry, the M-by-N matrix C.
// *          On exit, C is overwritten by Q*C or Q**T*C or C*Q**T or C*Q.
// *
// *  LDC     (input) INTEGER
// *          The leading dimension of the array C. LDC >= max(1,M).
// *
// *  WORK    (workspace/output) DOUBLE PRECISION array, dimension (LWORK)
// *          On exit, if INFO = 0, WORK(1) returns the optimal LWORK.
// *
// *  LWORK   (input) INTEGER
// *          The dimension of the array WORK.
// *          If SIDE = 'L', LWORK >= max(1,N);
// *          if SIDE = 'R', LWORK >= max(1,M).
// *          For optimum performance LWORK >= N*NB if SIDE = 'L', and
// *          LWORK >= M*NB if SIDE = 'R', where NB is the optimal
// *          blocksize.
// *
// *  INFO    (output) INTEGER
// *          = 0:  successful exit
// *          < 0:  if INFO = -i, the i-th argument had an illegal value
// *
// *  =====================================================================
// *
// *     .. Local Scalars ..
static boolean left= false;
static int i1= 0;
static int i2= 0;
static intW iinfo= new intW(0);
static int mi= 0;
static int nh= 0;
static int ni= 0;
static int nq= 0;
static int nw= 0;
// *     ..
// *     .. External Functions ..
// *     ..
// *     .. External Subroutines ..
// *     ..
// *     .. Intrinsic Functions ..
// *     ..
// *     .. Executable Statements ..
// *
// *     Test the input arguments
// *

public static void dormhr (String side,
String trans,
int m,
int n,
int ilo,
int ihi,
double [] a, int _a_offset,
int lda,
double [] tau, int _tau_offset,
double [] c, int _c_offset,
int Ldc,
double [] work, int _work_offset,
int lwork,
intW info)  {

info.val = 0;
left = (side.toLowerCase().charAt(0) == "L".toLowerCase().charAt(0));
// *
// *     NQ is the order of Q and NW is the minimum dimension of WORK
// *
if (left)  {
    nq = m;
nw = n;
}              // Close if()
else  {
  nq = n;
nw = m;
}              //  Close else.
if (!left && !(side.toLowerCase().charAt(0) == "R".toLowerCase().charAt(0)))  {
    info.val = -1;
}              // Close if()
else if (!(trans.toLowerCase().charAt(0) == "N".toLowerCase().charAt(0)) && !(trans.toLowerCase().charAt(0) == "T".toLowerCase().charAt(0)))  {
    info.val = -2;
}              // Close else if()
else if (m < 0)  {
    info.val = -3;
}              // Close else if()
else if (n < 0)  {
    info.val = -4;
}              // Close else if()
else if (ilo < 1 || ilo > Math.max(1, nq) )  {
    info.val = -5;
}              // Close else if()
else if (ihi < Math.min(ilo, nq)  || ihi > nq)  {
    info.val = -6;
}              // Close else if()
else if (lda < Math.max(1, nq) )  {
    info.val = -8;
}              // Close else if()
else if (Ldc < Math.max(1, m) )  {
    info.val = -11;
}              // Close else if()
else if (lwork < Math.max(1, nw) )  {
    info.val = -13;
}              // Close else if()
if (info.val != 0)  {
    Xerbla.xerbla("DORMHR",-info.val);
Dummy.go_to("Dormhr",999999);
}              // Close if()
// *
// *     Quick return if possible
// *
nh = ihi-ilo;
if (m == 0 || n == 0 || nh == 0)  {
    work[(1)- 1+ _work_offset] = (double)(1);
Dummy.go_to("Dormhr",999999);
}              // Close if()
// *
if (left)  {
    mi = nh;
ni = n;
i1 = ilo+1;
i2 = 1;
}              // Close if()
else  {
  mi = m;
ni = nh;
i1 = 1;
i2 = ilo+1;
}              //  Close else.
// *
Dormqr.dormqr(side,trans,mi,ni,nh,a,(ilo+1)- 1+(ilo- 1)*lda+ _a_offset,lda,tau,(ilo)- 1+ _tau_offset,c,(i1)- 1+(i2- 1)*Ldc+ _c_offset,Ldc,work,_work_offset,lwork,iinfo);
Dummy.go_to("Dormhr",999999);
// *
// *     End of DORMHR
// *
Dummy.label("Dormhr",999999);
return;
   }
} // End class.
