/*
 *  Produced by f2java.  f2java is part of the Fortran-
 *  -to-Java project at the University of Tennessee Netlib
 *  numerical software repository.
 *
 *  Original authorship for the BLAS and LAPACK numerical
 *  routines may be found in the Fortran source, available at
 *  www.netlib.org.
 *
 *  Fortran input file: jlapack.f
 *
 *  The f2j compiler code was written by
 *  David M. Doolin (doolin@cs.utk.edu) and
 *  Keith  Seymour (seymour@cs.utk.edu)
 */

package org.netlib.lapack;
import java.lang.*;
import org.netlib.util.*;



public class Dormql {

// *
// *  -- LAPACK routine (version 2.0) --
// *     Univ. of Tennessee, Univ. of California Berkeley, NAG Ltd.,
// *     Courant Institute, Argonne National Lab, and Rice University
// *     September 30, 1994
// *
// *     .. Scalar Arguments ..
// *     ..
// *     .. Array Arguments ..
// *     ..
// *
// *  Purpose
// *  =======
// *
// *  DORMQL overwrites the general real M-by-N matrix C with
// *
// *                  SIDE = 'L'     SIDE = 'R'
// *  TRANS = 'N':      Q * C          C * Q
// *  TRANS = 'T':      Q**T * C       C * Q**T
// *
// *  where Q is a real orthogonal matrix defined as the product of k
// *  elementary reflectors
// *
// *        Q = H(k) . . . H(2) H(1)
// *
// *  as returned by DGEQLF. Q is of order M if SIDE = 'L' and of order N
// *  if SIDE = 'R'.
// *
// *  Arguments
// *  =========
// *
// *  SIDE    (input) CHARACTER*1
// *          = 'L': apply Q or Q**T from the Left;
// *          = 'R': apply Q or Q**T from the Right.
// *
// *  TRANS   (input) CHARACTER*1
// *          = 'N':  No transpose, apply Q;
// *          = 'T':  Transpose, apply Q**T.
// *
// *  M       (input) INTEGER
// *          The number of rows of the matrix C. M >= 0.
// *
// *  N       (input) INTEGER
// *          The number of columns of the matrix C. N >= 0.
// *
// *  K       (input) INTEGER
// *          The number of elementary reflectors whose product defines
// *          the matrix Q.
// *          If SIDE = 'L', M >= K >= 0;
// *          if SIDE = 'R', N >= K >= 0.
// *
// *  A       (input) DOUBLE PRECISION array, dimension (LDA,K)
// *          The i-th column must contain the vector which defines the
// *          elementary reflector H(i), for i = 1,2,...,k, as returned by
// *          DGEQLF in the last k columns of its array argument A.
// *          A is modified by the routine but restored on exit.
// *
// *  LDA     (input) INTEGER
// *          The leading dimension of the array A.
// *          If SIDE = 'L', LDA >= max(1,M);
// *          if SIDE = 'R', LDA >= max(1,N).
// *
// *  TAU     (input) DOUBLE PRECISION array, dimension (K)
// *          TAU(i) must contain the scalar factor of the elementary
// *          reflector H(i), as returned by DGEQLF.
// *
// *  C       (input/output) DOUBLE PRECISION array, dimension (LDC,N)
// *          On entry, the M-by-N matrix C.
// *          On exit, C is overwritten by Q*C or Q**T*C or C*Q**T or C*Q.
// *
// *  LDC     (input) INTEGER
// *          The leading dimension of the array C. LDC >= max(1,M).
// *
// *  WORK    (workspace/output) DOUBLE PRECISION array, dimension (LWORK)
// *          On exit, if INFO = 0, WORK(1) returns the optimal LWORK.
// *
// *  LWORK   (input) INTEGER
// *          The dimension of the array WORK.
// *          If SIDE = 'L', LWORK >= max(1,N);
// *          if SIDE = 'R', LWORK >= max(1,M).
// *          For optimum performance LWORK >= N*NB if SIDE = 'L', and
// *          LWORK >= M*NB if SIDE = 'R', where NB is the optimal
// *          blocksize.
// *
// *  INFO    (output) INTEGER
// *          = 0:  successful exit
// *          < 0:  if INFO = -i, the i-th argument had an illegal value
// *
// *  =====================================================================
// *
// *     .. Parameters ..
static int nbmax= 64;
static int ldt= nbmax+1;
// *     ..
// *     .. Local Scalars ..
static boolean left= false;
static boolean notran= false;
static int i= 0;
static int i1= 0;
static int i2= 0;
static int i3= 0;
static int ib= 0;
static intW iinfo= new intW(0);
static int iws= 0;
static int ldwork= 0;
static int mi= 0;
static int nb= 0;
static int nbmin= 0;
static int ni= 0;
static int nq= 0;
static int nw= 0;
// *     ..
// *     .. Local Arrays ..
static double [] t= new double[(ldt) * (nbmax)];
// *     ..
// *     .. External Functions ..
// *     ..
// *     .. External Subroutines ..
// *     ..
// *     .. Intrinsic Functions ..
// *     ..
// *     .. Executable Statements ..
// *
// *     Test the input arguments
// *

public static void dormql (String side,
String trans,
int m,
int n,
int k,
double [] a, int _a_offset,
int lda,
double [] tau, int _tau_offset,
double [] c, int _c_offset,
int Ldc,
double [] work, int _work_offset,
int lwork,
intW info)  {

info.val = 0;
left = (side.toLowerCase().charAt(0) == "L".toLowerCase().charAt(0));
notran = (trans.toLowerCase().charAt(0) == "N".toLowerCase().charAt(0));
// *
// *     NQ is the order of Q and NW is the minimum dimension of WORK
// *
if (left)  {
    nq = m;
nw = n;
}              // Close if()
else  {
  nq = n;
nw = m;
}              //  Close else.
if (!left && !(side.toLowerCase().charAt(0) == "R".toLowerCase().charAt(0)))  {
    info.val = -1;
}              // Close if()
else if (!notran && !(trans.toLowerCase().charAt(0) == "T".toLowerCase().charAt(0)))  {
    info.val = -2;
}              // Close else if()
else if (m < 0)  {
    info.val = -3;
}              // Close else if()
else if (n < 0)  {
    info.val = -4;
}              // Close else if()
else if (k < 0 || k > nq)  {
    info.val = -5;
}              // Close else if()
else if (lda < Math.max(1, nq) )  {
    info.val = -7;
}              // Close else if()
else if (Ldc < Math.max(1, m) )  {
    info.val = -10;
}              // Close else if()
else if (lwork < Math.max(1, nw) )  {
    info.val = -12;
}              // Close else if()
if (info.val != 0)  {
    Xerbla.xerbla("DORMQL",-info.val);
Dummy.go_to("Dormql",999999);
}              // Close if()
// *
// *     Quick return if possible
// *
if (m == 0 || n == 0 || k == 0)  {
    work[(1)- 1+ _work_offset] = (double)(1);
Dummy.go_to("Dormql",999999);
}              // Close if()
// *
// *     Determine the block size.  NB may be at most NBMAX, where NBMAX
// *     is used to define the local array T.
// *
nb = (int)(Math.min(nbmax, Ilaenv.ilaenv(1,"DORMQL",side+trans,m,n,k,-1)) );
nbmin = 2;
ldwork = nw;
if (nb > 1 && nb < k)  {
    iws = nw*nb;
if (lwork < iws)  {
    nb = lwork/ldwork;
nbmin = (int)(Math.max(2, Ilaenv.ilaenv(2,"DORMQL",side+trans,m,n,k,-1)) );
}              // Close if()
}              // Close if()
else  {
  iws = nw;
}              //  Close else.
// *
if (nb < nbmin || nb >= k)  {
    // *
// *        Use unblocked code
// *
Dorm2l.dorm2l(side,trans,m,n,k,a,_a_offset,lda,tau,_tau_offset,c,_c_offset,Ldc,work,_work_offset,iinfo);
}              // Close if()
else  {
  // *
// *        Use blocked code
// *
if ((left && notran) || (!left && !notran))  {
    i1 = 1;
i2 = k;
i3 = nb;
}              // Close if()
else  {
  i1 = ((k-1)/nb)*nb+1;
i2 = 1;
i3 = -nb;
}              //  Close else.
// *
if (left)  {
    ni = n;
}              // Close if()
else  {
  mi = m;
}              //  Close else.
// *
{
int _i_inc = i3;
forloop10:
for (i = i1; (_i_inc < 0) ? i >= i2 : i <= i2; i += _i_inc) {
ib = (int)(Math.min(nb, k-i+1) );
// *
// *           Form the triangular factor of the block reflector
// *           H = H(i+ib-1) . . . H(i+1) H(i)
// *
Dlarft.dlarft("Backward","Columnwise",nq-k+i+ib-1,ib,a,(1)- 1+(i- 1)*lda+ _a_offset,lda,tau,(i)- 1+ _tau_offset,t,0,ldt);
if (left)  {
    // *
// *              H or H' is applied to C(1:m-k+i+ib-1,1:n)
// *
mi = m-k+i+ib-1;
}              // Close if()
else  {
  // *
// *              H or H' is applied to C(1:m,1:n-k+i+ib-1)
// *
ni = n-k+i+ib-1;
}              //  Close else.
// *
// *           Apply H or H'
// *
Dlarfb.dlarfb(side,trans,"Backward","Columnwise",mi,ni,ib,a,(1)- 1+(i- 1)*lda+ _a_offset,lda,t,0,ldt,c,_c_offset,Ldc,work,_work_offset,ldwork);
Dummy.label("Dormql",10);
}              //  Close for() loop. 
}
}              //  Close else.
work[(1)- 1+ _work_offset] = (double)(iws);
Dummy.go_to("Dormql",999999);
// *
// *     End of DORMQL
// *
Dummy.label("Dormql",999999);
return;
   }
} // End class.
