/*
 *  Produced by f2java.  f2java is part of the Fortran-
 *  -to-Java project at the University of Tennessee Netlib
 *  numerical software repository.
 *
 *  Original authorship for the BLAS and LAPACK numerical
 *  routines may be found in the Fortran source, available at
 *  www.netlib.org.
 *
 *  Fortran input file: jlapack.f
 *
 *  The f2j compiler code was written by
 *  David M. Doolin (doolin@cs.utk.edu) and
 *  Keith  Seymour (seymour@cs.utk.edu)
 */

package org.netlib.lapack;
import java.lang.*;
import org.netlib.util.*;



public class Dpbequ {

// *
// *  -- LAPACK routine (version 2.0) --
// *     Univ. of Tennessee, Univ. of California Berkeley, NAG Ltd.,
// *     Courant Institute, Argonne National Lab, and Rice University
// *     March 31, 1993
// *
// *     .. Scalar Arguments ..
// *     ..
// *     .. Array Arguments ..
// *     ..
// *
// *  Purpose
// *  =======
// *
// *  DPBEQU computes row and column scalings intended to equilibrate a
// *  symmetric positive definite band matrix A and reduce its condition
// *  number (with respect to the two-norm).  S contains the scale factors,
// *  S(i) = 1/sqrt(A(i,i)), chosen so that the scaled matrix B with
// *  elements B(i,j) = S(i)*A(i,j)*S(j) has ones on the diagonal.  This
// *  choice of S puts the condition number of B within a factor N of the
// *  smallest possible condition number over all possible diagonal
// *  scalings.
// *
// *  Arguments
// *  =========
// *
// *  UPLO    (input) CHARACTER*1
// *          = 'U':  Upper triangular of A is stored;
// *          = 'L':  Lower triangular of A is stored.
// *
// *  N       (input) INTEGER
// *          The order of the matrix A.  N >= 0.
// *
// *  KD      (input) INTEGER
// *          The number of superdiagonals of the matrix A if UPLO = 'U',
// *          or the number of subdiagonals if UPLO = 'L'.  KD >= 0.
// *
// *  AB      (input) DOUBLE PRECISION array, dimension (LDAB,N)
// *          The upper or lower triangle of the symmetric band matrix A,
// *          stored in the first KD+1 rows of the array.  The j-th column
// *          of A is stored in the j-th column of the array AB as follows:
// *          if UPLO = 'U', AB(kd+1+i-j,j) = A(i,j) for max(1,j-kd)<=i<=j;
// *          if UPLO = 'L', AB(1+i-j,j)    = A(i,j) for j<=i<=min(n,j+kd).
// *
// *  LDAB     (input) INTEGER
// *          The leading dimension of the array A.  LDAB >= KD+1.
// *
// *  S       (output) DOUBLE PRECISION array, dimension (N)
// *          If INFO = 0, S contains the scale factors for A.
// *
// *  SCOND   (output) DOUBLE PRECISION
// *          If INFO = 0, S contains the ratio of the smallest S(i) to
// *          the largest S(i).  If SCOND >= 0.1 and AMAX is neither too
// *          large nor too small, it is not worth scaling by S.
// *
// *  AMAX    (output) DOUBLE PRECISION
// *          Absolute value of largest matrix element.  If AMAX is very
// *          close to overflow or very close to underflow, the matrix
// *          should be scaled.
// *
// *  INFO    (output) INTEGER
// *          = 0:  successful exit
// *          < 0:  if INFO = -i, the i-th argument had an illegal value.
// *          > 0:  if INFO = i, the i-th diagonal element is nonpositive.
// *
// *  =====================================================================
// *
// *     .. Parameters ..
static double zero= 0.0e+0;
static double one= 1.0e+0;
// *     ..
// *     .. Local Scalars ..
static boolean upper= false;
static int i= 0;
static int j= 0;
static double smin= 0.0;
// *     ..
// *     .. External Functions ..
// *     ..
// *     .. External Subroutines ..
// *     ..
// *     .. Intrinsic Functions ..
// *     ..
// *     .. Executable Statements ..
// *
// *     Test the input parameters.
// *

public static void dpbequ (String uplo,
int n,
int kd,
double [] ab, int _ab_offset,
int ldab,
double [] s, int _s_offset,
doubleW scond,
doubleW amax,
intW info)  {

info.val = 0;
upper = (uplo.toLowerCase().charAt(0) == "U".toLowerCase().charAt(0));
if (!upper && !(uplo.toLowerCase().charAt(0) == "L".toLowerCase().charAt(0)))  {
    info.val = -1;
}              // Close if()
else if (n < 0)  {
    info.val = -2;
}              // Close else if()
else if (kd < 0)  {
    info.val = -3;
}              // Close else if()
else if (ldab < kd+1)  {
    info.val = -5;
}              // Close else if()
if (info.val != 0)  {
    Xerbla.xerbla("DPBEQU",-info.val);
Dummy.go_to("Dpbequ",999999);
}              // Close if()
// *
// *     Quick return if possible
// *
if (n == 0)  {
    scond.val = one;
amax.val = zero;
Dummy.go_to("Dpbequ",999999);
}              // Close if()
// *
if (upper)  {
    j = kd+1;
}              // Close if()
else  {
  j = 1;
}              //  Close else.
// *
// *     Initialize SMIN and AMAX.
// *
s[(1)- 1+ _s_offset] = ab[(j)- 1+(1- 1)*ldab+ _ab_offset];
smin = s[(1)- 1+ _s_offset];
amax.val = s[(1)- 1+ _s_offset];
// *
// *     Find the minimum and maximum diagonal elements.
// *
{
forloop10:
for (i = 2; i <= n; i++) {
s[(i)- 1+ _s_offset] = ab[(j)- 1+(i- 1)*ldab+ _ab_offset];
smin = Math.min(smin, s[(i)- 1+ _s_offset]) ;
amax.val = Math.max(amax.val, s[(i)- 1+ _s_offset]) ;
Dummy.label("Dpbequ",10);
}              //  Close for() loop. 
}
// *
if (smin <= zero)  {
    // *
// *        Find the first non-positive diagonal element and return.
// *
{
forloop20:
for (i = 1; i <= n; i++) {
if (s[(i)- 1+ _s_offset] <= zero)  {
    info.val = i;
Dummy.go_to("Dpbequ",999999);
}              // Close if()
Dummy.label("Dpbequ",20);
}              //  Close for() loop. 
}
}              // Close if()
else  {
  // *
// *        Set the scale factors to the reciprocals
// *        of the diagonal elements.
// *
{
forloop30:
for (i = 1; i <= n; i++) {
s[(i)- 1+ _s_offset] = one/Math.sqrt(s[(i)- 1+ _s_offset]);
Dummy.label("Dpbequ",30);
}              //  Close for() loop. 
}
// *
// *        Compute SCOND = min(S(I)) / max(S(I))
// *
scond.val = Math.sqrt(smin)/Math.sqrt(amax.val);
}              //  Close else.
Dummy.go_to("Dpbequ",999999);
// *
// *     End of DPBEQU
// *
Dummy.label("Dpbequ",999999);
return;
   }
} // End class.
