/*
 *  Produced by f2java.  f2java is part of the Fortran-
 *  -to-Java project at the University of Tennessee Netlib
 *  numerical software repository.
 *
 *  Original authorship for the BLAS and LAPACK numerical
 *  routines may be found in the Fortran source, available at
 *  www.netlib.org.
 *
 *  Fortran input file: jlapack.f
 *
 *  The f2j compiler code was written by
 *  David M. Doolin (doolin@cs.utk.edu) and
 *  Keith  Seymour (seymour@cs.utk.edu)
 */

package org.netlib.lapack;
import java.lang.*;
import org.netlib.util.*;

import org.netlib.blas.*;


public class Dpbstf {

// *
// *  -- LAPACK routine (version 2.0) --
// *     Univ. of Tennessee, Univ. of California Berkeley, NAG Ltd.,
// *     Courant Institute, Argonne National Lab, and Rice University
// *     September 30, 1994
// *
// *     .. Scalar Arguments ..
// *     ..
// *     .. Array Arguments ..
// *     ..
// *
// *  Purpose
// *  =======
// *
// *  DPBSTF computes a split Cholesky factorization of a real
// *  symmetric positive definite band matrix A.
// *
// *  This routine is designed to be used in conjunction with DSBGST.
// *
// *  The factorization has the form  A = S**T*S  where S is a band matrix
// *  of the same bandwidth as A and the following structure:
// *
// *    S = ( U    )
// *        ( M  L )
// *
// *  where U is upper triangular of order m = (n+kd)/2, and L is lower
// *  triangular of order n-m.
// *
// *  Arguments
// *  =========
// *
// *  UPLO    (input) CHARACTER*1
// *          = 'U':  Upper triangle of A is stored;
// *          = 'L':  Lower triangle of A is stored.
// *
// *  N       (input) INTEGER
// *          The order of the matrix A.  N >= 0.
// *
// *  KD      (input) INTEGER
// *          The number of superdiagonals of the matrix A if UPLO = 'U',
// *          or the number of subdiagonals if UPLO = 'L'.  KD >= 0.
// *
// *  AB      (input/output) DOUBLE PRECISION array, dimension (LDAB,N)
// *          On entry, the upper or lower triangle of the symmetric band
// *          matrix A, stored in the first kd+1 rows of the array.  The
// *          j-th column of A is stored in the j-th column of the array AB
// *          as follows:
// *          if UPLO = 'U', AB(kd+1+i-j,j) = A(i,j) for max(1,j-kd)<=i<=j;
// *          if UPLO = 'L', AB(1+i-j,j)    = A(i,j) for j<=i<=min(n,j+kd).
// *
// *          On exit, if INFO = 0, the factor S from the split Cholesky
// *          factorization A = S**T*S. See Further Details.
// *
// *  LDAB    (input) INTEGER
// *          The leading dimension of the array AB.  LDAB >= KD+1.
// *
// *  INFO    (output) INTEGER
// *          = 0: successful exit
// *          < 0: if INFO = -i, the i-th argument had an illegal value
// *          > 0: if INFO = i, the factorization could not be completed,
// *               because the updated element a(i,i) was negative; the
// *               matrix A is not positive definite.
// *
// *  Further Details
// *  ===============
// *
// *  The band storage scheme is illustrated by the following example, when
// *  N = 7, KD = 2:
// *
// *  S = ( s11  s12  s13                     )
// *      (      s22  s23  s24                )
// *      (           s33  s34                )
// *      (                s44                )
// *      (           s53  s54  s55           )
// *      (                s64  s65  s66      )
// *      (                     s75  s76  s77 )
// *
// *  If UPLO = 'U', the array AB holds:
// *
// *  on entry:                          on exit:
// *
// *   *    *   a13  a24  a35  a46  a57   *    *   s13  s24  s53  s64  s75
// *   *   a12  a23  a34  a45  a56  a67   *   s12  s23  s34  s54  s65  s76
// *  a11  a22  a33  a44  a55  a66  a77  s11  s22  s33  s44  s55  s66  s77
// *
// *  If UPLO = 'L', the array AB holds:
// *
// *  on entry:                          on exit:
// *
// *  a11  a22  a33  a44  a55  a66  a77  s11  s22  s33  s44  s55  s66  s77
// *  a21  a32  a43  a54  a65  a76   *   s12  s23  s34  s54  s65  s76   *
// *  a31  a42  a53  a64  a64   *    *   s13  s24  s53  s64  s75   *    *
// *
// *  Array elements marked * are not used by the routine.
// *
// *  =====================================================================
// *
// *     .. Parameters ..
static double one= 1.0e+0;
static double zero= 0.0e+0;
// *     ..
// *     .. Local Scalars ..
static boolean upper= false;
static int j= 0;
static int kld= 0;
static int km= 0;
static int m= 0;
static double ajj= 0.0;
// *     ..
// *     .. External Functions ..
// *     ..
// *     .. External Subroutines ..
// *     ..
// *     .. Intrinsic Functions ..
// *     ..
// *     .. Executable Statements ..
// *
// *     Test the input parameters.
// *

public static void dpbstf (String uplo,
int n,
int kd,
double [] ab, int _ab_offset,
int ldab,
intW info)  {

info.val = 0;
upper = (uplo.toLowerCase().charAt(0) == "U".toLowerCase().charAt(0));
if (!upper && !(uplo.toLowerCase().charAt(0) == "L".toLowerCase().charAt(0)))  {
    info.val = -1;
}              // Close if()
else if (n < 0)  {
    info.val = -2;
}              // Close else if()
else if (kd < 0)  {
    info.val = -3;
}              // Close else if()
else if (ldab < kd+1)  {
    info.val = -5;
}              // Close else if()
if (info.val != 0)  {
    Xerbla.xerbla("DPBSTF",-info.val);
Dummy.go_to("Dpbstf",999999);
}              // Close if()
// *
// *     Quick return if possible
// *
if (n == 0)  
    Dummy.go_to("Dpbstf",999999);
// *
kld = (int)(Math.max(1, ldab-1) );
// *
// *     Set the splitting point m.
// *
m = (n+kd)/2;
// *
if (upper)  {
    // *
// *        Factorize A(m+1:n,m+1:n) as L**T*L, and update A(1:m,1:m).
// *
{
int _j_inc = -1;
forloop10:
for (j = n; j >= m+1; j += _j_inc) {
// *
// *           Compute s(j,j) and test for non-positive-definiteness.
// *
ajj = ab[(kd+1)- 1+(j- 1)*ldab+ _ab_offset];
if (ajj <= zero)  
    Dummy.go_to("Dpbstf",50);
ajj = Math.sqrt(ajj);
ab[(kd+1)- 1+(j- 1)*ldab+ _ab_offset] = ajj;
km = (int)(Math.min(j-1, kd) );
// *
// *           Compute elements j-km:j-1 of the j-th column and update the
// *           the leading submatrix within the band.
// *
Dscal.dscal(km,one/ajj,ab,(kd+1-km)- 1+(j- 1)*ldab+ _ab_offset,1);
Dsyr.dsyr("Upper",km,-one,ab,(kd+1-km)- 1+(j- 1)*ldab+ _ab_offset,1,ab,(kd+1)- 1+(j-km- 1)*ldab+ _ab_offset,kld);
Dummy.label("Dpbstf",10);
}              //  Close for() loop. 
}
// *
// *        Factorize the updated submatrix A(1:m,1:m) as U**T*U.
// *
{
forloop20:
for (j = 1; j <= m; j++) {
// *
// *           Compute s(j,j) and test for non-positive-definiteness.
// *
ajj = ab[(kd+1)- 1+(j- 1)*ldab+ _ab_offset];
if (ajj <= zero)  
    Dummy.go_to("Dpbstf",50);
ajj = Math.sqrt(ajj);
ab[(kd+1)- 1+(j- 1)*ldab+ _ab_offset] = ajj;
km = (int)(Math.min(kd, m-j) );
// *
// *           Compute elements j+1:j+km of the j-th row and update the
// *           trailing submatrix within the band.
// *
if (km > 0)  {
    Dscal.dscal(km,one/ajj,ab,(kd)- 1+(j+1- 1)*ldab+ _ab_offset,kld);
Dsyr.dsyr("Upper",km,-one,ab,(kd)- 1+(j+1- 1)*ldab+ _ab_offset,kld,ab,(kd+1)- 1+(j+1- 1)*ldab+ _ab_offset,kld);
}              // Close if()
Dummy.label("Dpbstf",20);
}              //  Close for() loop. 
}
}              // Close if()
else  {
  // *
// *        Factorize A(m+1:n,m+1:n) as L**T*L, and update A(1:m,1:m).
// *
{
int _j_inc = -1;
forloop30:
for (j = n; j >= m+1; j += _j_inc) {
// *
// *           Compute s(j,j) and test for non-positive-definiteness.
// *
ajj = ab[(1)- 1+(j- 1)*ldab+ _ab_offset];
if (ajj <= zero)  
    Dummy.go_to("Dpbstf",50);
ajj = Math.sqrt(ajj);
ab[(1)- 1+(j- 1)*ldab+ _ab_offset] = ajj;
km = (int)(Math.min(j-1, kd) );
// *
// *           Compute elements j-km:j-1 of the j-th row and update the
// *           trailing submatrix within the band.
// *
Dscal.dscal(km,one/ajj,ab,(km+1)- 1+(j-km- 1)*ldab+ _ab_offset,kld);
Dsyr.dsyr("Lower",km,-one,ab,(km+1)- 1+(j-km- 1)*ldab+ _ab_offset,kld,ab,(1)- 1+(j-km- 1)*ldab+ _ab_offset,kld);
Dummy.label("Dpbstf",30);
}              //  Close for() loop. 
}
// *
// *        Factorize the updated submatrix A(1:m,1:m) as U**T*U.
// *
{
forloop40:
for (j = 1; j <= m; j++) {
// *
// *           Compute s(j,j) and test for non-positive-definiteness.
// *
ajj = ab[(1)- 1+(j- 1)*ldab+ _ab_offset];
if (ajj <= zero)  
    Dummy.go_to("Dpbstf",50);
ajj = Math.sqrt(ajj);
ab[(1)- 1+(j- 1)*ldab+ _ab_offset] = ajj;
km = (int)(Math.min(kd, m-j) );
// *
// *           Compute elements j+1:j+km of the j-th column and update the
// *           trailing submatrix within the band.
// *
if (km > 0)  {
    Dscal.dscal(km,one/ajj,ab,(2)- 1+(j- 1)*ldab+ _ab_offset,1);
Dsyr.dsyr("Lower",km,-one,ab,(2)- 1+(j- 1)*ldab+ _ab_offset,1,ab,(1)- 1+(j+1- 1)*ldab+ _ab_offset,kld);
}              // Close if()
Dummy.label("Dpbstf",40);
}              //  Close for() loop. 
}
}              //  Close else.
Dummy.go_to("Dpbstf",999999);
// *
label50:
   Dummy.label("Dpbstf",50);
info.val = j;
Dummy.go_to("Dpbstf",999999);
// *
// *     End of DPBSTF
// *
Dummy.label("Dpbstf",999999);
return;
   }
} // End class.
