/*
 *  Produced by f2java.  f2java is part of the Fortran-
 *  -to-Java project at the University of Tennessee Netlib
 *  numerical software repository.
 *
 *  Original authorship for the BLAS and LAPACK numerical
 *  routines may be found in the Fortran source, available at
 *  www.netlib.org.
 *
 *  Fortran input file: jlapack.f
 *
 *  The f2j compiler code was written by
 *  David M. Doolin (doolin@cs.utk.edu) and
 *  Keith  Seymour (seymour@cs.utk.edu)
 */

package org.netlib.lapack;
import java.lang.*;
import org.netlib.util.*;

import org.netlib.blas.*;


public class Dpbtf2 {

// *
// *  -- LAPACK routine (version 2.0) --
// *     Univ. of Tennessee, Univ. of California Berkeley, NAG Ltd.,
// *     Courant Institute, Argonne National Lab, and Rice University
// *     February 29, 1992
// *
// *     .. Scalar Arguments ..
// *     ..
// *     .. Array Arguments ..
// *     ..
// *
// *  Purpose
// *  =======
// *
// *  DPBTF2 computes the Cholesky factorization of a real symmetric
// *  positive definite band matrix A.
// *
// *  The factorization has the form
// *     A = U' * U ,  if UPLO = 'U', or
// *     A = L  * L',  if UPLO = 'L',
// *  where U is an upper triangular matrix, U' is the transpose of U, and
// *  L is lower triangular.
// *
// *  This is the unblocked version of the algorithm, calling Level 2 BLAS.
// *
// *  Arguments
// *  =========
// *
// *  UPLO    (input) CHARACTER*1
// *          Specifies whether the upper or lower triangular part of the
// *          symmetric matrix A is stored:
// *          = 'U':  Upper triangular
// *          = 'L':  Lower triangular
// *
// *  N       (input) INTEGER
// *          The order of the matrix A.  N >= 0.
// *
// *  KD      (input) INTEGER
// *          The number of super-diagonals of the matrix A if UPLO = 'U',
// *          or the number of sub-diagonals if UPLO = 'L'.  KD >= 0.
// *
// *  AB      (input/output) DOUBLE PRECISION array, dimension (LDAB,N)
// *          On entry, the upper or lower triangle of the symmetric band
// *          matrix A, stored in the first KD+1 rows of the array.  The
// *          j-th column of A is stored in the j-th column of the array AB
// *          as follows:
// *          if UPLO = 'U', AB(kd+1+i-j,j) = A(i,j) for max(1,j-kd)<=i<=j;
// *          if UPLO = 'L', AB(1+i-j,j)    = A(i,j) for j<=i<=min(n,j+kd).
// *
// *          On exit, if INFO = 0, the triangular factor U or L from the
// *          Cholesky factorization A = U'*U or A = L*L' of the band
// *          matrix A, in the same storage format as A.
// *
// *  LDAB    (input) INTEGER
// *          The leading dimension of the array AB.  LDAB >= KD+1.
// *
// *  INFO    (output) INTEGER
// *          = 0: successful exit
// *          < 0: if INFO = -k, the k-th argument had an illegal value
// *          > 0: if INFO = k, the leading minor of order k is not
// *               positive definite, and the factorization could not be
// *               completed.
// *
// *  Further Details
// *  ===============
// *
// *  The band storage scheme is illustrated by the following example, when
// *  N = 6, KD = 2, and UPLO = 'U':
// *
// *  On entry:                       On exit:
// *
// *      *    *   a13  a24  a35  a46      *    *   u13  u24  u35  u46
// *      *   a12  a23  a34  a45  a56      *   u12  u23  u34  u45  u56
// *     a11  a22  a33  a44  a55  a66     u11  u22  u33  u44  u55  u66
// *
// *  Similarly, if UPLO = 'L' the format of A is as follows:
// *
// *  On entry:                       On exit:
// *
// *     a11  a22  a33  a44  a55  a66     l11  l22  l33  l44  l55  l66
// *     a21  a32  a43  a54  a65   *      l21  l32  l43  l54  l65   *
// *     a31  a42  a53  a64   *    *      l31  l42  l53  l64   *    *
// *
// *  Array elements marked * are not used by the routine.
// *
// *  =====================================================================
// *
// *     .. Parameters ..
static double one= 1.0e+0;
static double zero= 0.0e+0;
// *     ..
// *     .. Local Scalars ..
static boolean upper= false;
static int j= 0;
static int kld= 0;
static int kn= 0;
static double ajj= 0.0;
// *     ..
// *     .. External Functions ..
// *     ..
// *     .. External Subroutines ..
// *     ..
// *     .. Intrinsic Functions ..
// *     ..
// *     .. Executable Statements ..
// *
// *     Test the input parameters.
// *

public static void dpbtf2 (String uplo,
int n,
int kd,
double [] ab, int _ab_offset,
int ldab,
intW info)  {

info.val = 0;
upper = (uplo.toLowerCase().charAt(0) == "U".toLowerCase().charAt(0));
if (!upper && !(uplo.toLowerCase().charAt(0) == "L".toLowerCase().charAt(0)))  {
    info.val = -1;
}              // Close if()
else if (n < 0)  {
    info.val = -2;
}              // Close else if()
else if (kd < 0)  {
    info.val = -3;
}              // Close else if()
else if (ldab < kd+1)  {
    info.val = -5;
}              // Close else if()
if (info.val != 0)  {
    Xerbla.xerbla("DPBTF2",-info.val);
Dummy.go_to("Dpbtf2",999999);
}              // Close if()
// *
// *     Quick return if possible
// *
if (n == 0)  
    Dummy.go_to("Dpbtf2",999999);
// *
kld = (int)(Math.max(1, ldab-1) );
// *
if (upper)  {
    // *
// *        Compute the Cholesky factorization A = U'*U.
// *
{
forloop10:
for (j = 1; j <= n; j++) {
// *
// *           Compute U(J,J) and test for non-positive-definiteness.
// *
ajj = ab[(kd+1)- 1+(j- 1)*ldab+ _ab_offset];
if (ajj <= zero)  
    Dummy.go_to("Dpbtf2",30);
ajj = Math.sqrt(ajj);
ab[(kd+1)- 1+(j- 1)*ldab+ _ab_offset] = ajj;
// *
// *           Compute elements J+1:J+KN of row J and update the
// *           trailing submatrix within the band.
// *
kn = (int)(Math.min(kd, n-j) );
if (kn > 0)  {
    Dscal.dscal(kn,one/ajj,ab,(kd)- 1+(j+1- 1)*ldab+ _ab_offset,kld);
Dsyr.dsyr("Upper",kn,-one,ab,(kd)- 1+(j+1- 1)*ldab+ _ab_offset,kld,ab,(kd+1)- 1+(j+1- 1)*ldab+ _ab_offset,kld);
}              // Close if()
Dummy.label("Dpbtf2",10);
}              //  Close for() loop. 
}
}              // Close if()
else  {
  // *
// *        Compute the Cholesky factorization A = L*L'.
// *
{
forloop20:
for (j = 1; j <= n; j++) {
// *
// *           Compute L(J,J) and test for non-positive-definiteness.
// *
ajj = ab[(1)- 1+(j- 1)*ldab+ _ab_offset];
if (ajj <= zero)  
    Dummy.go_to("Dpbtf2",30);
ajj = Math.sqrt(ajj);
ab[(1)- 1+(j- 1)*ldab+ _ab_offset] = ajj;
// *
// *           Compute elements J+1:J+KN of column J and update the
// *           trailing submatrix within the band.
// *
kn = (int)(Math.min(kd, n-j) );
if (kn > 0)  {
    Dscal.dscal(kn,one/ajj,ab,(2)- 1+(j- 1)*ldab+ _ab_offset,1);
Dsyr.dsyr("Lower",kn,-one,ab,(2)- 1+(j- 1)*ldab+ _ab_offset,1,ab,(1)- 1+(j+1- 1)*ldab+ _ab_offset,kld);
}              // Close if()
Dummy.label("Dpbtf2",20);
}              //  Close for() loop. 
}
}              //  Close else.
Dummy.go_to("Dpbtf2",999999);
// *
label30:
   Dummy.label("Dpbtf2",30);
info.val = j;
Dummy.go_to("Dpbtf2",999999);
// *
// *     End of DPBTF2
// *
Dummy.label("Dpbtf2",999999);
return;
   }
} // End class.
