/*
 *  Produced by f2java.  f2java is part of the Fortran-
 *  -to-Java project at the University of Tennessee Netlib
 *  numerical software repository.
 *
 *  Original authorship for the BLAS and LAPACK numerical
 *  routines may be found in the Fortran source, available at
 *  www.netlib.org.
 *
 *  Fortran input file: jlapack.f
 *
 *  The f2j compiler code was written by
 *  David M. Doolin (doolin@cs.utk.edu) and
 *  Keith  Seymour (seymour@cs.utk.edu)
 */

package org.netlib.lapack;
import java.lang.*;
import org.netlib.util.*;

import org.netlib.blas.*;


public class Dpbtrf {

// *
// *  -- LAPACK routine (version 2.0) --
// *     Univ. of Tennessee, Univ. of California Berkeley, NAG Ltd.,
// *     Courant Institute, Argonne National Lab, and Rice University
// *     March 31, 1993
// *
// *     .. Scalar Arguments ..
// *     ..
// *     .. Array Arguments ..
// *     ..
// *
// *  Purpose
// *  =======
// *
// *  DPBTRF computes the Cholesky factorization of a real symmetric
// *  positive definite band matrix A.
// *
// *  The factorization has the form
// *     A = U**T * U,  if UPLO = 'U', or
// *     A = L  * L**T,  if UPLO = 'L',
// *  where U is an upper triangular matrix and L is lower triangular.
// *
// *  Arguments
// *  =========
// *
// *  UPLO    (input) CHARACTER*1
// *          = 'U':  Upper triangle of A is stored;
// *          = 'L':  Lower triangle of A is stored.
// *
// *  N       (input) INTEGER
// *          The order of the matrix A.  N >= 0.
// *
// *  KD      (input) INTEGER
// *          The number of superdiagonals of the matrix A if UPLO = 'U',
// *          or the number of subdiagonals if UPLO = 'L'.  KD >= 0.
// *
// *  AB      (input/output) DOUBLE PRECISION array, dimension (LDAB,N)
// *          On entry, the upper or lower triangle of the symmetric band
// *          matrix A, stored in the first KD+1 rows of the array.  The
// *          j-th column of A is stored in the j-th column of the array AB
// *          as follows:
// *          if UPLO = 'U', AB(kd+1+i-j,j) = A(i,j) for max(1,j-kd)<=i<=j;
// *          if UPLO = 'L', AB(1+i-j,j)    = A(i,j) for j<=i<=min(n,j+kd).
// *
// *          On exit, if INFO = 0, the triangular factor U or L from the
// *          Cholesky factorization A = U**T*U or A = L*L**T of the band
// *          matrix A, in the same storage format as A.
// *
// *  LDAB    (input) INTEGER
// *          The leading dimension of the array AB.  LDAB >= KD+1.
// *
// *  INFO    (output) INTEGER
// *          = 0:  successful exit
// *          < 0:  if INFO = -i, the i-th argument had an illegal value
// *          > 0:  if INFO = i, the leading minor of order i is not
// *                positive definite, and the factorization could not be
// *                completed.
// *
// *  Further Details
// *  ===============
// *
// *  The band storage scheme is illustrated by the following example, when
// *  N = 6, KD = 2, and UPLO = 'U':
// *
// *  On entry:                       On exit:
// *
// *      *    *   a13  a24  a35  a46      *    *   u13  u24  u35  u46
// *      *   a12  a23  a34  a45  a56      *   u12  u23  u34  u45  u56
// *     a11  a22  a33  a44  a55  a66     u11  u22  u33  u44  u55  u66
// *
// *  Similarly, if UPLO = 'L' the format of A is as follows:
// *
// *  On entry:                       On exit:
// *
// *     a11  a22  a33  a44  a55  a66     l11  l22  l33  l44  l55  l66
// *     a21  a32  a43  a54  a65   *      l21  l32  l43  l54  l65   *
// *     a31  a42  a53  a64   *    *      l31  l42  l53  l64   *    *
// *
// *  Array elements marked * are not used by the routine.
// *
// *  Contributed by
// *  Peter Mayes and Giuseppe Radicati, IBM ECSEC, Rome, March 23, 1989
// *
// *  =====================================================================
// *
// *     .. Parameters ..
static double one= 1.0e+0;
static double zero= 0.0e+0;
static int nbmax= 32;
static int ldwork= nbmax+1;
// *     ..
// *     .. Local Scalars ..
static int i= 0;
static int i2= 0;
static int i3= 0;
static int ib= 0;
static intW ii= new intW(0);
static int j= 0;
static int jj= 0;
static int nb= 0;
// *     ..
// *     .. Local Arrays ..
static double [] work= new double[(ldwork) * (nbmax)];
// *     ..
// *     .. External Functions ..
// *     ..
// *     .. External Subroutines ..
// *     ..
// *     .. Intrinsic Functions ..
// *     ..
// *     .. Executable Statements ..
// *
// *     Test the input parameters.
// *

public static void dpbtrf (String uplo,
int n,
int kd,
double [] ab, int _ab_offset,
int ldab,
intW info)  {

info.val = 0;
if ((!(uplo.toLowerCase().charAt(0) == "U".toLowerCase().charAt(0))) && (!(uplo.toLowerCase().charAt(0) == "L".toLowerCase().charAt(0))))  {
    info.val = -1;
}              // Close if()
else if (n < 0)  {
    info.val = -2;
}              // Close else if()
else if (kd < 0)  {
    info.val = -3;
}              // Close else if()
else if (ldab < kd+1)  {
    info.val = -5;
}              // Close else if()
if (info.val != 0)  {
    Xerbla.xerbla("DPBTRF",-info.val);
Dummy.go_to("Dpbtrf",999999);
}              // Close if()
// *
// *     Quick return if possible
// *
if (n == 0)  
    Dummy.go_to("Dpbtrf",999999);
// *
// *     Determine the block size for this environment
// *
nb = Ilaenv.ilaenv(1,"DPBTRF",uplo,n,kd,-1,-1);
// *
// *     The block size must not exceed the semi-bandwidth KD, and must not
// *     exceed the limit set by the size of the local array WORK.
// *
nb = (int)(Math.min(nb, nbmax) );
// *
if (nb <= 1 || nb > kd)  {
    // *
// *        Use unblocked code
// *
Dpbtf2.dpbtf2(uplo,n,kd,ab,_ab_offset,ldab,info);
}              // Close if()
else  {
  // *
// *        Use blocked code
// *
if ((uplo.toLowerCase().charAt(0) == "U".toLowerCase().charAt(0)))  {
    // *
// *           Compute the Cholesky factorization of a symmetric band
// *           matrix, given the upper triangle of the matrix in band
// *           storage.
// *
// *           Zero the upper triangle of the work array.
// *
{
forloop20:
for (j = 1; j <= nb; j++) {
{
forloop10:
for (i = 1; i <= j-1; i++) {
work[(i)- 1+(j- 1)*ldwork] = zero;
Dummy.label("Dpbtrf",10);
}              //  Close for() loop. 
}
Dummy.label("Dpbtrf",20);
}              //  Close for() loop. 
}
// *
// *           Process the band matrix one diagonal block at a time.
// *
{
int _i_inc = nb;
forloop70:
for (i = 1; (_i_inc < 0) ? i >= n : i <= n; i += _i_inc) {
ib = (int)(Math.min(nb, n-i+1) );
// *
// *              Factorize the diagonal block
// *
Dpotf2.dpotf2(uplo,ib,ab,(kd+1)- 1+(i- 1)*ldab+ _ab_offset,ldab-1,ii);
if (ii.val != 0)  {
    info.val = i+ii.val-1;
Dummy.go_to("Dpbtrf",150);
}              // Close if()
if (i+ib <= n)  {
    // *
// *                 Update the relevant part of the trailing submatrix.
// *                 If A11 denotes the diagonal block which has just been
// *                 factorized, then we need to update the remaining
// *                 blocks in the diagram:
// *
// *                    A11   A12   A13
// *                          A22   A23
// *                                A33
// *
// *                 The numbers of rows and columns in the partitioning
// *                 are IB, I2, I3 respectively. The blocks A12, A22 and
// *                 A23 are empty if IB = KD. The upper triangle of A13
// *                 lies outside the band.
// *
i2 = (int)(Math.min(kd-ib, n-i-ib+1) );
i3 = (int)(Math.min(ib, n-i-kd+1) );
// *
if (i2 > 0)  {
    // *
// *                    Update A12
// *
Dtrsm.dtrsm("Left","Upper","Transpose","Non-unit",ib,i2,one,ab,(kd+1)- 1+(i- 1)*ldab+ _ab_offset,ldab-1,ab,(kd+1-ib)- 1+(i+ib- 1)*ldab+ _ab_offset,ldab-1);
// *
// *                    Update A22
// *
Dsyrk.dsyrk("Upper","Transpose",i2,ib,-one,ab,(kd+1-ib)- 1+(i+ib- 1)*ldab+ _ab_offset,ldab-1,one,ab,(kd+1)- 1+(i+ib- 1)*ldab+ _ab_offset,ldab-1);
}              // Close if()
// *
if (i3 > 0)  {
    // *
// *                    Copy the lower triangle of A13 into the work array.
// *
{
forloop40:
for (jj = 1; jj <= i3; jj++) {
{
forloop30:
for (ii.val = jj; ii.val <= ib; ii.val++) {
work[(ii.val)- 1+(jj- 1)*ldwork] = ab[(ii.val-jj+1)- 1+(jj+i+kd-1- 1)*ldab+ _ab_offset];
Dummy.label("Dpbtrf",30);
}              //  Close for() loop. 
}
Dummy.label("Dpbtrf",40);
}              //  Close for() loop. 
}
// *
// *                    Update A13 (in the work array).
// *
Dtrsm.dtrsm("Left","Upper","Transpose","Non-unit",ib,i3,one,ab,(kd+1)- 1+(i- 1)*ldab+ _ab_offset,ldab-1,work,0,ldwork);
// *
// *                    Update A23
// *
if (i2 > 0)  
    Dgemm.dgemm("Transpose","No Transpose",i2,i3,ib,-one,ab,(kd+1-ib)- 1+(i+ib- 1)*ldab+ _ab_offset,ldab-1,work,0,ldwork,one,ab,(1+ib)- 1+(i+kd- 1)*ldab+ _ab_offset,ldab-1);
// *
// *                    Update A33
// *
Dsyrk.dsyrk("Upper","Transpose",i3,ib,-one,work,0,ldwork,one,ab,(kd+1)- 1+(i+kd- 1)*ldab+ _ab_offset,ldab-1);
// *
// *                    Copy the lower triangle of A13 back into place.
// *
{
forloop60:
for (jj = 1; jj <= i3; jj++) {
{
forloop50:
for (ii.val = jj; ii.val <= ib; ii.val++) {
ab[(ii.val-jj+1)- 1+(jj+i+kd-1- 1)*ldab+ _ab_offset] = work[(ii.val)- 1+(jj- 1)*ldwork];
Dummy.label("Dpbtrf",50);
}              //  Close for() loop. 
}
Dummy.label("Dpbtrf",60);
}              //  Close for() loop. 
}
}              // Close if()
}              // Close if()
Dummy.label("Dpbtrf",70);
}              //  Close for() loop. 
}
}              // Close if()
else  {
  // *
// *           Compute the Cholesky factorization of a symmetric band
// *           matrix, given the lower triangle of the matrix in band
// *           storage.
// *
// *           Zero the lower triangle of the work array.
// *
{
forloop90:
for (j = 1; j <= nb; j++) {
{
forloop80:
for (i = j+1; i <= nb; i++) {
work[(i)- 1+(j- 1)*ldwork] = zero;
Dummy.label("Dpbtrf",80);
}              //  Close for() loop. 
}
Dummy.label("Dpbtrf",90);
}              //  Close for() loop. 
}
// *
// *           Process the band matrix one diagonal block at a time.
// *
{
int _i_inc = nb;
forloop140:
for (i = 1; (_i_inc < 0) ? i >= n : i <= n; i += _i_inc) {
ib = (int)(Math.min(nb, n-i+1) );
// *
// *              Factorize the diagonal block
// *
Dpotf2.dpotf2(uplo,ib,ab,(1)- 1+(i- 1)*ldab+ _ab_offset,ldab-1,ii);
if (ii.val != 0)  {
    info.val = i+ii.val-1;
Dummy.go_to("Dpbtrf",150);
}              // Close if()
if (i+ib <= n)  {
    // *
// *                 Update the relevant part of the trailing submatrix.
// *                 If A11 denotes the diagonal block which has just been
// *                 factorized, then we need to update the remaining
// *                 blocks in the diagram:
// *
// *                    A11
// *                    A21   A22
// *                    A31   A32   A33
// *
// *                 The numbers of rows and columns in the partitioning
// *                 are IB, I2, I3 respectively. The blocks A21, A22 and
// *                 A32 are empty if IB = KD. The lower triangle of A31
// *                 lies outside the band.
// *
i2 = (int)(Math.min(kd-ib, n-i-ib+1) );
i3 = (int)(Math.min(ib, n-i-kd+1) );
// *
if (i2 > 0)  {
    // *
// *                    Update A21
// *
Dtrsm.dtrsm("Right","Lower","Transpose","Non-unit",i2,ib,one,ab,(1)- 1+(i- 1)*ldab+ _ab_offset,ldab-1,ab,(1+ib)- 1+(i- 1)*ldab+ _ab_offset,ldab-1);
// *
// *                    Update A22
// *
Dsyrk.dsyrk("Lower","No Transpose",i2,ib,-one,ab,(1+ib)- 1+(i- 1)*ldab+ _ab_offset,ldab-1,one,ab,(1)- 1+(i+ib- 1)*ldab+ _ab_offset,ldab-1);
}              // Close if()
// *
if (i3 > 0)  {
    // *
// *                    Copy the upper triangle of A31 into the work array.
// *
{
forloop110:
for (jj = 1; jj <= ib; jj++) {
{
forloop100:
for (ii.val = 1; ii.val <= Math.min(jj, i3) ; ii.val++) {
work[(ii.val)- 1+(jj- 1)*ldwork] = ab[(kd+1-jj+ii.val)- 1+(jj+i-1- 1)*ldab+ _ab_offset];
Dummy.label("Dpbtrf",100);
}              //  Close for() loop. 
}
Dummy.label("Dpbtrf",110);
}              //  Close for() loop. 
}
// *
// *                    Update A31 (in the work array).
// *
Dtrsm.dtrsm("Right","Lower","Transpose","Non-unit",i3,ib,one,ab,(1)- 1+(i- 1)*ldab+ _ab_offset,ldab-1,work,0,ldwork);
// *
// *                    Update A32
// *
if (i2 > 0)  
    Dgemm.dgemm("No transpose","Transpose",i3,i2,ib,-one,work,0,ldwork,ab,(1+ib)- 1+(i- 1)*ldab+ _ab_offset,ldab-1,one,ab,(1+kd-ib)- 1+(i+ib- 1)*ldab+ _ab_offset,ldab-1);
// *
// *                    Update A33
// *
Dsyrk.dsyrk("Lower","No Transpose",i3,ib,-one,work,0,ldwork,one,ab,(1)- 1+(i+kd- 1)*ldab+ _ab_offset,ldab-1);
// *
// *                    Copy the upper triangle of A31 back into place.
// *
{
forloop130:
for (jj = 1; jj <= ib; jj++) {
{
forloop120:
for (ii.val = 1; ii.val <= Math.min(jj, i3) ; ii.val++) {
ab[(kd+1-jj+ii.val)- 1+(jj+i-1- 1)*ldab+ _ab_offset] = work[(ii.val)- 1+(jj- 1)*ldwork];
Dummy.label("Dpbtrf",120);
}              //  Close for() loop. 
}
Dummy.label("Dpbtrf",130);
}              //  Close for() loop. 
}
}              // Close if()
}              // Close if()
Dummy.label("Dpbtrf",140);
}              //  Close for() loop. 
}
}              //  Close else.
}              //  Close else.
Dummy.go_to("Dpbtrf",999999);
// *
label150:
   Dummy.label("Dpbtrf",150);
Dummy.go_to("Dpbtrf",999999);
// *
// *     End of DPBTRF
// *
Dummy.label("Dpbtrf",999999);
return;
   }
} // End class.
