/*
 *  Produced by f2java.  f2java is part of the Fortran-
 *  -to-Java project at the University of Tennessee Netlib
 *  numerical software repository.
 *
 *  Original authorship for the BLAS and LAPACK numerical
 *  routines may be found in the Fortran source, available at
 *  www.netlib.org.
 *
 *  Fortran input file: jlapack.f
 *
 *  The f2j compiler code was written by
 *  David M. Doolin (doolin@cs.utk.edu) and
 *  Keith  Seymour (seymour@cs.utk.edu)
 */

package org.netlib.lapack;
import java.lang.*;
import org.netlib.util.*;

import org.netlib.blas.*;


public class Dpbtrs {

// *
// *  -- LAPACK routine (version 2.0) --
// *     Univ. of Tennessee, Univ. of California Berkeley, NAG Ltd.,
// *     Courant Institute, Argonne National Lab, and Rice University
// *     September 30, 1994
// *
// *     .. Scalar Arguments ..
// *     ..
// *     .. Array Arguments ..
// *     ..
// *
// *  Purpose
// *  =======
// *
// *  DPBTRS solves a system of linear equations A*X = B with a symmetric
// *  positive definite band matrix A using the Cholesky factorization
// *  A = U**T*U or A = L*L**T computed by DPBTRF.
// *
// *  Arguments
// *  =========
// *
// *  UPLO    (input) CHARACTER*1
// *          = 'U':  Upper triangular factor stored in AB;
// *          = 'L':  Lower triangular factor stored in AB.
// *
// *  N       (input) INTEGER
// *          The order of the matrix A.  N >= 0.
// *
// *  KD      (input) INTEGER
// *          The number of superdiagonals of the matrix A if UPLO = 'U',
// *          or the number of subdiagonals if UPLO = 'L'.  KD >= 0.
// *
// *  NRHS    (input) INTEGER
// *          The number of right hand sides, i.e., the number of columns
// *          of the matrix B.  NRHS >= 0.
// *
// *  AB      (input) DOUBLE PRECISION array, dimension (LDAB,N)
// *          The triangular factor U or L from the Cholesky factorization
// *          A = U**T*U or A = L*L**T of the band matrix A, stored in the
// *          first KD+1 rows of the array.  The j-th column of U or L is
// *          stored in the j-th column of the array AB as follows:
// *          if UPLO ='U', AB(kd+1+i-j,j) = U(i,j) for max(1,j-kd)<=i<=j;
// *          if UPLO ='L', AB(1+i-j,j)    = L(i,j) for j<=i<=min(n,j+kd).
// *
// *  LDAB    (input) INTEGER
// *          The leading dimension of the array AB.  LDAB >= KD+1.
// *
// *  B       (input/output) DOUBLE PRECISION array, dimension (LDB,NRHS)
// *          On entry, the right hand side matrix B.
// *          On exit, the solution matrix X.
// *
// *  LDB     (input) INTEGER
// *          The leading dimension of the array B.  LDB >= max(1,N).
// *
// *  INFO    (output) INTEGER
// *          = 0:  successful exit
// *          < 0:  if INFO = -i, the i-th argument had an illegal value
// *
// *  =====================================================================
// *
// *     .. Local Scalars ..
static boolean upper= false;
static int j= 0;
// *     ..
// *     .. External Functions ..
// *     ..
// *     .. External Subroutines ..
// *     ..
// *     .. Intrinsic Functions ..
// *     ..
// *     .. Executable Statements ..
// *
// *     Test the input parameters.
// *

public static void dpbtrs (String uplo,
int n,
int kd,
int nrhs,
double [] ab, int _ab_offset,
int ldab,
double [] b, int _b_offset,
int ldb,
intW info)  {

info.val = 0;
upper = (uplo.toLowerCase().charAt(0) == "U".toLowerCase().charAt(0));
if (!upper && !(uplo.toLowerCase().charAt(0) == "L".toLowerCase().charAt(0)))  {
    info.val = -1;
}              // Close if()
else if (n < 0)  {
    info.val = -2;
}              // Close else if()
else if (kd < 0)  {
    info.val = -3;
}              // Close else if()
else if (nrhs < 0)  {
    info.val = -4;
}              // Close else if()
else if (ldab < kd+1)  {
    info.val = -6;
}              // Close else if()
else if (ldb < Math.max(1, n) )  {
    info.val = -8;
}              // Close else if()
if (info.val != 0)  {
    Xerbla.xerbla("DPBTRS",-info.val);
Dummy.go_to("Dpbtrs",999999);
}              // Close if()
// *
// *     Quick return if possible
// *
if (n == 0 || nrhs == 0)  
    Dummy.go_to("Dpbtrs",999999);
// *
if (upper)  {
    // *
// *        Solve A*X = B where A = U'*U.
// *
{
forloop10:
for (j = 1; j <= nrhs; j++) {
// *
// *           Solve U'*X = B, overwriting B with X.
// *
Dtbsv.dtbsv("Upper","Transpose","Non-unit",n,kd,ab,_ab_offset,ldab,b,(1)- 1+(j- 1)*ldb+ _b_offset,1);
// *
// *           Solve U*X = B, overwriting B with X.
// *
Dtbsv.dtbsv("Upper","No transpose","Non-unit",n,kd,ab,_ab_offset,ldab,b,(1)- 1+(j- 1)*ldb+ _b_offset,1);
Dummy.label("Dpbtrs",10);
}              //  Close for() loop. 
}
}              // Close if()
else  {
  // *
// *        Solve A*X = B where A = L*L'.
// *
{
forloop20:
for (j = 1; j <= nrhs; j++) {
// *
// *           Solve L*X = B, overwriting B with X.
// *
Dtbsv.dtbsv("Lower","No transpose","Non-unit",n,kd,ab,_ab_offset,ldab,b,(1)- 1+(j- 1)*ldb+ _b_offset,1);
// *
// *           Solve L'*X = B, overwriting B with X.
// *
Dtbsv.dtbsv("Lower","Transpose","Non-unit",n,kd,ab,_ab_offset,ldab,b,(1)- 1+(j- 1)*ldb+ _b_offset,1);
Dummy.label("Dpbtrs",20);
}              //  Close for() loop. 
}
}              //  Close else.
// *
Dummy.go_to("Dpbtrs",999999);
// *
// *     End of DPBTRS
// *
Dummy.label("Dpbtrs",999999);
return;
   }
} // End class.
