/*
 *  Produced by f2java.  f2java is part of the Fortran-
 *  -to-Java project at the University of Tennessee Netlib
 *  numerical software repository.
 *
 *  Original authorship for the BLAS and LAPACK numerical
 *  routines may be found in the Fortran source, available at
 *  www.netlib.org.
 *
 *  Fortran input file: jlapack.f
 *
 *  The f2j compiler code was written by
 *  David M. Doolin (doolin@cs.utk.edu) and
 *  Keith  Seymour (seymour@cs.utk.edu)
 */

package org.netlib.lapack;
import java.lang.*;
import org.netlib.util.*;



public class Dpoequ {

// *
// *  -- LAPACK routine (version 2.0) --
// *     Univ. of Tennessee, Univ. of California Berkeley, NAG Ltd.,
// *     Courant Institute, Argonne National Lab, and Rice University
// *     March 31, 1993
// *
// *     .. Scalar Arguments ..
// *     ..
// *     .. Array Arguments ..
// *     ..
// *
// *  Purpose
// *  =======
// *
// *  DPOEQU computes row and column scalings intended to equilibrate a
// *  symmetric positive definite matrix A and reduce its condition number
// *  (with respect to the two-norm).  S contains the scale factors,
// *  S(i) = 1/sqrt(A(i,i)), chosen so that the scaled matrix B with
// *  elements B(i,j) = S(i)*A(i,j)*S(j) has ones on the diagonal.  This
// *  choice of S puts the condition number of B within a factor N of the
// *  smallest possible condition number over all possible diagonal
// *  scalings.
// *
// *  Arguments
// *  =========
// *
// *  N       (input) INTEGER
// *          The order of the matrix A.  N >= 0.
// *
// *  A       (input) DOUBLE PRECISION array, dimension (LDA,N)
// *          The N-by-N symmetric positive definite matrix whose scaling
// *          factors are to be computed.  Only the diagonal elements of A
// *          are referenced.
// *
// *  LDA     (input) INTEGER
// *          The leading dimension of the array A.  LDA >= max(1,N).
// *
// *  S       (output) DOUBLE PRECISION array, dimension (N)
// *          If INFO = 0, S contains the scale factors for A.
// *
// *  SCOND   (output) DOUBLE PRECISION
// *          If INFO = 0, S contains the ratio of the smallest S(i) to
// *          the largest S(i).  If SCOND >= 0.1 and AMAX is neither too
// *          large nor too small, it is not worth scaling by S.
// *
// *  AMAX    (output) DOUBLE PRECISION
// *          Absolute value of largest matrix element.  If AMAX is very
// *          close to overflow or very close to underflow, the matrix
// *          should be scaled.
// *
// *  INFO    (output) INTEGER
// *          = 0:  successful exit
// *          < 0:  if INFO = -i, the i-th argument had an illegal value
// *          > 0:  if INFO = i, the i-th diagonal element is nonpositive.
// *
// *  =====================================================================
// *
// *     .. Parameters ..
static double zero= 0.0e+0;
static double one= 1.0e+0;
// *     ..
// *     .. Local Scalars ..
static int i= 0;
static double smin= 0.0;
// *     ..
// *     .. External Subroutines ..
// *     ..
// *     .. Intrinsic Functions ..
// *     ..
// *     .. Executable Statements ..
// *
// *     Test the input parameters.
// *

public static void dpoequ (int n,
double [] a, int _a_offset,
int lda,
double [] s, int _s_offset,
doubleW scond,
doubleW amax,
intW info)  {

info.val = 0;
if (n < 0)  {
    info.val = -1;
}              // Close if()
else if (lda < Math.max(1, n) )  {
    info.val = -3;
}              // Close else if()
if (info.val != 0)  {
    Xerbla.xerbla("DPOEQU",-info.val);
Dummy.go_to("Dpoequ",999999);
}              // Close if()
// *
// *     Quick return if possible
// *
if (n == 0)  {
    scond.val = one;
amax.val = zero;
Dummy.go_to("Dpoequ",999999);
}              // Close if()
// *
// *     Find the minimum and maximum diagonal elements.
// *
s[(1)- 1+ _s_offset] = a[(1)- 1+(1- 1)*lda+ _a_offset];
smin = s[(1)- 1+ _s_offset];
amax.val = s[(1)- 1+ _s_offset];
{
forloop10:
for (i = 2; i <= n; i++) {
s[(i)- 1+ _s_offset] = a[(i)- 1+(i- 1)*lda+ _a_offset];
smin = Math.min(smin, s[(i)- 1+ _s_offset]) ;
amax.val = Math.max(amax.val, s[(i)- 1+ _s_offset]) ;
Dummy.label("Dpoequ",10);
}              //  Close for() loop. 
}
// *
if (smin <= zero)  {
    // *
// *        Find the first non-positive diagonal element and return.
// *
{
forloop20:
for (i = 1; i <= n; i++) {
if (s[(i)- 1+ _s_offset] <= zero)  {
    info.val = i;
Dummy.go_to("Dpoequ",999999);
}              // Close if()
Dummy.label("Dpoequ",20);
}              //  Close for() loop. 
}
}              // Close if()
else  {
  // *
// *        Set the scale factors to the reciprocals
// *        of the diagonal elements.
// *
{
forloop30:
for (i = 1; i <= n; i++) {
s[(i)- 1+ _s_offset] = one/Math.sqrt(s[(i)- 1+ _s_offset]);
Dummy.label("Dpoequ",30);
}              //  Close for() loop. 
}
// *
// *        Compute SCOND = min(S(I)) / max(S(I))
// *
scond.val = Math.sqrt(smin)/Math.sqrt(amax.val);
}              //  Close else.
Dummy.go_to("Dpoequ",999999);
// *
// *     End of DPOEQU
// *
Dummy.label("Dpoequ",999999);
return;
   }
} // End class.
