/*
 *  Produced by f2java.  f2java is part of the Fortran-
 *  -to-Java project at the University of Tennessee Netlib
 *  numerical software repository.
 *
 *  Original authorship for the BLAS and LAPACK numerical
 *  routines may be found in the Fortran source, available at
 *  www.netlib.org.
 *
 *  Fortran input file: jlapack.f
 *
 *  The f2j compiler code was written by
 *  David M. Doolin (doolin@cs.utk.edu) and
 *  Keith  Seymour (seymour@cs.utk.edu)
 */

package org.netlib.lapack;
import java.lang.*;
import org.netlib.util.*;



public class Dposvx {

// *
// *  -- LAPACK driver routine (version 2.0) --
// *     Univ. of Tennessee, Univ. of California Berkeley, NAG Ltd.,
// *     Courant Institute, Argonne National Lab, and Rice University
// *     September 30, 1994
// *
// *     .. Scalar Arguments ..
// *     ..
// *     .. Array Arguments ..
// *     ..
// *
// *  Purpose
// *  =======
// *
// *  DPOSVX uses the Cholesky factorization A = U**T*U or A = L*L**T to
// *  compute the solution to a real system of linear equations
// *     A * X = B,
// *  where A is an N-by-N symmetric positive definite matrix and X and B
// *  are N-by-NRHS matrices.
// *
// *  Error bounds on the solution and a condition estimate are also
// *  provided.
// *
// *  Description
// *  ===========
// *
// *  The following steps are performed:
// *
// *  1. If FACT = 'E', real scaling factors are computed to equilibrate
// *     the system:
// *        diag(S) * A * diag(S) * inv(diag(S)) * X = diag(S) * B
// *     Whether or not the system will be equilibrated depends on the
// *     scaling of the matrix A, but if equilibration is used, A is
// *     overwritten by diag(S)*A*diag(S) and B by diag(S)*B.
// *
// *  2. If FACT = 'N' or 'E', the Cholesky decomposition is used to
// *     factor the matrix A (after equilibration if FACT = 'E') as
// *        A = U**T* U,  if UPLO = 'U', or
// *        A = L * L**T,  if UPLO = 'L',
// *     where U is an upper triangular matrix and L is a lower triangular
// *     matrix.
// *
// *  3. The factored form of A is used to estimate the condition number
// *     of the matrix A.  If the reciprocal of the condition number is
// *     less than machine precision, steps 4-6 are skipped.
// *
// *  4. The system of equations is solved for X using the factored form
// *     of A.
// *
// *  5. Iterative refinement is applied to improve the computed solution
// *     matrix and calculate error bounds and backward error estimates
// *     for it.
// *
// *  6. If equilibration was used, the matrix X is premultiplied by
// *     diag(S) so that it solves the original system before
// *     equilibration.
// *
// *  Arguments
// *  =========
// *
// *  FACT    (input) CHARACTER*1
// *          Specifies whether or not the factored form of the matrix A is
// *          supplied on entry, and if not, whether the matrix A should be
// *          equilibrated before it is factored.
// *          = 'F':  On entry, AF contains the factored form of A.
// *                  If EQUED = 'Y', the matrix A has been equilibrated
// *                  with scaling factors given by S.  A and AF will not
// *                  be modified.
// *          = 'N':  The matrix A will be copied to AF and factored.
// *          = 'E':  The matrix A will be equilibrated if necessary, then
// *                  copied to AF and factored.
// *
// *  UPLO    (input) CHARACTER*1
// *          = 'U':  Upper triangle of A is stored;
// *          = 'L':  Lower triangle of A is stored.
// *
// *  N       (input) INTEGER
// *          The number of linear equations, i.e., the order of the
// *          matrix A.  N >= 0.
// *
// *  NRHS    (input) INTEGER
// *          The number of right hand sides, i.e., the number of columns
// *          of the matrices B and X.  NRHS >= 0.
// *
// *  A       (input/output) DOUBLE PRECISION array, dimension (LDA,N)
// *          On entry, the symmetric matrix A, except if FACT = 'F' and
// *          EQUED = 'Y', then A must contain the equilibrated matrix
// *          diag(S)*A*diag(S).  If UPLO = 'U', the leading
// *          N-by-N upper triangular part of A contains the upper
// *          triangular part of the matrix A, and the strictly lower
// *          triangular part of A is not referenced.  If UPLO = 'L', the
// *          leading N-by-N lower triangular part of A contains the lower
// *          triangular part of the matrix A, and the strictly upper
// *          triangular part of A is not referenced.  A is not modified if
// *          FACT = 'F' or 'N', or if FACT = 'E' and EQUED = 'N' on exit.
// *
// *          On exit, if FACT = 'E' and EQUED = 'Y', A is overwritten by
// *          diag(S)*A*diag(S).
// *
// *  LDA     (input) INTEGER
// *          The leading dimension of the array A.  LDA >= max(1,N).
// *
// *  AF      (input or output) DOUBLE PRECISION array, dimension (LDAF,N)
// *          If FACT = 'F', then AF is an input argument and on entry
// *          contains the triangular factor U or L from the Cholesky
// *          factorization A = U**T*U or A = L*L**T, in the same storage
// *          format as A.  If EQUED .ne. 'N', then AF is the factored form
// *          of the equilibrated matrix diag(S)*A*diag(S).
// *
// *          If FACT = 'N', then AF is an output argument and on exit
// *          returns the triangular factor U or L from the Cholesky
// *          factorization A = U**T*U or A = L*L**T of the original
// *          matrix A.
// *
// *          If FACT = 'E', then AF is an output argument and on exit
// *          returns the triangular factor U or L from the Cholesky
// *          factorization A = U**T*U or A = L*L**T of the equilibrated
// *          matrix A (see the description of A for the form of the
// *          equilibrated matrix).
// *
// *  LDAF    (input) INTEGER
// *          The leading dimension of the array AF.  LDAF >= max(1,N).
// *
// *  EQUED   (input or output) CHARACTER*1
// *          Specifies the form of equilibration that was done.
// *          = 'N':  No equilibration (always true if FACT = 'N').
// *          = 'Y':  Equilibration was done, i.e., A has been replaced by
// *                  diag(S) * A * diag(S).
// *          EQUED is an input argument if FACT = 'F'; otherwise, it is an
// *          output argument.
// *
// *  S       (input or output) DOUBLE PRECISION array, dimension (N)
// *          The scale factors for A; not accessed if EQUED = 'N'.  S is
// *          an input argument if FACT = 'F'; otherwise, S is an output
// *          argument.  If FACT = 'F' and EQUED = 'Y', each element of S
// *          must be positive.
// *
// *  B       (input/output) DOUBLE PRECISION array, dimension (LDB,NRHS)
// *          On entry, the N-by-NRHS right hand side matrix B.
// *          On exit, if EQUED = 'N', B is not modified; if EQUED = 'Y',
// *          B is overwritten by diag(S) * B.
// *
// *  LDB     (input) INTEGER
// *          The leading dimension of the array B.  LDB >= max(1,N).
// *
// *  X       (output) DOUBLE PRECISION array, dimension (LDX,NRHS)
// *          If INFO = 0, the N-by-NRHS solution matrix X to the original
// *          system of equations.  Note that if EQUED = 'Y', A and B are
// *          modified on exit, and the solution to the equilibrated system
// *          is inv(diag(S))*X.
// *
// *  LDX     (input) INTEGER
// *          The leading dimension of the array X.  LDX >= max(1,N).
// *
// *  RCOND   (output) DOUBLE PRECISION
// *          The estimate of the reciprocal condition number of the matrix
// *          A after equilibration (if done).  If RCOND is less than the
// *          machine precision (in particular, if RCOND = 0), the matrix
// *          is singular to working precision.  This condition is
// *          indicated by a return code of INFO > 0, and the solution and
// *          error bounds are not computed.
// *
// *  FERR    (output) DOUBLE PRECISION array, dimension (NRHS)
// *          The estimated forward error bound for each solution vector
// *          X(j) (the j-th column of the solution matrix X).
// *          If XTRUE is the true solution corresponding to X(j), FERR(j)
// *          is an estimated upper bound for the magnitude of the largest
// *          element in (X(j) - XTRUE) divided by the magnitude of the
// *          largest element in X(j).  The estimate is as reliable as
// *          the estimate for RCOND, and is almost always a slight
// *          overestimate of the true error.
// *
// *  BERR    (output) DOUBLE PRECISION array, dimension (NRHS)
// *          The componentwise relative backward error of each solution
// *          vector X(j) (i.e., the smallest relative change in
// *          any element of A or B that makes X(j) an exact solution).
// *
// *  WORK    (workspace) DOUBLE PRECISION array, dimension (3*N)
// *
// *  IWORK   (workspace) INTEGER array, dimension (N)
// *
// *  INFO    (output) INTEGER
// *          = 0: successful exit
// *          < 0: if INFO = -i, the i-th argument had an illegal value
// *          > 0: if INFO = i, and i is
// *               <= N: the leading minor of order i of A
// *                     is not positive definite, so the factorization
// *                     could not be completed, and the solution and error
// *                     bounds could not be computed.
// *               = N+1: RCOND is less than machine precision.  The
// *                     factorization has been completed, but the matrix
// *                     is singular to working precision, and the solution
// *                     and error bounds have not been computed.
// *
// *  =====================================================================
// *
// *     .. Parameters ..
static double zero= 0.0e+0;
static double one= 1.0e+0;
// *     ..
// *     .. Local Scalars ..
static boolean equil= false;
static boolean nofact= false;
static boolean rcequ= false;
static int i= 0;
static intW infequ= new intW(0);
static int j= 0;
static doubleW amax= new doubleW(0.0);
static double anorm= 0.0;
static double bignum= 0.0;
static doubleW scond= new doubleW(0.0);
static double smax= 0.0;
static double smin= 0.0;
static double smlnum= 0.0;
// *     ..
// *     .. External Functions ..
// *     ..
// *     .. External Subroutines ..
// *     ..
// *     .. Intrinsic Functions ..
// *     ..
// *     .. Executable Statements ..
// *

public static void dposvx (String fact,
String uplo,
int n,
int nrhs,
double [] a, int _a_offset,
int lda,
double [] af, int _af_offset,
int ldaf,
StringW equed,
double [] s, int _s_offset,
double [] b, int _b_offset,
int ldb,
double [] x, int _x_offset,
int ldx,
doubleW rcond,
double [] ferr, int _ferr_offset,
double [] berr, int _berr_offset,
double [] work, int _work_offset,
int [] iwork, int _iwork_offset,
intW info)  {

info.val = 0;
nofact = (fact.toLowerCase().charAt(0) == "N".toLowerCase().charAt(0));
equil = (fact.toLowerCase().charAt(0) == "E".toLowerCase().charAt(0));
if (nofact || equil)  {
    equed.val = "N";
rcequ = false;
}              // Close if()
else  {
  rcequ = (equed.val.toLowerCase().charAt(0) == "Y".toLowerCase().charAt(0));
smlnum = Dlamch.dlamch("Safe minimum");
bignum = one/smlnum;
}              //  Close else.
// *
// *     Test the input parameters.
// *
if (!nofact && !equil && !(fact.toLowerCase().charAt(0) == "F".toLowerCase().charAt(0)))  {
    info.val = -1;
}              // Close if()
else if (!(uplo.toLowerCase().charAt(0) == "U".toLowerCase().charAt(0)) && !(uplo.toLowerCase().charAt(0) == "L".toLowerCase().charAt(0)))  {
    info.val = -2;
}              // Close else if()
else if (n < 0)  {
    info.val = -3;
}              // Close else if()
else if (nrhs < 0)  {
    info.val = -4;
}              // Close else if()
else if (lda < Math.max(1, n) )  {
    info.val = -6;
}              // Close else if()
else if (ldaf < Math.max(1, n) )  {
    info.val = -8;
}              // Close else if()
else if ((fact.toLowerCase().charAt(0) == "F".toLowerCase().charAt(0)) && !(rcequ || (equed.val.toLowerCase().charAt(0) == "N".toLowerCase().charAt(0))))  {
    info.val = -9;
}              // Close else if()
else  {
  if (rcequ)  {
    smin = bignum;
smax = zero;
{
forloop10:
for (j = 1; j <= n; j++) {
smin = Math.min(smin, s[(j)- 1+ _s_offset]) ;
smax = Math.max(smax, s[(j)- 1+ _s_offset]) ;
Dummy.label("Dposvx",10);
}              //  Close for() loop. 
}
if (smin <= zero)  {
    info.val = -10;
}              // Close if()
else if (n > 0)  {
    scond.val = Math.max(smin, smlnum) /Math.min(smax, bignum) ;
}              // Close else if()
else  {
  scond.val = one;
}              //  Close else.
}              // Close if()
if (info.val == 0)  {
    if (ldb < Math.max(1, n) )  {
    info.val = -12;
}              // Close if()
else if (ldx < Math.max(1, n) )  {
    info.val = -14;
}              // Close else if()
}              // Close if()
}              //  Close else.
// *
if (info.val != 0)  {
    Xerbla.xerbla("DPOSVX",-info.val);
Dummy.go_to("Dposvx",999999);
}              // Close if()
// *
if (equil)  {
    // *
// *        Compute row and column scalings to equilibrate the matrix A.
// *
Dpoequ.dpoequ(n,a,_a_offset,lda,s,_s_offset,scond,amax,infequ);
if (infequ.val == 0)  {
    // *
// *           Equilibrate the matrix.
// *
Dlaqsy.dlaqsy(uplo,n,a,_a_offset,lda,s,_s_offset,scond.val,amax.val,equed);
rcequ = (equed.val.toLowerCase().charAt(0) == "Y".toLowerCase().charAt(0));
}              // Close if()
}              // Close if()
// *
// *     Scale the right hand side.
// *
if (rcequ)  {
    {
forloop30:
for (j = 1; j <= nrhs; j++) {
{
forloop20:
for (i = 1; i <= n; i++) {
b[(i)- 1+(j- 1)*ldb+ _b_offset] = s[(i)- 1+ _s_offset]*b[(i)- 1+(j- 1)*ldb+ _b_offset];
Dummy.label("Dposvx",20);
}              //  Close for() loop. 
}
Dummy.label("Dposvx",30);
}              //  Close for() loop. 
}
}              // Close if()
// *
if (nofact || equil)  {
    // *
// *        Compute the Cholesky factorization A = U'*U or A = L*L'.
// *
Dlacpy.dlacpy(uplo,n,n,a,_a_offset,lda,af,_af_offset,ldaf);
Dpotrf.dpotrf(uplo,n,af,_af_offset,ldaf,info);
// *
// *        Return if INFO is non-zero.
// *
if (info.val != 0)  {
    if (info.val > 0)  
    rcond.val = zero;
Dummy.go_to("Dposvx",999999);
}              // Close if()
}              // Close if()
// *
// *     Compute the norm of the matrix A.
// *
anorm = Dlansy.dlansy("1",uplo,n,a,_a_offset,lda,work,_work_offset);
// *
// *     Compute the reciprocal of the condition number of A.
// *
Dpocon.dpocon(uplo,n,af,_af_offset,ldaf,anorm,rcond,work,_work_offset,iwork,_iwork_offset,info);
// *
// *     Return if the matrix is singular to working precision.
// *
if (rcond.val < Dlamch.dlamch("Epsilon"))  {
    info.val = n+1;
Dummy.go_to("Dposvx",999999);
}              // Close if()
// *
// *     Compute the solution matrix X.
// *
Dlacpy.dlacpy("Full",n,nrhs,b,_b_offset,ldb,x,_x_offset,ldx);
Dpotrs.dpotrs(uplo,n,nrhs,af,_af_offset,ldaf,x,_x_offset,ldx,info);
// *
// *     Use iterative refinement to improve the computed solution and
// *     compute error bounds and backward error estimates for it.
// *
Dporfs.dporfs(uplo,n,nrhs,a,_a_offset,lda,af,_af_offset,ldaf,b,_b_offset,ldb,x,_x_offset,ldx,ferr,_ferr_offset,berr,_berr_offset,work,_work_offset,iwork,_iwork_offset,info);
// *
// *     Transform the solution matrix X to a solution of the original
// *     system.
// *
if (rcequ)  {
    {
forloop50:
for (j = 1; j <= nrhs; j++) {
{
forloop40:
for (i = 1; i <= n; i++) {
x[(i)- 1+(j- 1)*ldx+ _x_offset] = s[(i)- 1+ _s_offset]*x[(i)- 1+(j- 1)*ldx+ _x_offset];
Dummy.label("Dposvx",40);
}              //  Close for() loop. 
}
Dummy.label("Dposvx",50);
}              //  Close for() loop. 
}
{
forloop60:
for (j = 1; j <= nrhs; j++) {
ferr[(j)- 1+ _ferr_offset] = ferr[(j)- 1+ _ferr_offset]/scond.val;
Dummy.label("Dposvx",60);
}              //  Close for() loop. 
}
}              // Close if()
// *
Dummy.go_to("Dposvx",999999);
// *
// *     End of DPOSVX
// *
Dummy.label("Dposvx",999999);
return;
   }
} // End class.
