/*
 *  Produced by f2java.  f2java is part of the Fortran-
 *  -to-Java project at the University of Tennessee Netlib
 *  numerical software repository.
 *
 *  Original authorship for the BLAS and LAPACK numerical
 *  routines may be found in the Fortran source, available at
 *  www.netlib.org.
 *
 *  Fortran input file: jlapack.f
 *
 *  The f2j compiler code was written by
 *  David M. Doolin (doolin@cs.utk.edu) and
 *  Keith  Seymour (seymour@cs.utk.edu)
 */

package org.netlib.lapack;
import java.lang.*;
import org.netlib.util.*;

import org.netlib.blas.*;


public class Dpotf2 {

// *
// *  -- LAPACK routine (version 2.0) --
// *     Univ. of Tennessee, Univ. of California Berkeley, NAG Ltd.,
// *     Courant Institute, Argonne National Lab, and Rice University
// *     February 29, 1992
// *
// *     .. Scalar Arguments ..
// *     ..
// *     .. Array Arguments ..
// *     ..
// *
// *  Purpose
// *  =======
// *
// *  DPOTF2 computes the Cholesky factorization of a real symmetric
// *  positive definite matrix A.
// *
// *  The factorization has the form
// *     A = U' * U ,  if UPLO = 'U', or
// *     A = L  * L',  if UPLO = 'L',
// *  where U is an upper triangular matrix and L is lower triangular.
// *
// *  This is the unblocked version of the algorithm, calling Level 2 BLAS.
// *
// *  Arguments
// *  =========
// *
// *  UPLO    (input) CHARACTER*1
// *          Specifies whether the upper or lower triangular part of the
// *          symmetric matrix A is stored.
// *          = 'U':  Upper triangular
// *          = 'L':  Lower triangular
// *
// *  N       (input) INTEGER
// *          The order of the matrix A.  N >= 0.
// *
// *  A       (input/output) DOUBLE PRECISION array, dimension (LDA,N)
// *          On entry, the symmetric matrix A.  If UPLO = 'U', the leading
// *          n by n upper triangular part of A contains the upper
// *          triangular part of the matrix A, and the strictly lower
// *          triangular part of A is not referenced.  If UPLO = 'L', the
// *          leading n by n lower triangular part of A contains the lower
// *          triangular part of the matrix A, and the strictly upper
// *          triangular part of A is not referenced.
// *
// *          On exit, if INFO = 0, the factor U or L from the Cholesky
// *          factorization A = U'*U  or A = L*L'.
// *
// *  LDA     (input) INTEGER
// *          The leading dimension of the array A.  LDA >= max(1,N).
// *
// *  INFO    (output) INTEGER
// *          = 0: successful exit
// *          < 0: if INFO = -k, the k-th argument had an illegal value
// *          > 0: if INFO = k, the leading minor of order k is not
// *               positive definite, and the factorization could not be
// *               completed.
// *
// *  =====================================================================
// *
// *     .. Parameters ..
static double one= 1.0e+0;
static double zero= 0.0e+0;
// *     ..
// *     .. Local Scalars ..
static boolean upper= false;
static int j= 0;
static double ajj= 0.0;
// *     ..
// *     .. External Functions ..
// *     ..
// *     .. External Subroutines ..
// *     ..
// *     .. Intrinsic Functions ..
// *     ..
// *     .. Executable Statements ..
// *
// *     Test the input parameters.
// *

public static void dpotf2 (String uplo,
int n,
double [] a, int _a_offset,
int lda,
intW info)  {

info.val = 0;
upper = (uplo.toLowerCase().charAt(0) == "U".toLowerCase().charAt(0));
if (!upper && !(uplo.toLowerCase().charAt(0) == "L".toLowerCase().charAt(0)))  {
    info.val = -1;
}              // Close if()
else if (n < 0)  {
    info.val = -2;
}              // Close else if()
else if (lda < Math.max(1, n) )  {
    info.val = -4;
}              // Close else if()
if (info.val != 0)  {
    Xerbla.xerbla("DPOTF2",-info.val);
Dummy.go_to("Dpotf2",999999);
}              // Close if()
// *
// *     Quick return if possible
// *
if (n == 0)  
    Dummy.go_to("Dpotf2",999999);
// *
if (upper)  {
    // *
// *        Compute the Cholesky factorization A = U'*U.
// *
{
forloop10:
for (j = 1; j <= n; j++) {
// *
// *           Compute U(J,J) and test for non-positive-definiteness.
// *
ajj = a[(j)- 1+(j- 1)*lda+ _a_offset]-Ddot.ddot(j-1,a,(1)- 1+(j- 1)*lda+ _a_offset,1,a,(1)- 1+(j- 1)*lda+ _a_offset,1);
if (ajj <= zero)  {
    a[(j)- 1+(j- 1)*lda+ _a_offset] = ajj;
Dummy.go_to("Dpotf2",30);
}              // Close if()
ajj = Math.sqrt(ajj);
a[(j)- 1+(j- 1)*lda+ _a_offset] = ajj;
// *
// *           Compute elements J+1:N of row J.
// *
if (j < n)  {
    Dgemv.dgemv("Transpose",j-1,n-j,-one,a,(1)- 1+(j+1- 1)*lda+ _a_offset,lda,a,(1)- 1+(j- 1)*lda+ _a_offset,1,one,a,(j)- 1+(j+1- 1)*lda+ _a_offset,lda);
Dscal.dscal(n-j,one/ajj,a,(j)- 1+(j+1- 1)*lda+ _a_offset,lda);
}              // Close if()
Dummy.label("Dpotf2",10);
}              //  Close for() loop. 
}
}              // Close if()
else  {
  // *
// *        Compute the Cholesky factorization A = L*L'.
// *
{
forloop20:
for (j = 1; j <= n; j++) {
// *
// *           Compute L(J,J) and test for non-positive-definiteness.
// *
ajj = a[(j)- 1+(j- 1)*lda+ _a_offset]-Ddot.ddot(j-1,a,(j)- 1+(1- 1)*lda+ _a_offset,lda,a,(j)- 1+(1- 1)*lda+ _a_offset,lda);
if (ajj <= zero)  {
    a[(j)- 1+(j- 1)*lda+ _a_offset] = ajj;
Dummy.go_to("Dpotf2",30);
}              // Close if()
ajj = Math.sqrt(ajj);
a[(j)- 1+(j- 1)*lda+ _a_offset] = ajj;
// *
// *           Compute elements J+1:N of column J.
// *
if (j < n)  {
    Dgemv.dgemv("No transpose",n-j,j-1,-one,a,(j+1)- 1+(1- 1)*lda+ _a_offset,lda,a,(j)- 1+(1- 1)*lda+ _a_offset,lda,one,a,(j+1)- 1+(j- 1)*lda+ _a_offset,1);
Dscal.dscal(n-j,one/ajj,a,(j+1)- 1+(j- 1)*lda+ _a_offset,1);
}              // Close if()
Dummy.label("Dpotf2",20);
}              //  Close for() loop. 
}
}              //  Close else.
Dummy.go_to("Dpotf2",40);
// *
label30:
   Dummy.label("Dpotf2",30);
info.val = j;
// *
label40:
   Dummy.label("Dpotf2",40);
Dummy.go_to("Dpotf2",999999);
// *
// *     End of DPOTF2
// *
Dummy.label("Dpotf2",999999);
return;
   }
} // End class.
