/*
 *  Produced by f2java.  f2java is part of the Fortran-
 *  -to-Java project at the University of Tennessee Netlib
 *  numerical software repository.
 *
 *  Original authorship for the BLAS and LAPACK numerical
 *  routines may be found in the Fortran source, available at
 *  www.netlib.org.
 *
 *  Fortran input file: jlapack.f
 *
 *  The f2j compiler code was written by
 *  David M. Doolin (doolin@cs.utk.edu) and
 *  Keith  Seymour (seymour@cs.utk.edu)
 */

package org.netlib.lapack;
import java.lang.*;
import org.netlib.util.*;

import org.netlib.blas.*;


public class Dpotrf {

// *
// *  -- LAPACK routine (version 2.0) --
// *     Univ. of Tennessee, Univ. of California Berkeley, NAG Ltd.,
// *     Courant Institute, Argonne National Lab, and Rice University
// *     March 31, 1993
// *
// *     .. Scalar Arguments ..
// *     ..
// *     .. Array Arguments ..
// *     ..
// *
// *  Purpose
// *  =======
// *
// *  DPOTRF computes the Cholesky factorization of a real symmetric
// *  positive definite matrix A.
// *
// *  The factorization has the form
// *     A = U**T * U,  if UPLO = 'U', or
// *     A = L  * L**T,  if UPLO = 'L',
// *  where U is an upper triangular matrix and L is lower triangular.
// *
// *  This is the block version of the algorithm, calling Level 3 BLAS.
// *
// *  Arguments
// *  =========
// *
// *  UPLO    (input) CHARACTER*1
// *          = 'U':  Upper triangle of A is stored;
// *          = 'L':  Lower triangle of A is stored.
// *
// *  N       (input) INTEGER
// *          The order of the matrix A.  N >= 0.
// *
// *  A       (input/output) DOUBLE PRECISION array, dimension (LDA,N)
// *          On entry, the symmetric matrix A.  If UPLO = 'U', the leading
// *          N-by-N upper triangular part of A contains the upper
// *          triangular part of the matrix A, and the strictly lower
// *          triangular part of A is not referenced.  If UPLO = 'L', the
// *          leading N-by-N lower triangular part of A contains the lower
// *          triangular part of the matrix A, and the strictly upper
// *          triangular part of A is not referenced.
// *
// *          On exit, if INFO = 0, the factor U or L from the Cholesky
// *          factorization A = U**T*U or A = L*L**T.
// *
// *  LDA     (input) INTEGER
// *          The leading dimension of the array A.  LDA >= max(1,N).
// *
// *  INFO    (output) INTEGER
// *          = 0:  successful exit
// *          < 0:  if INFO = -i, the i-th argument had an illegal value
// *          > 0:  if INFO = i, the leading minor of order i is not
// *                positive definite, and the factorization could not be
// *                completed.
// *
// *  =====================================================================
// *
// *     .. Parameters ..
static double one= 1.0e+0;
// *     ..
// *     .. Local Scalars ..
static boolean upper= false;
static int j= 0;
static int jb= 0;
static int nb= 0;
// *     ..
// *     .. External Functions ..
// *     ..
// *     .. External Subroutines ..
// *     ..
// *     .. Intrinsic Functions ..
// *     ..
// *     .. Executable Statements ..
// *
// *     Test the input parameters.
// *

public static void dpotrf (String uplo,
int n,
double [] a, int _a_offset,
int lda,
intW info)  {

info.val = 0;
upper = (uplo.toLowerCase().charAt(0) == "U".toLowerCase().charAt(0));
if (!upper && !(uplo.toLowerCase().charAt(0) == "L".toLowerCase().charAt(0)))  {
    info.val = -1;
}              // Close if()
else if (n < 0)  {
    info.val = -2;
}              // Close else if()
else if (lda < Math.max(1, n) )  {
    info.val = -4;
}              // Close else if()
if (info.val != 0)  {
    Xerbla.xerbla("DPOTRF",-info.val);
Dummy.go_to("Dpotrf",999999);
}              // Close if()
// *
// *     Quick return if possible
// *
if (n == 0)  
    Dummy.go_to("Dpotrf",999999);
// *
// *     Determine the block size for this environment.
// *
nb = Ilaenv.ilaenv(1,"DPOTRF",uplo,n,-1,-1,-1);
if (nb <= 1 || nb >= n)  {
    // *
// *        Use unblocked code.
// *
Dpotf2.dpotf2(uplo,n,a,_a_offset,lda,info);
}              // Close if()
else  {
  // *
// *        Use blocked code.
// *
if (upper)  {
    // *
// *           Compute the Cholesky factorization A = U'*U.
// *
{
int _j_inc = nb;
forloop10:
for (j = 1; (_j_inc < 0) ? j >= n : j <= n; j += _j_inc) {
// *
// *              Update and factorize the current diagonal block and test
// *              for non-positive-definiteness.
// *
jb = (int)(Math.min(nb, n-j+1) );
Dsyrk.dsyrk("Upper","Transpose",jb,j-1,-one,a,(1)- 1+(j- 1)*lda+ _a_offset,lda,one,a,(j)- 1+(j- 1)*lda+ _a_offset,lda);
Dpotf2.dpotf2("Upper",jb,a,(j)- 1+(j- 1)*lda+ _a_offset,lda,info);
if (info.val != 0)  
    Dummy.go_to("Dpotrf",30);
if (j+jb <= n)  {
    // *
// *                 Compute the current block row.
// *
Dgemm.dgemm("Transpose","No transpose",jb,n-j-jb+1,j-1,-one,a,(1)- 1+(j- 1)*lda+ _a_offset,lda,a,(1)- 1+(j+jb- 1)*lda+ _a_offset,lda,one,a,(j)- 1+(j+jb- 1)*lda+ _a_offset,lda);
Dtrsm.dtrsm("Left","Upper","Transpose","Non-unit",jb,n-j-jb+1,one,a,(j)- 1+(j- 1)*lda+ _a_offset,lda,a,(j)- 1+(j+jb- 1)*lda+ _a_offset,lda);
}              // Close if()
Dummy.label("Dpotrf",10);
}              //  Close for() loop. 
}
// *
}              // Close if()
else  {
  // *
// *           Compute the Cholesky factorization A = L*L'.
// *
{
int _j_inc = nb;
forloop20:
for (j = 1; (_j_inc < 0) ? j >= n : j <= n; j += _j_inc) {
// *
// *              Update and factorize the current diagonal block and test
// *              for non-positive-definiteness.
// *
jb = (int)(Math.min(nb, n-j+1) );
Dsyrk.dsyrk("Lower","No transpose",jb,j-1,-one,a,(j)- 1+(1- 1)*lda+ _a_offset,lda,one,a,(j)- 1+(j- 1)*lda+ _a_offset,lda);
Dpotf2.dpotf2("Lower",jb,a,(j)- 1+(j- 1)*lda+ _a_offset,lda,info);
if (info.val != 0)  
    Dummy.go_to("Dpotrf",30);
if (j+jb <= n)  {
    // *
// *                 Compute the current block column.
// *
Dgemm.dgemm("No transpose","Transpose",n-j-jb+1,jb,j-1,-one,a,(j+jb)- 1+(1- 1)*lda+ _a_offset,lda,a,(j)- 1+(1- 1)*lda+ _a_offset,lda,one,a,(j+jb)- 1+(j- 1)*lda+ _a_offset,lda);
Dtrsm.dtrsm("Right","Lower","Transpose","Non-unit",n-j-jb+1,jb,one,a,(j)- 1+(j- 1)*lda+ _a_offset,lda,a,(j+jb)- 1+(j- 1)*lda+ _a_offset,lda);
}              // Close if()
Dummy.label("Dpotrf",20);
}              //  Close for() loop. 
}
}              //  Close else.
}              //  Close else.
Dummy.go_to("Dpotrf",40);
// *
label30:
   Dummy.label("Dpotrf",30);
info.val = info.val+j-1;
// *
label40:
   Dummy.label("Dpotrf",40);
Dummy.go_to("Dpotrf",999999);
// *
// *     End of DPOTRF
// *
Dummy.label("Dpotrf",999999);
return;
   }
} // End class.
