/*
 *  Produced by f2java.  f2java is part of the Fortran-
 *  -to-Java project at the University of Tennessee Netlib
 *  numerical software repository.
 *
 *  Original authorship for the BLAS and LAPACK numerical
 *  routines may be found in the Fortran source, available at
 *  www.netlib.org.
 *
 *  Fortran input file: jlapack.f
 *
 *  The f2j compiler code was written by
 *  David M. Doolin (doolin@cs.utk.edu) and
 *  Keith  Seymour (seymour@cs.utk.edu)
 */

package org.netlib.lapack;
import java.lang.*;
import org.netlib.util.*;

import org.netlib.blas.*;


public class Dppcon {

// *
// *  -- LAPACK routine (version 2.0) --
// *     Univ. of Tennessee, Univ. of California Berkeley, NAG Ltd.,
// *     Courant Institute, Argonne National Lab, and Rice University
// *     March 31, 1993
// *
// *     .. Scalar Arguments ..
// *     ..
// *     .. Array Arguments ..
// *     ..
// *
// *  Purpose
// *  =======
// *
// *  DPPCON estimates the reciprocal of the condition number (in the
// *  1-norm) of a real symmetric positive definite packed matrix using
// *  the Cholesky factorization A = U**T*U or A = L*L**T computed by
// *  DPPTRF.
// *
// *  An estimate is obtained for norm(inv(A)), and the reciprocal of the
// *  condition number is computed as RCOND = 1 / (ANORM * norm(inv(A))).
// *
// *  Arguments
// *  =========
// *
// *  UPLO    (input) CHARACTER*1
// *          = 'U':  Upper triangle of A is stored;
// *          = 'L':  Lower triangle of A is stored.
// *
// *  N       (input) INTEGER
// *          The order of the matrix A.  N >= 0.
// *
// *  AP      (input) DOUBLE PRECISION array, dimension (N*(N+1)/2)
// *          The triangular factor U or L from the Cholesky factorization
// *          A = U**T*U or A = L*L**T, packed columnwise in a linear
// *          array.  The j-th column of U or L is stored in the array AP
// *          as follows:
// *          if UPLO = 'U', AP(i + (j-1)*j/2) = U(i,j) for 1<=i<=j;
// *          if UPLO = 'L', AP(i + (j-1)*(2n-j)/2) = L(i,j) for j<=i<=n.
// *
// *  ANORM   (input) DOUBLE PRECISION
// *          The 1-norm (or infinity-norm) of the symmetric matrix A.
// *
// *  RCOND   (output) DOUBLE PRECISION
// *          The reciprocal of the condition number of the matrix A,
// *          computed as RCOND = 1/(ANORM * AINVNM), where AINVNM is an
// *          estimate of the 1-norm of inv(A) computed in this routine.
// *
// *  WORK    (workspace) DOUBLE PRECISION array, dimension (3*N)
// *
// *  IWORK   (workspace) INTEGER array, dimension (N)
// *
// *  INFO    (output) INTEGER
// *          = 0:  successful exit
// *          < 0:  if INFO = -i, the i-th argument had an illegal value
// *
// *  =====================================================================
// *
// *     .. Parameters ..
static double one= 1.0e+0;
static double zero= 0.0e+0;
// *     ..
// *     .. Local Scalars ..
static boolean upper= false;
static String normin= new String(" ");
static int ix= 0;
static intW kase= new intW(0);
static doubleW ainvnm= new doubleW(0.0);
static double scale= 0.0;
static doubleW scalel= new doubleW(0.0);
static doubleW scaleu= new doubleW(0.0);
static double smlnum= 0.0;
// *     ..
// *     .. External Functions ..
// *     ..
// *     .. External Subroutines ..
// *     ..
// *     .. Intrinsic Functions ..
// *     ..
// *     .. Executable Statements ..
// *
// *     Test the input parameters.
// *

public static void dppcon (String uplo,
int n,
double [] ap, int _ap_offset,
double anorm,
doubleW rcond,
double [] work, int _work_offset,
int [] iwork, int _iwork_offset,
intW info)  {

info.val = 0;
upper = (uplo.toLowerCase().charAt(0) == "U".toLowerCase().charAt(0));
if (!upper && !(uplo.toLowerCase().charAt(0) == "L".toLowerCase().charAt(0)))  {
    info.val = -1;
}              // Close if()
else if (n < 0)  {
    info.val = -2;
}              // Close else if()
else if (anorm < zero)  {
    info.val = -4;
}              // Close else if()
if (info.val != 0)  {
    Xerbla.xerbla("DPPCON",-info.val);
Dummy.go_to("Dppcon",999999);
}              // Close if()
// *
// *     Quick return if possible
// *
rcond.val = zero;
if (n == 0)  {
    rcond.val = one;
Dummy.go_to("Dppcon",999999);
}              // Close if()
else if (anorm == zero)  {
    Dummy.go_to("Dppcon",999999);
}              // Close else if()
// *
smlnum = Dlamch.dlamch("Safe minimum");
// *
// *     Estimate the 1-norm of the inverse.
// *
kase.val = 0;
normin = "N";
label10:
   Dummy.label("Dppcon",10);
Dlacon.dlacon(n,work,(n+1)- 1+ _work_offset,work,_work_offset,iwork,_iwork_offset,ainvnm,kase);
if (kase.val != 0)  {
    if (upper)  {
    // *
// *           Multiply by inv(U').
// *
Dlatps.dlatps("Upper","Transpose","Non-unit",normin,n,ap,_ap_offset,work,_work_offset,scalel,work,(2*n+1)- 1+ _work_offset,info);
normin = "Y";
// *
// *           Multiply by inv(U).
// *
Dlatps.dlatps("Upper","No transpose","Non-unit",normin,n,ap,_ap_offset,work,_work_offset,scaleu,work,(2*n+1)- 1+ _work_offset,info);
}              // Close if()
else  {
  // *
// *           Multiply by inv(L).
// *
Dlatps.dlatps("Lower","No transpose","Non-unit",normin,n,ap,_ap_offset,work,_work_offset,scalel,work,(2*n+1)- 1+ _work_offset,info);
normin = "Y";
// *
// *           Multiply by inv(L').
// *
Dlatps.dlatps("Lower","Transpose","Non-unit",normin,n,ap,_ap_offset,work,_work_offset,scaleu,work,(2*n+1)- 1+ _work_offset,info);
}              //  Close else.
// *
// *        Multiply by 1/SCALE if doing so will not cause overflow.
// *
scale = scalel.val*scaleu.val;
if (scale != one)  {
    ix = Idamax.idamax(n,work,_work_offset,1);
if (scale < Math.abs(work[(ix)- 1+ _work_offset])*smlnum || scale == zero)  
    Dummy.go_to("Dppcon",20);
Drscl.drscl(n,scale,work,_work_offset,1);
}              // Close if()
Dummy.go_to("Dppcon",10);
}              // Close if()
// *
// *     Compute the estimate of the reciprocal condition number.
// *
if (ainvnm.val != zero)  
    rcond.val = (one/ainvnm.val)/anorm;
// *
label20:
   Dummy.label("Dppcon",20);
Dummy.go_to("Dppcon",999999);
// *
// *     End of DPPCON
// *
Dummy.label("Dppcon",999999);
return;
   }
} // End class.
