/*
 *  Produced by f2java.  f2java is part of the Fortran-
 *  -to-Java project at the University of Tennessee Netlib
 *  numerical software repository.
 *
 *  Original authorship for the BLAS and LAPACK numerical
 *  routines may be found in the Fortran source, available at
 *  www.netlib.org.
 *
 *  Fortran input file: jlapack.f
 *
 *  The f2j compiler code was written by
 *  David M. Doolin (doolin@cs.utk.edu) and
 *  Keith  Seymour (seymour@cs.utk.edu)
 */

package org.netlib.lapack;
import java.lang.*;
import org.netlib.util.*;



public class Dppequ {

// *
// *  -- LAPACK routine (version 2.0) --
// *     Univ. of Tennessee, Univ. of California Berkeley, NAG Ltd.,
// *     Courant Institute, Argonne National Lab, and Rice University
// *     March 31, 1993
// *
// *     .. Scalar Arguments ..
// *     ..
// *     .. Array Arguments ..
// *     ..
// *
// *  Purpose
// *  =======
// *
// *  DPPEQU computes row and column scalings intended to equilibrate a
// *  symmetric positive definite matrix A in packed storage and reduce
// *  its condition number (with respect to the two-norm).  S contains the
// *  scale factors, S(i)=1/sqrt(A(i,i)), chosen so that the scaled matrix
// *  B with elements B(i,j)=S(i)*A(i,j)*S(j) has ones on the diagonal.
// *  This choice of S puts the condition number of B within a factor N of
// *  the smallest possible condition number over all possible diagonal
// *  scalings.
// *
// *  Arguments
// *  =========
// *
// *  UPLO    (input) CHARACTER*1
// *          = 'U':  Upper triangle of A is stored;
// *          = 'L':  Lower triangle of A is stored.
// *
// *  N       (input) INTEGER
// *          The order of the matrix A.  N >= 0.
// *
// *  AP      (input) DOUBLE PRECISION array, dimension (N*(N+1)/2)
// *          The upper or lower triangle of the symmetric matrix A, packed
// *          columnwise in a linear array.  The j-th column of A is stored
// *          in the array AP as follows:
// *          if UPLO = 'U', AP(i + (j-1)*j/2) = A(i,j) for 1<=i<=j;
// *          if UPLO = 'L', AP(i + (j-1)*(2n-j)/2) = A(i,j) for j<=i<=n.
// *
// *  S       (output) DOUBLE PRECISION array, dimension (N)
// *          If INFO = 0, S contains the scale factors for A.
// *
// *  SCOND   (output) DOUBLE PRECISION
// *          If INFO = 0, S contains the ratio of the smallest S(i) to
// *          the largest S(i).  If SCOND >= 0.1 and AMAX is neither too
// *          large nor too small, it is not worth scaling by S.
// *
// *  AMAX    (output) DOUBLE PRECISION
// *          Absolute value of largest matrix element.  If AMAX is very
// *          close to overflow or very close to underflow, the matrix
// *          should be scaled.
// *
// *  INFO    (output) INTEGER
// *          = 0:  successful exit
// *          < 0:  if INFO = -i, the i-th argument had an illegal value
// *          > 0:  if INFO = i, the i-th diagonal element is nonpositive.
// *
// *  =====================================================================
// *
// *     .. Parameters ..
static double one= 1.0e+0;
static double zero= 0.0e+0;
// *     ..
// *     .. Local Scalars ..
static boolean upper= false;
static int i= 0;
static int jj= 0;
static double smin= 0.0;
// *     ..
// *     .. External Functions ..
// *     ..
// *     .. External Subroutines ..
// *     ..
// *     .. Intrinsic Functions ..
// *     ..
// *     .. Executable Statements ..
// *
// *     Test the input parameters.
// *

public static void dppequ (String uplo,
int n,
double [] ap, int _ap_offset,
double [] s, int _s_offset,
doubleW scond,
doubleW amax,
intW info)  {

info.val = 0;
upper = (uplo.toLowerCase().charAt(0) == "U".toLowerCase().charAt(0));
if (!upper && !(uplo.toLowerCase().charAt(0) == "L".toLowerCase().charAt(0)))  {
    info.val = -1;
}              // Close if()
else if (n < 0)  {
    info.val = -2;
}              // Close else if()
if (info.val != 0)  {
    Xerbla.xerbla("DPPEQU",-info.val);
Dummy.go_to("Dppequ",999999);
}              // Close if()
// *
// *     Quick return if possible
// *
if (n == 0)  {
    scond.val = one;
amax.val = zero;
Dummy.go_to("Dppequ",999999);
}              // Close if()
// *
// *     Initialize SMIN and AMAX.
// *
s[(1)- 1+ _s_offset] = ap[(1)- 1+ _ap_offset];
smin = s[(1)- 1+ _s_offset];
amax.val = s[(1)- 1+ _s_offset];
// *
if (upper)  {
    // *
// *        UPLO = 'U':  Upper triangle of A is stored.
// *        Find the minimum and maximum diagonal elements.
// *
jj = 1;
{
forloop10:
for (i = 2; i <= n; i++) {
jj = jj+i;
s[(i)- 1+ _s_offset] = ap[(jj)- 1+ _ap_offset];
smin = Math.min(smin, s[(i)- 1+ _s_offset]) ;
amax.val = Math.max(amax.val, s[(i)- 1+ _s_offset]) ;
Dummy.label("Dppequ",10);
}              //  Close for() loop. 
}
// *
}              // Close if()
else  {
  // *
// *        UPLO = 'L':  Lower triangle of A is stored.
// *        Find the minimum and maximum diagonal elements.
// *
jj = 1;
{
forloop20:
for (i = 2; i <= n; i++) {
jj = jj+n-i+2;
s[(i)- 1+ _s_offset] = ap[(jj)- 1+ _ap_offset];
smin = Math.min(smin, s[(i)- 1+ _s_offset]) ;
amax.val = Math.max(amax.val, s[(i)- 1+ _s_offset]) ;
Dummy.label("Dppequ",20);
}              //  Close for() loop. 
}
}              //  Close else.
// *
if (smin <= zero)  {
    // *
// *        Find the first non-positive diagonal element and return.
// *
{
forloop30:
for (i = 1; i <= n; i++) {
if (s[(i)- 1+ _s_offset] <= zero)  {
    info.val = i;
Dummy.go_to("Dppequ",999999);
}              // Close if()
Dummy.label("Dppequ",30);
}              //  Close for() loop. 
}
}              // Close if()
else  {
  // *
// *        Set the scale factors to the reciprocals
// *        of the diagonal elements.
// *
{
forloop40:
for (i = 1; i <= n; i++) {
s[(i)- 1+ _s_offset] = one/Math.sqrt(s[(i)- 1+ _s_offset]);
Dummy.label("Dppequ",40);
}              //  Close for() loop. 
}
// *
// *        Compute SCOND = min(S(I)) / max(S(I))
// *
scond.val = Math.sqrt(smin)/Math.sqrt(amax.val);
}              //  Close else.
Dummy.go_to("Dppequ",999999);
// *
// *     End of DPPEQU
// *
Dummy.label("Dppequ",999999);
return;
   }
} // End class.
