/*
 *  Produced by f2java.  f2java is part of the Fortran-
 *  -to-Java project at the University of Tennessee Netlib
 *  numerical software repository.
 *
 *  Original authorship for the BLAS and LAPACK numerical
 *  routines may be found in the Fortran source, available at
 *  www.netlib.org.
 *
 *  Fortran input file: jlapack.f
 *
 *  The f2j compiler code was written by
 *  David M. Doolin (doolin@cs.utk.edu) and
 *  Keith  Seymour (seymour@cs.utk.edu)
 */

package org.netlib.lapack;
import java.lang.*;
import org.netlib.util.*;

import org.netlib.blas.*;


public class Dpptrf {

// *
// *  -- LAPACK routine (version 2.0) --
// *     Univ. of Tennessee, Univ. of California Berkeley, NAG Ltd.,
// *     Courant Institute, Argonne National Lab, and Rice University
// *     March 31, 1993
// *
// *     .. Scalar Arguments ..
// *     ..
// *     .. Array Arguments ..
// *     ..
// *
// *  Purpose
// *  =======
// *
// *  DPPTRF computes the Cholesky factorization of a real symmetric
// *  positive definite matrix A stored in packed format.
// *
// *  The factorization has the form
// *     A = U**T * U,  if UPLO = 'U', or
// *     A = L  * L**T,  if UPLO = 'L',
// *  where U is an upper triangular matrix and L is lower triangular.
// *
// *  Arguments
// *  =========
// *
// *  UPLO    (input) CHARACTER*1
// *          = 'U':  Upper triangle of A is stored;
// *          = 'L':  Lower triangle of A is stored.
// *
// *  N       (input) INTEGER
// *          The order of the matrix A.  N >= 0.
// *
// *  AP      (input/output) DOUBLE PRECISION array, dimension (N*(N+1)/2)
// *          On entry, the upper or lower triangle of the symmetric matrix
// *          A, packed columnwise in a linear array.  The j-th column of A
// *          is stored in the array AP as follows:
// *          if UPLO = 'U', AP(i + (j-1)*j/2) = A(i,j) for 1<=i<=j;
// *          if UPLO = 'L', AP(i + (j-1)*(2n-j)/2) = A(i,j) for j<=i<=n.
// *          See below for further details.
// *
// *          On exit, if INFO = 0, the triangular factor U or L from the
// *          Cholesky factorization A = U**T*U or A = L*L**T, in the same
// *          storage format as A.
// *
// *  INFO    (output) INTEGER
// *          = 0:  successful exit
// *          < 0:  if INFO = -i, the i-th argument had an illegal value
// *          > 0:  if INFO = i, the leading minor of order i is not
// *                positive definite, and the factorization could not be
// *                completed.
// *
// *  Further Details
// *  ======= =======
// *
// *  The packed storage scheme is illustrated by the following example
// *  when N = 4, UPLO = 'U':
// *
// *  Two-dimensional storage of the symmetric matrix A:
// *
// *     a11 a12 a13 a14
// *         a22 a23 a24
// *             a33 a34     (aij = aji)
// *                 a44
// *
// *  Packed storage of the upper triangle of A:
// *
// *  AP = [ a11, a12, a22, a13, a23, a33, a14, a24, a34, a44 ]
// *
// *  =====================================================================
// *
// *     .. Parameters ..
static double one= 1.0e+0;
static double zero= 0.0e+0;
// *     ..
// *     .. Local Scalars ..
static boolean upper= false;
static int j= 0;
static int jc= 0;
static int jj= 0;
static double ajj= 0.0;
// *     ..
// *     .. External Functions ..
// *     ..
// *     .. External Subroutines ..
// *     ..
// *     .. Intrinsic Functions ..
// *     ..
// *     .. Executable Statements ..
// *
// *     Test the input parameters.
// *

public static void dpptrf (String uplo,
int n,
double [] ap, int _ap_offset,
intW info)  {

info.val = 0;
upper = (uplo.toLowerCase().charAt(0) == "U".toLowerCase().charAt(0));
if (!upper && !(uplo.toLowerCase().charAt(0) == "L".toLowerCase().charAt(0)))  {
    info.val = -1;
}              // Close if()
else if (n < 0)  {
    info.val = -2;
}              // Close else if()
if (info.val != 0)  {
    Xerbla.xerbla("DPPTRF",-info.val);
Dummy.go_to("Dpptrf",999999);
}              // Close if()
// *
// *     Quick return if possible
// *
if (n == 0)  
    Dummy.go_to("Dpptrf",999999);
// *
if (upper)  {
    // *
// *        Compute the Cholesky factorization A = U'*U.
// *
jj = 0;
{
forloop10:
for (j = 1; j <= n; j++) {
jc = jj+1;
jj = jj+j;
// *
// *           Compute elements 1:J-1 of column J.
// *
if (j > 1)  
    Dtpsv.dtpsv("Upper","Transpose","Non-unit",j-1,ap,_ap_offset,ap,(jc)- 1+ _ap_offset,1);
// *
// *           Compute U(J,J) and test for non-positive-definiteness.
// *
ajj = ap[(jj)- 1+ _ap_offset]-Ddot.ddot(j-1,ap,(jc)- 1+ _ap_offset,1,ap,(jc)- 1+ _ap_offset,1);
if (ajj <= zero)  {
    ap[(jj)- 1+ _ap_offset] = ajj;
Dummy.go_to("Dpptrf",30);
}              // Close if()
ap[(jj)- 1+ _ap_offset] = Math.sqrt(ajj);
Dummy.label("Dpptrf",10);
}              //  Close for() loop. 
}
}              // Close if()
else  {
  // *
// *        Compute the Cholesky factorization A = L*L'.
// *
jj = 1;
{
forloop20:
for (j = 1; j <= n; j++) {
// *
// *           Compute L(J,J) and test for non-positive-definiteness.
// *
ajj = ap[(jj)- 1+ _ap_offset];
if (ajj <= zero)  {
    ap[(jj)- 1+ _ap_offset] = ajj;
Dummy.go_to("Dpptrf",30);
}              // Close if()
ajj = Math.sqrt(ajj);
ap[(jj)- 1+ _ap_offset] = ajj;
// *
// *           Compute elements J+1:N of column J and update the trailing
// *           submatrix.
// *
if (j < n)  {
    Dscal.dscal(n-j,one/ajj,ap,(jj+1)- 1+ _ap_offset,1);
Dspr.dspr("Lower",n-j,-one,ap,(jj+1)- 1+ _ap_offset,1,ap,(jj+n-j+1)- 1+ _ap_offset);
jj = jj+n-j+1;
}              // Close if()
Dummy.label("Dpptrf",20);
}              //  Close for() loop. 
}
}              //  Close else.
Dummy.go_to("Dpptrf",40);
// *
label30:
   Dummy.label("Dpptrf",30);
info.val = j;
// *
label40:
   Dummy.label("Dpptrf",40);
Dummy.go_to("Dpptrf",999999);
// *
// *     End of DPPTRF
// *
Dummy.label("Dpptrf",999999);
return;
   }
} // End class.
