/*
 *  Produced by f2java.  f2java is part of the Fortran-
 *  -to-Java project at the University of Tennessee Netlib
 *  numerical software repository.
 *
 *  Original authorship for the BLAS and LAPACK numerical
 *  routines may be found in the Fortran source, available at
 *  www.netlib.org.
 *
 *  Fortran input file: jlapack.f
 *
 *  The f2j compiler code was written by
 *  David M. Doolin (doolin@cs.utk.edu) and
 *  Keith  Seymour (seymour@cs.utk.edu)
 */

package org.netlib.lapack;
import java.lang.*;
import org.netlib.util.*;

import org.netlib.blas.*;


public class Dpptrs {

// *
// *  -- LAPACK routine (version 2.0) --
// *     Univ. of Tennessee, Univ. of California Berkeley, NAG Ltd.,
// *     Courant Institute, Argonne National Lab, and Rice University
// *     March 31, 1993
// *
// *     .. Scalar Arguments ..
// *     ..
// *     .. Array Arguments ..
// *     ..
// *
// *  Purpose
// *  =======
// *
// *  DPPTRS solves a system of linear equations A*X = B with a symmetric
// *  positive definite matrix A in packed storage using the Cholesky
// *  factorization A = U**T*U or A = L*L**T computed by DPPTRF.
// *
// *  Arguments
// *  =========
// *
// *  UPLO    (input) CHARACTER*1
// *          = 'U':  Upper triangle of A is stored;
// *          = 'L':  Lower triangle of A is stored.
// *
// *  N       (input) INTEGER
// *          The order of the matrix A.  N >= 0.
// *
// *  NRHS    (input) INTEGER
// *          The number of right hand sides, i.e., the number of columns
// *          of the matrix B.  NRHS >= 0.
// *
// *  AP      (input) DOUBLE PRECISION array, dimension (N*(N+1)/2)
// *          The triangular factor U or L from the Cholesky factorization
// *          A = U**T*U or A = L*L**T, packed columnwise in a linear
// *          array.  The j-th column of U or L is stored in the array AP
// *          as follows:
// *          if UPLO = 'U', AP(i + (j-1)*j/2) = U(i,j) for 1<=i<=j;
// *          if UPLO = 'L', AP(i + (j-1)*(2n-j)/2) = L(i,j) for j<=i<=n.
// *
// *  B       (input/output) DOUBLE PRECISION array, dimension (LDB,NRHS)
// *          On entry, the right hand side matrix B.
// *          On exit, the solution matrix X.
// *
// *  LDB     (input) INTEGER
// *          The leading dimension of the array B.  LDB >= max(1,N).
// *
// *  INFO    (output) INTEGER
// *          = 0:  successful exit
// *          < 0:  if INFO = -i, the i-th argument had an illegal value
// *
// *  =====================================================================
// *
// *     .. Local Scalars ..
static boolean upper= false;
static int i= 0;
// *     ..
// *     .. External Functions ..
// *     ..
// *     .. External Subroutines ..
// *     ..
// *     .. Intrinsic Functions ..
// *     ..
// *     .. Executable Statements ..
// *
// *     Test the input parameters.
// *

public static void dpptrs (String uplo,
int n,
int nrhs,
double [] ap, int _ap_offset,
double [] b, int _b_offset,
int ldb,
intW info)  {

info.val = 0;
upper = (uplo.toLowerCase().charAt(0) == "U".toLowerCase().charAt(0));
if (!upper && !(uplo.toLowerCase().charAt(0) == "L".toLowerCase().charAt(0)))  {
    info.val = -1;
}              // Close if()
else if (n < 0)  {
    info.val = -2;
}              // Close else if()
else if (nrhs < 0)  {
    info.val = -3;
}              // Close else if()
else if (ldb < Math.max(1, n) )  {
    info.val = -6;
}              // Close else if()
if (info.val != 0)  {
    Xerbla.xerbla("DPPTRS",-info.val);
Dummy.go_to("Dpptrs",999999);
}              // Close if()
// *
// *     Quick return if possible
// *
if (n == 0 || nrhs == 0)  
    Dummy.go_to("Dpptrs",999999);
// *
if (upper)  {
    // *
// *        Solve A*X = B where A = U'*U.
// *
{
forloop10:
for (i = 1; i <= nrhs; i++) {
// *
// *           Solve U'*X = B, overwriting B with X.
// *
Dtpsv.dtpsv("Upper","Transpose","Non-unit",n,ap,_ap_offset,b,(1)- 1+(i- 1)*ldb+ _b_offset,1);
// *
// *           Solve U*X = B, overwriting B with X.
// *
Dtpsv.dtpsv("Upper","No transpose","Non-unit",n,ap,_ap_offset,b,(1)- 1+(i- 1)*ldb+ _b_offset,1);
Dummy.label("Dpptrs",10);
}              //  Close for() loop. 
}
}              // Close if()
else  {
  // *
// *        Solve A*X = B where A = L*L'.
// *
{
forloop20:
for (i = 1; i <= nrhs; i++) {
// *
// *           Solve L*Y = B, overwriting B with X.
// *
Dtpsv.dtpsv("Lower","No transpose","Non-unit",n,ap,_ap_offset,b,(1)- 1+(i- 1)*ldb+ _b_offset,1);
// *
// *           Solve L'*X = Y, overwriting B with X.
// *
Dtpsv.dtpsv("Lower","Transpose","Non-unit",n,ap,_ap_offset,b,(1)- 1+(i- 1)*ldb+ _b_offset,1);
Dummy.label("Dpptrs",20);
}              //  Close for() loop. 
}
}              //  Close else.
// *
Dummy.go_to("Dpptrs",999999);
// *
// *     End of DPPTRS
// *
Dummy.label("Dpptrs",999999);
return;
   }
} // End class.
