/*
 *  Produced by f2java.  f2java is part of the Fortran-
 *  -to-Java project at the University of Tennessee Netlib
 *  numerical software repository.
 *
 *  Original authorship for the BLAS and LAPACK numerical
 *  routines may be found in the Fortran source, available at
 *  www.netlib.org.
 *
 *  Fortran input file: jlapack.f
 *
 *  The f2j compiler code was written by
 *  David M. Doolin (doolin@cs.utk.edu) and
 *  Keith  Seymour (seymour@cs.utk.edu)
 */

package org.netlib.lapack;
import java.lang.*;
import org.netlib.util.*;

import org.netlib.blas.*;


public class Dptcon {

// *
// *  -- LAPACK routine (version 2.0) --
// *     Univ. of Tennessee, Univ. of California Berkeley, NAG Ltd.,
// *     Courant Institute, Argonne National Lab, and Rice University
// *     March 31, 1993
// *
// *     .. Scalar Arguments ..
// *     ..
// *     .. Array Arguments ..
// *     ..
// *
// *  Purpose
// *  =======
// *
// *  DPTCON computes the reciprocal of the condition number (in the
// *  1-norm) of a real symmetric positive definite tridiagonal matrix
// *  using the factorization A = L*D*L**T or A = U**T*D*U computed by
// *  DPTTRF.
// *
// *  Norm(inv(A)) is computed by a direct method, and the reciprocal of
// *  the condition number is computed as
// *               RCOND = 1 / (ANORM * norm(inv(A))).
// *
// *  Arguments
// *  =========
// *
// *  N       (input) INTEGER
// *          The order of the matrix A.  N >= 0.
// *
// *  D       (input) DOUBLE PRECISION array, dimension (N)
// *          The n diagonal elements of the diagonal matrix D from the
// *          factorization of A, as computed by DPTTRF.
// *
// *  E       (input) DOUBLE PRECISION array, dimension (N-1)
// *          The (n-1) off-diagonal elements of the unit bidiagonal factor
// *          U or L from the factorization of A,  as computed by DPTTRF.
// *
// *  ANORM   (input) DOUBLE PRECISION
// *          The 1-norm of the original matrix A.
// *
// *  RCOND   (output) DOUBLE PRECISION
// *          The reciprocal of the condition number of the matrix A,
// *          computed as RCOND = 1/(ANORM * AINVNM), where AINVNM is the
// *          1-norm of inv(A) computed in this routine.
// *
// *  WORK    (workspace) DOUBLE PRECISION array, dimension (N)
// *
// *  INFO    (output) INTEGER
// *          = 0:  successful exit
// *          < 0:  if INFO = -i, the i-th argument had an illegal value
// *
// *  Further Details
// *  ===============
// *
// *  The method used is described in Nicholas J. Higham, "Efficient
// *  Algorithms for Computing the Condition Number of a Tridiagonal
// *  Matrix", SIAM J. Sci. Stat. Comput., Vol. 7, No. 1, January 1986.
// *
// *  =====================================================================
// *
// *     .. Parameters ..
static double one= 1.0e+0;
static double zero= 0.0e+0;
// *     ..
// *     .. Local Scalars ..
static int i= 0;
static int ix= 0;
static double ainvnm= 0.0;
// *     ..
// *     .. External Functions ..
// *     ..
// *     .. External Subroutines ..
// *     ..
// *     .. Intrinsic Functions ..
// *     ..
// *     .. Executable Statements ..
// *
// *     Test the input arguments.
// *

public static void dptcon (int n,
double [] d, int _d_offset,
double [] e, int _e_offset,
double anorm,
doubleW rcond,
double [] work, int _work_offset,
intW info)  {

info.val = 0;
if (n < 0)  {
    info.val = -1;
}              // Close if()
else if (anorm < zero)  {
    info.val = -4;
}              // Close else if()
if (info.val != 0)  {
    Xerbla.xerbla("DPTCON",-info.val);
Dummy.go_to("Dptcon",999999);
}              // Close if()
// *
// *     Quick return if possible
// *
rcond.val = zero;
if (n == 0)  {
    rcond.val = one;
Dummy.go_to("Dptcon",999999);
}              // Close if()
else if (anorm == zero)  {
    Dummy.go_to("Dptcon",999999);
}              // Close else if()
// *
// *     Check that D(1:N) is positive.
// *
{
forloop10:
for (i = 1; i <= n; i++) {
if (d[(i)- 1+ _d_offset] <= zero)  
    Dummy.go_to("Dptcon",999999);
Dummy.label("Dptcon",10);
}              //  Close for() loop. 
}
// *
// *     Solve M(A) * x = e, where M(A) = (m(i,j)) is given by
// *
// *        m(i,j) =  abs(A(i,j)), i = j,
// *        m(i,j) = -abs(A(i,j)), i .ne. j,
// *
// *     and e = [ 1, 1, ..., 1 ]'.  Note M(A) = M(L)*D*M(L)'.
// *
// *     Solve M(L) * x = e.
// *
work[(1)- 1+ _work_offset] = one;
{
forloop20:
for (i = 2; i <= n; i++) {
work[(i)- 1+ _work_offset] = one+work[(i-1)- 1+ _work_offset]*Math.abs(e[(i-1)- 1+ _e_offset]);
Dummy.label("Dptcon",20);
}              //  Close for() loop. 
}
// *
// *     Solve D * M(L)' * x = b.
// *
work[(n)- 1+ _work_offset] = work[(n)- 1+ _work_offset]/d[(n)- 1+ _d_offset];
{
int _i_inc = -1;
forloop30:
for (i = n-1; i >= 1; i += _i_inc) {
work[(i)- 1+ _work_offset] = work[(i)- 1+ _work_offset]/d[(i)- 1+ _d_offset]+work[(i+1)- 1+ _work_offset]*Math.abs(e[(i)- 1+ _e_offset]);
Dummy.label("Dptcon",30);
}              //  Close for() loop. 
}
// *
// *     Compute AINVNM = max(x(i)), 1<=i<=n.
// *
ix = Idamax.idamax(n,work,_work_offset,1);
ainvnm = Math.abs(work[(ix)- 1+ _work_offset]);
// *
// *     Compute the reciprocal condition number.
// *
if (ainvnm != zero)  
    rcond.val = (one/ainvnm)/anorm;
// *
Dummy.go_to("Dptcon",999999);
// *
// *     End of DPTCON
// *
Dummy.label("Dptcon",999999);
return;
   }
} // End class.
