/*
 *  Produced by f2java.  f2java is part of the Fortran-
 *  -to-Java project at the University of Tennessee Netlib
 *  numerical software repository.
 *
 *  Original authorship for the BLAS and LAPACK numerical
 *  routines may be found in the Fortran source, available at
 *  www.netlib.org.
 *
 *  Fortran input file: jlapack.f
 *
 *  The f2j compiler code was written by
 *  David M. Doolin (doolin@cs.utk.edu) and
 *  Keith  Seymour (seymour@cs.utk.edu)
 */

package org.netlib.lapack;
import java.lang.*;
import org.netlib.util.*;



public class Dpteqr {

// *
// *  -- LAPACK routine (version 2.0) --
// *     Univ. of Tennessee, Univ. of California Berkeley, NAG Ltd.,
// *     Courant Institute, Argonne National Lab, and Rice University
// *     September 30, 1994
// *
// *     .. Scalar Arguments ..
// *     ..
// *     .. Array Arguments ..
// *     ..
// *
// *  Purpose
// *  =======
// *
// *  DPTEQR computes all eigenvalues and, optionally, eigenvectors of a
// *  symmetric positive definite tridiagonal matrix by first factoring the
// *  matrix using DPTTRF, and then calling DBDSQR to compute the singular
// *  values of the bidiagonal factor.
// *
// *  This routine computes the eigenvalues of the positive definite
// *  tridiagonal matrix to high relative accuracy.  This means that if the
// *  eigenvalues range over many orders of magnitude in size, then the
// *  small eigenvalues and corresponding eigenvectors will be computed
// *  more accurately than, for example, with the standard QR method.
// *
// *  The eigenvectors of a full or band symmetric positive definite matrix
// *  can also be found if DSYTRD, DSPTRD, or DSBTRD has been used to
// *  reduce this matrix to tridiagonal form. (The reduction to tridiagonal
// *  form, however, may preclude the possibility of obtaining high
// *  relative accuracy in the small eigenvalues of the original matrix, if
// *  these eigenvalues range over many orders of magnitude.)
// *
// *  Arguments
// *  =========
// *
// *  COMPZ   (input) CHARACTER*1
// *          = 'N':  Compute eigenvalues only.
// *          = 'V':  Compute eigenvectors of original symmetric
// *                  matrix also.  Array Z contains the orthogonal
// *                  matrix used to reduce the original matrix to
// *                  tridiagonal form.
// *          = 'I':  Compute eigenvectors of tridiagonal matrix also.
// *
// *  N       (input) INTEGER
// *          The order of the matrix.  N >= 0.
// *
// *  D       (input/output) DOUBLE PRECISION array, dimension (N)
// *          On entry, the n diagonal elements of the tridiagonal
// *          matrix.
// *          On normal exit, D contains the eigenvalues, in descending
// *          order.
// *
// *  E       (input/output) DOUBLE PRECISION array, dimension (N-1)
// *          On entry, the (n-1) subdiagonal elements of the tridiagonal
// *          matrix.
// *          On exit, E has been destroyed.
// *
// *  Z       (input/output) DOUBLE PRECISION array, dimension (LDZ, N)
// *          On entry, if COMPZ = 'V', the orthogonal matrix used in the
// *          reduction to tridiagonal form.
// *          On exit, if COMPZ = 'V', the orthonormal eigenvectors of the
// *          original symmetric matrix;
// *          if COMPZ = 'I', the orthonormal eigenvectors of the
// *          tridiagonal matrix.
// *          If INFO > 0 on exit, Z contains the eigenvectors associated
// *          with only the stored eigenvalues.
// *          If  COMPZ = 'N', then Z is not referenced.
// *
// *  LDZ     (input) INTEGER
// *          The leading dimension of the array Z.  LDZ >= 1, and if
// *          COMPZ = 'V' or 'I', LDZ >= max(1,N).
// *
// *  WORK    (workspace) DOUBLE PRECISION array, dimension (LWORK)
// *          If  COMPZ = 'N', then LWORK = 2*N
// *          If  COMPZ = 'V' or 'I', then LWORK = MAX(1,4*N-4)
// *
// *  INFO    (output) INTEGER
// *          = 0:  successful exit.
// *          < 0:  if INFO = -i, the i-th argument had an illegal value.
// *          > 0:  if INFO = i, and i is:
// *                <= N  the Cholesky factorization of the matrix could
// *                      not be performed because the i-th principal minor
// *                      was not positive definite.
// *                > N   the SVD algorithm failed to converge;
// *                      if INFO = N+i, i off-diagonal elements of the
// *                      bidiagonal factor did not converge to zero.
// *
// *  =====================================================================
// *
// *     .. Parameters ..
static double zero= 0.0e0;
static double one= 1.0e0;
// *     ..
// *     .. External Functions ..
// *     ..
// *     .. External Subroutines ..
// *     ..
// *     .. Local Arrays ..
static double [] c= new double[(1) * (1)];
static double [] vt= new double[(1) * (1)];
// *     ..
// *     .. Local Scalars ..
static int i= 0;
static int icompz= 0;
static int nru= 0;
// *     ..
// *     .. Intrinsic Functions ..
// *     ..
// *     .. Executable Statements ..
// *
// *     Test the input parameters.
// *

public static void dpteqr (String compz,
int n,
double [] d, int _d_offset,
double [] e, int _e_offset,
double [] z, int _z_offset,
int ldz,
double [] work, int _work_offset,
intW info)  {

info.val = 0;
// *
if ((compz.toLowerCase().charAt(0) == "N".toLowerCase().charAt(0)))  {
    icompz = 0;
}              // Close if()
else if ((compz.toLowerCase().charAt(0) == "V".toLowerCase().charAt(0)))  {
    icompz = 1;
}              // Close else if()
else if ((compz.toLowerCase().charAt(0) == "I".toLowerCase().charAt(0)))  {
    icompz = 2;
}              // Close else if()
else  {
  icompz = -1;
}              //  Close else.
if (icompz < 0)  {
    info.val = -1;
}              // Close if()
else if (n < 0)  {
    info.val = -2;
}              // Close else if()
else if ((ldz < 1) || (icompz > 0 && ldz < Math.max(1, n) ))  {
    info.val = -6;
}              // Close else if()
if (info.val != 0)  {
    Xerbla.xerbla("DPTEQR",-info.val);
Dummy.go_to("Dpteqr",999999);
}              // Close if()
// *
// *     Quick return if possible
// *
if (n == 0)  
    Dummy.go_to("Dpteqr",999999);
// *
if (n == 1)  {
    if (icompz > 0)  
    z[(1)- 1+(1- 1)*ldz+ _z_offset] = one;
Dummy.go_to("Dpteqr",999999);
}              // Close if()
if (icompz == 2)  
    Dlaset.dlaset("Full",n,n,zero,one,z,_z_offset,ldz);
// *
// *     Call DPTTRF to factor the matrix.
// *
Dpttrf.dpttrf(n,d,_d_offset,e,_e_offset,info);
if (info.val != 0)  
    Dummy.go_to("Dpteqr",999999);
{
forloop10:
for (i = 1; i <= n; i++) {
d[(i)- 1+ _d_offset] = Math.sqrt(d[(i)- 1+ _d_offset]);
Dummy.label("Dpteqr",10);
}              //  Close for() loop. 
}
{
forloop20:
for (i = 1; i <= n-1; i++) {
e[(i)- 1+ _e_offset] = e[(i)- 1+ _e_offset]*d[(i)- 1+ _d_offset];
Dummy.label("Dpteqr",20);
}              //  Close for() loop. 
}
// *
// *     Call DBDSQR to compute the singular values/vectors of the
// *     bidiagonal factor.
// *
if (icompz > 0)  {
    nru = n;
}              // Close if()
else  {
  nru = 0;
}              //  Close else.
Dbdsqr.dbdsqr("Lower",n,0,nru,0,d,_d_offset,e,_e_offset,vt,0,1,z,_z_offset,ldz,c,0,1,work,_work_offset,info);
// *
// *     Square the singular values.
// *
if (info.val == 0)  {
    {
forloop30:
for (i = 1; i <= n; i++) {
d[(i)- 1+ _d_offset] = d[(i)- 1+ _d_offset]*d[(i)- 1+ _d_offset];
Dummy.label("Dpteqr",30);
}              //  Close for() loop. 
}
}              // Close if()
else  {
  info.val = n+info.val;
}              //  Close else.
// *
Dummy.go_to("Dpteqr",999999);
// *
// *     End of DPTEQR
// *
Dummy.label("Dpteqr",999999);
return;
   }
} // End class.
