/*
 *  Produced by f2java.  f2java is part of the Fortran-
 *  -to-Java project at the University of Tennessee Netlib
 *  numerical software repository.
 *
 *  Original authorship for the BLAS and LAPACK numerical
 *  routines may be found in the Fortran source, available at
 *  www.netlib.org.
 *
 *  Fortran input file: jlapack.f
 *
 *  The f2j compiler code was written by
 *  David M. Doolin (doolin@cs.utk.edu) and
 *  Keith  Seymour (seymour@cs.utk.edu)
 */

package org.netlib.lapack;
import java.lang.*;
import org.netlib.util.*;

import org.netlib.blas.*;


public class Dptrfs {

// *
// *  -- LAPACK routine (version 2.0) --
// *     Univ. of Tennessee, Univ. of California Berkeley, NAG Ltd.,
// *     Courant Institute, Argonne National Lab, and Rice University
// *     September 30, 1994
// *
// *     .. Scalar Arguments ..
// *     ..
// *     .. Array Arguments ..
// *     ..
// *
// *  Purpose
// *  =======
// *
// *  DPTRFS improves the computed solution to a system of linear
// *  equations when the coefficient matrix is symmetric positive definite
// *  and tridiagonal, and provides error bounds and backward error
// *  estimates for the solution.
// *
// *  Arguments
// *  =========
// *
// *  N       (input) INTEGER
// *          The order of the matrix A.  N >= 0.
// *
// *  NRHS    (input) INTEGER
// *          The number of right hand sides, i.e., the number of columns
// *          of the matrix B.  NRHS >= 0.
// *
// *  D       (input) DOUBLE PRECISION array, dimension (N)
// *          The n diagonal elements of the tridiagonal matrix A.
// *
// *  E       (input) DOUBLE PRECISION array, dimension (N-1)
// *          The (n-1) subdiagonal elements of the tridiagonal matrix A.
// *
// *  DF      (input) DOUBLE PRECISION array, dimension (N)
// *          The n diagonal elements of the diagonal matrix D from the
// *          factorization computed by DPTTRF.
// *
// *  EF      (input) DOUBLE PRECISION array, dimension (N-1)
// *          The (n-1) subdiagonal elements of the unit bidiagonal factor
// *          L from the factorization computed by DPTTRF.
// *
// *  B       (input) DOUBLE PRECISION array, dimension (LDB,NRHS)
// *          The right hand side matrix B.
// *
// *  LDB     (input) INTEGER
// *          The leading dimension of the array B.  LDB >= max(1,N).
// *
// *  X       (input/output) DOUBLE PRECISION array, dimension (LDX,NRHS)
// *          On entry, the solution matrix X, as computed by DPTTRS.
// *          On exit, the improved solution matrix X.
// *
// *  LDX     (input) INTEGER
// *          The leading dimension of the array X.  LDX >= max(1,N).
// *
// *  FERR    (output) DOUBLE PRECISION array, dimension (NRHS)
// *          The forward error bound for each solution vector
// *          X(j) (the j-th column of the solution matrix X).
// *          If XTRUE is the true solution corresponding to X(j), FERR(j)
// *          is an estimated upper bound for the magnitude of the largest
// *          element in (X(j) - XTRUE) divided by the magnitude of the
// *          largest element in X(j).
// *
// *  BERR    (output) DOUBLE PRECISION array, dimension (NRHS)
// *          The componentwise relative backward error of each solution
// *          vector X(j) (i.e., the smallest relative change in
// *          any element of A or B that makes X(j) an exact solution).
// *
// *  WORK    (workspace) DOUBLE PRECISION array, dimension (2*N)
// *
// *  INFO    (output) INTEGER
// *          = 0:  successful exit
// *          < 0:  if INFO = -i, the i-th argument had an illegal value
// *
// *  Internal Parameters
// *  ===================
// *
// *  ITMAX is the maximum number of steps of iterative refinement.
// *
// *  =====================================================================
// *
// *     .. Parameters ..
static int itmax= 5;
static double zero= 0.0e+0;
static double one= 1.0e+0;
static double two= 2.0e+0;
static double three= 3.0e+0;
// *     ..
// *     .. Local Scalars ..
static int count= 0;
static int i= 0;
static int ix= 0;
static int j= 0;
static int nz= 0;
static double bi= 0.0;
static double cx= 0.0;
static double dx= 0.0;
static double eps= 0.0;
static double ex= 0.0;
static double lstres= 0.0;
static double s= 0.0;
static double safe1= 0.0;
static double safe2= 0.0;
static double safmin= 0.0;
// *     ..
// *     .. External Subroutines ..
// *     ..
// *     .. Intrinsic Functions ..
// *     ..
// *     .. External Functions ..
// *     ..
// *     .. Executable Statements ..
// *
// *     Test the input parameters.
// *

public static void dptrfs (int n,
int nrhs,
double [] d, int _d_offset,
double [] e, int _e_offset,
double [] df, int _df_offset,
double [] ef, int _ef_offset,
double [] b, int _b_offset,
int ldb,
double [] x, int _x_offset,
int ldx,
double [] ferr, int _ferr_offset,
double [] berr, int _berr_offset,
double [] work, int _work_offset,
intW info)  {

info.val = 0;
if (n < 0)  {
    info.val = -1;
}              // Close if()
else if (nrhs < 0)  {
    info.val = -2;
}              // Close else if()
else if (ldb < Math.max(1, n) )  {
    info.val = -8;
}              // Close else if()
else if (ldx < Math.max(1, n) )  {
    info.val = -10;
}              // Close else if()
if (info.val != 0)  {
    Xerbla.xerbla("DPTRFS",-info.val);
Dummy.go_to("Dptrfs",999999);
}              // Close if()
// *
// *     Quick return if possible
// *
if (n == 0 || nrhs == 0)  {
    {
forloop10:
for (j = 1; j <= nrhs; j++) {
ferr[(j)- 1+ _ferr_offset] = zero;
berr[(j)- 1+ _berr_offset] = zero;
Dummy.label("Dptrfs",10);
}              //  Close for() loop. 
}
Dummy.go_to("Dptrfs",999999);
}              // Close if()
// *
// *     NZ = maximum number of nonzero elements in each row of A, plus 1
// *
nz = 4;
eps = Dlamch.dlamch("Epsilon");
safmin = Dlamch.dlamch("Safe minimum");
safe1 = nz*safmin;
safe2 = safe1/eps;
// *
// *     Do for each right hand side
// *
{
forloop90:
for (j = 1; j <= nrhs; j++) {
// *
count = 1;
lstres = three;
label20:
   Dummy.label("Dptrfs",20);
// *
// *        Loop until stopping criterion is satisfied.
// *
// *        Compute residual R = B - A * X.  Also compute
// *        abs(A)*abs(x) + abs(b) for use in the backward error bound.
// *
if (n == 1)  {
    bi = b[(1)- 1+(j- 1)*ldb+ _b_offset];
dx = d[(1)- 1+ _d_offset]*x[(1)- 1+(j- 1)*ldx+ _x_offset];
work[(n+1)- 1+ _work_offset] = bi-dx;
work[(1)- 1+ _work_offset] = Math.abs(bi)+Math.abs(dx);
}              // Close if()
else  {
  bi = b[(1)- 1+(j- 1)*ldb+ _b_offset];
dx = d[(1)- 1+ _d_offset]*x[(1)- 1+(j- 1)*ldx+ _x_offset];
ex = e[(1)- 1+ _e_offset]*x[(2)- 1+(j- 1)*ldx+ _x_offset];
work[(n+1)- 1+ _work_offset] = bi-dx-ex;
work[(1)- 1+ _work_offset] = Math.abs(bi)+Math.abs(dx)+Math.abs(ex);
{
forloop30:
for (i = 2; i <= n-1; i++) {
bi = b[(i)- 1+(j- 1)*ldb+ _b_offset];
cx = e[(i-1)- 1+ _e_offset]*x[(i-1)- 1+(j- 1)*ldx+ _x_offset];
dx = d[(i)- 1+ _d_offset]*x[(i)- 1+(j- 1)*ldx+ _x_offset];
ex = e[(i)- 1+ _e_offset]*x[(i+1)- 1+(j- 1)*ldx+ _x_offset];
work[(n+i)- 1+ _work_offset] = bi-cx-dx-ex;
work[(i)- 1+ _work_offset] = Math.abs(bi)+Math.abs(cx)+Math.abs(dx)+Math.abs(ex);
Dummy.label("Dptrfs",30);
}              //  Close for() loop. 
}
bi = b[(n)- 1+(j- 1)*ldb+ _b_offset];
cx = e[(n-1)- 1+ _e_offset]*x[(n-1)- 1+(j- 1)*ldx+ _x_offset];
dx = d[(n)- 1+ _d_offset]*x[(n)- 1+(j- 1)*ldx+ _x_offset];
work[(n+n)- 1+ _work_offset] = bi-cx-dx;
work[(n)- 1+ _work_offset] = Math.abs(bi)+Math.abs(cx)+Math.abs(dx);
}              //  Close else.
// *
// *        Compute componentwise relative backward error from formula
// *
// *        max(i) ( abs(R(i)) / ( abs(A)*abs(X) + abs(B) )(i) )
// *
// *        where abs(Z) is the componentwise absolute value of the matrix
// *        or vector Z.  If the i-th component of the denominator is less
// *        than SAFE2, then SAFE1 is added to the i-th components of the
// *        numerator and denominator before dividing.
// *
s = zero;
{
forloop40:
for (i = 1; i <= n; i++) {
if (work[(i)- 1+ _work_offset] > safe2)  {
    s = Math.max(s, Math.abs(work[(n+i)- 1+ _work_offset])/work[(i)- 1+ _work_offset]) ;
}              // Close if()
else  {
  s = Math.max(s, (Math.abs(work[(n+i)- 1+ _work_offset])+safe1)/(work[(i)- 1+ _work_offset]+safe1)) ;
}              //  Close else.
Dummy.label("Dptrfs",40);
}              //  Close for() loop. 
}
berr[(j)- 1+ _berr_offset] = s;
// *
// *        Test stopping criterion. Continue iterating if
// *           1) The residual BERR(J) is larger than machine epsilon, and
// *           2) BERR(J) decreased by at least a factor of 2 during the
// *              last iteration, and
// *           3) At most ITMAX iterations tried.
// *
if (berr[(j)- 1+ _berr_offset] > eps && two*berr[(j)- 1+ _berr_offset] <= lstres && count <= itmax)  {
    // *
// *           Update solution and try again.
// *
Dpttrs.dpttrs(n,1,df,_df_offset,ef,_ef_offset,work,(n+1)- 1+ _work_offset,n,info);
Daxpy.daxpy(n,one,work,(n+1)- 1+ _work_offset,1,x,(1)- 1+(j- 1)*ldx+ _x_offset,1);
lstres = berr[(j)- 1+ _berr_offset];
count = count+1;
Dummy.go_to("Dptrfs",20);
}              // Close if()
// *
// *        Bound error from formula
// *
// *        norm(X - XTRUE) / norm(X) .le. FERR =
// *        norm( abs(inv(A))*
// *           ( abs(R) + NZ*EPS*( abs(A)*abs(X)+abs(B) ))) / norm(X)
// *
// *        where
// *          norm(Z) is the magnitude of the largest component of Z
// *          inv(A) is the inverse of A
// *          abs(Z) is the componentwise absolute value of the matrix or
// *             vector Z
// *          NZ is the maximum number of nonzeros in any row of A, plus 1
// *          EPS is machine epsilon
// *
// *        The i-th component of abs(R)+NZ*EPS*(abs(A)*abs(X)+abs(B))
// *        is incremented by SAFE1 if the i-th component of
// *        abs(A)*abs(X) + abs(B) is less than SAFE2.
// *
{
forloop50:
for (i = 1; i <= n; i++) {
if (work[(i)- 1+ _work_offset] > safe2)  {
    work[(i)- 1+ _work_offset] = Math.abs(work[(n+i)- 1+ _work_offset])+nz*eps*work[(i)- 1+ _work_offset];
}              // Close if()
else  {
  work[(i)- 1+ _work_offset] = Math.abs(work[(n+i)- 1+ _work_offset])+nz*eps*work[(i)- 1+ _work_offset]+safe1;
}              //  Close else.
Dummy.label("Dptrfs",50);
}              //  Close for() loop. 
}
ix = Idamax.idamax(n,work,_work_offset,1);
ferr[(j)- 1+ _ferr_offset] = work[(ix)- 1+ _work_offset];
// *
// *        Estimate the norm of inv(A).
// *
// *        Solve M(A) * x = e, where M(A) = (m(i,j)) is given by
// *
// *           m(i,j) =  abs(A(i,j)), i = j,
// *           m(i,j) = -abs(A(i,j)), i .ne. j,
// *
// *        and e = [ 1, 1, ..., 1 ]'.  Note M(A) = M(L)*D*M(L)'.
// *
// *        Solve M(L) * x = e.
// *
work[(1)- 1+ _work_offset] = one;
{
forloop60:
for (i = 2; i <= n; i++) {
work[(i)- 1+ _work_offset] = one+work[(i-1)- 1+ _work_offset]*Math.abs(ef[(i-1)- 1+ _ef_offset]);
Dummy.label("Dptrfs",60);
}              //  Close for() loop. 
}
// *
// *        Solve D * M(L)' * x = b.
// *
work[(n)- 1+ _work_offset] = work[(n)- 1+ _work_offset]/df[(n)- 1+ _df_offset];
{
int _i_inc = -1;
forloop70:
for (i = n-1; i >= 1; i += _i_inc) {
work[(i)- 1+ _work_offset] = work[(i)- 1+ _work_offset]/df[(i)- 1+ _df_offset]+work[(i+1)- 1+ _work_offset]*Math.abs(ef[(i)- 1+ _ef_offset]);
Dummy.label("Dptrfs",70);
}              //  Close for() loop. 
}
// *
// *        Compute norm(inv(A)) = max(x(i)), 1<=i<=n.
// *
ix = Idamax.idamax(n,work,_work_offset,1);
ferr[(j)- 1+ _ferr_offset] = ferr[(j)- 1+ _ferr_offset]*Math.abs(work[(ix)- 1+ _work_offset]);
// *
// *        Normalize error.
// *
lstres = zero;
{
forloop80:
for (i = 1; i <= n; i++) {
lstres = Math.max(lstres, Math.abs(x[(i)- 1+(j- 1)*ldx+ _x_offset])) ;
Dummy.label("Dptrfs",80);
}              //  Close for() loop. 
}
if (lstres != zero)  
    ferr[(j)- 1+ _ferr_offset] = ferr[(j)- 1+ _ferr_offset]/lstres;
// *
Dummy.label("Dptrfs",90);
}              //  Close for() loop. 
}
// *
Dummy.go_to("Dptrfs",999999);
// *
// *     End of DPTRFS
// *
Dummy.label("Dptrfs",999999);
return;
   }
} // End class.
