/*
 *  Produced by f2java.  f2java is part of the Fortran-
 *  -to-Java project at the University of Tennessee Netlib
 *  numerical software repository.
 *
 *  Original authorship for the BLAS and LAPACK numerical
 *  routines may be found in the Fortran source, available at
 *  www.netlib.org.
 *
 *  Fortran input file: jlapack.f
 *
 *  The f2j compiler code was written by
 *  David M. Doolin (doolin@cs.utk.edu) and
 *  Keith  Seymour (seymour@cs.utk.edu)
 */

package org.netlib.lapack;
import java.lang.*;
import org.netlib.util.*;



public class Dptsv {

// *
// *  -- LAPACK routine (version 2.0) --
// *     Univ. of Tennessee, Univ. of California Berkeley, NAG Ltd.,
// *     Courant Institute, Argonne National Lab, and Rice University
// *     March 31, 1993
// *
// *     .. Scalar Arguments ..
// *     ..
// *     .. Array Arguments ..
// *     ..
// *
// *  Purpose
// *  =======
// *
// *  DPTSV computes the solution to a real system of linear equations
// *  A*X = B, where A is an N-by-N symmetric positive definite tridiagonal
// *  matrix, and X and B are N-by-NRHS matrices.
// *
// *  A is factored as A = L*D*L**T, and the factored form of A is then
// *  used to solve the system of equations.
// *
// *  Arguments
// *  =========
// *
// *  N       (input) INTEGER
// *          The order of the matrix A.  N >= 0.
// *
// *  NRHS    (input) INTEGER
// *          The number of right hand sides, i.e., the number of columns
// *          of the matrix B.  NRHS >= 0.
// *
// *  D       (input/output) DOUBLE PRECISION array, dimension (N)
// *          On entry, the n diagonal elements of the tridiagonal matrix
// *          A.  On exit, the n diagonal elements of the diagonal matrix
// *          D from the factorization A = L*D*L**T.
// *
// *  E       (input/output) DOUBLE PRECISION array, dimension (N-1)
// *          On entry, the (n-1) subdiagonal elements of the tridiagonal
// *          matrix A.  On exit, the (n-1) subdiagonal elements of the
// *          unit bidiagonal factor L from the L*D*L**T factorization of
// *          A.  (E can also be regarded as the superdiagonal of the unit
// *          bidiagonal factor U from the U**T*D*U factorization of A.)
// *
// *  B       (input/output) DOUBLE PRECISION array, dimension (LDB,N)
// *          On entry, the N-by-NRHS right hand side matrix B.
// *          On exit, if INFO = 0, the N-by-NRHS solution matrix X.
// *
// *  LDB     (input) INTEGER
// *          The leading dimension of the array B.  LDB >= max(1,N).
// *
// *  INFO    (output) INTEGER
// *          = 0:  successful exit
// *          < 0:  if INFO = -i, the i-th argument had an illegal value
// *          > 0:  if INFO = i, the leading minor of order i is not
// *                positive definite, and the solution has not been
// *                computed.  The factorization has not been completed
// *                unless i = N.
// *
// *  =====================================================================
// *
// *     .. External Subroutines ..
// *     ..
// *     .. Intrinsic Functions ..
// *     ..
// *     .. Executable Statements ..
// *
// *     Test the input parameters.
// *

public static void dptsv (int n,
int nrhs,
double [] d, int _d_offset,
double [] e, int _e_offset,
double [] b, int _b_offset,
int ldb,
intW info)  {

info.val = 0;
if (n < 0)  {
    info.val = -1;
}              // Close if()
else if (nrhs < 0)  {
    info.val = -2;
}              // Close else if()
else if (ldb < Math.max(1, n) )  {
    info.val = -6;
}              // Close else if()
if (info.val != 0)  {
    Xerbla.xerbla("DPTSV ",-info.val);
Dummy.go_to("Dptsv",999999);
}              // Close if()
// *
// *     Compute the L*D*L' (or U'*D*U) factorization of A.
// *
Dpttrf.dpttrf(n,d,_d_offset,e,_e_offset,info);
if (info.val == 0)  {
    // *
// *        Solve the system A*X = B, overwriting B with X.
// *
Dpttrs.dpttrs(n,nrhs,d,_d_offset,e,_e_offset,b,_b_offset,ldb,info);
}              // Close if()
Dummy.go_to("Dptsv",999999);
// *
// *     End of DPTSV
// *
Dummy.label("Dptsv",999999);
return;
   }
} // End class.
