/*
 *  Produced by f2java.  f2java is part of the Fortran-
 *  -to-Java project at the University of Tennessee Netlib
 *  numerical software repository.
 *
 *  Original authorship for the BLAS and LAPACK numerical
 *  routines may be found in the Fortran source, available at
 *  www.netlib.org.
 *
 *  Fortran input file: jlapack.f
 *
 *  The f2j compiler code was written by
 *  David M. Doolin (doolin@cs.utk.edu) and
 *  Keith  Seymour (seymour@cs.utk.edu)
 */

package org.netlib.lapack;
import java.lang.*;
import org.netlib.util.*;

import org.netlib.blas.*;


public class Dsbev {

// *
// *  -- LAPACK driver routine (version 2.0) --
// *     Univ. of Tennessee, Univ. of California Berkeley, NAG Ltd.,
// *     Courant Institute, Argonne National Lab, and Rice University
// *     September 30, 1994
// *
// *     .. Scalar Arguments ..
// *     ..
// *     .. Array Arguments ..
// *     ..
// *
// *  Purpose
// *  =======
// *
// *  DSBEV computes all the eigenvalues and, optionally, eigenvectors of
// *  a real symmetric band matrix A.
// *
// *  Arguments
// *  =========
// *
// *  JOBZ    (input) CHARACTER*1
// *          = 'N':  Compute eigenvalues only;
// *          = 'V':  Compute eigenvalues and eigenvectors.
// *
// *  UPLO    (input) CHARACTER*1
// *          = 'U':  Upper triangle of A is stored;
// *          = 'L':  Lower triangle of A is stored.
// *
// *  N       (input) INTEGER
// *          The order of the matrix A.  N >= 0.
// *
// *  KD      (input) INTEGER
// *          The number of superdiagonals of the matrix A if UPLO = 'U',
// *          or the number of subdiagonals if UPLO = 'L'.  KD >= 0.
// *
// *  AB      (input/output) DOUBLE PRECISION array, dimension (LDAB, N)
// *          On entry, the upper or lower triangle of the symmetric band
// *          matrix A, stored in the first KD+1 rows of the array.  The
// *          j-th column of A is stored in the j-th column of the array AB
// *          as follows:
// *          if UPLO = 'U', AB(kd+1+i-j,j) = A(i,j) for max(1,j-kd)<=i<=j;
// *          if UPLO = 'L', AB(1+i-j,j)    = A(i,j) for j<=i<=min(n,j+kd).
// *
// *          On exit, AB is overwritten by values generated during the
// *          reduction to tridiagonal form.  If UPLO = 'U', the first
// *          superdiagonal and the diagonal of the tridiagonal matrix T
// *          are returned in rows KD and KD+1 of AB, and if UPLO = 'L',
// *          the diagonal and first subdiagonal of T are returned in the
// *          first two rows of AB.
// *
// *  LDAB    (input) INTEGER
// *          The leading dimension of the array AB.  LDAB >= KD + 1.
// *
// *  W       (output) DOUBLE PRECISION array, dimension (N)
// *          If INFO = 0, the eigenvalues in ascending order.
// *
// *  Z       (output) DOUBLE PRECISION array, dimension (LDZ, N)
// *          If JOBZ = 'V', then if INFO = 0, Z contains the orthonormal
// *          eigenvectors of the matrix A, with the i-th column of Z
// *          holding the eigenvector associated with W(i).
// *          If JOBZ = 'N', then Z is not referenced.
// *
// *  LDZ     (input) INTEGER
// *          The leading dimension of the array Z.  LDZ >= 1, and if
// *          JOBZ = 'V', LDZ >= max(1,N).
// *
// *  WORK    (workspace) DOUBLE PRECISION array, dimension (max(1,3*N-2))
// *
// *  INFO    (output) INTEGER
// *          = 0:  successful exit
// *          < 0:  if INFO = -i, the i-th argument had an illegal value
// *          > 0:  if INFO = i, the algorithm failed to converge; i
// *                off-diagonal elements of an intermediate tridiagonal
// *                form did not converge to zero.
// *
// *  =====================================================================
// *
// *     .. Parameters ..
static double zero= 0.0e0;
static double one= 1.0e0;
// *     ..
// *     .. Local Scalars ..
static boolean lower= false;
static boolean wantz= false;
static intW iinfo= new intW(0);
static int imax= 0;
static int inde= 0;
static int indwrk= 0;
static int iscale= 0;
static double anrm= 0.0;
static double bignum= 0.0;
static double eps= 0.0;
static double rmax= 0.0;
static double rmin= 0.0;
static double safmin= 0.0;
static double sigma= 0.0;
static double smlnum= 0.0;
// *     ..
// *     .. External Functions ..
// *     ..
// *     .. External Subroutines ..
// *     ..
// *     .. Intrinsic Functions ..
// *     ..
// *     .. Executable Statements ..
// *
// *     Test the input parameters.
// *

public static void dsbev (String jobz,
String uplo,
int n,
int kd,
double [] ab, int _ab_offset,
int ldab,
double [] w, int _w_offset,
double [] z, int _z_offset,
int ldz,
double [] work, int _work_offset,
intW info)  {

wantz = (jobz.toLowerCase().charAt(0) == "V".toLowerCase().charAt(0));
lower = (uplo.toLowerCase().charAt(0) == "L".toLowerCase().charAt(0));
// *
info.val = 0;
if (!(wantz || (jobz.toLowerCase().charAt(0) == "N".toLowerCase().charAt(0))))  {
    info.val = -1;
}              // Close if()
else if (!(lower || (uplo.toLowerCase().charAt(0) == "U".toLowerCase().charAt(0))))  {
    info.val = -2;
}              // Close else if()
else if (n < 0)  {
    info.val = -3;
}              // Close else if()
else if (kd < 0)  {
    info.val = -4;
}              // Close else if()
else if (ldab < kd+1)  {
    info.val = -6;
}              // Close else if()
else if (ldz < 1 || (wantz && ldz < n))  {
    info.val = -9;
}              // Close else if()
// *
if (info.val != 0)  {
    Xerbla.xerbla("DSBEV ",-info.val);
Dummy.go_to("Dsbev",999999);
}              // Close if()
// *
// *     Quick return if possible
// *
if (n == 0)  
    Dummy.go_to("Dsbev",999999);
// *
if (n == 1)  {
    w[(1)- 1+ _w_offset] = ab[(1)- 1+(1- 1)*ldab+ _ab_offset];
if (wantz)  
    z[(1)- 1+(1- 1)*ldz+ _z_offset] = one;
Dummy.go_to("Dsbev",999999);
}              // Close if()
// *
// *     Get machine constants.
// *
safmin = Dlamch.dlamch("Safe minimum");
eps = Dlamch.dlamch("Precision");
smlnum = safmin/eps;
bignum = one/smlnum;
rmin = Math.sqrt(smlnum);
rmax = Math.sqrt(bignum);
// *
// *     Scale matrix to allowable range, if necessary.
// *
anrm = Dlansb.dlansb("M",uplo,n,kd,ab,_ab_offset,ldab,work,_work_offset);
iscale = 0;
if (anrm > zero && anrm < rmin)  {
    iscale = 1;
sigma = rmin/anrm;
}              // Close if()
else if (anrm > rmax)  {
    iscale = 1;
sigma = rmax/anrm;
}              // Close else if()
if (iscale == 1)  {
    if (lower)  {
    Dlascl.dlascl("B",kd,kd,one,sigma,n,n,ab,_ab_offset,ldab,info);
}              // Close if()
else  {
  Dlascl.dlascl("Q",kd,kd,one,sigma,n,n,ab,_ab_offset,ldab,info);
}              //  Close else.
}              // Close if()
// *
// *     Call DSBTRD to reduce symmetric band matrix to tridiagonal form.
// *
inde = 1;
indwrk = inde+n;
Dsbtrd.dsbtrd(jobz,uplo,n,kd,ab,_ab_offset,ldab,w,_w_offset,work,(inde)- 1+ _work_offset,z,_z_offset,ldz,work,(indwrk)- 1+ _work_offset,iinfo);
// *
// *     For eigenvalues only, call DSTERF.  For eigenvectors, call SSTEQR.
// *
if (!wantz)  {
    Dsterf.dsterf(n,w,_w_offset,work,(inde)- 1+ _work_offset,info);
}              // Close if()
else  {
  Dsteqr.dsteqr(jobz,n,w,_w_offset,work,(inde)- 1+ _work_offset,z,_z_offset,ldz,work,(indwrk)- 1+ _work_offset,info);
}              //  Close else.
// *
// *     If matrix was scaled, then rescale eigenvalues appropriately.
// *
if (iscale == 1)  {
    if (info.val == 0)  {
    imax = n;
}              // Close if()
else  {
  imax = info.val-1;
}              //  Close else.
Dscal.dscal(imax,one/sigma,w,_w_offset,1);
}              // Close if()
// *
Dummy.go_to("Dsbev",999999);
// *
// *     End of DSBEV
// *
Dummy.label("Dsbev",999999);
return;
   }
} // End class.
