/*
 *  Produced by f2java.  f2java is part of the Fortran-
 *  -to-Java project at the University of Tennessee Netlib
 *  numerical software repository.
 *
 *  Original authorship for the BLAS and LAPACK numerical
 *  routines may be found in the Fortran source, available at
 *  www.netlib.org.
 *
 *  Fortran input file: jlapack.f
 *
 *  The f2j compiler code was written by
 *  David M. Doolin (doolin@cs.utk.edu) and
 *  Keith  Seymour (seymour@cs.utk.edu)
 */

package org.netlib.lapack;
import java.lang.*;
import org.netlib.util.*;

import org.netlib.blas.*;


public class Dsbgst {

// *
// *  -- LAPACK routine (version 2.0) --
// *     Univ. of Tennessee, Univ. of California Berkeley, NAG Ltd.,
// *     Courant Institute, Argonne National Lab, and Rice University
// *     September 30, 1994
// *
// *     .. Scalar Arguments ..
// *     ..
// *     .. Array Arguments ..
// *     ..
// *
// *  Purpose
// *  =======
// *
// *  DSBGST reduces a real symmetric-definite banded generalized
// *  eigenproblem  A*x = lambda*B*x  to standard form  C*y = lambda*y,
// *  such that C has the same bandwidth as A.
// *
// *  B must have been previously factorized as S**T*S by DPBSTF, using a
// *  split Cholesky factorization. A is overwritten by C = X**T*A*X, where
// *  X = S**(-1)*Q and Q is an orthogonal matrix chosen to preserve the
// *  bandwidth of A.
// *
// *  Arguments
// *  =========
// *
// *  VECT    (input) CHARACTER*1
// *          = 'N':  do not form the transformation matrix X;
// *          = 'V':  form X.
// *
// *  UPLO    (input) CHARACTER*1
// *          = 'U':  Upper triangle of A is stored;
// *          = 'L':  Lower triangle of A is stored.
// *
// *  N       (input) INTEGER
// *          The order of the matrices A and B.  N >= 0.
// *
// *  KA      (input) INTEGER
// *          The number of superdiagonals of the matrix A if UPLO = 'U',
// *          or the number of subdiagonals if UPLO = 'L'.  KA >= 0.
// *
// *  KB      (input) INTEGER
// *          The number of superdiagonals of the matrix B if UPLO = 'U',
// *          or the number of subdiagonals if UPLO = 'L'.  KA >= KB >= 0.
// *
// *  AB      (input/output) DOUBLE PRECISION array, dimension (LDAB,N)
// *          On entry, the upper or lower triangle of the symmetric band
// *          matrix A, stored in the first ka+1 rows of the array.  The
// *          j-th column of A is stored in the j-th column of the array AB
// *          as follows:
// *          if UPLO = 'U', AB(ka+1+i-j,j) = A(i,j) for max(1,j-ka)<=i<=j;
// *          if UPLO = 'L', AB(1+i-j,j)    = A(i,j) for j<=i<=min(n,j+ka).
// *
// *          On exit, the transformed matrix X**T*A*X, stored in the same
// *          format as A.
// *
// *  LDAB    (input) INTEGER
// *          The leading dimension of the array AB.  LDAB >= KA+1.
// *
// *  BB      (input) DOUBLE PRECISION array, dimension (LDBB,N)
// *          The banded factor S from the split Cholesky factorization of
// *          B, as returned by DPBSTF, stored in the first KB+1 rows of
// *          the array.
// *
// *  LDBB    (input) INTEGER
// *          The leading dimension of the array BB.  LDBB >= KB+1.
// *
// *  X       (output) DOUBLE PRECISION array, dimension (LDX,N)
// *          If VECT = 'V', the n-by-n matrix X.
// *          If VECT = 'N', the array X is not referenced.
// *
// *  LDX     (input) INTEGER
// *          The leading dimension of the array X.
// *          LDX >= max(1,N) if VECT = 'V'; LDX >= 1 otherwise.
// *
// *  WORK    (workspace) DOUBLE PRECISION array, dimension (2*N)
// *
// *  INFO    (output) INTEGER
// *          = 0:  successful exit
// *          < 0:  if INFO = -i, the i-th argument had an illegal value.
// *
// *  =====================================================================
// *
// *     .. Parameters ..
static double zero= 0.0e+0;
static double one= 1.0e+0;
// *     ..
// *     .. Local Scalars ..
static boolean update= false;
static boolean upper= false;
static boolean wantx= false;
static int i= 0;
static int i0= 0;
static int i1= 0;
static int i2= 0;
static int inca= 0;
static int j= 0;
static int j1= 0;
static int j1t= 0;
static int j2= 0;
static int j2t= 0;
static int k= 0;
static int ka1= 0;
static int kb1= 0;
static int kbt= 0;
static int l= 0;
static int m= 0;
static int nr= 0;
static int nrt= 0;
static int nx= 0;
static double bii= 0.0;
static doubleW ra= new doubleW(0.0);
static double ra1= 0.0;
static double t= 0.0;
// *     ..
// *     .. External Functions ..
// *     ..
// *     .. External Subroutines ..
// *     ..
// *     .. Intrinsic Functions ..
// *     ..
// *     .. Executable Statements ..
// *
// *     Test the input parameters
// *

public static void dsbgst (String vect,
String uplo,
int n,
int ka,
int kb,
double [] ab, int _ab_offset,
int ldab,
double [] bb, int _bb_offset,
int ldbb,
double [] x, int _x_offset,
int ldx,
double [] work, int _work_offset,
intW info)  {

wantx = (vect.toLowerCase().charAt(0) == "V".toLowerCase().charAt(0));
upper = (uplo.toLowerCase().charAt(0) == "U".toLowerCase().charAt(0));
ka1 = ka+1;
kb1 = kb+1;
info.val = 0;
if (!wantx && !(vect.toLowerCase().charAt(0) == "N".toLowerCase().charAt(0)))  {
    info.val = -1;
}              // Close if()
else if (!upper && !(uplo.toLowerCase().charAt(0) == "L".toLowerCase().charAt(0)))  {
    info.val = -2;
}              // Close else if()
else if (n < 0)  {
    info.val = -3;
}              // Close else if()
else if (ka < 0)  {
    info.val = -4;
}              // Close else if()
else if (kb < 0)  {
    info.val = -5;
}              // Close else if()
else if (ldab < ka+1)  {
    info.val = -7;
}              // Close else if()
else if (ldbb < kb+1)  {
    info.val = -9;
}              // Close else if()
else if (ldx < 1 || wantx && ldx < Math.max(1, n) )  {
    info.val = -11;
}              // Close else if()
if (info.val != 0)  {
    Xerbla.xerbla("DSBGST",-info.val);
Dummy.go_to("Dsbgst",999999);
}              // Close if()
// *
// *     Quick return if possible
// *
if (n == 0)  
    Dummy.go_to("Dsbgst",999999);
// *
inca = ldab*ka1;
// *
// *     Initialize X to the unit matrix, if needed
// *
if (wantx)  
    Dlaset.dlaset("Full",n,n,zero,one,x,_x_offset,ldx);
// *
// *     Set M to the splitting point m. It must be the same value as is
// *     used in DPBSTF. The chosen value allows the arrays WORK and RWORK
// *     to be of dimension (N).
// *
m = (n+kb)/2;
// *
// *     The routine works in two phases, corresponding to the two halves
// *     of the split Cholesky factorization of B as S**T*S where
// *
// *     S = ( U    )
// *         ( M  L )
// *
// *     with U upper triangular of order m, and L lower triangular of
// *     order n-m. S has the same bandwidth as B.
// *
// *     S is treated as a product of elementary matrices:
// *
// *     S = S(m)*S(m-1)*...*S(2)*S(1)*S(m+1)*S(m+2)*...*S(n-1)*S(n)
// *
// *     where S(i) is determined by the i-th row of S.
// *
// *     In phase 1, the index i takes the values n, n-1, ... , m+1;
// *     in phase 2, it takes the values 1, 2, ... , m.
// *
// *     For each value of i, the current matrix A is updated by forming
// *     inv(S(i))**T*A*inv(S(i)). This creates a triangular bulge outside
// *     the band of A. The bulge is then pushed down toward the bottom of
// *     A in phase 1, and up toward the top of A in phase 2, by applying
// *     plane rotations.
// *
// *     There are kb*(kb+1)/2 elements in the bulge, but at most 2*kb-1
// *     of them are linearly independent, so annihilating a bulge requires
// *     only 2*kb-1 plane rotations. The rotations are divided into a 1st
// *     set of kb-1 rotations, and a 2nd set of kb rotations.
// *
// *     Wherever possible, rotations are generated and applied in vector
// *     operations of length NR between the indices J1 and J2 (sometimes
// *     replaced by modified values NRT, J1T or J2T).
// *
// *     The cosines and sines of the rotations are stored in the array
// *     WORK. The cosines of the 1st set of rotations are stored in
// *     elements n+2:n+m-kb-1 and the sines of the 1st set in elements
// *     2:m-kb-1; the cosines of the 2nd set are stored in elements
// *     n+m-kb+1:2*n and the sines of the second set in elements m-kb+1:n.
// *
// *     The bulges are not formed explicitly; nonzero elements outside the
// *     band are created only when they are required for generating new
// *     rotations; they are stored in the array WORK, in positions where
// *     they are later overwritten by the sines of the rotations which
// *     annihilate them.
// *
// *     **************************** Phase 1 *****************************
// *
// *     The logical structure of this phase is:
// *
// *     UPDATE = .TRUE.
// *     DO I = N, M + 1, -1
// *        use S(i) to update A and create a new bulge
// *        apply rotations to push all bulges KA positions downward
// *     END DO
// *     UPDATE = .FALSE.
// *     DO I = M + KA + 1, N - 1
// *        apply rotations to push all bulges KA positions downward
// *     END DO
// *
// *     To avoid duplicating code, the two loops are merged.
// *
update = true;
i = n+1;
label10:
   Dummy.label("Dsbgst",10);
if (update)  {
    i = i-1;
kbt = (int)(Math.min(kb, i-1) );
i0 = i-1;
i1 = (int)(Math.min(n, i+ka) );
i2 = i-kbt+ka1;
if (i < m+1)  {
    update = false;
i = i+1;
i0 = m;
if (ka == 0)  
    Dummy.go_to("Dsbgst",480);
Dummy.go_to("Dsbgst",10);
}              // Close if()
}              // Close if()
else  {
  i = i+ka;
if (i > n-1)  
    Dummy.go_to("Dsbgst",480);
}              //  Close else.
// *
if (upper)  {
    // *
// *        Transform A, working with the upper triangle
// *
if (update)  {
    // *
// *           Form  inv(S(i))**T * A * inv(S(i))
// *
bii = bb[(kb1)- 1+(i- 1)*ldbb+ _bb_offset];
{
forloop20:
for (j = i; j <= i1; j++) {
ab[(i-j+ka1)- 1+(j- 1)*ldab+ _ab_offset] = ab[(i-j+ka1)- 1+(j- 1)*ldab+ _ab_offset]/bii;
Dummy.label("Dsbgst",20);
}              //  Close for() loop. 
}
{
forloop30:
for (j = (int)(Math.max(1, i-ka) ); j <= i; j++) {
ab[(j-i+ka1)- 1+(i- 1)*ldab+ _ab_offset] = ab[(j-i+ka1)- 1+(i- 1)*ldab+ _ab_offset]/bii;
Dummy.label("Dsbgst",30);
}              //  Close for() loop. 
}
{
forloop60:
for (k = i-kbt; k <= i-1; k++) {
{
forloop40:
for (j = i-kbt; j <= k; j++) {
ab[(j-k+ka1)- 1+(k- 1)*ldab+ _ab_offset] = ab[(j-k+ka1)- 1+(k- 1)*ldab+ _ab_offset]-bb[(j-i+kb1)- 1+(i- 1)*ldbb+ _bb_offset]*ab[(k-i+ka1)- 1+(i- 1)*ldab+ _ab_offset]-bb[(k-i+kb1)- 1+(i- 1)*ldbb+ _bb_offset]*ab[(j-i+ka1)- 1+(i- 1)*ldab+ _ab_offset]+ab[(ka1)- 1+(i- 1)*ldab+ _ab_offset]*bb[(j-i+kb1)- 1+(i- 1)*ldbb+ _bb_offset]*bb[(k-i+kb1)- 1+(i- 1)*ldbb+ _bb_offset];
Dummy.label("Dsbgst",40);
}              //  Close for() loop. 
}
{
forloop50:
for (j = (int)(Math.max(1, i-ka) ); j <= i-kbt-1; j++) {
ab[(j-k+ka1)- 1+(k- 1)*ldab+ _ab_offset] = ab[(j-k+ka1)- 1+(k- 1)*ldab+ _ab_offset]-bb[(k-i+kb1)- 1+(i- 1)*ldbb+ _bb_offset]*ab[(j-i+ka1)- 1+(i- 1)*ldab+ _ab_offset];
Dummy.label("Dsbgst",50);
}              //  Close for() loop. 
}
Dummy.label("Dsbgst",60);
}              //  Close for() loop. 
}
{
forloop80:
for (j = i; j <= i1; j++) {
{
forloop70:
for (k = (int)(Math.max(j-ka, i-kbt) ); k <= i-1; k++) {
ab[(k-j+ka1)- 1+(j- 1)*ldab+ _ab_offset] = ab[(k-j+ka1)- 1+(j- 1)*ldab+ _ab_offset]-bb[(k-i+kb1)- 1+(i- 1)*ldbb+ _bb_offset]*ab[(i-j+ka1)- 1+(j- 1)*ldab+ _ab_offset];
Dummy.label("Dsbgst",70);
}              //  Close for() loop. 
}
Dummy.label("Dsbgst",80);
}              //  Close for() loop. 
}
// *
if (wantx)  {
    // *
// *              post-multiply X by inv(S(i))
// *
Dscal.dscal(n-m,one/bii,x,(m+1)- 1+(i- 1)*ldx+ _x_offset,1);
if (kbt > 0)  
    Dger.dger(n-m,kbt,-one,x,(m+1)- 1+(i- 1)*ldx+ _x_offset,1,bb,(kb1-kbt)- 1+(i- 1)*ldbb+ _bb_offset,1,x,(m+1)- 1+(i-kbt- 1)*ldx+ _x_offset,ldx);
}              // Close if()
// *
// *           store a(i,i1) in RA1 for use in next loop over K
// *
ra1 = ab[(i-i1+ka1)- 1+(i1- 1)*ldab+ _ab_offset];
}              // Close if()
// *
// *        Generate and apply vectors of rotations to chase all the
// *        existing bulges KA positions down toward the bottom of the
// *        band
// *
{
forloop130:
for (k = 1; k <= kb-1; k++) {
if (update)  {
    // *
// *              Determine the rotations which would annihilate the bulge
// *              which has in theory just been created
// *
if (i-k+ka < n && i-k > 1)  {
    // *
// *                 generate rotation to annihilate a(i,i-k+ka+1)
// *
dlartg_adapter(ab[(k+1)- 1+(i-k+ka- 1)*ldab+ _ab_offset],ra1,work,(n+i-k+ka-m)- 1+ _work_offset,work,(i-k+ka-m)- 1+ _work_offset,ra);
// *
// *                 create nonzero element a(i-k,i-k+ka+1) outside the
// *                 band and store it in WORK(i-k)
// *
t = -bb[(kb1-k)- 1+(i- 1)*ldbb+ _bb_offset]*ra1;
work[(i-k)- 1+ _work_offset] = work[(n+i-k+ka-m)- 1+ _work_offset]*t-work[(i-k+ka-m)- 1+ _work_offset]*ab[(1)- 1+(i-k+ka- 1)*ldab+ _ab_offset];
ab[(1)- 1+(i-k+ka- 1)*ldab+ _ab_offset] = work[(i-k+ka-m)- 1+ _work_offset]*t+work[(n+i-k+ka-m)- 1+ _work_offset]*ab[(1)- 1+(i-k+ka- 1)*ldab+ _ab_offset];
ra1 = ra.val;
}              // Close if()
}              // Close if()
j2 = (int)(i-k-1+Math.max(1, k-i0+2) *ka1);
nr = (n-j2+ka)/ka1;
j1 = j2+(nr-1)*ka1;
if (update)  {
    j2t = (int)(Math.max(j2, i+2*ka-k+1) );
}              // Close if()
else  {
  j2t = j2;
}              //  Close else.
nrt = (n-j2t+ka)/ka1;
{
int _j_inc = ka1;
forloop90:
for (j = j2t; (_j_inc < 0) ? j >= j1 : j <= j1; j += _j_inc) {
// *
// *              create nonzero element a(j-ka,j+1) outside the band
// *              and store it in WORK(j-m)
// *
work[(j-m)- 1+ _work_offset] = work[(j-m)- 1+ _work_offset]*ab[(1)- 1+(j+1- 1)*ldab+ _ab_offset];
ab[(1)- 1+(j+1- 1)*ldab+ _ab_offset] = work[(n+j-m)- 1+ _work_offset]*ab[(1)- 1+(j+1- 1)*ldab+ _ab_offset];
Dummy.label("Dsbgst",90);
}              //  Close for() loop. 
}
// *
// *           generate rotations in 1st set to annihilate elements which
// *           have been created outside the band
// *
if (nrt > 0)  
    Dlargv.dlargv(nrt,ab,(1)- 1+(j2t- 1)*ldab+ _ab_offset,inca,work,(j2t-m)- 1+ _work_offset,ka1,work,(n+j2t-m)- 1+ _work_offset,ka1);
if (nr > 0)  {
    // *
// *              apply rotations in 1st set from the right
// *
{
forloop100:
for (l = 1; l <= ka-1; l++) {
Dlartv.dlartv(nr,ab,(ka1-l)- 1+(j2- 1)*ldab+ _ab_offset,inca,ab,(ka-l)- 1+(j2+1- 1)*ldab+ _ab_offset,inca,work,(n+j2-m)- 1+ _work_offset,work,(j2-m)- 1+ _work_offset,ka1);
Dummy.label("Dsbgst",100);
}              //  Close for() loop. 
}
// *
// *              apply rotations in 1st set from both sides to diagonal
// *              blocks
// *
Dlar2v.dlar2v(nr,ab,(ka1)- 1+(j2- 1)*ldab+ _ab_offset,ab,(ka1)- 1+(j2+1- 1)*ldab+ _ab_offset,ab,(ka)- 1+(j2+1- 1)*ldab+ _ab_offset,inca,work,(n+j2-m)- 1+ _work_offset,work,(j2-m)- 1+ _work_offset,ka1);
// *
}              // Close if()
// *
// *           start applying rotations in 1st set from the left
// *
{
int _l_inc = -1;
forloop110:
for (l = ka-1; l >= kb-k+1; l += _l_inc) {
nrt = (n-j2+l)/ka1;
if (nrt > 0)  
    Dlartv.dlartv(nrt,ab,(l)- 1+(j2+ka1-l- 1)*ldab+ _ab_offset,inca,ab,(l+1)- 1+(j2+ka1-l- 1)*ldab+ _ab_offset,inca,work,(n+j2-m)- 1+ _work_offset,work,(j2-m)- 1+ _work_offset,ka1);
Dummy.label("Dsbgst",110);
}              //  Close for() loop. 
}
// *
if (wantx)  {
    // *
// *              post-multiply X by product of rotations in 1st set
// *
{
int _j_inc = ka1;
forloop120:
for (j = j2; (_j_inc < 0) ? j >= j1 : j <= j1; j += _j_inc) {
Drot.drot(n-m,x,(m+1)- 1+(j- 1)*ldx+ _x_offset,1,x,(m+1)- 1+(j+1- 1)*ldx+ _x_offset,1,work[(n+j-m)- 1+ _work_offset],work[(j-m)- 1+ _work_offset]);
Dummy.label("Dsbgst",120);
}              //  Close for() loop. 
}
}              // Close if()
Dummy.label("Dsbgst",130);
}              //  Close for() loop. 
}
// *
if (update)  {
    if (i2 <= n && kbt > 0)  {
    // *
// *              create nonzero element a(i-kbt,i-kbt+ka+1) outside the
// *              band and store it in WORK(i-kbt)
// *
work[(i-kbt)- 1+ _work_offset] = -bb[(kb1-kbt)- 1+(i- 1)*ldbb+ _bb_offset]*ra1;
}              // Close if()
}              // Close if()
// *
{
int _k_inc = -1;
forloop170:
for (k = kb; k >= 1; k += _k_inc) {
if (update)  {
    j2 = (int)(i-k-1+Math.max(2, k-i0+1) *ka1);
}              // Close if()
else  {
  j2 = (int)(i-k-1+Math.max(1, k-i0+1) *ka1);
}              //  Close else.
// *
// *           finish applying rotations in 2nd set from the left
// *
{
int _l_inc = -1;
forloop140:
for (l = kb-k; l >= 1; l += _l_inc) {
nrt = (n-j2+ka+l)/ka1;
if (nrt > 0)  
    Dlartv.dlartv(nrt,ab,(l)- 1+(j2-l+1- 1)*ldab+ _ab_offset,inca,ab,(l+1)- 1+(j2-l+1- 1)*ldab+ _ab_offset,inca,work,(n+j2-ka)- 1+ _work_offset,work,(j2-ka)- 1+ _work_offset,ka1);
Dummy.label("Dsbgst",140);
}              //  Close for() loop. 
}
nr = (n-j2+ka)/ka1;
j1 = j2+(nr-1)*ka1;
{
int _j_inc = -ka1;
forloop150:
for (j = j1; (_j_inc < 0) ? j >= j2 : j <= j2; j += _j_inc) {
work[(j)- 1+ _work_offset] = work[(j-ka)- 1+ _work_offset];
work[(n+j)- 1+ _work_offset] = work[(n+j-ka)- 1+ _work_offset];
Dummy.label("Dsbgst",150);
}              //  Close for() loop. 
}
{
int _j_inc = ka1;
forloop160:
for (j = j2; (_j_inc < 0) ? j >= j1 : j <= j1; j += _j_inc) {
// *
// *              create nonzero element a(j-ka,j+1) outside the band
// *              and store it in WORK(j)
// *
work[(j)- 1+ _work_offset] = work[(j)- 1+ _work_offset]*ab[(1)- 1+(j+1- 1)*ldab+ _ab_offset];
ab[(1)- 1+(j+1- 1)*ldab+ _ab_offset] = work[(n+j)- 1+ _work_offset]*ab[(1)- 1+(j+1- 1)*ldab+ _ab_offset];
Dummy.label("Dsbgst",160);
}              //  Close for() loop. 
}
if (update)  {
    if (i-k < n-ka && k <= kbt)  
    work[(i-k+ka)- 1+ _work_offset] = work[(i-k)- 1+ _work_offset];
}              // Close if()
Dummy.label("Dsbgst",170);
}              //  Close for() loop. 
}
// *
{
int _k_inc = -1;
forloop210:
for (k = kb; k >= 1; k += _k_inc) {
j2 = (int)(i-k-1+Math.max(1, k-i0+1) *ka1);
nr = (n-j2+ka)/ka1;
j1 = j2+(nr-1)*ka1;
if (nr > 0)  {
    // *
// *              generate rotations in 2nd set to annihilate elements
// *              which have been created outside the band
// *
Dlargv.dlargv(nr,ab,(1)- 1+(j2- 1)*ldab+ _ab_offset,inca,work,(j2)- 1+ _work_offset,ka1,work,(n+j2)- 1+ _work_offset,ka1);
// *
// *              apply rotations in 2nd set from the right
// *
{
forloop180:
for (l = 1; l <= ka-1; l++) {
Dlartv.dlartv(nr,ab,(ka1-l)- 1+(j2- 1)*ldab+ _ab_offset,inca,ab,(ka-l)- 1+(j2+1- 1)*ldab+ _ab_offset,inca,work,(n+j2)- 1+ _work_offset,work,(j2)- 1+ _work_offset,ka1);
Dummy.label("Dsbgst",180);
}              //  Close for() loop. 
}
// *
// *              apply rotations in 2nd set from both sides to diagonal
// *              blocks
// *
Dlar2v.dlar2v(nr,ab,(ka1)- 1+(j2- 1)*ldab+ _ab_offset,ab,(ka1)- 1+(j2+1- 1)*ldab+ _ab_offset,ab,(ka)- 1+(j2+1- 1)*ldab+ _ab_offset,inca,work,(n+j2)- 1+ _work_offset,work,(j2)- 1+ _work_offset,ka1);
// *
}              // Close if()
// *
// *           start applying rotations in 2nd set from the left
// *
{
int _l_inc = -1;
forloop190:
for (l = ka-1; l >= kb-k+1; l += _l_inc) {
nrt = (n-j2+l)/ka1;
if (nrt > 0)  
    Dlartv.dlartv(nrt,ab,(l)- 1+(j2+ka1-l- 1)*ldab+ _ab_offset,inca,ab,(l+1)- 1+(j2+ka1-l- 1)*ldab+ _ab_offset,inca,work,(n+j2)- 1+ _work_offset,work,(j2)- 1+ _work_offset,ka1);
Dummy.label("Dsbgst",190);
}              //  Close for() loop. 
}
// *
if (wantx)  {
    // *
// *              post-multiply X by product of rotations in 2nd set
// *
{
int _j_inc = ka1;
forloop200:
for (j = j2; (_j_inc < 0) ? j >= j1 : j <= j1; j += _j_inc) {
Drot.drot(n-m,x,(m+1)- 1+(j- 1)*ldx+ _x_offset,1,x,(m+1)- 1+(j+1- 1)*ldx+ _x_offset,1,work[(n+j)- 1+ _work_offset],work[(j)- 1+ _work_offset]);
Dummy.label("Dsbgst",200);
}              //  Close for() loop. 
}
}              // Close if()
Dummy.label("Dsbgst",210);
}              //  Close for() loop. 
}
// *
{
forloop230:
for (k = 1; k <= kb-1; k++) {
j2 = (int)(i-k-1+Math.max(1, k-i0+2) *ka1);
// *
// *           finish applying rotations in 1st set from the left
// *
{
int _l_inc = -1;
forloop220:
for (l = kb-k; l >= 1; l += _l_inc) {
nrt = (n-j2+l)/ka1;
if (nrt > 0)  
    Dlartv.dlartv(nrt,ab,(l)- 1+(j2+ka1-l- 1)*ldab+ _ab_offset,inca,ab,(l+1)- 1+(j2+ka1-l- 1)*ldab+ _ab_offset,inca,work,(n+j2-m)- 1+ _work_offset,work,(j2-m)- 1+ _work_offset,ka1);
Dummy.label("Dsbgst",220);
}              //  Close for() loop. 
}
Dummy.label("Dsbgst",230);
}              //  Close for() loop. 
}
// *
{
int _j_inc = -1;
forloop240:
for (j = n-1; j >= i2+ka; j += _j_inc) {
work[(n+j-m)- 1+ _work_offset] = work[(n+j-ka-m)- 1+ _work_offset];
work[(j-m)- 1+ _work_offset] = work[(j-ka-m)- 1+ _work_offset];
Dummy.label("Dsbgst",240);
}              //  Close for() loop. 
}
// *
}              // Close if()
else  {
  // *
// *        Transform A, working with the lower triangle
// *
if (update)  {
    // *
// *           Form  inv(S(i))**T * A * inv(S(i))
// *
bii = bb[(1)- 1+(i- 1)*ldbb+ _bb_offset];
{
forloop250:
for (j = i; j <= i1; j++) {
ab[(j-i+1)- 1+(i- 1)*ldab+ _ab_offset] = ab[(j-i+1)- 1+(i- 1)*ldab+ _ab_offset]/bii;
Dummy.label("Dsbgst",250);
}              //  Close for() loop. 
}
{
forloop260:
for (j = (int)(Math.max(1, i-ka) ); j <= i; j++) {
ab[(i-j+1)- 1+(j- 1)*ldab+ _ab_offset] = ab[(i-j+1)- 1+(j- 1)*ldab+ _ab_offset]/bii;
Dummy.label("Dsbgst",260);
}              //  Close for() loop. 
}
{
forloop290:
for (k = i-kbt; k <= i-1; k++) {
{
forloop270:
for (j = i-kbt; j <= k; j++) {
ab[(k-j+1)- 1+(j- 1)*ldab+ _ab_offset] = ab[(k-j+1)- 1+(j- 1)*ldab+ _ab_offset]-bb[(i-j+1)- 1+(j- 1)*ldbb+ _bb_offset]*ab[(i-k+1)- 1+(k- 1)*ldab+ _ab_offset]-bb[(i-k+1)- 1+(k- 1)*ldbb+ _bb_offset]*ab[(i-j+1)- 1+(j- 1)*ldab+ _ab_offset]+ab[(1)- 1+(i- 1)*ldab+ _ab_offset]*bb[(i-j+1)- 1+(j- 1)*ldbb+ _bb_offset]*bb[(i-k+1)- 1+(k- 1)*ldbb+ _bb_offset];
Dummy.label("Dsbgst",270);
}              //  Close for() loop. 
}
{
forloop280:
for (j = (int)(Math.max(1, i-ka) ); j <= i-kbt-1; j++) {
ab[(k-j+1)- 1+(j- 1)*ldab+ _ab_offset] = ab[(k-j+1)- 1+(j- 1)*ldab+ _ab_offset]-bb[(i-k+1)- 1+(k- 1)*ldbb+ _bb_offset]*ab[(i-j+1)- 1+(j- 1)*ldab+ _ab_offset];
Dummy.label("Dsbgst",280);
}              //  Close for() loop. 
}
Dummy.label("Dsbgst",290);
}              //  Close for() loop. 
}
{
forloop310:
for (j = i; j <= i1; j++) {
{
forloop300:
for (k = (int)(Math.max(j-ka, i-kbt) ); k <= i-1; k++) {
ab[(j-k+1)- 1+(k- 1)*ldab+ _ab_offset] = ab[(j-k+1)- 1+(k- 1)*ldab+ _ab_offset]-bb[(i-k+1)- 1+(k- 1)*ldbb+ _bb_offset]*ab[(j-i+1)- 1+(i- 1)*ldab+ _ab_offset];
Dummy.label("Dsbgst",300);
}              //  Close for() loop. 
}
Dummy.label("Dsbgst",310);
}              //  Close for() loop. 
}
// *
if (wantx)  {
    // *
// *              post-multiply X by inv(S(i))
// *
Dscal.dscal(n-m,one/bii,x,(m+1)- 1+(i- 1)*ldx+ _x_offset,1);
if (kbt > 0)  
    Dger.dger(n-m,kbt,-one,x,(m+1)- 1+(i- 1)*ldx+ _x_offset,1,bb,(kbt+1)- 1+(i-kbt- 1)*ldbb+ _bb_offset,ldbb-1,x,(m+1)- 1+(i-kbt- 1)*ldx+ _x_offset,ldx);
}              // Close if()
// *
// *           store a(i1,i) in RA1 for use in next loop over K
// *
ra1 = ab[(i1-i+1)- 1+(i- 1)*ldab+ _ab_offset];
}              // Close if()
// *
// *        Generate and apply vectors of rotations to chase all the
// *        existing bulges KA positions down toward the bottom of the
// *        band
// *
{
forloop360:
for (k = 1; k <= kb-1; k++) {
if (update)  {
    // *
// *              Determine the rotations which would annihilate the bulge
// *              which has in theory just been created
// *
if (i-k+ka < n && i-k > 1)  {
    // *
// *                 generate rotation to annihilate a(i-k+ka+1,i)
// *
dlartg_adapter(ab[(ka1-k)- 1+(i- 1)*ldab+ _ab_offset],ra1,work,(n+i-k+ka-m)- 1+ _work_offset,work,(i-k+ka-m)- 1+ _work_offset,ra);
// *
// *                 create nonzero element a(i-k+ka+1,i-k) outside the
// *                 band and store it in WORK(i-k)
// *
t = -bb[(k+1)- 1+(i-k- 1)*ldbb+ _bb_offset]*ra1;
work[(i-k)- 1+ _work_offset] = work[(n+i-k+ka-m)- 1+ _work_offset]*t-work[(i-k+ka-m)- 1+ _work_offset]*ab[(ka1)- 1+(i-k- 1)*ldab+ _ab_offset];
ab[(ka1)- 1+(i-k- 1)*ldab+ _ab_offset] = work[(i-k+ka-m)- 1+ _work_offset]*t+work[(n+i-k+ka-m)- 1+ _work_offset]*ab[(ka1)- 1+(i-k- 1)*ldab+ _ab_offset];
ra1 = ra.val;
}              // Close if()
}              // Close if()
j2 = (int)(i-k-1+Math.max(1, k-i0+2) *ka1);
nr = (n-j2+ka)/ka1;
j1 = j2+(nr-1)*ka1;
if (update)  {
    j2t = (int)(Math.max(j2, i+2*ka-k+1) );
}              // Close if()
else  {
  j2t = j2;
}              //  Close else.
nrt = (n-j2t+ka)/ka1;
{
int _j_inc = ka1;
forloop320:
for (j = j2t; (_j_inc < 0) ? j >= j1 : j <= j1; j += _j_inc) {
// *
// *              create nonzero element a(j+1,j-ka) outside the band
// *              and store it in WORK(j-m)
// *
work[(j-m)- 1+ _work_offset] = work[(j-m)- 1+ _work_offset]*ab[(ka1)- 1+(j-ka+1- 1)*ldab+ _ab_offset];
ab[(ka1)- 1+(j-ka+1- 1)*ldab+ _ab_offset] = work[(n+j-m)- 1+ _work_offset]*ab[(ka1)- 1+(j-ka+1- 1)*ldab+ _ab_offset];
Dummy.label("Dsbgst",320);
}              //  Close for() loop. 
}
// *
// *           generate rotations in 1st set to annihilate elements which
// *           have been created outside the band
// *
if (nrt > 0)  
    Dlargv.dlargv(nrt,ab,(ka1)- 1+(j2t-ka- 1)*ldab+ _ab_offset,inca,work,(j2t-m)- 1+ _work_offset,ka1,work,(n+j2t-m)- 1+ _work_offset,ka1);
if (nr > 0)  {
    // *
// *              apply rotations in 1st set from the left
// *
{
forloop330:
for (l = 1; l <= ka-1; l++) {
Dlartv.dlartv(nr,ab,(l+1)- 1+(j2-l- 1)*ldab+ _ab_offset,inca,ab,(l+2)- 1+(j2-l- 1)*ldab+ _ab_offset,inca,work,(n+j2-m)- 1+ _work_offset,work,(j2-m)- 1+ _work_offset,ka1);
Dummy.label("Dsbgst",330);
}              //  Close for() loop. 
}
// *
// *              apply rotations in 1st set from both sides to diagonal
// *              blocks
// *
Dlar2v.dlar2v(nr,ab,(1)- 1+(j2- 1)*ldab+ _ab_offset,ab,(1)- 1+(j2+1- 1)*ldab+ _ab_offset,ab,(2)- 1+(j2- 1)*ldab+ _ab_offset,inca,work,(n+j2-m)- 1+ _work_offset,work,(j2-m)- 1+ _work_offset,ka1);
// *
}              // Close if()
// *
// *           start applying rotations in 1st set from the right
// *
{
int _l_inc = -1;
forloop340:
for (l = ka-1; l >= kb-k+1; l += _l_inc) {
nrt = (n-j2+l)/ka1;
if (nrt > 0)  
    Dlartv.dlartv(nrt,ab,(ka1-l+1)- 1+(j2- 1)*ldab+ _ab_offset,inca,ab,(ka1-l)- 1+(j2+1- 1)*ldab+ _ab_offset,inca,work,(n+j2-m)- 1+ _work_offset,work,(j2-m)- 1+ _work_offset,ka1);
Dummy.label("Dsbgst",340);
}              //  Close for() loop. 
}
// *
if (wantx)  {
    // *
// *              post-multiply X by product of rotations in 1st set
// *
{
int _j_inc = ka1;
forloop350:
for (j = j2; (_j_inc < 0) ? j >= j1 : j <= j1; j += _j_inc) {
Drot.drot(n-m,x,(m+1)- 1+(j- 1)*ldx+ _x_offset,1,x,(m+1)- 1+(j+1- 1)*ldx+ _x_offset,1,work[(n+j-m)- 1+ _work_offset],work[(j-m)- 1+ _work_offset]);
Dummy.label("Dsbgst",350);
}              //  Close for() loop. 
}
}              // Close if()
Dummy.label("Dsbgst",360);
}              //  Close for() loop. 
}
// *
if (update)  {
    if (i2 <= n && kbt > 0)  {
    // *
// *              create nonzero element a(i-kbt+ka+1,i-kbt) outside the
// *              band and store it in WORK(i-kbt)
// *
work[(i-kbt)- 1+ _work_offset] = -bb[(kbt+1)- 1+(i-kbt- 1)*ldbb+ _bb_offset]*ra1;
}              // Close if()
}              // Close if()
// *
{
int _k_inc = -1;
forloop400:
for (k = kb; k >= 1; k += _k_inc) {
if (update)  {
    j2 = (int)(i-k-1+Math.max(2, k-i0+1) *ka1);
}              // Close if()
else  {
  j2 = (int)(i-k-1+Math.max(1, k-i0+1) *ka1);
}              //  Close else.
// *
// *           finish applying rotations in 2nd set from the right
// *
{
int _l_inc = -1;
forloop370:
for (l = kb-k; l >= 1; l += _l_inc) {
nrt = (n-j2+ka+l)/ka1;
if (nrt > 0)  
    Dlartv.dlartv(nrt,ab,(ka1-l+1)- 1+(j2-ka- 1)*ldab+ _ab_offset,inca,ab,(ka1-l)- 1+(j2-ka+1- 1)*ldab+ _ab_offset,inca,work,(n+j2-ka)- 1+ _work_offset,work,(j2-ka)- 1+ _work_offset,ka1);
Dummy.label("Dsbgst",370);
}              //  Close for() loop. 
}
nr = (n-j2+ka)/ka1;
j1 = j2+(nr-1)*ka1;
{
int _j_inc = -ka1;
forloop380:
for (j = j1; (_j_inc < 0) ? j >= j2 : j <= j2; j += _j_inc) {
work[(j)- 1+ _work_offset] = work[(j-ka)- 1+ _work_offset];
work[(n+j)- 1+ _work_offset] = work[(n+j-ka)- 1+ _work_offset];
Dummy.label("Dsbgst",380);
}              //  Close for() loop. 
}
{
int _j_inc = ka1;
forloop390:
for (j = j2; (_j_inc < 0) ? j >= j1 : j <= j1; j += _j_inc) {
// *
// *              create nonzero element a(j+1,j-ka) outside the band
// *              and store it in WORK(j)
// *
work[(j)- 1+ _work_offset] = work[(j)- 1+ _work_offset]*ab[(ka1)- 1+(j-ka+1- 1)*ldab+ _ab_offset];
ab[(ka1)- 1+(j-ka+1- 1)*ldab+ _ab_offset] = work[(n+j)- 1+ _work_offset]*ab[(ka1)- 1+(j-ka+1- 1)*ldab+ _ab_offset];
Dummy.label("Dsbgst",390);
}              //  Close for() loop. 
}
if (update)  {
    if (i-k < n-ka && k <= kbt)  
    work[(i-k+ka)- 1+ _work_offset] = work[(i-k)- 1+ _work_offset];
}              // Close if()
Dummy.label("Dsbgst",400);
}              //  Close for() loop. 
}
// *
{
int _k_inc = -1;
forloop440:
for (k = kb; k >= 1; k += _k_inc) {
j2 = (int)(i-k-1+Math.max(1, k-i0+1) *ka1);
nr = (n-j2+ka)/ka1;
j1 = j2+(nr-1)*ka1;
if (nr > 0)  {
    // *
// *              generate rotations in 2nd set to annihilate elements
// *              which have been created outside the band
// *
Dlargv.dlargv(nr,ab,(ka1)- 1+(j2-ka- 1)*ldab+ _ab_offset,inca,work,(j2)- 1+ _work_offset,ka1,work,(n+j2)- 1+ _work_offset,ka1);
// *
// *              apply rotations in 2nd set from the left
// *
{
forloop410:
for (l = 1; l <= ka-1; l++) {
Dlartv.dlartv(nr,ab,(l+1)- 1+(j2-l- 1)*ldab+ _ab_offset,inca,ab,(l+2)- 1+(j2-l- 1)*ldab+ _ab_offset,inca,work,(n+j2)- 1+ _work_offset,work,(j2)- 1+ _work_offset,ka1);
Dummy.label("Dsbgst",410);
}              //  Close for() loop. 
}
// *
// *              apply rotations in 2nd set from both sides to diagonal
// *              blocks
// *
Dlar2v.dlar2v(nr,ab,(1)- 1+(j2- 1)*ldab+ _ab_offset,ab,(1)- 1+(j2+1- 1)*ldab+ _ab_offset,ab,(2)- 1+(j2- 1)*ldab+ _ab_offset,inca,work,(n+j2)- 1+ _work_offset,work,(j2)- 1+ _work_offset,ka1);
// *
}              // Close if()
// *
// *           start applying rotations in 2nd set from the right
// *
{
int _l_inc = -1;
forloop420:
for (l = ka-1; l >= kb-k+1; l += _l_inc) {
nrt = (n-j2+l)/ka1;
if (nrt > 0)  
    Dlartv.dlartv(nrt,ab,(ka1-l+1)- 1+(j2- 1)*ldab+ _ab_offset,inca,ab,(ka1-l)- 1+(j2+1- 1)*ldab+ _ab_offset,inca,work,(n+j2)- 1+ _work_offset,work,(j2)- 1+ _work_offset,ka1);
Dummy.label("Dsbgst",420);
}              //  Close for() loop. 
}
// *
if (wantx)  {
    // *
// *              post-multiply X by product of rotations in 2nd set
// *
{
int _j_inc = ka1;
forloop430:
for (j = j2; (_j_inc < 0) ? j >= j1 : j <= j1; j += _j_inc) {
Drot.drot(n-m,x,(m+1)- 1+(j- 1)*ldx+ _x_offset,1,x,(m+1)- 1+(j+1- 1)*ldx+ _x_offset,1,work[(n+j)- 1+ _work_offset],work[(j)- 1+ _work_offset]);
Dummy.label("Dsbgst",430);
}              //  Close for() loop. 
}
}              // Close if()
Dummy.label("Dsbgst",440);
}              //  Close for() loop. 
}
// *
{
forloop460:
for (k = 1; k <= kb-1; k++) {
j2 = (int)(i-k-1+Math.max(1, k-i0+2) *ka1);
// *
// *           finish applying rotations in 1st set from the right
// *
{
int _l_inc = -1;
forloop450:
for (l = kb-k; l >= 1; l += _l_inc) {
nrt = (n-j2+l)/ka1;
if (nrt > 0)  
    Dlartv.dlartv(nrt,ab,(ka1-l+1)- 1+(j2- 1)*ldab+ _ab_offset,inca,ab,(ka1-l)- 1+(j2+1- 1)*ldab+ _ab_offset,inca,work,(n+j2-m)- 1+ _work_offset,work,(j2-m)- 1+ _work_offset,ka1);
Dummy.label("Dsbgst",450);
}              //  Close for() loop. 
}
Dummy.label("Dsbgst",460);
}              //  Close for() loop. 
}
// *
{
int _j_inc = -1;
forloop470:
for (j = n-1; j >= i2+ka; j += _j_inc) {
work[(n+j-m)- 1+ _work_offset] = work[(n+j-ka-m)- 1+ _work_offset];
work[(j-m)- 1+ _work_offset] = work[(j-ka-m)- 1+ _work_offset];
Dummy.label("Dsbgst",470);
}              //  Close for() loop. 
}
// *
}              //  Close else.
// *
Dummy.go_to("Dsbgst",10);
// *
label480:
   Dummy.label("Dsbgst",480);
// *
// *     **************************** Phase 2 *****************************
// *
// *     The logical structure of this phase is:
// *
// *     UPDATE = .TRUE.
// *     DO I = 1, M
// *        use S(i) to update A and create a new bulge
// *        apply rotations to push all bulges KA positions upward
// *     END DO
// *     UPDATE = .FALSE.
// *     DO I = M - KA - 1, 2, -1
// *        apply rotations to push all bulges KA positions upward
// *     END DO
// *
// *     To avoid duplicating code, the two loops are merged.
// *
update = true;
i = 0;
label490:
   Dummy.label("Dsbgst",490);
if (update)  {
    i = i+1;
kbt = (int)(Math.min(kb, m-i) );
i0 = i+1;
i1 = (int)(Math.max(1, i-ka) );
i2 = i+kbt-ka1;
if (i > m)  {
    update = false;
i = i-1;
i0 = m+1;
if (ka == 0)  
    Dummy.go_to("Dsbgst",999999);
Dummy.go_to("Dsbgst",490);
}              // Close if()
}              // Close if()
else  {
  i = i-ka;
if (i < 2)  
    Dummy.go_to("Dsbgst",999999);
}              //  Close else.
// *
if (i < m-kbt)  {
    nx = m;
}              // Close if()
else  {
  nx = n;
}              //  Close else.
// *
if (upper)  {
    // *
// *        Transform A, working with the upper triangle
// *
if (update)  {
    // *
// *           Form  inv(S(i))**T * A * inv(S(i))
// *
bii = bb[(kb1)- 1+(i- 1)*ldbb+ _bb_offset];
{
forloop500:
for (j = i1; j <= i; j++) {
ab[(j-i+ka1)- 1+(i- 1)*ldab+ _ab_offset] = ab[(j-i+ka1)- 1+(i- 1)*ldab+ _ab_offset]/bii;
Dummy.label("Dsbgst",500);
}              //  Close for() loop. 
}
{
forloop510:
for (j = i; j <= Math.min(n, i+ka) ; j++) {
ab[(i-j+ka1)- 1+(j- 1)*ldab+ _ab_offset] = ab[(i-j+ka1)- 1+(j- 1)*ldab+ _ab_offset]/bii;
Dummy.label("Dsbgst",510);
}              //  Close for() loop. 
}
{
forloop540:
for (k = i+1; k <= i+kbt; k++) {
{
forloop520:
for (j = k; j <= i+kbt; j++) {
ab[(k-j+ka1)- 1+(j- 1)*ldab+ _ab_offset] = ab[(k-j+ka1)- 1+(j- 1)*ldab+ _ab_offset]-bb[(i-j+kb1)- 1+(j- 1)*ldbb+ _bb_offset]*ab[(i-k+ka1)- 1+(k- 1)*ldab+ _ab_offset]-bb[(i-k+kb1)- 1+(k- 1)*ldbb+ _bb_offset]*ab[(i-j+ka1)- 1+(j- 1)*ldab+ _ab_offset]+ab[(ka1)- 1+(i- 1)*ldab+ _ab_offset]*bb[(i-j+kb1)- 1+(j- 1)*ldbb+ _bb_offset]*bb[(i-k+kb1)- 1+(k- 1)*ldbb+ _bb_offset];
Dummy.label("Dsbgst",520);
}              //  Close for() loop. 
}
{
forloop530:
for (j = i+kbt+1; j <= Math.min(n, i+ka) ; j++) {
ab[(k-j+ka1)- 1+(j- 1)*ldab+ _ab_offset] = ab[(k-j+ka1)- 1+(j- 1)*ldab+ _ab_offset]-bb[(i-k+kb1)- 1+(k- 1)*ldbb+ _bb_offset]*ab[(i-j+ka1)- 1+(j- 1)*ldab+ _ab_offset];
Dummy.label("Dsbgst",530);
}              //  Close for() loop. 
}
Dummy.label("Dsbgst",540);
}              //  Close for() loop. 
}
{
forloop560:
for (j = i1; j <= i; j++) {
{
forloop550:
for (k = i+1; k <= Math.min(j+ka, i+kbt) ; k++) {
ab[(j-k+ka1)- 1+(k- 1)*ldab+ _ab_offset] = ab[(j-k+ka1)- 1+(k- 1)*ldab+ _ab_offset]-bb[(i-k+kb1)- 1+(k- 1)*ldbb+ _bb_offset]*ab[(j-i+ka1)- 1+(i- 1)*ldab+ _ab_offset];
Dummy.label("Dsbgst",550);
}              //  Close for() loop. 
}
Dummy.label("Dsbgst",560);
}              //  Close for() loop. 
}
// *
if (wantx)  {
    // *
// *              post-multiply X by inv(S(i))
// *
Dscal.dscal(nx,one/bii,x,(1)- 1+(i- 1)*ldx+ _x_offset,1);
if (kbt > 0)  
    Dger.dger(nx,kbt,-one,x,(1)- 1+(i- 1)*ldx+ _x_offset,1,bb,(kb)- 1+(i+1- 1)*ldbb+ _bb_offset,ldbb-1,x,(1)- 1+(i+1- 1)*ldx+ _x_offset,ldx);
}              // Close if()
// *
// *           store a(i1,i) in RA1 for use in next loop over K
// *
ra1 = ab[(i1-i+ka1)- 1+(i- 1)*ldab+ _ab_offset];
}              // Close if()
// *
// *        Generate and apply vectors of rotations to chase all the
// *        existing bulges KA positions up toward the top of the band
// *
{
forloop610:
for (k = 1; k <= kb-1; k++) {
if (update)  {
    // *
// *              Determine the rotations which would annihilate the bulge
// *              which has in theory just been created
// *
if (i+k-ka1 > 0 && i+k < m)  {
    // *
// *                 generate rotation to annihilate a(i+k-ka-1,i)
// *
dlartg_adapter(ab[(k+1)- 1+(i- 1)*ldab+ _ab_offset],ra1,work,(n+i+k-ka)- 1+ _work_offset,work,(i+k-ka)- 1+ _work_offset,ra);
// *
// *                 create nonzero element a(i+k-ka-1,i+k) outside the
// *                 band and store it in WORK(m-kb+i+k)
// *
t = -bb[(kb1-k)- 1+(i+k- 1)*ldbb+ _bb_offset]*ra1;
work[(m-kb+i+k)- 1+ _work_offset] = work[(n+i+k-ka)- 1+ _work_offset]*t-work[(i+k-ka)- 1+ _work_offset]*ab[(1)- 1+(i+k- 1)*ldab+ _ab_offset];
ab[(1)- 1+(i+k- 1)*ldab+ _ab_offset] = work[(i+k-ka)- 1+ _work_offset]*t+work[(n+i+k-ka)- 1+ _work_offset]*ab[(1)- 1+(i+k- 1)*ldab+ _ab_offset];
ra1 = ra.val;
}              // Close if()
}              // Close if()
j2 = (int)(i+k+1-Math.max(1, k+i0-m+1) *ka1);
nr = (j2+ka-1)/ka1;
j1 = j2-(nr-1)*ka1;
if (update)  {
    j2t = (int)(Math.min(j2, i-2*ka+k-1) );
}              // Close if()
else  {
  j2t = j2;
}              //  Close else.
nrt = (j2t+ka-1)/ka1;
{
int _j_inc = ka1;
forloop570:
for (j = j1; (_j_inc < 0) ? j >= j2t : j <= j2t; j += _j_inc) {
// *
// *              create nonzero element a(j-1,j+ka) outside the band
// *              and store it in WORK(j)
// *
work[(j)- 1+ _work_offset] = work[(j)- 1+ _work_offset]*ab[(1)- 1+(j+ka-1- 1)*ldab+ _ab_offset];
ab[(1)- 1+(j+ka-1- 1)*ldab+ _ab_offset] = work[(n+j)- 1+ _work_offset]*ab[(1)- 1+(j+ka-1- 1)*ldab+ _ab_offset];
Dummy.label("Dsbgst",570);
}              //  Close for() loop. 
}
// *
// *           generate rotations in 1st set to annihilate elements which
// *           have been created outside the band
// *
if (nrt > 0)  
    Dlargv.dlargv(nrt,ab,(1)- 1+(j1+ka- 1)*ldab+ _ab_offset,inca,work,(j1)- 1+ _work_offset,ka1,work,(n+j1)- 1+ _work_offset,ka1);
if (nr > 0)  {
    // *
// *              apply rotations in 1st set from the left
// *
{
forloop580:
for (l = 1; l <= ka-1; l++) {
Dlartv.dlartv(nr,ab,(ka1-l)- 1+(j1+l- 1)*ldab+ _ab_offset,inca,ab,(ka-l)- 1+(j1+l- 1)*ldab+ _ab_offset,inca,work,(n+j1)- 1+ _work_offset,work,(j1)- 1+ _work_offset,ka1);
Dummy.label("Dsbgst",580);
}              //  Close for() loop. 
}
// *
// *              apply rotations in 1st set from both sides to diagonal
// *              blocks
// *
Dlar2v.dlar2v(nr,ab,(ka1)- 1+(j1- 1)*ldab+ _ab_offset,ab,(ka1)- 1+(j1-1- 1)*ldab+ _ab_offset,ab,(ka)- 1+(j1- 1)*ldab+ _ab_offset,inca,work,(n+j1)- 1+ _work_offset,work,(j1)- 1+ _work_offset,ka1);
// *
}              // Close if()
// *
// *           start applying rotations in 1st set from the right
// *
{
int _l_inc = -1;
forloop590:
for (l = ka-1; l >= kb-k+1; l += _l_inc) {
nrt = (j2+l-1)/ka1;
j1t = j2-(nrt-1)*ka1;
if (nrt > 0)  
    Dlartv.dlartv(nrt,ab,(l)- 1+(j1t- 1)*ldab+ _ab_offset,inca,ab,(l+1)- 1+(j1t-1- 1)*ldab+ _ab_offset,inca,work,(n+j1t)- 1+ _work_offset,work,(j1t)- 1+ _work_offset,ka1);
Dummy.label("Dsbgst",590);
}              //  Close for() loop. 
}
// *
if (wantx)  {
    // *
// *              post-multiply X by product of rotations in 1st set
// *
{
int _j_inc = ka1;
forloop600:
for (j = j1; (_j_inc < 0) ? j >= j2 : j <= j2; j += _j_inc) {
Drot.drot(nx,x,(1)- 1+(j- 1)*ldx+ _x_offset,1,x,(1)- 1+(j-1- 1)*ldx+ _x_offset,1,work[(n+j)- 1+ _work_offset],work[(j)- 1+ _work_offset]);
Dummy.label("Dsbgst",600);
}              //  Close for() loop. 
}
}              // Close if()
Dummy.label("Dsbgst",610);
}              //  Close for() loop. 
}
// *
if (update)  {
    if (i2 > 0 && kbt > 0)  {
    // *
// *              create nonzero element a(i+kbt-ka-1,i+kbt) outside the
// *              band and store it in WORK(m-kb+i+kbt)
// *
work[(m-kb+i+kbt)- 1+ _work_offset] = -bb[(kb1-kbt)- 1+(i+kbt- 1)*ldbb+ _bb_offset]*ra1;
}              // Close if()
}              // Close if()
// *
{
int _k_inc = -1;
forloop650:
for (k = kb; k >= 1; k += _k_inc) {
if (update)  {
    j2 = (int)(i+k+1-Math.max(2, k+i0-m) *ka1);
}              // Close if()
else  {
  j2 = (int)(i+k+1-Math.max(1, k+i0-m) *ka1);
}              //  Close else.
// *
// *           finish applying rotations in 2nd set from the right
// *
{
int _l_inc = -1;
forloop620:
for (l = kb-k; l >= 1; l += _l_inc) {
nrt = (j2+ka+l-1)/ka1;
j1t = j2-(nrt-1)*ka1;
if (nrt > 0)  
    Dlartv.dlartv(nrt,ab,(l)- 1+(j1t+ka- 1)*ldab+ _ab_offset,inca,ab,(l+1)- 1+(j1t+ka-1- 1)*ldab+ _ab_offset,inca,work,(n+m-kb+j1t+ka)- 1+ _work_offset,work,(m-kb+j1t+ka)- 1+ _work_offset,ka1);
Dummy.label("Dsbgst",620);
}              //  Close for() loop. 
}
nr = (j2+ka-1)/ka1;
j1 = j2-(nr-1)*ka1;
{
int _j_inc = ka1;
forloop630:
for (j = j1; (_j_inc < 0) ? j >= j2 : j <= j2; j += _j_inc) {
work[(m-kb+j)- 1+ _work_offset] = work[(m-kb+j+ka)- 1+ _work_offset];
work[(n+m-kb+j)- 1+ _work_offset] = work[(n+m-kb+j+ka)- 1+ _work_offset];
Dummy.label("Dsbgst",630);
}              //  Close for() loop. 
}
{
int _j_inc = ka1;
forloop640:
for (j = j1; (_j_inc < 0) ? j >= j2 : j <= j2; j += _j_inc) {
// *
// *              create nonzero element a(j-1,j+ka) outside the band
// *              and store it in WORK(m-kb+j)
// *
work[(m-kb+j)- 1+ _work_offset] = work[(m-kb+j)- 1+ _work_offset]*ab[(1)- 1+(j+ka-1- 1)*ldab+ _ab_offset];
ab[(1)- 1+(j+ka-1- 1)*ldab+ _ab_offset] = work[(n+m-kb+j)- 1+ _work_offset]*ab[(1)- 1+(j+ka-1- 1)*ldab+ _ab_offset];
Dummy.label("Dsbgst",640);
}              //  Close for() loop. 
}
if (update)  {
    if (i+k > ka1 && k <= kbt)  
    work[(m-kb+i+k-ka)- 1+ _work_offset] = work[(m-kb+i+k)- 1+ _work_offset];
}              // Close if()
Dummy.label("Dsbgst",650);
}              //  Close for() loop. 
}
// *
{
int _k_inc = -1;
forloop690:
for (k = kb; k >= 1; k += _k_inc) {
j2 = (int)(i+k+1-Math.max(1, k+i0-m) *ka1);
nr = (j2+ka-1)/ka1;
j1 = j2-(nr-1)*ka1;
if (nr > 0)  {
    // *
// *              generate rotations in 2nd set to annihilate elements
// *              which have been created outside the band
// *
Dlargv.dlargv(nr,ab,(1)- 1+(j1+ka- 1)*ldab+ _ab_offset,inca,work,(m-kb+j1)- 1+ _work_offset,ka1,work,(n+m-kb+j1)- 1+ _work_offset,ka1);
// *
// *              apply rotations in 2nd set from the left
// *
{
forloop660:
for (l = 1; l <= ka-1; l++) {
Dlartv.dlartv(nr,ab,(ka1-l)- 1+(j1+l- 1)*ldab+ _ab_offset,inca,ab,(ka-l)- 1+(j1+l- 1)*ldab+ _ab_offset,inca,work,(n+m-kb+j1)- 1+ _work_offset,work,(m-kb+j1)- 1+ _work_offset,ka1);
Dummy.label("Dsbgst",660);
}              //  Close for() loop. 
}
// *
// *              apply rotations in 2nd set from both sides to diagonal
// *              blocks
// *
Dlar2v.dlar2v(nr,ab,(ka1)- 1+(j1- 1)*ldab+ _ab_offset,ab,(ka1)- 1+(j1-1- 1)*ldab+ _ab_offset,ab,(ka)- 1+(j1- 1)*ldab+ _ab_offset,inca,work,(n+m-kb+j1)- 1+ _work_offset,work,(m-kb+j1)- 1+ _work_offset,ka1);
// *
}              // Close if()
// *
// *           start applying rotations in 2nd set from the right
// *
{
int _l_inc = -1;
forloop670:
for (l = ka-1; l >= kb-k+1; l += _l_inc) {
nrt = (j2+l-1)/ka1;
j1t = j2-(nrt-1)*ka1;
if (nrt > 0)  
    Dlartv.dlartv(nrt,ab,(l)- 1+(j1t- 1)*ldab+ _ab_offset,inca,ab,(l+1)- 1+(j1t-1- 1)*ldab+ _ab_offset,inca,work,(n+m-kb+j1t)- 1+ _work_offset,work,(m-kb+j1t)- 1+ _work_offset,ka1);
Dummy.label("Dsbgst",670);
}              //  Close for() loop. 
}
// *
if (wantx)  {
    // *
// *              post-multiply X by product of rotations in 2nd set
// *
{
int _j_inc = ka1;
forloop680:
for (j = j1; (_j_inc < 0) ? j >= j2 : j <= j2; j += _j_inc) {
Drot.drot(nx,x,(1)- 1+(j- 1)*ldx+ _x_offset,1,x,(1)- 1+(j-1- 1)*ldx+ _x_offset,1,work[(n+m-kb+j)- 1+ _work_offset],work[(m-kb+j)- 1+ _work_offset]);
Dummy.label("Dsbgst",680);
}              //  Close for() loop. 
}
}              // Close if()
Dummy.label("Dsbgst",690);
}              //  Close for() loop. 
}
// *
{
forloop710:
for (k = 1; k <= kb-1; k++) {
j2 = (int)(i+k+1-Math.max(1, k+i0-m+1) *ka1);
// *
// *           finish applying rotations in 1st set from the right
// *
{
int _l_inc = -1;
forloop700:
for (l = kb-k; l >= 1; l += _l_inc) {
nrt = (j2+l-1)/ka1;
j1t = j2-(nrt-1)*ka1;
if (nrt > 0)  
    Dlartv.dlartv(nrt,ab,(l)- 1+(j1t- 1)*ldab+ _ab_offset,inca,ab,(l+1)- 1+(j1t-1- 1)*ldab+ _ab_offset,inca,work,(n+j1t)- 1+ _work_offset,work,(j1t)- 1+ _work_offset,ka1);
Dummy.label("Dsbgst",700);
}              //  Close for() loop. 
}
Dummy.label("Dsbgst",710);
}              //  Close for() loop. 
}
// *
{
forloop720:
for (j = 2; j <= i2-ka; j++) {
work[(n+j)- 1+ _work_offset] = work[(n+j+ka)- 1+ _work_offset];
work[(j)- 1+ _work_offset] = work[(j+ka)- 1+ _work_offset];
Dummy.label("Dsbgst",720);
}              //  Close for() loop. 
}
// *
}              // Close if()
else  {
  // *
// *        Transform A, working with the lower triangle
// *
if (update)  {
    // *
// *           Form  inv(S(i))**T * A * inv(S(i))
// *
bii = bb[(1)- 1+(i- 1)*ldbb+ _bb_offset];
{
forloop730:
for (j = i1; j <= i; j++) {
ab[(i-j+1)- 1+(j- 1)*ldab+ _ab_offset] = ab[(i-j+1)- 1+(j- 1)*ldab+ _ab_offset]/bii;
Dummy.label("Dsbgst",730);
}              //  Close for() loop. 
}
{
forloop740:
for (j = i; j <= Math.min(n, i+ka) ; j++) {
ab[(j-i+1)- 1+(i- 1)*ldab+ _ab_offset] = ab[(j-i+1)- 1+(i- 1)*ldab+ _ab_offset]/bii;
Dummy.label("Dsbgst",740);
}              //  Close for() loop. 
}
{
forloop770:
for (k = i+1; k <= i+kbt; k++) {
{
forloop750:
for (j = k; j <= i+kbt; j++) {
ab[(j-k+1)- 1+(k- 1)*ldab+ _ab_offset] = ab[(j-k+1)- 1+(k- 1)*ldab+ _ab_offset]-bb[(j-i+1)- 1+(i- 1)*ldbb+ _bb_offset]*ab[(k-i+1)- 1+(i- 1)*ldab+ _ab_offset]-bb[(k-i+1)- 1+(i- 1)*ldbb+ _bb_offset]*ab[(j-i+1)- 1+(i- 1)*ldab+ _ab_offset]+ab[(1)- 1+(i- 1)*ldab+ _ab_offset]*bb[(j-i+1)- 1+(i- 1)*ldbb+ _bb_offset]*bb[(k-i+1)- 1+(i- 1)*ldbb+ _bb_offset];
Dummy.label("Dsbgst",750);
}              //  Close for() loop. 
}
{
forloop760:
for (j = i+kbt+1; j <= Math.min(n, i+ka) ; j++) {
ab[(j-k+1)- 1+(k- 1)*ldab+ _ab_offset] = ab[(j-k+1)- 1+(k- 1)*ldab+ _ab_offset]-bb[(k-i+1)- 1+(i- 1)*ldbb+ _bb_offset]*ab[(j-i+1)- 1+(i- 1)*ldab+ _ab_offset];
Dummy.label("Dsbgst",760);
}              //  Close for() loop. 
}
Dummy.label("Dsbgst",770);
}              //  Close for() loop. 
}
{
forloop790:
for (j = i1; j <= i; j++) {
{
forloop780:
for (k = i+1; k <= Math.min(j+ka, i+kbt) ; k++) {
ab[(k-j+1)- 1+(j- 1)*ldab+ _ab_offset] = ab[(k-j+1)- 1+(j- 1)*ldab+ _ab_offset]-bb[(k-i+1)- 1+(i- 1)*ldbb+ _bb_offset]*ab[(i-j+1)- 1+(j- 1)*ldab+ _ab_offset];
Dummy.label("Dsbgst",780);
}              //  Close for() loop. 
}
Dummy.label("Dsbgst",790);
}              //  Close for() loop. 
}
// *
if (wantx)  {
    // *
// *              post-multiply X by inv(S(i))
// *
Dscal.dscal(nx,one/bii,x,(1)- 1+(i- 1)*ldx+ _x_offset,1);
if (kbt > 0)  
    Dger.dger(nx,kbt,-one,x,(1)- 1+(i- 1)*ldx+ _x_offset,1,bb,(2)- 1+(i- 1)*ldbb+ _bb_offset,1,x,(1)- 1+(i+1- 1)*ldx+ _x_offset,ldx);
}              // Close if()
// *
// *           store a(i,i1) in RA1 for use in next loop over K
// *
ra1 = ab[(i-i1+1)- 1+(i1- 1)*ldab+ _ab_offset];
}              // Close if()
// *
// *        Generate and apply vectors of rotations to chase all the
// *        existing bulges KA positions up toward the top of the band
// *
{
forloop840:
for (k = 1; k <= kb-1; k++) {
if (update)  {
    // *
// *              Determine the rotations which would annihilate the bulge
// *              which has in theory just been created
// *
if (i+k-ka1 > 0 && i+k < m)  {
    // *
// *                 generate rotation to annihilate a(i,i+k-ka-1)
// *
dlartg_adapter(ab[(ka1-k)- 1+(i+k-ka- 1)*ldab+ _ab_offset],ra1,work,(n+i+k-ka)- 1+ _work_offset,work,(i+k-ka)- 1+ _work_offset,ra);
// *
// *                 create nonzero element a(i+k,i+k-ka-1) outside the
// *                 band and store it in WORK(m-kb+i+k)
// *
t = -bb[(k+1)- 1+(i- 1)*ldbb+ _bb_offset]*ra1;
work[(m-kb+i+k)- 1+ _work_offset] = work[(n+i+k-ka)- 1+ _work_offset]*t-work[(i+k-ka)- 1+ _work_offset]*ab[(ka1)- 1+(i+k-ka- 1)*ldab+ _ab_offset];
ab[(ka1)- 1+(i+k-ka- 1)*ldab+ _ab_offset] = work[(i+k-ka)- 1+ _work_offset]*t+work[(n+i+k-ka)- 1+ _work_offset]*ab[(ka1)- 1+(i+k-ka- 1)*ldab+ _ab_offset];
ra1 = ra.val;
}              // Close if()
}              // Close if()
j2 = (int)(i+k+1-Math.max(1, k+i0-m+1) *ka1);
nr = (j2+ka-1)/ka1;
j1 = j2-(nr-1)*ka1;
if (update)  {
    j2t = (int)(Math.min(j2, i-2*ka+k-1) );
}              // Close if()
else  {
  j2t = j2;
}              //  Close else.
nrt = (j2t+ka-1)/ka1;
{
int _j_inc = ka1;
forloop800:
for (j = j1; (_j_inc < 0) ? j >= j2t : j <= j2t; j += _j_inc) {
// *
// *              create nonzero element a(j+ka,j-1) outside the band
// *              and store it in WORK(j)
// *
work[(j)- 1+ _work_offset] = work[(j)- 1+ _work_offset]*ab[(ka1)- 1+(j-1- 1)*ldab+ _ab_offset];
ab[(ka1)- 1+(j-1- 1)*ldab+ _ab_offset] = work[(n+j)- 1+ _work_offset]*ab[(ka1)- 1+(j-1- 1)*ldab+ _ab_offset];
Dummy.label("Dsbgst",800);
}              //  Close for() loop. 
}
// *
// *           generate rotations in 1st set to annihilate elements which
// *           have been created outside the band
// *
if (nrt > 0)  
    Dlargv.dlargv(nrt,ab,(ka1)- 1+(j1- 1)*ldab+ _ab_offset,inca,work,(j1)- 1+ _work_offset,ka1,work,(n+j1)- 1+ _work_offset,ka1);
if (nr > 0)  {
    // *
// *              apply rotations in 1st set from the right
// *
{
forloop810:
for (l = 1; l <= ka-1; l++) {
Dlartv.dlartv(nr,ab,(l+1)- 1+(j1- 1)*ldab+ _ab_offset,inca,ab,(l+2)- 1+(j1-1- 1)*ldab+ _ab_offset,inca,work,(n+j1)- 1+ _work_offset,work,(j1)- 1+ _work_offset,ka1);
Dummy.label("Dsbgst",810);
}              //  Close for() loop. 
}
// *
// *              apply rotations in 1st set from both sides to diagonal
// *              blocks
// *
Dlar2v.dlar2v(nr,ab,(1)- 1+(j1- 1)*ldab+ _ab_offset,ab,(1)- 1+(j1-1- 1)*ldab+ _ab_offset,ab,(2)- 1+(j1-1- 1)*ldab+ _ab_offset,inca,work,(n+j1)- 1+ _work_offset,work,(j1)- 1+ _work_offset,ka1);
// *
}              // Close if()
// *
// *           start applying rotations in 1st set from the left
// *
{
int _l_inc = -1;
forloop820:
for (l = ka-1; l >= kb-k+1; l += _l_inc) {
nrt = (j2+l-1)/ka1;
j1t = j2-(nrt-1)*ka1;
if (nrt > 0)  
    Dlartv.dlartv(nrt,ab,(ka1-l+1)- 1+(j1t-ka1+l- 1)*ldab+ _ab_offset,inca,ab,(ka1-l)- 1+(j1t-ka1+l- 1)*ldab+ _ab_offset,inca,work,(n+j1t)- 1+ _work_offset,work,(j1t)- 1+ _work_offset,ka1);
Dummy.label("Dsbgst",820);
}              //  Close for() loop. 
}
// *
if (wantx)  {
    // *
// *              post-multiply X by product of rotations in 1st set
// *
{
int _j_inc = ka1;
forloop830:
for (j = j1; (_j_inc < 0) ? j >= j2 : j <= j2; j += _j_inc) {
Drot.drot(nx,x,(1)- 1+(j- 1)*ldx+ _x_offset,1,x,(1)- 1+(j-1- 1)*ldx+ _x_offset,1,work[(n+j)- 1+ _work_offset],work[(j)- 1+ _work_offset]);
Dummy.label("Dsbgst",830);
}              //  Close for() loop. 
}
}              // Close if()
Dummy.label("Dsbgst",840);
}              //  Close for() loop. 
}
// *
if (update)  {
    if (i2 > 0 && kbt > 0)  {
    // *
// *              create nonzero element a(i+kbt,i+kbt-ka-1) outside the
// *              band and store it in WORK(m-kb+i+kbt)
// *
work[(m-kb+i+kbt)- 1+ _work_offset] = -bb[(kbt+1)- 1+(i- 1)*ldbb+ _bb_offset]*ra1;
}              // Close if()
}              // Close if()
// *
{
int _k_inc = -1;
forloop880:
for (k = kb; k >= 1; k += _k_inc) {
if (update)  {
    j2 = (int)(i+k+1-Math.max(2, k+i0-m) *ka1);
}              // Close if()
else  {
  j2 = (int)(i+k+1-Math.max(1, k+i0-m) *ka1);
}              //  Close else.
// *
// *           finish applying rotations in 2nd set from the left
// *
{
int _l_inc = -1;
forloop850:
for (l = kb-k; l >= 1; l += _l_inc) {
nrt = (j2+ka+l-1)/ka1;
j1t = j2-(nrt-1)*ka1;
if (nrt > 0)  
    Dlartv.dlartv(nrt,ab,(ka1-l+1)- 1+(j1t+l-1- 1)*ldab+ _ab_offset,inca,ab,(ka1-l)- 1+(j1t+l-1- 1)*ldab+ _ab_offset,inca,work,(n+m-kb+j1t+ka)- 1+ _work_offset,work,(m-kb+j1t+ka)- 1+ _work_offset,ka1);
Dummy.label("Dsbgst",850);
}              //  Close for() loop. 
}
nr = (j2+ka-1)/ka1;
j1 = j2-(nr-1)*ka1;
{
int _j_inc = ka1;
forloop860:
for (j = j1; (_j_inc < 0) ? j >= j2 : j <= j2; j += _j_inc) {
work[(m-kb+j)- 1+ _work_offset] = work[(m-kb+j+ka)- 1+ _work_offset];
work[(n+m-kb+j)- 1+ _work_offset] = work[(n+m-kb+j+ka)- 1+ _work_offset];
Dummy.label("Dsbgst",860);
}              //  Close for() loop. 
}
{
int _j_inc = ka1;
forloop870:
for (j = j1; (_j_inc < 0) ? j >= j2 : j <= j2; j += _j_inc) {
// *
// *              create nonzero element a(j+ka,j-1) outside the band
// *              and store it in WORK(m-kb+j)
// *
work[(m-kb+j)- 1+ _work_offset] = work[(m-kb+j)- 1+ _work_offset]*ab[(ka1)- 1+(j-1- 1)*ldab+ _ab_offset];
ab[(ka1)- 1+(j-1- 1)*ldab+ _ab_offset] = work[(n+m-kb+j)- 1+ _work_offset]*ab[(ka1)- 1+(j-1- 1)*ldab+ _ab_offset];
Dummy.label("Dsbgst",870);
}              //  Close for() loop. 
}
if (update)  {
    if (i+k > ka1 && k <= kbt)  
    work[(m-kb+i+k-ka)- 1+ _work_offset] = work[(m-kb+i+k)- 1+ _work_offset];
}              // Close if()
Dummy.label("Dsbgst",880);
}              //  Close for() loop. 
}
// *
{
int _k_inc = -1;
forloop920:
for (k = kb; k >= 1; k += _k_inc) {
j2 = (int)(i+k+1-Math.max(1, k+i0-m) *ka1);
nr = (j2+ka-1)/ka1;
j1 = j2-(nr-1)*ka1;
if (nr > 0)  {
    // *
// *              generate rotations in 2nd set to annihilate elements
// *              which have been created outside the band
// *
Dlargv.dlargv(nr,ab,(ka1)- 1+(j1- 1)*ldab+ _ab_offset,inca,work,(m-kb+j1)- 1+ _work_offset,ka1,work,(n+m-kb+j1)- 1+ _work_offset,ka1);
// *
// *              apply rotations in 2nd set from the right
// *
{
forloop890:
for (l = 1; l <= ka-1; l++) {
Dlartv.dlartv(nr,ab,(l+1)- 1+(j1- 1)*ldab+ _ab_offset,inca,ab,(l+2)- 1+(j1-1- 1)*ldab+ _ab_offset,inca,work,(n+m-kb+j1)- 1+ _work_offset,work,(m-kb+j1)- 1+ _work_offset,ka1);
Dummy.label("Dsbgst",890);
}              //  Close for() loop. 
}
// *
// *              apply rotations in 2nd set from both sides to diagonal
// *              blocks
// *
Dlar2v.dlar2v(nr,ab,(1)- 1+(j1- 1)*ldab+ _ab_offset,ab,(1)- 1+(j1-1- 1)*ldab+ _ab_offset,ab,(2)- 1+(j1-1- 1)*ldab+ _ab_offset,inca,work,(n+m-kb+j1)- 1+ _work_offset,work,(m-kb+j1)- 1+ _work_offset,ka1);
// *
}              // Close if()
// *
// *           start applying rotations in 2nd set from the left
// *
{
int _l_inc = -1;
forloop900:
for (l = ka-1; l >= kb-k+1; l += _l_inc) {
nrt = (j2+l-1)/ka1;
j1t = j2-(nrt-1)*ka1;
if (nrt > 0)  
    Dlartv.dlartv(nrt,ab,(ka1-l+1)- 1+(j1t-ka1+l- 1)*ldab+ _ab_offset,inca,ab,(ka1-l)- 1+(j1t-ka1+l- 1)*ldab+ _ab_offset,inca,work,(n+m-kb+j1t)- 1+ _work_offset,work,(m-kb+j1t)- 1+ _work_offset,ka1);
Dummy.label("Dsbgst",900);
}              //  Close for() loop. 
}
// *
if (wantx)  {
    // *
// *              post-multiply X by product of rotations in 2nd set
// *
{
int _j_inc = ka1;
forloop910:
for (j = j1; (_j_inc < 0) ? j >= j2 : j <= j2; j += _j_inc) {
Drot.drot(nx,x,(1)- 1+(j- 1)*ldx+ _x_offset,1,x,(1)- 1+(j-1- 1)*ldx+ _x_offset,1,work[(n+m-kb+j)- 1+ _work_offset],work[(m-kb+j)- 1+ _work_offset]);
Dummy.label("Dsbgst",910);
}              //  Close for() loop. 
}
}              // Close if()
Dummy.label("Dsbgst",920);
}              //  Close for() loop. 
}
// *
{
forloop940:
for (k = 1; k <= kb-1; k++) {
j2 = (int)(i+k+1-Math.max(1, k+i0-m+1) *ka1);
// *
// *           finish applying rotations in 1st set from the left
// *
{
int _l_inc = -1;
forloop930:
for (l = kb-k; l >= 1; l += _l_inc) {
nrt = (j2+l-1)/ka1;
j1t = j2-(nrt-1)*ka1;
if (nrt > 0)  
    Dlartv.dlartv(nrt,ab,(ka1-l+1)- 1+(j1t-ka1+l- 1)*ldab+ _ab_offset,inca,ab,(ka1-l)- 1+(j1t-ka1+l- 1)*ldab+ _ab_offset,inca,work,(n+j1t)- 1+ _work_offset,work,(j1t)- 1+ _work_offset,ka1);
Dummy.label("Dsbgst",930);
}              //  Close for() loop. 
}
Dummy.label("Dsbgst",940);
}              //  Close for() loop. 
}
// *
{
forloop950:
for (j = 2; j <= i2-ka; j++) {
work[(n+j)- 1+ _work_offset] = work[(n+j+ka)- 1+ _work_offset];
work[(j)- 1+ _work_offset] = work[(j+ka)- 1+ _work_offset];
Dummy.label("Dsbgst",950);
}              //  Close for() loop. 
}
// *
}              //  Close else.
// *
Dummy.go_to("Dsbgst",490);
// *
// *     End of DSBGST
// *
Dummy.label("Dsbgst",999999);
return;
   }
// adapter for dlartg
private static void dlartg_adapter(double arg0 ,double arg1 ,double [] arg2 , int arg2_offset ,double [] arg3 , int arg3_offset ,doubleW arg4 )
{
doubleW _f2j_tmp2 = new doubleW(arg2[arg2_offset]);
doubleW _f2j_tmp3 = new doubleW(arg3[arg3_offset]);

Dlartg.dlartg(arg0,arg1,_f2j_tmp2,_f2j_tmp3,arg4);

arg2[arg2_offset] = _f2j_tmp2.val;
arg3[arg3_offset] = _f2j_tmp3.val;
}

} // End class.
