/*
 *  Produced by f2java.  f2java is part of the Fortran-
 *  -to-Java project at the University of Tennessee Netlib
 *  numerical software repository.
 *
 *  Original authorship for the BLAS and LAPACK numerical
 *  routines may be found in the Fortran source, available at
 *  www.netlib.org.
 *
 *  Fortran input file: jlapack.f
 *
 *  The f2j compiler code was written by
 *  David M. Doolin (doolin@cs.utk.edu) and
 *  Keith  Seymour (seymour@cs.utk.edu)
 */

package org.netlib.lapack;
import java.lang.*;
import org.netlib.util.*;



public class Dsbgv {

// *
// *  -- LAPACK driver routine (version 2.0) --
// *     Univ. of Tennessee, Univ. of California Berkeley, NAG Ltd.,
// *     Courant Institute, Argonne National Lab, and Rice University
// *     September 30, 1994
// *
// *     .. Scalar Arguments ..
// *     ..
// *     .. Array Arguments ..
// *     ..
// *
// *  Purpose
// *  =======
// *
// *  DSBGV computes all the eigenvalues, and optionally, the eigenvectors
// *  of a real generalized symmetric-definite banded eigenproblem, of
// *  the form A*x=(lambda)*B*x. Here A and B are assumed to be symmetric
// *  and banded, and B is also positive definite.
// *
// *  Arguments
// *  =========
// *
// *  JOBZ    (input) CHARACTER*1
// *          = 'N':  Compute eigenvalues only;
// *          = 'V':  Compute eigenvalues and eigenvectors.
// *
// *  UPLO    (input) CHARACTER*1
// *          = 'U':  Upper triangles of A and B are stored;
// *          = 'L':  Lower triangles of A and B are stored.
// *
// *  N       (input) INTEGER
// *          The order of the matrices A and B.  N >= 0.
// *
// *  KA      (input) INTEGER
// *          The number of superdiagonals of the matrix A if UPLO = 'U',
// *          or the number of subdiagonals if UPLO = 'L'. KA >= 0.
// *
// *  KB      (input) INTEGER
// *          The number of superdiagonals of the matrix B if UPLO = 'U',
// *          or the number of subdiagonals if UPLO = 'L'. KB >= 0.
// *
// *  AB      (input/output) DOUBLE PRECISION array, dimension (LDAB, N)
// *          On entry, the upper or lower triangle of the symmetric band
// *          matrix A, stored in the first ka+1 rows of the array.  The
// *          j-th column of A is stored in the j-th column of the array AB
// *          as follows:
// *          if UPLO = 'U', AB(ka+1+i-j,j) = A(i,j) for max(1,j-ka)<=i<=j;
// *          if UPLO = 'L', AB(1+i-j,j)    = A(i,j) for j<=i<=min(n,j+ka).
// *
// *          On exit, the contents of AB are destroyed.
// *
// *  LDAB    (input) INTEGER
// *          The leading dimension of the array AB.  LDAB >= KA+1.
// *
// *  BB      (input/output) DOUBLE PRECISION array, dimension (LDBB, N)
// *          On entry, the upper or lower triangle of the symmetric band
// *          matrix B, stored in the first kb+1 rows of the array.  The
// *          j-th column of B is stored in the j-th column of the array BB
// *          as follows:
// *          if UPLO = 'U', BB(kb+1+i-j,j) = B(i,j) for max(1,j-kb)<=i<=j;
// *          if UPLO = 'L', BB(1+i-j,j)    = B(i,j) for j<=i<=min(n,j+kb).
// *
// *          On exit, the factor S from the split Cholesky factorization
// *          B = S**T*S, as returned by DPBSTF.
// *
// *  LDBB    (input) INTEGER
// *          The leading dimension of the array BB.  LDBB >= KB+1.
// *
// *  W       (output) DOUBLE PRECISION array, dimension (N)
// *          If INFO = 0, the eigenvalues in ascending order.
// *
// *  Z       (output) DOUBLE PRECISION array, dimension (LDZ, N)
// *          If JOBZ = 'V', then if INFO = 0, Z contains the matrix Z of
// *          eigenvectors, with the i-th column of Z holding the
// *          eigenvector associated with W(i). The eigenvectors are
// *          normalized so that Z**T*B*Z = I.
// *          If JOBZ = 'N', then Z is not referenced.
// *
// *  LDZ     (input) INTEGER
// *          The leading dimension of the array Z.  LDZ >= 1, and if
// *          JOBZ = 'V', LDZ >= N.
// *
// *  WORK    (workspace) DOUBLE PRECISION array, dimension (3*N)
// *
// *  INFO    (output) INTEGER
// *          = 0:  successful exit
// *          < 0:  if INFO = -i, the i-th argument had an illegal value
// *          > 0:  if INFO = i, and i is:
// *             <= N:  the algorithm failed to converge:
// *                    i off-diagonal elements of an intermediate
// *                    tridiagonal form did not converge to zero;
// *             > N:   if INFO = N + i, for 1 <= i <= N, then DPBSTF
// *                    returned INFO = i: B is not positive definite.
// *                    The factorization of B could not be completed and
// *                    no eigenvalues or eigenvectors were computed.
// *
// *  =====================================================================
// *
// *     .. Local Scalars ..
static boolean upper= false;
static boolean wantz= false;
static String vect= new String(" ");
static intW iinfo= new intW(0);
static int inde= 0;
static int indwrk= 0;
// *     ..
// *     .. External Functions ..
// *     ..
// *     .. External Subroutines ..
// *     ..
// *     .. Executable Statements ..
// *
// *     Test the input parameters.
// *

public static void dsbgv (String jobz,
String uplo,
int n,
int ka,
int kb,
double [] ab, int _ab_offset,
int ldab,
double [] bb, int _bb_offset,
int ldbb,
double [] w, int _w_offset,
double [] z, int _z_offset,
int ldz,
double [] work, int _work_offset,
intW info)  {

wantz = (jobz.toLowerCase().charAt(0) == "V".toLowerCase().charAt(0));
upper = (uplo.toLowerCase().charAt(0) == "U".toLowerCase().charAt(0));
// *
info.val = 0;
if (!(wantz || (jobz.toLowerCase().charAt(0) == "N".toLowerCase().charAt(0))))  {
    info.val = -1;
}              // Close if()
else if (!(upper || (uplo.toLowerCase().charAt(0) == "L".toLowerCase().charAt(0))))  {
    info.val = -2;
}              // Close else if()
else if (n < 0)  {
    info.val = -3;
}              // Close else if()
else if (ka < 0)  {
    info.val = -4;
}              // Close else if()
else if (kb < 0 || kb > ka)  {
    info.val = -5;
}              // Close else if()
else if (ldab < ka+1)  {
    info.val = -7;
}              // Close else if()
else if (ldbb < kb+1)  {
    info.val = -9;
}              // Close else if()
else if (ldz < 1 || (wantz && ldz < n))  {
    info.val = -12;
}              // Close else if()
if (info.val != 0)  {
    Xerbla.xerbla("DSBGV ",-info.val);
Dummy.go_to("Dsbgv",999999);
}              // Close if()
// *
// *     Quick return if possible
// *
if (n == 0)  
    Dummy.go_to("Dsbgv",999999);
// *
// *     Form a split Cholesky factorization of B.
// *
Dpbstf.dpbstf(uplo,n,kb,bb,_bb_offset,ldbb,info);
if (info.val != 0)  {
    info.val = n+info.val;
Dummy.go_to("Dsbgv",999999);
}              // Close if()
// *
// *     Transform problem to standard eigenvalue problem.
// *
inde = 1;
indwrk = inde+n;
Dsbgst.dsbgst(jobz,uplo,n,ka,kb,ab,_ab_offset,ldab,bb,_bb_offset,ldbb,z,_z_offset,ldz,work,(indwrk)- 1+ _work_offset,iinfo);
// *
// *     Reduce to tridiagonal form.
// *
if (wantz)  {
    vect = "U";
}              // Close if()
else  {
  vect = "N";
}              //  Close else.
Dsbtrd.dsbtrd(vect,uplo,n,ka,ab,_ab_offset,ldab,w,_w_offset,work,(inde)- 1+ _work_offset,z,_z_offset,ldz,work,(indwrk)- 1+ _work_offset,iinfo);
// *
// *     For eigenvalues only, call DSTERF.  For eigenvectors, call SSTEQR.
// *
if (!wantz)  {
    Dsterf.dsterf(n,w,_w_offset,work,(inde)- 1+ _work_offset,info);
}              // Close if()
else  {
  Dsteqr.dsteqr(jobz,n,w,_w_offset,work,(inde)- 1+ _work_offset,z,_z_offset,ldz,work,(indwrk)- 1+ _work_offset,info);
}              //  Close else.
Dummy.go_to("Dsbgv",999999);
// *
// *     End of DSBGV
// *
Dummy.label("Dsbgv",999999);
return;
   }
} // End class.
