/*
 *  Produced by f2java.  f2java is part of the Fortran-
 *  -to-Java project at the University of Tennessee Netlib
 *  numerical software repository.
 *
 *  Original authorship for the BLAS and LAPACK numerical
 *  routines may be found in the Fortran source, available at
 *  www.netlib.org.
 *
 *  Fortran input file: jlapack.f
 *
 *  The f2j compiler code was written by
 *  David M. Doolin (doolin@cs.utk.edu) and
 *  Keith  Seymour (seymour@cs.utk.edu)
 */

package org.netlib.lapack;
import java.lang.*;
import org.netlib.util.*;

import org.netlib.blas.*;


public class Dsbtrd {

// *
// *  -- LAPACK routine (version 2.0) --
// *     Univ. of Tennessee, Univ. of California Berkeley, NAG Ltd.,
// *     Courant Institute, Argonne National Lab, and Rice University
// *     September 30, 1994
// *
// *     .. Scalar Arguments ..
// *     ..
// *     .. Array Arguments ..
// *     ..
// *
// *  Purpose
// *  =======
// *
// *  DSBTRD reduces a real symmetric band matrix A to symmetric
// *  tridiagonal form T by an orthogonal similarity transformation:
// *  Q**T * A * Q = T.
// *
// *  Arguments
// *  =========
// *
// *  VECT    (input) CHARACTER*1
// *          = 'N':  do not form Q;
// *          = 'V':  form Q;
// *          = 'U':  update a matrix X, by forming X*Q.
// *
// *  UPLO    (input) CHARACTER*1
// *          = 'U':  Upper triangle of A is stored;
// *          = 'L':  Lower triangle of A is stored.
// *
// *  N       (input) INTEGER
// *          The order of the matrix A.  N >= 0.
// *
// *  KD      (input) INTEGER
// *          The number of superdiagonals of the matrix A if UPLO = 'U',
// *          or the number of subdiagonals if UPLO = 'L'.  KD >= 0.
// *
// *  AB      (input/output) DOUBLE PRECISION array, dimension (LDAB,N)
// *          On entry, the upper or lower triangle of the symmetric band
// *          matrix A, stored in the first KD+1 rows of the array.  The
// *          j-th column of A is stored in the j-th column of the array AB
// *          as follows:
// *          if UPLO = 'U', AB(kd+1+i-j,j) = A(i,j) for max(1,j-kd)<=i<=j;
// *          if UPLO = 'L', AB(1+i-j,j)    = A(i,j) for j<=i<=min(n,j+kd).
// *          On exit, the diagonal elements of AB are overwritten by the
// *          diagonal elements of the tridiagonal matrix T; if KD > 0, the
// *          elements on the first superdiagonal (if UPLO = 'U') or the
// *          first subdiagonal (if UPLO = 'L') are overwritten by the
// *          off-diagonal elements of T; the rest of AB is overwritten by
// *          values generated during the reduction.
// *
// *  LDAB    (input) INTEGER
// *          The leading dimension of the array AB.  LDAB >= KD+1.
// *
// *  D       (output) DOUBLE PRECISION array, dimension (N)
// *          The diagonal elements of the tridiagonal matrix T.
// *
// *  E       (output) DOUBLE PRECISION array, dimension (N-1)
// *          The off-diagonal elements of the tridiagonal matrix T:
// *          E(i) = T(i,i+1) if UPLO = 'U'; E(i) = T(i+1,i) if UPLO = 'L'.
// *
// *  Q       (input/output) DOUBLE PRECISION array, dimension (LDQ,N)
// *          On entry, if VECT = 'U', then Q must contain an N-by-N
// *          matrix X; if VECT = 'N' or 'V', then Q need not be set.
// *
// *          On exit:
// *          if VECT = 'V', Q contains the N-by-N orthogonal matrix Q;
// *          if VECT = 'U', Q contains the product X*Q;
// *          if VECT = 'N', the array Q is not referenced.
// *
// *  LDQ     (input) INTEGER
// *          The leading dimension of the array Q.
// *          LDQ >= 1, and LDQ >= N if VECT = 'V' or 'U'.
// *
// *  WORK    (workspace) DOUBLE PRECISION array, dimension (N)
// *
// *  INFO    (output) INTEGER
// *          = 0:  successful exit
// *          < 0:  if INFO = -i, the i-th argument had an illegal value
// *
// *  =====================================================================
// *
// *     .. Parameters ..
static double zero= 0.0e+0;
static double one= 1.0e+0;
// *     ..
// *     .. Local Scalars ..
static boolean initq= false;
static boolean upper= false;
static boolean wantq= false;
static int i= 0;
static int inca= 0;
static int j= 0;
static int j1= 0;
static int j2= 0;
static int k= 0;
static int kd1= 0;
static int kdn= 0;
static int l= 0;
static int nr= 0;
static int nrt= 0;
static doubleW temp= new doubleW(0.0);
// *     ..
// *     .. External Subroutines ..
// *     ..
// *     .. Intrinsic Functions ..
// *     ..
// *     .. External Functions ..
// *     ..
// *     .. Executable Statements ..
// *
// *     Test the input parameters
// *

public static void dsbtrd (String vect,
String uplo,
int n,
int kd,
double [] ab, int _ab_offset,
int ldab,
double [] d, int _d_offset,
double [] e, int _e_offset,
double [] q, int _q_offset,
int ldq,
double [] work, int _work_offset,
intW info)  {

initq = (vect.toLowerCase().charAt(0) == "V".toLowerCase().charAt(0));
wantq = initq || (vect.toLowerCase().charAt(0) == "U".toLowerCase().charAt(0));
upper = (uplo.toLowerCase().charAt(0) == "U".toLowerCase().charAt(0));
kd1 = kd+1;
info.val = 0;
if (!wantq && !(vect.toLowerCase().charAt(0) == "N".toLowerCase().charAt(0)))  {
    info.val = -1;
}              // Close if()
else if (!upper && !(uplo.toLowerCase().charAt(0) == "L".toLowerCase().charAt(0)))  {
    info.val = -2;
}              // Close else if()
else if (n < 0)  {
    info.val = -3;
}              // Close else if()
else if (kd < 0)  {
    info.val = -4;
}              // Close else if()
else if (ldab < kd1)  {
    info.val = -6;
}              // Close else if()
else if (ldq < Math.max(1, n)  && wantq)  {
    info.val = -10;
}              // Close else if()
if (info.val != 0)  {
    Xerbla.xerbla("DSBTRD",-info.val);
Dummy.go_to("Dsbtrd",999999);
}              // Close if()
// *
// *     Quick return if possible
// *
if (n == 0)  
    Dummy.go_to("Dsbtrd",999999);
// *
// *     Initialize Q to the unit matrix, if needed
// *
if (initq)  
    Dlaset.dlaset("Full",n,n,zero,one,q,_q_offset,ldq);
// *
// *     Wherever possible, plane rotations are generated and applied in
// *     vector operations of length NR over the index set J1:J2:KD1.
// *
// *     The cosines and sines of the plane rotations are stored in the
// *     arrays D and WORK.
// *
inca = kd1*ldab;
kdn = (int)(Math.min(n-1, kd) );
if (upper)  {
    // *
if (kd > 1)  {
    // *
// *           Reduce to tridiagonal form, working with upper triangle
// *
nr = 0;
j1 = kdn+2;
j2 = 1;
// *
{
forloop60:
for (i = 1; i <= n-2; i++) {
// *
// *              Reduce i-th row of matrix to tridiagonal form
// *
{
int _k_inc = -1;
forloop50:
for (k = kdn+1; k >= 2; k += _k_inc) {
j1 = j1+kdn;
j2 = j2+kdn;
// *
if (nr > 0)  {
    // *
// *                    generate plane rotations to annihilate nonzero
// *                    elements which have been created outside the band
// *
Dlargv.dlargv(nr,ab,(1)- 1+(j1-1- 1)*ldab+ _ab_offset,inca,work,(j1)- 1+ _work_offset,kd1,d,(j1)- 1+ _d_offset,kd1);
// *
// *                    apply rotations from the right
// *
{
forloop10:
for (l = 1; l <= kd-1; l++) {
Dlartv.dlartv(nr,ab,(l+1)- 1+(j1-1- 1)*ldab+ _ab_offset,inca,ab,(l)- 1+(j1- 1)*ldab+ _ab_offset,inca,d,(j1)- 1+ _d_offset,work,(j1)- 1+ _work_offset,kd1);
Dummy.label("Dsbtrd",10);
}              //  Close for() loop. 
}
}              // Close if()
// *
if (k > 2)  {
    if (k <= n-i+1)  {
    // *
// *                       generate plane rotation to annihilate a(i,i+k-1)
// *                       within the band
// *
dlartg_adapter(ab[(kd-k+3)- 1+(i+k-2- 1)*ldab+ _ab_offset],ab[(kd-k+2)- 1+(i+k-1- 1)*ldab+ _ab_offset],d,(i+k-1)- 1+ _d_offset,work,(i+k-1)- 1+ _work_offset,temp);
ab[(kd-k+3)- 1+(i+k-2- 1)*ldab+ _ab_offset] = temp.val;
// *
// *                       apply rotation from the right
// *
Drot.drot(k-3,ab,(kd-k+4)- 1+(i+k-2- 1)*ldab+ _ab_offset,1,ab,(kd-k+3)- 1+(i+k-1- 1)*ldab+ _ab_offset,1,d[(i+k-1)- 1+ _d_offset],work[(i+k-1)- 1+ _work_offset]);
}              // Close if()
nr = nr+1;
j1 = j1-kdn-1;
}              // Close if()
// *
// *                 apply plane rotations from both sides to diagonal
// *                 blocks
// *
if (nr > 0)  
    Dlar2v.dlar2v(nr,ab,(kd1)- 1+(j1-1- 1)*ldab+ _ab_offset,ab,(kd1)- 1+(j1- 1)*ldab+ _ab_offset,ab,(kd)- 1+(j1- 1)*ldab+ _ab_offset,inca,d,(j1)- 1+ _d_offset,work,(j1)- 1+ _work_offset,kd1);
// *
// *                 apply plane rotations from the left
// *
{
forloop20:
for (l = 1; l <= kd-1; l++) {
if (j2+l > n)  {
    nrt = nr-1;
}              // Close if()
else  {
  nrt = nr;
}              //  Close else.
if (nrt > 0)  
    Dlartv.dlartv(nrt,ab,(kd-l)- 1+(j1+l- 1)*ldab+ _ab_offset,inca,ab,(kd-l+1)- 1+(j1+l- 1)*ldab+ _ab_offset,inca,d,(j1)- 1+ _d_offset,work,(j1)- 1+ _work_offset,kd1);
Dummy.label("Dsbtrd",20);
}              //  Close for() loop. 
}
// *
if (wantq)  {
    // *
// *                    accumulate product of plane rotations in Q
// *
{
int _j_inc = kd1;
forloop30:
for (j = j1; (_j_inc < 0) ? j >= j2 : j <= j2; j += _j_inc) {
Drot.drot(n,q,(1)- 1+(j-1- 1)*ldq+ _q_offset,1,q,(1)- 1+(j- 1)*ldq+ _q_offset,1,d[(j)- 1+ _d_offset],work[(j)- 1+ _work_offset]);
Dummy.label("Dsbtrd",30);
}              //  Close for() loop. 
}
}              // Close if()
// *
if (j2+kdn > n)  {
    // *
// *                    adjust J2 to keep within the bounds of the matrix
// *
nr = nr-1;
j2 = j2-kdn-1;
}              // Close if()
// *
{
int _j_inc = kd1;
forloop40:
for (j = j1; (_j_inc < 0) ? j >= j2 : j <= j2; j += _j_inc) {
// *
// *                    create nonzero element a(j-1,j+kd) outside the band
// *                    and store it in WORK
// *
work[(j+kd)- 1+ _work_offset] = work[(j)- 1+ _work_offset]*ab[(1)- 1+(j+kd- 1)*ldab+ _ab_offset];
ab[(1)- 1+(j+kd- 1)*ldab+ _ab_offset] = d[(j)- 1+ _d_offset]*ab[(1)- 1+(j+kd- 1)*ldab+ _ab_offset];
Dummy.label("Dsbtrd",40);
}              //  Close for() loop. 
}
Dummy.label("Dsbtrd",50);
}              //  Close for() loop. 
}
Dummy.label("Dsbtrd",60);
}              //  Close for() loop. 
}
}              // Close if()
// *
if (kd > 0)  {
    // *
// *           copy off-diagonal elements to E
// *
{
forloop70:
for (i = 1; i <= n-1; i++) {
e[(i)- 1+ _e_offset] = ab[(kd)- 1+(i+1- 1)*ldab+ _ab_offset];
Dummy.label("Dsbtrd",70);
}              //  Close for() loop. 
}
}              // Close if()
else  {
  // *
// *           set E to zero if original matrix was diagonal
// *
{
forloop80:
for (i = 1; i <= n-1; i++) {
e[(i)- 1+ _e_offset] = zero;
Dummy.label("Dsbtrd",80);
}              //  Close for() loop. 
}
}              //  Close else.
// *
// *        copy diagonal elements to D
// *
{
forloop90:
for (i = 1; i <= n; i++) {
d[(i)- 1+ _d_offset] = ab[(kd1)- 1+(i- 1)*ldab+ _ab_offset];
Dummy.label("Dsbtrd",90);
}              //  Close for() loop. 
}
// *
}              // Close if()
else  {
  // *
if (kd > 1)  {
    // *
// *           Reduce to tridiagonal form, working with lower triangle
// *
nr = 0;
j1 = kdn+2;
j2 = 1;
// *
{
forloop150:
for (i = 1; i <= n-2; i++) {
// *
// *              Reduce i-th column of matrix to tridiagonal form
// *
{
int _k_inc = -1;
forloop140:
for (k = kdn+1; k >= 2; k += _k_inc) {
j1 = j1+kdn;
j2 = j2+kdn;
// *
if (nr > 0)  {
    // *
// *                    generate plane rotations to annihilate nonzero
// *                    elements which have been created outside the band
// *
Dlargv.dlargv(nr,ab,(kd1)- 1+(j1-kd1- 1)*ldab+ _ab_offset,inca,work,(j1)- 1+ _work_offset,kd1,d,(j1)- 1+ _d_offset,kd1);
// *
// *                    apply plane rotations from one side
// *
{
forloop100:
for (l = 1; l <= kd-1; l++) {
Dlartv.dlartv(nr,ab,(kd1-l)- 1+(j1-kd1+l- 1)*ldab+ _ab_offset,inca,ab,(kd1-l+1)- 1+(j1-kd1+l- 1)*ldab+ _ab_offset,inca,d,(j1)- 1+ _d_offset,work,(j1)- 1+ _work_offset,kd1);
Dummy.label("Dsbtrd",100);
}              //  Close for() loop. 
}
}              // Close if()
// *
if (k > 2)  {
    if (k <= n-i+1)  {
    // *
// *                       generate plane rotation to annihilate a(i+k-1,i)
// *                       within the band
// *
dlartg_adapter(ab[(k-1)- 1+(i- 1)*ldab+ _ab_offset],ab[(k)- 1+(i- 1)*ldab+ _ab_offset],d,(i+k-1)- 1+ _d_offset,work,(i+k-1)- 1+ _work_offset,temp);
ab[(k-1)- 1+(i- 1)*ldab+ _ab_offset] = temp.val;
// *
// *                       apply rotation from the left
// *
Drot.drot(k-3,ab,(k-2)- 1+(i+1- 1)*ldab+ _ab_offset,ldab-1,ab,(k-1)- 1+(i+1- 1)*ldab+ _ab_offset,ldab-1,d[(i+k-1)- 1+ _d_offset],work[(i+k-1)- 1+ _work_offset]);
}              // Close if()
nr = nr+1;
j1 = j1-kdn-1;
}              // Close if()
// *
// *                 apply plane rotations from both sides to diagonal
// *                 blocks
// *
if (nr > 0)  
    Dlar2v.dlar2v(nr,ab,(1)- 1+(j1-1- 1)*ldab+ _ab_offset,ab,(1)- 1+(j1- 1)*ldab+ _ab_offset,ab,(2)- 1+(j1-1- 1)*ldab+ _ab_offset,inca,d,(j1)- 1+ _d_offset,work,(j1)- 1+ _work_offset,kd1);
// *
// *                 apply plane rotations from the right
// *
{
forloop110:
for (l = 1; l <= kd-1; l++) {
if (j2+l > n)  {
    nrt = nr-1;
}              // Close if()
else  {
  nrt = nr;
}              //  Close else.
if (nrt > 0)  
    Dlartv.dlartv(nrt,ab,(l+2)- 1+(j1-1- 1)*ldab+ _ab_offset,inca,ab,(l+1)- 1+(j1- 1)*ldab+ _ab_offset,inca,d,(j1)- 1+ _d_offset,work,(j1)- 1+ _work_offset,kd1);
Dummy.label("Dsbtrd",110);
}              //  Close for() loop. 
}
// *
if (wantq)  {
    // *
// *                    accumulate product of plane rotations in Q
// *
{
int _j_inc = kd1;
forloop120:
for (j = j1; (_j_inc < 0) ? j >= j2 : j <= j2; j += _j_inc) {
Drot.drot(n,q,(1)- 1+(j-1- 1)*ldq+ _q_offset,1,q,(1)- 1+(j- 1)*ldq+ _q_offset,1,d[(j)- 1+ _d_offset],work[(j)- 1+ _work_offset]);
Dummy.label("Dsbtrd",120);
}              //  Close for() loop. 
}
}              // Close if()
// *
if (j2+kdn > n)  {
    // *
// *                    adjust J2 to keep within the bounds of the matrix
// *
nr = nr-1;
j2 = j2-kdn-1;
}              // Close if()
// *
{
int _j_inc = kd1;
forloop130:
for (j = j1; (_j_inc < 0) ? j >= j2 : j <= j2; j += _j_inc) {
// *
// *                    create nonzero element a(j+kd,j-1) outside the
// *                    band and store it in WORK
// *
work[(j+kd)- 1+ _work_offset] = work[(j)- 1+ _work_offset]*ab[(kd1)- 1+(j- 1)*ldab+ _ab_offset];
ab[(kd1)- 1+(j- 1)*ldab+ _ab_offset] = d[(j)- 1+ _d_offset]*ab[(kd1)- 1+(j- 1)*ldab+ _ab_offset];
Dummy.label("Dsbtrd",130);
}              //  Close for() loop. 
}
Dummy.label("Dsbtrd",140);
}              //  Close for() loop. 
}
Dummy.label("Dsbtrd",150);
}              //  Close for() loop. 
}
}              // Close if()
// *
if (kd > 0)  {
    // *
// *           copy off-diagonal elements to E
// *
{
forloop160:
for (i = 1; i <= n-1; i++) {
e[(i)- 1+ _e_offset] = ab[(2)- 1+(i- 1)*ldab+ _ab_offset];
Dummy.label("Dsbtrd",160);
}              //  Close for() loop. 
}
}              // Close if()
else  {
  // *
// *           set E to zero if original matrix was diagonal
// *
{
forloop170:
for (i = 1; i <= n-1; i++) {
e[(i)- 1+ _e_offset] = zero;
Dummy.label("Dsbtrd",170);
}              //  Close for() loop. 
}
}              //  Close else.
// *
// *        copy diagonal elements to D
// *
{
forloop180:
for (i = 1; i <= n; i++) {
d[(i)- 1+ _d_offset] = ab[(1)- 1+(i- 1)*ldab+ _ab_offset];
Dummy.label("Dsbtrd",180);
}              //  Close for() loop. 
}
}              //  Close else.
// *
Dummy.go_to("Dsbtrd",999999);
// *
// *     End of DSBTRD
// *
Dummy.label("Dsbtrd",999999);
return;
   }
// adapter for dlartg
private static void dlartg_adapter(double arg0 ,double arg1 ,double [] arg2 , int arg2_offset ,double [] arg3 , int arg3_offset ,doubleW arg4 )
{
doubleW _f2j_tmp2 = new doubleW(arg2[arg2_offset]);
doubleW _f2j_tmp3 = new doubleW(arg3[arg3_offset]);

Dlartg.dlartg(arg0,arg1,_f2j_tmp2,_f2j_tmp3,arg4);

arg2[arg2_offset] = _f2j_tmp2.val;
arg3[arg3_offset] = _f2j_tmp3.val;
}

} // End class.
