/*
 *  Produced by f2java.  f2java is part of the Fortran-
 *  -to-Java project at the University of Tennessee Netlib
 *  numerical software repository.
 *
 *  Original authorship for the BLAS and LAPACK numerical
 *  routines may be found in the Fortran source, available at
 *  www.netlib.org.
 *
 *  Fortran input file: jlapack.f
 *
 *  The f2j compiler code was written by
 *  David M. Doolin (doolin@cs.utk.edu) and
 *  Keith  Seymour (seymour@cs.utk.edu)
 */

package org.netlib.lapack;
import java.lang.*;
import org.netlib.util.*;



public class Dspcon {

// *
// *  -- LAPACK routine (version 2.0) --
// *     Univ. of Tennessee, Univ. of California Berkeley, NAG Ltd.,
// *     Courant Institute, Argonne National Lab, and Rice University
// *     March 31, 1993
// *
// *     .. Scalar Arguments ..
// *     ..
// *     .. Array Arguments ..
// *     ..
// *
// *  Purpose
// *  =======
// *
// *  DSPCON estimates the reciprocal of the condition number (in the
// *  1-norm) of a real symmetric packed matrix A using the factorization
// *  A = U*D*U**T or A = L*D*L**T computed by DSPTRF.
// *
// *  An estimate is obtained for norm(inv(A)), and the reciprocal of the
// *  condition number is computed as RCOND = 1 / (ANORM * norm(inv(A))).
// *
// *  Arguments
// *  =========
// *
// *  UPLO    (input) CHARACTER*1
// *          Specifies whether the details of the factorization are stored
// *          as an upper or lower triangular matrix.
// *          = 'U':  Upper triangular, form is A = U*D*U**T;
// *          = 'L':  Lower triangular, form is A = L*D*L**T.
// *
// *  N       (input) INTEGER
// *          The order of the matrix A.  N >= 0.
// *
// *  AP      (input) DOUBLE PRECISION array, dimension (N*(N+1)/2)
// *          The block diagonal matrix D and the multipliers used to
// *          obtain the factor U or L as computed by DSPTRF, stored as a
// *          packed triangular matrix.
// *
// *  IPIV    (input) INTEGER array, dimension (N)
// *          Details of the interchanges and the block structure of D
// *          as determined by DSPTRF.
// *
// *  ANORM   (input) DOUBLE PRECISION
// *          The 1-norm of the original matrix A.
// *
// *  RCOND   (output) DOUBLE PRECISION
// *          The reciprocal of the condition number of the matrix A,
// *          computed as RCOND = 1/(ANORM * AINVNM), where AINVNM is an
// *          estimate of the 1-norm of inv(A) computed in this routine.
// *
// *  WORK    (workspace) DOUBLE PRECISION array, dimension (2*N)
// *
// *  IWORK    (workspace) INTEGER array, dimension (N)
// *
// *  INFO    (output) INTEGER
// *          = 0:  successful exit
// *          < 0:  if INFO = -i, the i-th argument had an illegal value
// *
// *  =====================================================================
// *
// *     .. Parameters ..
static double one= 1.0e+0;
static double zero= 0.0e+0;
// *     ..
// *     .. Local Scalars ..
static boolean upper= false;
static int i= 0;
static int ip= 0;
static intW kase= new intW(0);
static doubleW ainvnm= new doubleW(0.0);
// *     ..
// *     .. External Functions ..
// *     ..
// *     .. External Subroutines ..
// *     ..
// *     .. Executable Statements ..
// *
// *     Test the input parameters.
// *

public static void dspcon (String uplo,
int n,
double [] ap, int _ap_offset,
int [] ipiv, int _ipiv_offset,
double anorm,
doubleW rcond,
double [] work, int _work_offset,
int [] iwork, int _iwork_offset,
intW info)  {

info.val = 0;
upper = (uplo.toLowerCase().charAt(0) == "U".toLowerCase().charAt(0));
if (!upper && !(uplo.toLowerCase().charAt(0) == "L".toLowerCase().charAt(0)))  {
    info.val = -1;
}              // Close if()
else if (n < 0)  {
    info.val = -2;
}              // Close else if()
else if (anorm < zero)  {
    info.val = -5;
}              // Close else if()
if (info.val != 0)  {
    Xerbla.xerbla("DSPCON",-info.val);
Dummy.go_to("Dspcon",999999);
}              // Close if()
// *
// *     Quick return if possible
// *
rcond.val = zero;
if (n == 0)  {
    rcond.val = one;
Dummy.go_to("Dspcon",999999);
}              // Close if()
else if (anorm <= zero)  {
    Dummy.go_to("Dspcon",999999);
}              // Close else if()
// *
// *     Check that the diagonal matrix D is nonsingular.
// *
if (upper)  {
    // *
// *        Upper triangular storage: examine D from bottom to top
// *
ip = n*(n+1)/2;
{
int _i_inc = -1;
forloop10:
for (i = n; i >= 1; i += _i_inc) {
if (ipiv[(i)- 1+ _ipiv_offset] > 0 && ap[(ip)- 1+ _ap_offset] == zero)  
    Dummy.go_to("Dspcon",999999);
ip = ip-i;
Dummy.label("Dspcon",10);
}              //  Close for() loop. 
}
}              // Close if()
else  {
  // *
// *        Lower triangular storage: examine D from top to bottom.
// *
ip = 1;
{
forloop20:
for (i = 1; i <= n; i++) {
if (ipiv[(i)- 1+ _ipiv_offset] > 0 && ap[(ip)- 1+ _ap_offset] == zero)  
    Dummy.go_to("Dspcon",999999);
ip = ip+n-i+1;
Dummy.label("Dspcon",20);
}              //  Close for() loop. 
}
}              //  Close else.
// *
// *     Estimate the 1-norm of the inverse.
// *
kase.val = 0;
label30:
   Dummy.label("Dspcon",30);
Dlacon.dlacon(n,work,(n+1)- 1+ _work_offset,work,_work_offset,iwork,_iwork_offset,ainvnm,kase);
if (kase.val != 0)  {
    // *
// *        Multiply by inv(L*D*L') or inv(U*D*U').
// *
Dsptrs.dsptrs(uplo,n,1,ap,_ap_offset,ipiv,_ipiv_offset,work,_work_offset,n,info);
Dummy.go_to("Dspcon",30);
}              // Close if()
// *
// *     Compute the estimate of the reciprocal condition number.
// *
if (ainvnm.val != zero)  
    rcond.val = (one/ainvnm.val)/anorm;
// *
Dummy.go_to("Dspcon",999999);
// *
// *     End of DSPCON
// *
Dummy.label("Dspcon",999999);
return;
   }
} // End class.
