/*
 *  Produced by f2java.  f2java is part of the Fortran-
 *  -to-Java project at the University of Tennessee Netlib
 *  numerical software repository.
 *
 *  Original authorship for the BLAS and LAPACK numerical
 *  routines may be found in the Fortran source, available at
 *  www.netlib.org.
 *
 *  Fortran input file: jlapack.f
 *
 *  The f2j compiler code was written by
 *  David M. Doolin (doolin@cs.utk.edu) and
 *  Keith  Seymour (seymour@cs.utk.edu)
 */

package org.netlib.lapack;
import java.lang.*;
import org.netlib.util.*;

import org.netlib.blas.*;


public class Dspev {

// *
// *  -- LAPACK driver routine (version 2.0) --
// *     Univ. of Tennessee, Univ. of California Berkeley, NAG Ltd.,
// *     Courant Institute, Argonne National Lab, and Rice University
// *     March 31, 1993
// *
// *     .. Scalar Arguments ..
// *     ..
// *     .. Array Arguments ..
// *     ..
// *
// *  Purpose
// *  =======
// *
// *  DSPEV computes all the eigenvalues and, optionally, eigenvectors of a
// *  real symmetric matrix A in packed storage.
// *
// *  Arguments
// *  =========
// *
// *  JOBZ    (input) CHARACTER*1
// *          = 'N':  Compute eigenvalues only;
// *          = 'V':  Compute eigenvalues and eigenvectors.
// *
// *  UPLO    (input) CHARACTER*1
// *          = 'U':  Upper triangle of A is stored;
// *          = 'L':  Lower triangle of A is stored.
// *
// *  N       (input) INTEGER
// *          The order of the matrix A.  N >= 0.
// *
// *  AP      (input/output) DOUBLE PRECISION array, dimension (N*(N+1)/2)
// *          On entry, the upper or lower triangle of the symmetric matrix
// *          A, packed columnwise in a linear array.  The j-th column of A
// *          is stored in the array AP as follows:
// *          if UPLO = 'U', AP(i + (j-1)*j/2) = A(i,j) for 1<=i<=j;
// *          if UPLO = 'L', AP(i + (j-1)*(2*n-j)/2) = A(i,j) for j<=i<=n.
// *
// *          On exit, AP is overwritten by values generated during the
// *          reduction to tridiagonal form.  If UPLO = 'U', the diagonal
// *          and first superdiagonal of the tridiagonal matrix T overwrite
// *          the corresponding elements of A, and if UPLO = 'L', the
// *          diagonal and first subdiagonal of T overwrite the
// *          corresponding elements of A.
// *
// *  W       (output) DOUBLE PRECISION array, dimension (N)
// *          If INFO = 0, the eigenvalues in ascending order.
// *
// *  Z       (output) DOUBLE PRECISION array, dimension (LDZ, N)
// *          If JOBZ = 'V', then if INFO = 0, Z contains the orthonormal
// *          eigenvectors of the matrix A, with the i-th column of Z
// *          holding the eigenvector associated with W(i).
// *          If JOBZ = 'N', then Z is not referenced.
// *
// *  LDZ     (input) INTEGER
// *          The leading dimension of the array Z.  LDZ >= 1, and if
// *          JOBZ = 'V', LDZ >= max(1,N).
// *
// *  WORK    (workspace) DOUBLE PRECISION array, dimension (3*N)
// *
// *  INFO    (output) INTEGER
// *          = 0:  successful exit.
// *          < 0:  if INFO = -i, the i-th argument had an illegal value.
// *          > 0:  if INFO = i, the algorithm failed to converge; i
// *                off-diagonal elements of an intermediate tridiagonal
// *                form did not converge to zero.
// *
// *  =====================================================================
// *
// *     .. Parameters ..
static double zero= 0.0e0;
static double one= 1.0e0;
// *     ..
// *     .. Local Scalars ..
static boolean wantz= false;
static intW iinfo= new intW(0);
static int imax= 0;
static int inde= 0;
static int indtau= 0;
static int indwrk= 0;
static int iscale= 0;
static double anrm= 0.0;
static double bignum= 0.0;
static double eps= 0.0;
static double rmax= 0.0;
static double rmin= 0.0;
static double safmin= 0.0;
static double sigma= 0.0;
static double smlnum= 0.0;
// *     ..
// *     .. External Functions ..
// *     ..
// *     .. External Subroutines ..
// *     ..
// *     .. Intrinsic Functions ..
// *     ..
// *     .. Executable Statements ..
// *
// *     Test the input parameters.
// *

public static void dspev (String jobz,
String uplo,
int n,
double [] ap, int _ap_offset,
double [] w, int _w_offset,
double [] z, int _z_offset,
int ldz,
double [] work, int _work_offset,
intW info)  {

wantz = (jobz.toLowerCase().charAt(0) == "V".toLowerCase().charAt(0));
// *
info.val = 0;
if (!(wantz || (jobz.toLowerCase().charAt(0) == "N".toLowerCase().charAt(0))))  {
    info.val = -1;
}              // Close if()
else if (!((uplo.toLowerCase().charAt(0) == "U".toLowerCase().charAt(0)) || (uplo.toLowerCase().charAt(0) == "L".toLowerCase().charAt(0))))  {
    info.val = -2;
}              // Close else if()
else if (n < 0)  {
    info.val = -3;
}              // Close else if()
else if (ldz < 1 || (wantz && ldz < n))  {
    info.val = -7;
}              // Close else if()
// *
if (info.val != 0)  {
    Xerbla.xerbla("DSPEV ",-info.val);
Dummy.go_to("Dspev",999999);
}              // Close if()
// *
// *     Quick return if possible
// *
if (n == 0)  
    Dummy.go_to("Dspev",999999);
// *
if (n == 1)  {
    w[(1)- 1+ _w_offset] = ap[(1)- 1+ _ap_offset];
if (wantz)  
    z[(1)- 1+(1- 1)*ldz+ _z_offset] = one;
Dummy.go_to("Dspev",999999);
}              // Close if()
// *
// *     Get machine constants.
// *
safmin = Dlamch.dlamch("Safe minimum");
eps = Dlamch.dlamch("Precision");
smlnum = safmin/eps;
bignum = one/smlnum;
rmin = Math.sqrt(smlnum);
rmax = Math.sqrt(bignum);
// *
// *     Scale matrix to allowable range, if necessary.
// *
anrm = Dlansp.dlansp("M",uplo,n,ap,_ap_offset,work,_work_offset);
iscale = 0;
if (anrm > zero && anrm < rmin)  {
    iscale = 1;
sigma = rmin/anrm;
}              // Close if()
else if (anrm > rmax)  {
    iscale = 1;
sigma = rmax/anrm;
}              // Close else if()
if (iscale == 1)  {
    Dscal.dscal((n*(n+1))/2,sigma,ap,_ap_offset,1);
}              // Close if()
// *
// *     Call DSPTRD to reduce symmetric packed matrix to tridiagonal form.
// *
inde = 1;
indtau = inde+n;
Dsptrd.dsptrd(uplo,n,ap,_ap_offset,w,_w_offset,work,(inde)- 1+ _work_offset,work,(indtau)- 1+ _work_offset,iinfo);
// *
// *     For eigenvalues only, call DSTERF.  For eigenvectors, first call
// *     DOPGTR to generate the orthogonal matrix, then call DSTEQR.
// *
if (!wantz)  {
    Dsterf.dsterf(n,w,_w_offset,work,(inde)- 1+ _work_offset,info);
}              // Close if()
else  {
  indwrk = indtau+n;
Dopgtr.dopgtr(uplo,n,ap,_ap_offset,work,(indtau)- 1+ _work_offset,z,_z_offset,ldz,work,(indwrk)- 1+ _work_offset,iinfo);
Dsteqr.dsteqr(jobz,n,w,_w_offset,work,(inde)- 1+ _work_offset,z,_z_offset,ldz,work,(indtau)- 1+ _work_offset,info);
}              //  Close else.
// *
// *     If matrix was scaled, then rescale eigenvalues appropriately.
// *
if (iscale == 1)  {
    if (info.val == 0)  {
    imax = n;
}              // Close if()
else  {
  imax = info.val-1;
}              //  Close else.
Dscal.dscal(imax,one/sigma,w,_w_offset,1);
}              // Close if()
// *
Dummy.go_to("Dspev",999999);
// *
// *     End of DSPEV
// *
Dummy.label("Dspev",999999);
return;
   }
} // End class.
