/*
 *  Produced by f2java.  f2java is part of the Fortran-
 *  -to-Java project at the University of Tennessee Netlib
 *  numerical software repository.
 *
 *  Original authorship for the BLAS and LAPACK numerical
 *  routines may be found in the Fortran source, available at
 *  www.netlib.org.
 *
 *  Fortran input file: jlapack.f
 *
 *  The f2j compiler code was written by
 *  David M. Doolin (doolin@cs.utk.edu) and
 *  Keith  Seymour (seymour@cs.utk.edu)
 */

package org.netlib.lapack;
import java.lang.*;
import org.netlib.util.*;

import org.netlib.blas.*;


public class Dspevd {

// *
// *
// *  -- LAPACK driver routine (version 2.0) --
// *     Univ. of Tennessee, Univ. of California Berkeley, NAG Ltd.,
// *     Courant Institute, Argonne National Lab, and Rice University
// *     September 30, 1994
// *
// *     .. Scalar Arguments ..
// *     ..
// *     .. Array Arguments ..
// *     ..
// *
// *  Purpose
// *  =======
// *
// *  DSPEVD computes all the eigenvalues and, optionally, eigenvectors
// *  of a real symmetric matrix A in packed storage. If eigenvectors are
// *  desired, it uses a divide and conquer algorithm.
// *
// *  The divide and conquer algorithm makes very mild assumptions about
// *  floating point arithmetic. It will work on machines with a guard
// *  digit in add/subtract, or on those binary machines without guard
// *  digits which subtract like the Cray X-MP, Cray Y-MP, Cray C-90, or
// *  Cray-2. It could conceivably fail on hexadecimal or decimal machines
// *  without guard digits, but we know of none.
// *
// *  Arguments
// *  =========
// *
// *  JOBZ    (input) CHARACTER*1
// *          = 'N':  Compute eigenvalues only;
// *          = 'V':  Compute eigenvalues and eigenvectors.
// *
// *  UPLO    (input) CHARACTER*1
// *          = 'U':  Upper triangle of A is stored;
// *          = 'L':  Lower triangle of A is stored.
// *
// *  N       (input) INTEGER
// *          The order of the matrix A.  N >= 0.
// *
// *  AP      (input/output) DOUBLE PRECISION array, dimension (N*(N+1)/2)
// *          On entry, the upper or lower triangle of the symmetric matrix
// *          A, packed columnwise in a linear array.  The j-th column of A
// *          is stored in the array AP as follows:
// *          if UPLO = 'U', AP(i + (j-1)*j/2) = A(i,j) for 1<=i<=j;
// *          if UPLO = 'L', AP(i + (j-1)*(2*n-j)/2) = A(i,j) for j<=i<=n.
// *
// *          On exit, AP is overwritten by values generated during the
// *          reduction to tridiagonal form.  If UPLO = 'U', the diagonal
// *          and first superdiagonal of the tridiagonal matrix T overwrite
// *          the corresponding elements of A, and if UPLO = 'L', the
// *          diagonal and first subdiagonal of T overwrite the
// *          corresponding elements of A.
// *
// *  W       (output) DOUBLE PRECISION array, dimension (N)
// *          If INFO = 0, the eigenvalues in ascending order.
// *
// *  Z       (output) DOUBLE PRECISION array, dimension (LDZ, N)
// *          If JOBZ = 'V', then if INFO = 0, Z contains the orthonormal
// *          eigenvectors of the matrix A, with the i-th column of Z
// *          holding the eigenvector associated with W(i).
// *          If JOBZ = 'N', then Z is not referenced.
// *
// *  LDZ     (input) INTEGER
// *          The leading dimension of the array Z.  LDZ >= 1, and if
// *          JOBZ = 'V', LDZ >= max(1,N).
// *
// *  WORK    (workspace/output) DOUBLE PRECISION array,
// *                                         dimension (LWORK)
// *          On exit, if LWORK > 0, WORK(1) returns the optimal LWORK.
// *
// *  LWORK   (input) INTEGER
// *          The dimension of the array WORK.
// *          If N <= 1,               LWORK must be at least 1.
// *          If JOBZ = 'N' and N > 1, LWORK must be at least 2*N.
// *          If JOBZ = 'V' and N > 1, LWORK must be at least
// *                         ( 1 + 5*N + 2*N*lg N + 2*N**2 ),
// *                         where lg( N ) = smallest integer k such
// *                                         that 2**k >= N.
// *
// *  IWORK   (workspace/output) INTEGER array, dimension (LIWORK)
// *          On exit, if LIWORK > 0, IWORK(1) returns the optimal LIWORK.
// *
// *  LIWORK  (input) INTEGER
// *          The dimension of the array IWORK.
// *          If JOBZ  = 'N' or N <= 1, LIWORK must be at least 1.
// *          If JOBZ  = 'V' and N > 1, LIWORK must be at least 2 + 5*N.
// *
// *  INFO    (output) INTEGER
// *          = 0:  successful exit
// *          < 0:  if INFO = -i, the i-th argument had an illegal value.
// *          > 0:  if INFO = i, the algorithm failed to converge; i
// *                off-diagonal elements of an intermediate tridiagonal
// *                form did not converge to zero.
// *
// *  =====================================================================
// *
// *     .. Parameters ..
static double zero= 0.0e+0;
static double one= 1.0e+0;
static double two= 2.0e+0;
// *     ..
// *     .. Local Scalars ..
static boolean wantz= false;
static intW iinfo= new intW(0);
static int inde= 0;
static int indtau= 0;
static int indwrk= 0;
static int iscale= 0;
static int lgn= 0;
static int liwmin= 0;
static int llwork= 0;
static int lwmin= 0;
static double anrm= 0.0;
static double bignum= 0.0;
static double eps= 0.0;
static double rmax= 0.0;
static double rmin= 0.0;
static double safmin= 0.0;
static double sigma= 0.0;
static double smlnum= 0.0;
// *     ..
// *     .. External Functions ..
// *     ..
// *     .. External Subroutines ..
// *     ..
// *     .. Intrinsic Functions ..
// *     ..
// *     .. Executable Statements ..
// *
// *     Test the input parameters.
// *

public static void dspevd (String jobz,
String uplo,
int n,
double [] ap, int _ap_offset,
double [] w, int _w_offset,
double [] z, int _z_offset,
int ldz,
double [] work, int _work_offset,
int lwork,
int [] iwork, int _iwork_offset,
int liwork,
intW info)  {

wantz = (jobz.toLowerCase().charAt(0) == "V".toLowerCase().charAt(0));
// *
info.val = 0;
if (n <= 1)  {
    lgn = 0;
liwmin = 1;
lwmin = 1;
}              // Close if()
else  {
  lgn = (int)(Math.log((double)(n))/Math.log(two));
if (Math.pow(2, lgn) < n)  
    lgn = lgn+1;
if (Math.pow(2, lgn) < n)  
    lgn = lgn+1;
if (wantz)  {
    liwmin = 2+5*n;
lwmin = (int)(1+5*n+2*n*lgn+2*Math.pow(n, 2));
}              // Close if()
else  {
  liwmin = 1;
lwmin = 2*n;
}              //  Close else.
}              //  Close else.
if (!(wantz || (jobz.toLowerCase().charAt(0) == "N".toLowerCase().charAt(0))))  {
    info.val = -1;
}              // Close if()
else if (!((uplo.toLowerCase().charAt(0) == "U".toLowerCase().charAt(0)) || (uplo.toLowerCase().charAt(0) == "L".toLowerCase().charAt(0))))  {
    info.val = -2;
}              // Close else if()
else if (n < 0)  {
    info.val = -3;
}              // Close else if()
else if (ldz < 1 || (wantz && ldz < n))  {
    info.val = -7;
}              // Close else if()
else if (lwork < lwmin)  {
    info.val = -9;
}              // Close else if()
else if (liwork < liwmin)  {
    info.val = -11;
}              // Close else if()
// *
if (info.val != 0)  {
    Xerbla.xerbla("DSPEVD ",-info.val);
Dummy.go_to("Dspevd",10);
}              // Close if()
// *
// *     Quick return if possible
// *
if (n == 0)  
    Dummy.go_to("Dspevd",10);
// *
if (n == 1)  {
    w[(1)- 1+ _w_offset] = ap[(1)- 1+ _ap_offset];
if (wantz)  
    z[(1)- 1+(1- 1)*ldz+ _z_offset] = one;
Dummy.go_to("Dspevd",10);
}              // Close if()
// *
// *     Get machine constants.
// *
safmin = Dlamch.dlamch("Safe minimum");
eps = Dlamch.dlamch("Precision");
smlnum = safmin/eps;
bignum = one/smlnum;
rmin = Math.sqrt(smlnum);
rmax = Math.sqrt(bignum);
// *
// *     Scale matrix to allowable range, if necessary.
// *
anrm = Dlansp.dlansp("M",uplo,n,ap,_ap_offset,work,_work_offset);
iscale = 0;
if (anrm > zero && anrm < rmin)  {
    iscale = 1;
sigma = rmin/anrm;
}              // Close if()
else if (anrm > rmax)  {
    iscale = 1;
sigma = rmax/anrm;
}              // Close else if()
if (iscale == 1)  {
    Dscal.dscal((n*(n+1))/2,sigma,ap,_ap_offset,1);
}              // Close if()
// *
// *     Call DSPTRD to reduce symmetric packed matrix to tridiagonal form.
// *
inde = 1;
indtau = inde+n;
Dsptrd.dsptrd(uplo,n,ap,_ap_offset,w,_w_offset,work,(inde)- 1+ _work_offset,work,(indtau)- 1+ _work_offset,iinfo);
// *
// *     For eigenvalues only, call DSTERF.  For eigenvectors, first call
// *     DSTEDC to generate the eigenvector matrix, WORK(INDWRK), of the
// *     tridiagonal matrix, then call DOPMTR to multiply it by the
// *     Householder transformations represented in AP.
// *
if (!wantz)  {
    Dsterf.dsterf(n,w,_w_offset,work,(inde)- 1+ _work_offset,info);
}              // Close if()
else  {
  indwrk = indtau+n;
llwork = lwork-indwrk+1;
Dstedc.dstedc("I",n,w,_w_offset,work,(inde)- 1+ _work_offset,z,_z_offset,ldz,work,(indwrk)- 1+ _work_offset,llwork,iwork,_iwork_offset,liwork,info);
Dopmtr.dopmtr("L",uplo,"N",n,n,ap,_ap_offset,work,(indtau)- 1+ _work_offset,z,_z_offset,ldz,work,(indwrk)- 1+ _work_offset,iinfo);
}              //  Close else.
// *
// *     If matrix was scaled, then rescale eigenvalues appropriately.
// *
if (iscale == 1)  
    Dscal.dscal(n,one/sigma,w,_w_offset,1);
// *
label10:
   Dummy.label("Dspevd",10);
if (lwork > 0)  
    work[(1)- 1+ _work_offset] = (double)(lwmin);
if (liwork > 0)  
    iwork[(1)- 1+ _iwork_offset] = liwmin;
Dummy.go_to("Dspevd",999999);
// *
// *     End of DSPEVD
// *
Dummy.label("Dspevd",999999);
return;
   }
} // End class.
