/*
 *  Produced by f2java.  f2java is part of the Fortran-
 *  -to-Java project at the University of Tennessee Netlib
 *  numerical software repository.
 *
 *  Original authorship for the BLAS and LAPACK numerical
 *  routines may be found in the Fortran source, available at
 *  www.netlib.org.
 *
 *  Fortran input file: jlapack.f
 *
 *  The f2j compiler code was written by
 *  David M. Doolin (doolin@cs.utk.edu) and
 *  Keith  Seymour (seymour@cs.utk.edu)
 */

package org.netlib.lapack;
import java.lang.*;
import org.netlib.util.*;

import org.netlib.blas.*;


public class Dspgst {

// *
// *  -- LAPACK routine (version 2.0) --
// *     Univ. of Tennessee, Univ. of California Berkeley, NAG Ltd.,
// *     Courant Institute, Argonne National Lab, and Rice University
// *     March 31, 1993
// *
// *     .. Scalar Arguments ..
// *     ..
// *     .. Array Arguments ..
// *     ..
// *
// *  Purpose
// *  =======
// *
// *  DSPGST reduces a real symmetric-definite generalized eigenproblem
// *  to standard form, using packed storage.
// *
// *  If ITYPE = 1, the problem is A*x = lambda*B*x,
// *  and A is overwritten by inv(U**T)*A*inv(U) or inv(L)*A*inv(L**T)
// *
// *  If ITYPE = 2 or 3, the problem is A*B*x = lambda*x or
// *  B*A*x = lambda*x, and A is overwritten by U*A*U**T or L**T*A*L.
// *
// *  B must have been previously factorized as U**T*U or L*L**T by DPPTRF.
// *
// *  Arguments
// *  =========
// *
// *  ITYPE   (input) INTEGER
// *          = 1: compute inv(U**T)*A*inv(U) or inv(L)*A*inv(L**T);
// *          = 2 or 3: compute U*A*U**T or L**T*A*L.
// *
// *  UPLO    (input) CHARACTER
// *          = 'U':  Upper triangle of A is stored and B is factored as
// *                  U**T*U;
// *          = 'L':  Lower triangle of A is stored and B is factored as
// *                  L*L**T.
// *
// *  N       (input) INTEGER
// *          The order of the matrices A and B.  N >= 0.
// *
// *  AP      (input/output) DOUBLE PRECISION array, dimension (N*(N+1)/2)
// *          On entry, the upper or lower triangle of the symmetric matrix
// *          A, packed columnwise in a linear array.  The j-th column of A
// *          is stored in the array AP as follows:
// *          if UPLO = 'U', AP(i + (j-1)*j/2) = A(i,j) for 1<=i<=j;
// *          if UPLO = 'L', AP(i + (j-1)*(2n-j)/2) = A(i,j) for j<=i<=n.
// *
// *          On exit, if INFO = 0, the transformed matrix, stored in the
// *          same format as A.
// *
// *  BP      (input) DOUBLE PRECISION array, dimension (N*(N+1)/2)
// *          The triangular factor from the Cholesky factorization of B,
// *          stored in the same format as A, as returned by DPPTRF.
// *
// *  INFO    (output) INTEGER
// *          = 0:  successful exit
// *          < 0:  if INFO = -i, the i-th argument had an illegal value
// *
// *  =====================================================================
// *
// *     .. Parameters ..
static double one= 1.0e0;
static double half= 0.5e0;
// *     ..
// *     .. Local Scalars ..
static boolean upper= false;
static int j= 0;
static int j1= 0;
static int j1j1= 0;
static int jj= 0;
static int k= 0;
static int k1= 0;
static int k1k1= 0;
static int kk= 0;
static double ajj= 0.0;
static double akk= 0.0;
static double bjj= 0.0;
static double bkk= 0.0;
static double ct= 0.0;
// *     ..
// *     .. External Subroutines ..
// *     ..
// *     .. External Functions ..
// *     ..
// *     .. Executable Statements ..
// *
// *     Test the input parameters.
// *

public static void dspgst (int itype,
String uplo,
int n,
double [] ap, int _ap_offset,
double [] bp, int _bp_offset,
intW info)  {

info.val = 0;
upper = (uplo.toLowerCase().charAt(0) == "U".toLowerCase().charAt(0));
if (itype < 1 || itype > 3)  {
    info.val = -1;
}              // Close if()
else if (!upper && !(uplo.toLowerCase().charAt(0) == "L".toLowerCase().charAt(0)))  {
    info.val = -2;
}              // Close else if()
else if (n < 0)  {
    info.val = -3;
}              // Close else if()
if (info.val != 0)  {
    Xerbla.xerbla("DSPGST",-info.val);
Dummy.go_to("Dspgst",999999);
}              // Close if()
// *
if (itype == 1)  {
    if (upper)  {
    // *
// *           Compute inv(U')*A*inv(U)
// *
// *           J1 and JJ are the indices of A(1,j) and A(j,j)
// *
jj = 0;
{
forloop10:
for (j = 1; j <= n; j++) {
j1 = jj+1;
jj = jj+j;
// *
// *              Compute the j-th column of the upper triangle of A
// *
bjj = bp[(jj)- 1+ _bp_offset];
Dtpsv.dtpsv(uplo,"Transpose","Nonunit",j,bp,_bp_offset,ap,(j1)- 1+ _ap_offset,1);
Dspmv.dspmv(uplo,j-1,-one,ap,_ap_offset,bp,(j1)- 1+ _bp_offset,1,one,ap,(j1)- 1+ _ap_offset,1);
Dscal.dscal(j-1,one/bjj,ap,(j1)- 1+ _ap_offset,1);
ap[(jj)- 1+ _ap_offset] = (ap[(jj)- 1+ _ap_offset]-Ddot.ddot(j-1,ap,(j1)- 1+ _ap_offset,1,bp,(j1)- 1+ _bp_offset,1))/bjj;
Dummy.label("Dspgst",10);
}              //  Close for() loop. 
}
}              // Close if()
else  {
  // *
// *           Compute inv(L)*A*inv(L')
// *
// *           KK and K1K1 are the indices of A(k,k) and A(k+1,k+1)
// *
kk = 1;
{
forloop20:
for (k = 1; k <= n; k++) {
k1k1 = kk+n-k+1;
// *
// *              Update the lower triangle of A(k:n,k:n)
// *
akk = ap[(kk)- 1+ _ap_offset];
bkk = bp[(kk)- 1+ _bp_offset];
akk = akk/Math.pow(bkk, 2);
ap[(kk)- 1+ _ap_offset] = akk;
if (k < n)  {
    Dscal.dscal(n-k,one/bkk,ap,(kk+1)- 1+ _ap_offset,1);
ct = -half*akk;
Daxpy.daxpy(n-k,ct,bp,(kk+1)- 1+ _bp_offset,1,ap,(kk+1)- 1+ _ap_offset,1);
Dspr2.dspr2(uplo,n-k,-one,ap,(kk+1)- 1+ _ap_offset,1,bp,(kk+1)- 1+ _bp_offset,1,ap,(k1k1)- 1+ _ap_offset);
Daxpy.daxpy(n-k,ct,bp,(kk+1)- 1+ _bp_offset,1,ap,(kk+1)- 1+ _ap_offset,1);
Dtpsv.dtpsv(uplo,"No transpose","Non-unit",n-k,bp,(k1k1)- 1+ _bp_offset,ap,(kk+1)- 1+ _ap_offset,1);
}              // Close if()
kk = k1k1;
Dummy.label("Dspgst",20);
}              //  Close for() loop. 
}
}              //  Close else.
}              // Close if()
else  {
  if (upper)  {
    // *
// *           Compute U*A*U'
// *
// *           K1 and KK are the indices of A(1,k) and A(k,k)
// *
kk = 0;
{
forloop30:
for (k = 1; k <= n; k++) {
k1 = kk+1;
kk = kk+k;
// *
// *              Update the upper triangle of A(1:k,1:k)
// *
akk = ap[(kk)- 1+ _ap_offset];
bkk = bp[(kk)- 1+ _bp_offset];
Dtpmv.dtpmv(uplo,"No transpose","Non-unit",k-1,bp,_bp_offset,ap,(k1)- 1+ _ap_offset,1);
ct = half*akk;
Daxpy.daxpy(k-1,ct,bp,(k1)- 1+ _bp_offset,1,ap,(k1)- 1+ _ap_offset,1);
Dspr2.dspr2(uplo,k-1,one,ap,(k1)- 1+ _ap_offset,1,bp,(k1)- 1+ _bp_offset,1,ap,_ap_offset);
Daxpy.daxpy(k-1,ct,bp,(k1)- 1+ _bp_offset,1,ap,(k1)- 1+ _ap_offset,1);
Dscal.dscal(k-1,bkk,ap,(k1)- 1+ _ap_offset,1);
ap[(kk)- 1+ _ap_offset] = akk*Math.pow(bkk, 2);
Dummy.label("Dspgst",30);
}              //  Close for() loop. 
}
}              // Close if()
else  {
  // *
// *           Compute L'*A*L
// *
// *           JJ and J1J1 are the indices of A(j,j) and A(j+1,j+1)
// *
jj = 1;
{
forloop40:
for (j = 1; j <= n; j++) {
j1j1 = jj+n-j+1;
// *
// *              Compute the j-th column of the lower triangle of A
// *
ajj = ap[(jj)- 1+ _ap_offset];
bjj = bp[(jj)- 1+ _bp_offset];
ap[(jj)- 1+ _ap_offset] = ajj*bjj+Ddot.ddot(n-j,ap,(jj+1)- 1+ _ap_offset,1,bp,(jj+1)- 1+ _bp_offset,1);
Dscal.dscal(n-j,bjj,ap,(jj+1)- 1+ _ap_offset,1);
Dspmv.dspmv(uplo,n-j,one,ap,(j1j1)- 1+ _ap_offset,bp,(jj+1)- 1+ _bp_offset,1,one,ap,(jj+1)- 1+ _ap_offset,1);
Dtpmv.dtpmv(uplo,"Transpose","Non-unit",n-j+1,bp,(jj)- 1+ _bp_offset,ap,(jj)- 1+ _ap_offset,1);
jj = j1j1;
Dummy.label("Dspgst",40);
}              //  Close for() loop. 
}
}              //  Close else.
}              //  Close else.
Dummy.go_to("Dspgst",999999);
// *
// *     End of DSPGST
// *
Dummy.label("Dspgst",999999);
return;
   }
} // End class.
