/*
 *  Produced by f2java.  f2java is part of the Fortran-
 *  -to-Java project at the University of Tennessee Netlib
 *  numerical software repository.
 *
 *  Original authorship for the BLAS and LAPACK numerical
 *  routines may be found in the Fortran source, available at
 *  www.netlib.org.
 *
 *  Fortran input file: jlapack.f
 *
 *  The f2j compiler code was written by
 *  David M. Doolin (doolin@cs.utk.edu) and
 *  Keith  Seymour (seymour@cs.utk.edu)
 */

package org.netlib.lapack;
import java.lang.*;
import org.netlib.util.*;

import org.netlib.blas.*;


public class Dspgv {

// *
// *  -- LAPACK driver routine (version 2.0) --
// *     Univ. of Tennessee, Univ. of California Berkeley, NAG Ltd.,
// *     Courant Institute, Argonne National Lab, and Rice University
// *     September 30, 1994
// *
// *     .. Scalar Arguments ..
// *     ..
// *     .. Array Arguments ..
// *     ..
// *
// *  Purpose
// *  =======
// *
// *  DSPGV computes all the eigenvalues and, optionally, the eigenvectors
// *  of a real generalized symmetric-definite eigenproblem, of the form
// *  A*x=(lambda)*B*x,  A*Bx=(lambda)*x,  or B*A*x=(lambda)*x.
// *  Here A and B are assumed to be symmetric, stored in packed format,
// *  and B is also positive definite.
// *
// *  Arguments
// *  =========
// *
// *  ITYPE   (input) INTEGER
// *          Specifies the problem type to be solved:
// *          = 1:  A*x = (lambda)*B*x
// *          = 2:  A*B*x = (lambda)*x
// *          = 3:  B*A*x = (lambda)*x
// *
// *  JOBZ    (input) CHARACTER*1
// *          = 'N':  Compute eigenvalues only;
// *          = 'V':  Compute eigenvalues and eigenvectors.
// *
// *  UPLO    (input) CHARACTER*1
// *          = 'U':  Upper triangles of A and B are stored;
// *          = 'L':  Lower triangles of A and B are stored.
// *
// *  N       (input) INTEGER
// *          The order of the matrices A and B.  N >= 0.
// *
// *  AP      (input/output) DOUBLE PRECISION array, dimension
// *                            (N*(N+1)/2)
// *          On entry, the upper or lower triangle of the symmetric matrix
// *          A, packed columnwise in a linear array.  The j-th column of A
// *          is stored in the array AP as follows:
// *          if UPLO = 'U', AP(i + (j-1)*j/2) = A(i,j) for 1<=i<=j;
// *          if UPLO = 'L', AP(i + (j-1)*(2*n-j)/2) = A(i,j) for j<=i<=n.
// *
// *          On exit, the contents of AP are destroyed.
// *
// *  BP      (input/output) DOUBLE PRECISION array, dimension (N*(N+1)/2)
// *          On entry, the upper or lower triangle of the symmetric matrix
// *          B, packed columnwise in a linear array.  The j-th column of B
// *          is stored in the array BP as follows:
// *          if UPLO = 'U', BP(i + (j-1)*j/2) = B(i,j) for 1<=i<=j;
// *          if UPLO = 'L', BP(i + (j-1)*(2*n-j)/2) = B(i,j) for j<=i<=n.
// *
// *          On exit, the triangular factor U or L from the Cholesky
// *          factorization B = U**T*U or B = L*L**T, in the same storage
// *          format as B.
// *
// *  W       (output) DOUBLE PRECISION array, dimension (N)
// *          If INFO = 0, the eigenvalues in ascending order.
// *
// *  Z       (output) DOUBLE PRECISION array, dimension (LDZ, N)
// *          If JOBZ = 'V', then if INFO = 0, Z contains the matrix Z of
// *          eigenvectors.  The eigenvectors are normalized as follows:
// *          if ITYPE = 1 or 2, Z**T*B*Z = I;
// *          if ITYPE = 3, Z**T*inv(B)*Z = I.
// *          If JOBZ = 'N', then Z is not referenced.
// *
// *  LDZ     (input) INTEGER
// *          The leading dimension of the array Z.  LDZ >= 1, and if
// *          JOBZ = 'V', LDZ >= max(1,N).
// *
// *  WORK    (workspace) DOUBLE PRECISION array, dimension (3*N)
// *
// *  INFO    (output) INTEGER
// *          = 0:  successful exit
// *          < 0:  if INFO = -i, the i-th argument had an illegal value
// *          > 0:  DPPTRF or DSPEV returned an error code:
// *             <= N:  if INFO = i, DSPEV failed to converge;
// *                    i off-diagonal elements of an intermediate
// *                    tridiagonal form did not converge to zero.
// *             > N:   if INFO = n + i, for 1 <= i <= n, then the leading
// *                    minor of order i of B is not positive definite.
// *                    The factorization of B could not be completed and
// *                    no eigenvalues or eigenvectors were computed.
// *
// *  =====================================================================
// *
// *     .. Local Scalars ..
static boolean upper= false;
static boolean wantz= false;
static String trans= new String(" ");
static int j= 0;
static int neig= 0;
// *     ..
// *     .. External Functions ..
// *     ..
// *     .. External Subroutines ..
// *     ..
// *     .. Executable Statements ..
// *
// *     Test the input parameters.
// *

public static void dspgv (int itype,
String jobz,
String uplo,
int n,
double [] ap, int _ap_offset,
double [] bp, int _bp_offset,
double [] w, int _w_offset,
double [] z, int _z_offset,
int ldz,
double [] work, int _work_offset,
intW info)  {

wantz = (jobz.toLowerCase().charAt(0) == "V".toLowerCase().charAt(0));
upper = (uplo.toLowerCase().charAt(0) == "U".toLowerCase().charAt(0));
// *
info.val = 0;
if (itype < 0 || itype > 3)  {
    info.val = -1;
}              // Close if()
else if (!(wantz || (jobz.toLowerCase().charAt(0) == "N".toLowerCase().charAt(0))))  {
    info.val = -2;
}              // Close else if()
else if (!(upper || (uplo.toLowerCase().charAt(0) == "L".toLowerCase().charAt(0))))  {
    info.val = -3;
}              // Close else if()
else if (n < 0)  {
    info.val = -4;
}              // Close else if()
else if (ldz < 1 || (wantz && ldz < n))  {
    info.val = -9;
}              // Close else if()
if (info.val != 0)  {
    Xerbla.xerbla("DSPGV ",-info.val);
Dummy.go_to("Dspgv",999999);
}              // Close if()
// *
// *     Quick return if possible
// *
if (n == 0)  
    Dummy.go_to("Dspgv",999999);
// *
// *     Form a Cholesky factorization of B.
// *
Dpptrf.dpptrf(uplo,n,bp,_bp_offset,info);
if (info.val != 0)  {
    info.val = n+info.val;
Dummy.go_to("Dspgv",999999);
}              // Close if()
// *
// *     Transform problem to standard eigenvalue problem and solve.
// *
Dspgst.dspgst(itype,uplo,n,ap,_ap_offset,bp,_bp_offset,info);
Dspev.dspev(jobz,uplo,n,ap,_ap_offset,w,_w_offset,z,_z_offset,ldz,work,_work_offset,info);
// *
if (wantz)  {
    // *
// *        Backtransform eigenvectors to the original problem.
// *
neig = n;
if (info.val > 0)  
    neig = info.val-1;
if (itype == 1 || itype == 2)  {
    // *
// *           For A*x=(lambda)*B*x and A*B*x=(lambda)*x;
// *           backtransform eigenvectors: x = inv(L)'*y or inv(U)*y
// *
if (upper)  {
    trans = "N";
}              // Close if()
else  {
  trans = "T";
}              //  Close else.
// *
{
forloop10:
for (j = 1; j <= neig; j++) {
Dtpsv.dtpsv(uplo,trans,"Non-unit",n,bp,_bp_offset,z,(1)- 1+(j- 1)*ldz+ _z_offset,1);
Dummy.label("Dspgv",10);
}              //  Close for() loop. 
}
// *
}              // Close if()
else if (itype == 3)  {
    // *
// *           For B*A*x=(lambda)*x;
// *           backtransform eigenvectors: x = L*y or U'*y
// *
if (upper)  {
    trans = "T";
}              // Close if()
else  {
  trans = "N";
}              //  Close else.
// *
{
forloop20:
for (j = 1; j <= neig; j++) {
Dtpmv.dtpmv(uplo,trans,"Non-unit",n,bp,_bp_offset,z,(1)- 1+(j- 1)*ldz+ _z_offset,1);
Dummy.label("Dspgv",20);
}              //  Close for() loop. 
}
}              // Close else if()
}              // Close if()
Dummy.go_to("Dspgv",999999);
// *
// *     End of DSPGV
// *
Dummy.label("Dspgv",999999);
return;
   }
} // End class.
