/*
 *  Produced by f2java.  f2java is part of the Fortran-
 *  -to-Java project at the University of Tennessee Netlib
 *  numerical software repository.
 *
 *  Original authorship for the BLAS and LAPACK numerical
 *  routines may be found in the Fortran source, available at
 *  www.netlib.org.
 *
 *  Fortran input file: jlapack.f
 *
 *  The f2j compiler code was written by
 *  David M. Doolin (doolin@cs.utk.edu) and
 *  Keith  Seymour (seymour@cs.utk.edu)
 */

package org.netlib.lapack;
import java.lang.*;
import org.netlib.util.*;

import org.netlib.blas.*;


public class Dsptrd {

// *
// *  -- LAPACK routine (version 2.0) --
// *     Univ. of Tennessee, Univ. of California Berkeley, NAG Ltd.,
// *     Courant Institute, Argonne National Lab, and Rice University
// *     March 31, 1993
// *
// *     .. Scalar Arguments ..
// *     ..
// *     .. Array Arguments ..
// *     ..
// *
// *  Purpose
// *  =======
// *
// *  DSPTRD reduces a real symmetric matrix A stored in packed form to
// *  symmetric tridiagonal form T by an orthogonal similarity
// *  transformation: Q**T * A * Q = T.
// *
// *  Arguments
// *  =========
// *
// *  UPLO    (input) CHARACTER*1
// *          = 'U':  Upper triangle of A is stored;
// *          = 'L':  Lower triangle of A is stored.
// *
// *  N       (input) INTEGER
// *          The order of the matrix A.  N >= 0.
// *
// *  AP      (input/output) DOUBLE PRECISION array, dimension (N*(N+1)/2)
// *          On entry, the upper or lower triangle of the symmetric matrix
// *          A, packed columnwise in a linear array.  The j-th column of A
// *          is stored in the array AP as follows:
// *          if UPLO = 'U', AP(i + (j-1)*j/2) = A(i,j) for 1<=i<=j;
// *          if UPLO = 'L', AP(i + (j-1)*(2*n-j)/2) = A(i,j) for j<=i<=n.
// *          On exit, if UPLO = 'U', the diagonal and first superdiagonal
// *          of A are overwritten by the corresponding elements of the
// *          tridiagonal matrix T, and the elements above the first
// *          superdiagonal, with the array TAU, represent the orthogonal
// *          matrix Q as a product of elementary reflectors; if UPLO
// *          = 'L', the diagonal and first subdiagonal of A are over-
// *          written by the corresponding elements of the tridiagonal
// *          matrix T, and the elements below the first subdiagonal, with
// *          the array TAU, represent the orthogonal matrix Q as a product
// *          of elementary reflectors. See Further Details.
// *
// *  D       (output) DOUBLE PRECISION array, dimension (N)
// *          The diagonal elements of the tridiagonal matrix T:
// *          D(i) = A(i,i).
// *
// *  E       (output) DOUBLE PRECISION array, dimension (N-1)
// *          The off-diagonal elements of the tridiagonal matrix T:
// *          E(i) = A(i,i+1) if UPLO = 'U', E(i) = A(i+1,i) if UPLO = 'L'.
// *
// *  TAU     (output) DOUBLE PRECISION array, dimension (N-1)
// *          The scalar factors of the elementary reflectors (see Further
// *          Details).
// *
// *  INFO    (output) INTEGER
// *          = 0:  successful exit
// *          < 0:  if INFO = -i, the i-th argument had an illegal value
// *
// *  Further Details
// *  ===============
// *
// *  If UPLO = 'U', the matrix Q is represented as a product of elementary
// *  reflectors
// *
// *     Q = H(n-1) . . . H(2) H(1).
// *
// *  Each H(i) has the form
// *
// *     H(i) = I - tau * v * v'
// *
// *  where tau is a real scalar, and v is a real vector with
// *  v(i+1:n) = 0 and v(i) = 1; v(1:i-1) is stored on exit in AP,
// *  overwriting A(1:i-1,i+1), and tau is stored in TAU(i).
// *
// *  If UPLO = 'L', the matrix Q is represented as a product of elementary
// *  reflectors
// *
// *     Q = H(1) H(2) . . . H(n-1).
// *
// *  Each H(i) has the form
// *
// *     H(i) = I - tau * v * v'
// *
// *  where tau is a real scalar, and v is a real vector with
// *  v(1:i) = 0 and v(i+1) = 1; v(i+2:n) is stored on exit in AP,
// *  overwriting A(i+2:n,i), and tau is stored in TAU(i).
// *
// *  =====================================================================
// *
// *     .. Parameters ..
static double one= 1.0e0;
static double zero= 0.0e0;
static double half= 1.0e0/2.0e0;
// *     ..
// *     .. Local Scalars ..
static boolean upper= false;
static int i= 0;
static int i1= 0;
static int i1i1= 0;
static int ii= 0;
static double alpha= 0.0;
static doubleW taui= new doubleW(0.0);
// *     ..
// *     .. External Subroutines ..
// *     ..
// *     .. External Functions ..
// *     ..
// *     .. Executable Statements ..
// *
// *     Test the input parameters
// *

public static void dsptrd (String uplo,
int n,
double [] ap, int _ap_offset,
double [] d, int _d_offset,
double [] e, int _e_offset,
double [] tau, int _tau_offset,
intW info)  {

info.val = 0;
upper = (uplo.toLowerCase().charAt(0) == "U".toLowerCase().charAt(0));
if (!upper && !(uplo.toLowerCase().charAt(0) == "L".toLowerCase().charAt(0)))  {
    info.val = -1;
}              // Close if()
else if (n < 0)  {
    info.val = -2;
}              // Close else if()
if (info.val != 0)  {
    Xerbla.xerbla("DSPTRD",-info.val);
Dummy.go_to("Dsptrd",999999);
}              // Close if()
// *
// *     Quick return if possible
// *
if (n <= 0)  
    Dummy.go_to("Dsptrd",999999);
// *
if (upper)  {
    // *
// *        Reduce the upper triangle of A.
// *        I1 is the index in AP of A(1,I+1).
// *
i1 = n*(n-1)/2+1;
{
int _i_inc = -1;
forloop10:
for (i = n-1; i >= 1; i += _i_inc) {
// *
// *           Generate elementary reflector H(i) = I - tau * v * v'
// *           to annihilate A(1:i-1,i+1)
// *
dlarfg_adapter(i,ap,(i1+i-1)- 1+ _ap_offset,ap,(i1)- 1+ _ap_offset,1,taui);
e[(i)- 1+ _e_offset] = ap[(i1+i-1)- 1+ _ap_offset];
// *
if (taui.val != zero)  {
    // *
// *              Apply H(i) from both sides to A(1:i,1:i)
// *
ap[(i1+i-1)- 1+ _ap_offset] = one;
// *
// *              Compute  y := tau * A * v  storing y in TAU(1:i)
// *
Dspmv.dspmv(uplo,i,taui.val,ap,_ap_offset,ap,(i1)- 1+ _ap_offset,1,zero,tau,_tau_offset,1);
// *
// *              Compute  w := y - 1/2 * tau * (y'*v) * v
// *
alpha = -half*taui.val*Ddot.ddot(i,tau,_tau_offset,1,ap,(i1)- 1+ _ap_offset,1);
Daxpy.daxpy(i,alpha,ap,(i1)- 1+ _ap_offset,1,tau,_tau_offset,1);
// *
// *              Apply the transformation as a rank-2 update:
// *                 A := A - v * w' - w * v'
// *
Dspr2.dspr2(uplo,i,-one,ap,(i1)- 1+ _ap_offset,1,tau,_tau_offset,1,ap,_ap_offset);
// *
ap[(i1+i-1)- 1+ _ap_offset] = e[(i)- 1+ _e_offset];
}              // Close if()
d[(i+1)- 1+ _d_offset] = ap[(i1+i)- 1+ _ap_offset];
tau[(i)- 1+ _tau_offset] = taui.val;
i1 = i1-i;
Dummy.label("Dsptrd",10);
}              //  Close for() loop. 
}
d[(1)- 1+ _d_offset] = ap[(1)- 1+ _ap_offset];
}              // Close if()
else  {
  // *
// *        Reduce the lower triangle of A. II is the index in AP of
// *        A(i,i) and I1I1 is the index of A(i+1,i+1).
// *
ii = 1;
{
forloop20:
for (i = 1; i <= n-1; i++) {
i1i1 = ii+n-i+1;
// *
// *           Generate elementary reflector H(i) = I - tau * v * v'
// *           to annihilate A(i+2:n,i)
// *
dlarfg_adapter(n-i,ap,(ii+1)- 1+ _ap_offset,ap,(ii+2)- 1+ _ap_offset,1,taui);
e[(i)- 1+ _e_offset] = ap[(ii+1)- 1+ _ap_offset];
// *
if (taui.val != zero)  {
    // *
// *              Apply H(i) from both sides to A(i+1:n,i+1:n)
// *
ap[(ii+1)- 1+ _ap_offset] = one;
// *
// *              Compute  y := tau * A * v  storing y in TAU(i:n-1)
// *
Dspmv.dspmv(uplo,n-i,taui.val,ap,(i1i1)- 1+ _ap_offset,ap,(ii+1)- 1+ _ap_offset,1,zero,tau,(i)- 1+ _tau_offset,1);
// *
// *              Compute  w := y - 1/2 * tau * (y'*v) * v
// *
alpha = -half*taui.val*Ddot.ddot(n-i,tau,(i)- 1+ _tau_offset,1,ap,(ii+1)- 1+ _ap_offset,1);
Daxpy.daxpy(n-i,alpha,ap,(ii+1)- 1+ _ap_offset,1,tau,(i)- 1+ _tau_offset,1);
// *
// *              Apply the transformation as a rank-2 update:
// *                 A := A - v * w' - w * v'
// *
Dspr2.dspr2(uplo,n-i,-one,ap,(ii+1)- 1+ _ap_offset,1,tau,(i)- 1+ _tau_offset,1,ap,(i1i1)- 1+ _ap_offset);
// *
ap[(ii+1)- 1+ _ap_offset] = e[(i)- 1+ _e_offset];
}              // Close if()
d[(i)- 1+ _d_offset] = ap[(ii)- 1+ _ap_offset];
tau[(i)- 1+ _tau_offset] = taui.val;
ii = i1i1;
Dummy.label("Dsptrd",20);
}              //  Close for() loop. 
}
d[(n)- 1+ _d_offset] = ap[(ii)- 1+ _ap_offset];
}              //  Close else.
// *
Dummy.go_to("Dsptrd",999999);
// *
// *     End of DSPTRD
// *
Dummy.label("Dsptrd",999999);
return;
   }
// adapter for dlarfg
private static void dlarfg_adapter(int arg0 ,double [] arg1 , int arg1_offset ,double [] arg2 , int arg2_offset ,int arg3 ,doubleW arg4 )
{
doubleW _f2j_tmp1 = new doubleW(arg1[arg1_offset]);

Dlarfg.dlarfg(arg0,_f2j_tmp1,arg2, arg2_offset,arg3,arg4);

arg1[arg1_offset] = _f2j_tmp1.val;
}

} // End class.
