/*
 *  Produced by f2java.  f2java is part of the Fortran-
 *  -to-Java project at the University of Tennessee Netlib
 *  numerical software repository.
 *
 *  Original authorship for the BLAS and LAPACK numerical
 *  routines may be found in the Fortran source, available at
 *  www.netlib.org.
 *
 *  Fortran input file: jlapack.f
 *
 *  The f2j compiler code was written by
 *  David M. Doolin (doolin@cs.utk.edu) and
 *  Keith  Seymour (seymour@cs.utk.edu)
 */

package org.netlib.lapack;
import java.lang.*;
import org.netlib.util.*;

import org.netlib.blas.*;


public class Dsptri {

// *
// *  -- LAPACK routine (version 2.0) --
// *     Univ. of Tennessee, Univ. of California Berkeley, NAG Ltd.,
// *     Courant Institute, Argonne National Lab, and Rice University
// *     March 31, 1993
// *
// *     .. Scalar Arguments ..
// *     ..
// *     .. Array Arguments ..
// *     ..
// *
// *  Purpose
// *  =======
// *
// *  DSPTRI computes the inverse of a real symmetric indefinite matrix
// *  A in packed storage using the factorization A = U*D*U**T or
// *  A = L*D*L**T computed by DSPTRF.
// *
// *  Arguments
// *  =========
// *
// *  UPLO    (input) CHARACTER*1
// *          Specifies whether the details of the factorization are stored
// *          as an upper or lower triangular matrix.
// *          = 'U':  Upper triangular, form is A = U*D*U**T;
// *          = 'L':  Lower triangular, form is A = L*D*L**T.
// *
// *  N       (input) INTEGER
// *          The order of the matrix A.  N >= 0.
// *
// *  AP      (input/output) DOUBLE PRECISION array, dimension (N*(N+1)/2)
// *          On entry, the block diagonal matrix D and the multipliers
// *          used to obtain the factor U or L as computed by DSPTRF,
// *          stored as a packed triangular matrix.
// *
// *          On exit, if INFO = 0, the (symmetric) inverse of the original
// *          matrix, stored as a packed triangular matrix. The j-th column
// *          of inv(A) is stored in the array AP as follows:
// *          if UPLO = 'U', AP(i + (j-1)*j/2) = inv(A)(i,j) for 1<=i<=j;
// *          if UPLO = 'L',
// *             AP(i + (j-1)*(2n-j)/2) = inv(A)(i,j) for j<=i<=n.
// *
// *  IPIV    (input) INTEGER array, dimension (N)
// *          Details of the interchanges and the block structure of D
// *          as determined by DSPTRF.
// *
// *  WORK    (workspace) DOUBLE PRECISION array, dimension (N)
// *
// *  INFO    (output) INTEGER
// *          = 0: successful exit
// *          < 0: if INFO = -i, the i-th argument had an illegal value
// *          > 0: if INFO = i, D(i,i) = 0; the matrix is singular and its
// *               inverse could not be computed.
// *
// *  =====================================================================
// *
// *     .. Parameters ..
static double one= 1.0e+0;
static double zero= 0.0e+0;
// *     ..
// *     .. Local Scalars ..
static boolean upper= false;
static int j= 0;
static int k= 0;
static int kc= 0;
static int kcnext= 0;
static int kp= 0;
static int kpc= 0;
static int kstep= 0;
static int kx= 0;
static int npp= 0;
static double ak= 0.0;
static double akkp1= 0.0;
static double akp1= 0.0;
static double d= 0.0;
static double t= 0.0;
static double temp= 0.0;
// *     ..
// *     .. External Functions ..
// *     ..
// *     .. External Subroutines ..
// *     ..
// *     .. Intrinsic Functions ..
// *     ..
// *     .. Executable Statements ..
// *
// *     Test the input parameters.
// *

public static void dsptri (String uplo,
int n,
double [] ap, int _ap_offset,
int [] ipiv, int _ipiv_offset,
double [] work, int _work_offset,
intW info)  {

info.val = 0;
upper = (uplo.toLowerCase().charAt(0) == "U".toLowerCase().charAt(0));
if (!upper && !(uplo.toLowerCase().charAt(0) == "L".toLowerCase().charAt(0)))  {
    info.val = -1;
}              // Close if()
else if (n < 0)  {
    info.val = -2;
}              // Close else if()
if (info.val != 0)  {
    Xerbla.xerbla("DSPTRI",-info.val);
Dummy.go_to("Dsptri",999999);
}              // Close if()
// *
// *     Quick return if possible
// *
if (n == 0)  
    Dummy.go_to("Dsptri",999999);
// *
// *     Check that the diagonal matrix D is nonsingular.
// *
if (upper)  {
    // *
// *        Upper triangular storage: examine D from bottom to top
// *
kp = n*(n+1)/2;
{
int _info_inc = -1;
forloop10:
for (info.val = n; info.val >= 1; info.val += _info_inc) {
if (ipiv[(info.val)- 1+ _ipiv_offset] > 0 && ap[(kp)- 1+ _ap_offset] == zero)  
    Dummy.go_to("Dsptri",999999);
kp = kp-info.val;
Dummy.label("Dsptri",10);
}              //  Close for() loop. 
}
}              // Close if()
else  {
  // *
// *        Lower triangular storage: examine D from top to bottom.
// *
kp = 1;
{
forloop20:
for (info.val = 1; info.val <= n; info.val++) {
if (ipiv[(info.val)- 1+ _ipiv_offset] > 0 && ap[(kp)- 1+ _ap_offset] == zero)  
    Dummy.go_to("Dsptri",999999);
kp = kp+n-info.val+1;
Dummy.label("Dsptri",20);
}              //  Close for() loop. 
}
}              //  Close else.
info.val = 0;
// *
if (upper)  {
    // *
// *        Compute inv(A) from the factorization A = U*D*U'.
// *
// *        K is the main loop index, increasing from 1 to N in steps of
// *        1 or 2, depending on the size of the diagonal blocks.
// *
k = 1;
kc = 1;
label30:
   Dummy.label("Dsptri",30);
// *
// *        If K > N, exit from loop.
// *
if (k > n)  
    Dummy.go_to("Dsptri",50);
// *
kcnext = kc+k;
if (ipiv[(k)- 1+ _ipiv_offset] > 0)  {
    // *
// *           1 x 1 diagonal block
// *
// *           Invert the diagonal block.
// *
ap[(kc+k-1)- 1+ _ap_offset] = one/ap[(kc+k-1)- 1+ _ap_offset];
// *
// *           Compute column K of the inverse.
// *
if (k > 1)  {
    Dcopy.dcopy(k-1,ap,(kc)- 1+ _ap_offset,1,work,_work_offset,1);
Dspmv.dspmv(uplo,k-1,-one,ap,_ap_offset,work,_work_offset,1,zero,ap,(kc)- 1+ _ap_offset,1);
ap[(kc+k-1)- 1+ _ap_offset] = ap[(kc+k-1)- 1+ _ap_offset]-Ddot.ddot(k-1,work,_work_offset,1,ap,(kc)- 1+ _ap_offset,1);
}              // Close if()
kstep = 1;
}              // Close if()
else  {
  // *
// *           2 x 2 diagonal block
// *
// *           Invert the diagonal block.
// *
t = Math.abs(ap[(kcnext+k-1)- 1+ _ap_offset]);
ak = ap[(kc+k-1)- 1+ _ap_offset]/t;
akp1 = ap[(kcnext+k)- 1+ _ap_offset]/t;
akkp1 = ap[(kcnext+k-1)- 1+ _ap_offset]/t;
d = t*(ak*akp1-one);
ap[(kc+k-1)- 1+ _ap_offset] = akp1/d;
ap[(kcnext+k)- 1+ _ap_offset] = ak/d;
ap[(kcnext+k-1)- 1+ _ap_offset] = -akkp1/d;
// *
// *           Compute columns K and K+1 of the inverse.
// *
if (k > 1)  {
    Dcopy.dcopy(k-1,ap,(kc)- 1+ _ap_offset,1,work,_work_offset,1);
Dspmv.dspmv(uplo,k-1,-one,ap,_ap_offset,work,_work_offset,1,zero,ap,(kc)- 1+ _ap_offset,1);
ap[(kc+k-1)- 1+ _ap_offset] = ap[(kc+k-1)- 1+ _ap_offset]-Ddot.ddot(k-1,work,_work_offset,1,ap,(kc)- 1+ _ap_offset,1);
ap[(kcnext+k-1)- 1+ _ap_offset] = ap[(kcnext+k-1)- 1+ _ap_offset]-Ddot.ddot(k-1,ap,(kc)- 1+ _ap_offset,1,ap,(kcnext)- 1+ _ap_offset,1);
Dcopy.dcopy(k-1,ap,(kcnext)- 1+ _ap_offset,1,work,_work_offset,1);
Dspmv.dspmv(uplo,k-1,-one,ap,_ap_offset,work,_work_offset,1,zero,ap,(kcnext)- 1+ _ap_offset,1);
ap[(kcnext+k)- 1+ _ap_offset] = ap[(kcnext+k)- 1+ _ap_offset]-Ddot.ddot(k-1,work,_work_offset,1,ap,(kcnext)- 1+ _ap_offset,1);
}              // Close if()
kstep = 2;
kcnext = kcnext+k+1;
}              //  Close else.
// *
kp = (int)(Math.abs(ipiv[(k)- 1+ _ipiv_offset]));
if (kp != k)  {
    // *
// *           Interchange rows and columns K and KP in the leading
// *           submatrix A(1:k+1,1:k+1)
// *
kpc = (kp-1)*kp/2+1;
Dswap.dswap(kp-1,ap,(kc)- 1+ _ap_offset,1,ap,(kpc)- 1+ _ap_offset,1);
kx = kpc+kp-1;
{
forloop40:
for (j = kp+1; j <= k-1; j++) {
kx = kx+j-1;
temp = ap[(kc+j-1)- 1+ _ap_offset];
ap[(kc+j-1)- 1+ _ap_offset] = ap[(kx)- 1+ _ap_offset];
ap[(kx)- 1+ _ap_offset] = temp;
Dummy.label("Dsptri",40);
}              //  Close for() loop. 
}
temp = ap[(kc+k-1)- 1+ _ap_offset];
ap[(kc+k-1)- 1+ _ap_offset] = ap[(kpc+kp-1)- 1+ _ap_offset];
ap[(kpc+kp-1)- 1+ _ap_offset] = temp;
if (kstep == 2)  {
    temp = ap[(kc+k+k-1)- 1+ _ap_offset];
ap[(kc+k+k-1)- 1+ _ap_offset] = ap[(kc+k+kp-1)- 1+ _ap_offset];
ap[(kc+k+kp-1)- 1+ _ap_offset] = temp;
}              // Close if()
}              // Close if()
// *
k = k+kstep;
kc = kcnext;
Dummy.go_to("Dsptri",30);
label50:
   Dummy.label("Dsptri",50);
// *
}              // Close if()
else  {
  // *
// *        Compute inv(A) from the factorization A = L*D*L'.
// *
// *        K is the main loop index, increasing from 1 to N in steps of
// *        1 or 2, depending on the size of the diagonal blocks.
// *
npp = n*(n+1)/2;
k = n;
kc = npp;
label60:
   Dummy.label("Dsptri",60);
// *
// *        If K < 1, exit from loop.
// *
if (k < 1)  
    Dummy.go_to("Dsptri",80);
// *
kcnext = kc-(n-k+2);
if (ipiv[(k)- 1+ _ipiv_offset] > 0)  {
    // *
// *           1 x 1 diagonal block
// *
// *           Invert the diagonal block.
// *
ap[(kc)- 1+ _ap_offset] = one/ap[(kc)- 1+ _ap_offset];
// *
// *           Compute column K of the inverse.
// *
if (k < n)  {
    Dcopy.dcopy(n-k,ap,(kc+1)- 1+ _ap_offset,1,work,_work_offset,1);
Dspmv.dspmv(uplo,n-k,-one,ap,(kc+n-k+1)- 1+ _ap_offset,work,_work_offset,1,zero,ap,(kc+1)- 1+ _ap_offset,1);
ap[(kc)- 1+ _ap_offset] = ap[(kc)- 1+ _ap_offset]-Ddot.ddot(n-k,work,_work_offset,1,ap,(kc+1)- 1+ _ap_offset,1);
}              // Close if()
kstep = 1;
}              // Close if()
else  {
  // *
// *           2 x 2 diagonal block
// *
// *           Invert the diagonal block.
// *
t = Math.abs(ap[(kcnext+1)- 1+ _ap_offset]);
ak = ap[(kcnext)- 1+ _ap_offset]/t;
akp1 = ap[(kc)- 1+ _ap_offset]/t;
akkp1 = ap[(kcnext+1)- 1+ _ap_offset]/t;
d = t*(ak*akp1-one);
ap[(kcnext)- 1+ _ap_offset] = akp1/d;
ap[(kc)- 1+ _ap_offset] = ak/d;
ap[(kcnext+1)- 1+ _ap_offset] = -akkp1/d;
// *
// *           Compute columns K-1 and K of the inverse.
// *
if (k < n)  {
    Dcopy.dcopy(n-k,ap,(kc+1)- 1+ _ap_offset,1,work,_work_offset,1);
Dspmv.dspmv(uplo,n-k,-one,ap,(kc+(n-k+1))- 1+ _ap_offset,work,_work_offset,1,zero,ap,(kc+1)- 1+ _ap_offset,1);
ap[(kc)- 1+ _ap_offset] = ap[(kc)- 1+ _ap_offset]-Ddot.ddot(n-k,work,_work_offset,1,ap,(kc+1)- 1+ _ap_offset,1);
ap[(kcnext+1)- 1+ _ap_offset] = ap[(kcnext+1)- 1+ _ap_offset]-Ddot.ddot(n-k,ap,(kc+1)- 1+ _ap_offset,1,ap,(kcnext+2)- 1+ _ap_offset,1);
Dcopy.dcopy(n-k,ap,(kcnext+2)- 1+ _ap_offset,1,work,_work_offset,1);
Dspmv.dspmv(uplo,n-k,-one,ap,(kc+(n-k+1))- 1+ _ap_offset,work,_work_offset,1,zero,ap,(kcnext+2)- 1+ _ap_offset,1);
ap[(kcnext)- 1+ _ap_offset] = ap[(kcnext)- 1+ _ap_offset]-Ddot.ddot(n-k,work,_work_offset,1,ap,(kcnext+2)- 1+ _ap_offset,1);
}              // Close if()
kstep = 2;
kcnext = kcnext-(n-k+3);
}              //  Close else.
// *
kp = (int)(Math.abs(ipiv[(k)- 1+ _ipiv_offset]));
if (kp != k)  {
    // *
// *           Interchange rows and columns K and KP in the trailing
// *           submatrix A(k-1:n,k-1:n)
// *
kpc = npp-(n-kp+1)*(n-kp+2)/2+1;
if (kp < n)  
    Dswap.dswap(n-kp,ap,(kc+kp-k+1)- 1+ _ap_offset,1,ap,(kpc+1)- 1+ _ap_offset,1);
kx = kc+kp-k;
{
forloop70:
for (j = k+1; j <= kp-1; j++) {
kx = kx+n-j+1;
temp = ap[(kc+j-k)- 1+ _ap_offset];
ap[(kc+j-k)- 1+ _ap_offset] = ap[(kx)- 1+ _ap_offset];
ap[(kx)- 1+ _ap_offset] = temp;
Dummy.label("Dsptri",70);
}              //  Close for() loop. 
}
temp = ap[(kc)- 1+ _ap_offset];
ap[(kc)- 1+ _ap_offset] = ap[(kpc)- 1+ _ap_offset];
ap[(kpc)- 1+ _ap_offset] = temp;
if (kstep == 2)  {
    temp = ap[(kc-n+k-1)- 1+ _ap_offset];
ap[(kc-n+k-1)- 1+ _ap_offset] = ap[(kc-n+kp-1)- 1+ _ap_offset];
ap[(kc-n+kp-1)- 1+ _ap_offset] = temp;
}              // Close if()
}              // Close if()
// *
k = k-kstep;
kc = kcnext;
Dummy.go_to("Dsptri",60);
label80:
   Dummy.label("Dsptri",80);
}              //  Close else.
// *
Dummy.go_to("Dsptri",999999);
// *
// *     End of DSPTRI
// *
Dummy.label("Dsptri",999999);
return;
   }
} // End class.
