/*
 *  Produced by f2java.  f2java is part of the Fortran-
 *  -to-Java project at the University of Tennessee Netlib
 *  numerical software repository.
 *
 *  Original authorship for the BLAS and LAPACK numerical
 *  routines may be found in the Fortran source, available at
 *  www.netlib.org.
 *
 *  Fortran input file: jlapack.f
 *
 *  The f2j compiler code was written by
 *  David M. Doolin (doolin@cs.utk.edu) and
 *  Keith  Seymour (seymour@cs.utk.edu)
 */

package org.netlib.lapack;
import java.lang.*;
import org.netlib.util.*;



public class Dsterf {

// *
// *  -- LAPACK routine (version 2.0) --
// *     Univ. of Tennessee, Univ. of California Berkeley, NAG Ltd.,
// *     Courant Institute, Argonne National Lab, and Rice University
// *     September 30, 1994
// *
// *     .. Scalar Arguments ..
// *     ..
// *     .. Array Arguments ..
// *     ..
// *
// *  Purpose
// *  =======
// *
// *  DSTERF computes all eigenvalues of a symmetric tridiagonal matrix
// *  using the Pal-Walker-Kahan variant of the QL or QR algorithm.
// *
// *  Arguments
// *  =========
// *
// *  N       (input) INTEGER
// *          The order of the matrix.  N >= 0.
// *
// *  D       (input/output) DOUBLE PRECISION array, dimension (N)
// *          On entry, the n diagonal elements of the tridiagonal matrix.
// *          On exit, if INFO = 0, the eigenvalues in ascending order.
// *
// *  E       (input/output) DOUBLE PRECISION array, dimension (N-1)
// *          On entry, the (n-1) subdiagonal elements of the tridiagonal
// *          matrix.
// *          On exit, E has been destroyed.
// *
// *  INFO    (output) INTEGER
// *          = 0:  successful exit
// *          < 0:  if INFO = -i, the i-th argument had an illegal value
// *          > 0:  the algorithm failed to find all of the eigenvalues in
// *                a total of 30*N iterations; if INFO = i, then i
// *                elements of E have not converged to zero.
// *
// *  =====================================================================
// *
// *     .. Parameters ..
static double zero= 0.0e0;
static double one= 1.0e0;
static double two= 2.0e0;
static double three= 3.0e0;
static int maxit= 30;
// *     ..
// *     .. Local Scalars ..
static int i= 0;
static int iscale= 0;
static int jtot= 0;
static int l= 0;
static int l1= 0;
static int lend= 0;
static int lendm1= 0;
static int lendp1= 0;
static int lendsv= 0;
static int lm1= 0;
static int lsv= 0;
static int m= 0;
static int mm1= 0;
static int nm1= 0;
static int nmaxit= 0;
static double alpha= 0.0;
static double anorm= 0.0;
static double bb= 0.0;
static double c= 0.0;
static double eps= 0.0;
static double eps2= 0.0;
static double gamma= 0.0;
static double oldc= 0.0;
static double oldgam= 0.0;
static double p= 0.0;
static double r= 0.0;
static doubleW rt1= new doubleW(0.0);
static doubleW rt2= new doubleW(0.0);
static double rte= 0.0;
static double s= 0.0;
static double safmax= 0.0;
static double safmin= 0.0;
static double sigma= 0.0;
static double ssfmax= 0.0;
static double ssfmin= 0.0;
static double tst= 0.0;
// *     ..
// *     .. External Functions ..
// *     ..
// *     .. External Subroutines ..
// *     ..
// *     .. Intrinsic Functions ..
// *     ..
// *     .. Executable Statements ..
// *
// *     Test the input parameters.
// *

public static void dsterf (int n,
double [] d, int _d_offset,
double [] e, int _e_offset,
intW info)  {

info.val = 0;
// *
// *     Quick return if possible
// *
if (n < 0)  {
    info.val = -1;
Xerbla.xerbla("DSTERF",-info.val);
Dummy.go_to("Dsterf",999999);
}              // Close if()
if (n <= 1)  
    Dummy.go_to("Dsterf",999999);
// *
// *     Determine the unit roundoff for this environment.
// *
eps = Dlamch.dlamch("E");
eps2 = Math.pow(eps, 2);
safmin = Dlamch.dlamch("S");
safmax = one/safmin;
ssfmax = Math.sqrt(safmax)/three;
ssfmin = Math.sqrt(safmin)/eps2;
// *
// *     Compute the eigenvalues of the tridiagonal matrix.
// *
nmaxit = n*maxit;
sigma = zero;
jtot = 0;
// *
// *     Determine where the matrix splits and choose QL or QR iteration
// *     for each block, according to whether top or bottom diagonal
// *     element is smaller.
// *
l1 = 1;
nm1 = n-1;
// *
label10:
   Dummy.label("Dsterf",10);
if (l1 > n)  
    Dummy.go_to("Dsterf",170);
if (l1 > 1)  
    e[(l1-1)- 1+ _e_offset] = zero;
if (l1 <= nm1)  {
    {
forloop20:
for (m = l1; m <= nm1; m++) {
tst = Math.abs(e[(m)- 1+ _e_offset]);
if (tst == zero)  
    Dummy.go_to("Dsterf",30);
if (tst <= (Math.sqrt(Math.abs(d[(m)- 1+ _d_offset]))*Math.sqrt(Math.abs(d[(m+1)- 1+ _d_offset])))*eps)  {
    e[(m)- 1+ _e_offset] = zero;
Dummy.go_to("Dsterf",30);
}              // Close if()
Dummy.label("Dsterf",20);
}              //  Close for() loop. 
}
}              // Close if()
m = n;
// *
label30:
   Dummy.label("Dsterf",30);
l = l1;
lsv = l;
lend = m;
lendsv = lend;
l1 = m+1;
if (lend == l)  
    Dummy.go_to("Dsterf",10);
// *
// *     Scale submatrix in rows and columns L to LEND
// *
anorm = Dlanst.dlanst("I",lend-l+1,d,(l)- 1+ _d_offset,e,(l)- 1+ _e_offset);
iscale = 0;
if (anorm > ssfmax)  {
    iscale = 1;
Dlascl.dlascl("G",0,0,anorm,ssfmax,lend-l+1,1,d,(l)- 1+ _d_offset,n,info);
Dlascl.dlascl("G",0,0,anorm,ssfmax,lend-l,1,e,(l)- 1+ _e_offset,n,info);
}              // Close if()
else if (anorm < ssfmin)  {
    iscale = 2;
Dlascl.dlascl("G",0,0,anorm,ssfmin,lend-l+1,1,d,(l)- 1+ _d_offset,n,info);
Dlascl.dlascl("G",0,0,anorm,ssfmin,lend-l,1,e,(l)- 1+ _e_offset,n,info);
}              // Close else if()
// *
{
forloop40:
for (i = l; i <= lend-1; i++) {
e[(i)- 1+ _e_offset] = Math.pow(e[(i)- 1+ _e_offset], 2);
Dummy.label("Dsterf",40);
}              //  Close for() loop. 
}
// *
// *     Choose between QL and QR iteration
// *
if (Math.abs(d[(lend)- 1+ _d_offset]) < Math.abs(d[(l)- 1+ _d_offset]))  {
    lend = lsv;
l = lendsv;
}              // Close if()
// *
if (lend >= l)  {
    // *
// *        QL Iteration
// *
// *        Look for small subdiagonal element.
// *
label50:
   Dummy.label("Dsterf",50);
if (l != lend)  {
    lendm1 = lend-1;
{
forloop60:
for (m = l; m <= lendm1; m++) {
tst = Math.abs(e[(m)- 1+ _e_offset]);
if (tst <= eps2*Math.abs(d[(m)- 1+ _d_offset]*d[(m+1)- 1+ _d_offset]))  
    Dummy.go_to("Dsterf",70);
Dummy.label("Dsterf",60);
}              //  Close for() loop. 
}
}              // Close if()
// *
m = lend;
// *
label70:
   Dummy.label("Dsterf",70);
if (m < lend)  
    e[(m)- 1+ _e_offset] = zero;
p = d[(l)- 1+ _d_offset];
if (m == l)  
    Dummy.go_to("Dsterf",90);
// *
// *        If remaining matrix is 2 by 2, use DLAE2 to compute its
// *        eigenvalues.
// *
if (m == l+1)  {
    rte = Math.sqrt(e[(l)- 1+ _e_offset]);
Dlae2.dlae2(d[(l)- 1+ _d_offset],rte,d[(l+1)- 1+ _d_offset],rt1,rt2);
d[(l)- 1+ _d_offset] = rt1.val;
d[(l+1)- 1+ _d_offset] = rt2.val;
e[(l)- 1+ _e_offset] = zero;
l = l+2;
if (l <= lend)  
    Dummy.go_to("Dsterf",50);
Dummy.go_to("Dsterf",150);
}              // Close if()
// *
if (jtot == nmaxit)  
    Dummy.go_to("Dsterf",150);
jtot = jtot+1;
// *
// *        Form shift.
// *
rte = Math.sqrt(e[(l)- 1+ _e_offset]);
sigma = (d[(l+1)- 1+ _d_offset]-p)/(two*rte);
r = Dlapy2.dlapy2(sigma,one);
sigma = p-(rte/(sigma+((sigma) >= 0 ? Math.abs(r) : -Math.abs(r))));
// *
c = one;
s = zero;
gamma = d[(m)- 1+ _d_offset]-sigma;
p = gamma*gamma;
// *
// *        Inner loop
// *
mm1 = m-1;
{
int _i_inc = -1;
forloop80:
for (i = mm1; i >= l; i += _i_inc) {
bb = e[(i)- 1+ _e_offset];
r = p+bb;
if (i != m-1)  
    e[(i+1)- 1+ _e_offset] = s*r;
oldc = c;
c = p/r;
s = bb/r;
oldgam = gamma;
alpha = d[(i)- 1+ _d_offset];
gamma = c*(alpha-sigma)-s*oldgam;
d[(i+1)- 1+ _d_offset] = oldgam+(alpha-gamma);
if (c != zero)  {
    p = (gamma*gamma)/c;
}              // Close if()
else  {
  p = oldc*bb;
}              //  Close else.
Dummy.label("Dsterf",80);
}              //  Close for() loop. 
}
// *
e[(l)- 1+ _e_offset] = s*p;
d[(l)- 1+ _d_offset] = sigma+gamma;
Dummy.go_to("Dsterf",50);
// *
// *        Eigenvalue found.
// *
label90:
   Dummy.label("Dsterf",90);
d[(l)- 1+ _d_offset] = p;
// *
l = l+1;
if (l <= lend)  
    Dummy.go_to("Dsterf",50);
Dummy.go_to("Dsterf",150);
// *
}              // Close if()
else  {
  // *
// *        QR Iteration
// *
// *        Look for small superdiagonal element.
// *
label100:
   Dummy.label("Dsterf",100);
if (l != lend)  {
    lendp1 = lend+1;
{
int _m_inc = -1;
forloop110:
for (m = l; m >= lendp1; m += _m_inc) {
tst = Math.abs(e[(m-1)- 1+ _e_offset]);
if (tst <= eps2*Math.abs(d[(m)- 1+ _d_offset]*d[(m-1)- 1+ _d_offset]))  
    Dummy.go_to("Dsterf",120);
Dummy.label("Dsterf",110);
}              //  Close for() loop. 
}
}              // Close if()
// *
m = lend;
// *
label120:
   Dummy.label("Dsterf",120);
if (m > lend)  
    e[(m-1)- 1+ _e_offset] = zero;
p = d[(l)- 1+ _d_offset];
if (m == l)  
    Dummy.go_to("Dsterf",140);
// *
// *        If remaining matrix is 2 by 2, use DLAE2 to compute its
// *        eigenvalues.
// *
if (m == l-1)  {
    rte = Math.sqrt(e[(l-1)- 1+ _e_offset]);
Dlae2.dlae2(d[(l)- 1+ _d_offset],rte,d[(l-1)- 1+ _d_offset],rt1,rt2);
d[(l)- 1+ _d_offset] = rt1.val;
d[(l-1)- 1+ _d_offset] = rt2.val;
e[(l-1)- 1+ _e_offset] = zero;
l = l-2;
if (l >= lend)  
    Dummy.go_to("Dsterf",100);
Dummy.go_to("Dsterf",150);
}              // Close if()
// *
if (jtot == nmaxit)  
    Dummy.go_to("Dsterf",150);
jtot = jtot+1;
// *
// *        Form shift.
// *
rte = Math.sqrt(e[(l-1)- 1+ _e_offset]);
sigma = (d[(l-1)- 1+ _d_offset]-p)/(two*rte);
r = Dlapy2.dlapy2(sigma,one);
sigma = p-(rte/(sigma+((sigma) >= 0 ? Math.abs(r) : -Math.abs(r))));
// *
c = one;
s = zero;
gamma = d[(m)- 1+ _d_offset]-sigma;
p = gamma*gamma;
// *
// *        Inner loop
// *
lm1 = l-1;
{
forloop130:
for (i = m; i <= lm1; i++) {
bb = e[(i)- 1+ _e_offset];
r = p+bb;
if (i != m)  
    e[(i-1)- 1+ _e_offset] = s*r;
oldc = c;
c = p/r;
s = bb/r;
oldgam = gamma;
alpha = d[(i+1)- 1+ _d_offset];
gamma = c*(alpha-sigma)-s*oldgam;
d[(i)- 1+ _d_offset] = oldgam+(alpha-gamma);
if (c != zero)  {
    p = (gamma*gamma)/c;
}              // Close if()
else  {
  p = oldc*bb;
}              //  Close else.
Dummy.label("Dsterf",130);
}              //  Close for() loop. 
}
// *
e[(lm1)- 1+ _e_offset] = s*p;
d[(l)- 1+ _d_offset] = sigma+gamma;
Dummy.go_to("Dsterf",100);
// *
// *        Eigenvalue found.
// *
label140:
   Dummy.label("Dsterf",140);
d[(l)- 1+ _d_offset] = p;
// *
l = l-1;
if (l >= lend)  
    Dummy.go_to("Dsterf",100);
Dummy.go_to("Dsterf",150);
// *
}              //  Close else.
// *
// *     Undo scaling if necessary
// *
label150:
   Dummy.label("Dsterf",150);
if (iscale == 1)  
    Dlascl.dlascl("G",0,0,ssfmax,anorm,lendsv-lsv+1,1,d,(lsv)- 1+ _d_offset,n,info);
if (iscale == 2)  
    Dlascl.dlascl("G",0,0,ssfmin,anorm,lendsv-lsv+1,1,d,(lsv)- 1+ _d_offset,n,info);
// *
// *     Check for no convergence to an eigenvalue after a total
// *     of N*MAXIT iterations.
// *
if (jtot == nmaxit)  {
    {
forloop160:
for (i = 1; i <= n-1; i++) {
if (e[(i)- 1+ _e_offset] != zero)  
    info.val = info.val+1;
Dummy.label("Dsterf",160);
}              //  Close for() loop. 
}
Dummy.go_to("Dsterf",999999);
}              // Close if()
Dummy.go_to("Dsterf",10);
// *
// *     Sort eigenvalues in increasing order.
// *
label170:
   Dummy.label("Dsterf",170);
Dlasrt.dlasrt("I",n,d,_d_offset,info);
// *
Dummy.go_to("Dsterf",999999);
// *
// *     End of DSTERF
// *
Dummy.label("Dsterf",999999);
return;
   }
} // End class.
