/*
 *  Produced by f2java.  f2java is part of the Fortran-
 *  -to-Java project at the University of Tennessee Netlib
 *  numerical software repository.
 *
 *  Original authorship for the BLAS and LAPACK numerical
 *  routines may be found in the Fortran source, available at
 *  www.netlib.org.
 *
 *  Fortran input file: jlapack.f
 *
 *  The f2j compiler code was written by
 *  David M. Doolin (doolin@cs.utk.edu) and
 *  Keith  Seymour (seymour@cs.utk.edu)
 */

package org.netlib.lapack;
import java.lang.*;
import org.netlib.util.*;

import org.netlib.blas.*;


public class Dstev {

// *
// *  -- LAPACK driver routine (version 2.0) --
// *     Univ. of Tennessee, Univ. of California Berkeley, NAG Ltd.,
// *     Courant Institute, Argonne National Lab, and Rice University
// *     September 30, 1994
// *
// *     .. Scalar Arguments ..
// *     ..
// *     .. Array Arguments ..
// *     ..
// *
// *  Purpose
// *  =======
// *
// *  DSTEV computes all eigenvalues and, optionally, eigenvectors of a
// *  real symmetric tridiagonal matrix A.
// *
// *  Arguments
// *  =========
// *
// *  JOBZ    (input) CHARACTER*1
// *          = 'N':  Compute eigenvalues only;
// *          = 'V':  Compute eigenvalues and eigenvectors.
// *
// *  N       (input) INTEGER
// *          The order of the matrix.  N >= 0.
// *
// *  D       (input/output) DOUBLE PRECISION array, dimension (N)
// *          On entry, the n diagonal elements of the tridiagonal matrix
// *          A.
// *          On exit, if INFO = 0, the eigenvalues in ascending order.
// *
// *  E       (input/output) DOUBLE PRECISION array, dimension (N)
// *          On entry, the (n-1) subdiagonal elements of the tridiagonal
// *          matrix A, stored in elements 1 to N-1 of E; E(N) need not
// *          be set, but is used by the routine.
// *          On exit, the contents of E are destroyed.
// *
// *  Z       (output) DOUBLE PRECISION array, dimension (LDZ, N)
// *          If JOBZ = 'V', then if INFO = 0, Z contains the orthonormal
// *          eigenvectors of the matrix A, with the i-th column of Z
// *          holding the eigenvector associated with D(i).
// *          If JOBZ = 'N', then Z is not referenced.
// *
// *  LDZ     (input) INTEGER
// *          The leading dimension of the array Z.  LDZ >= 1, and if
// *          JOBZ = 'V', LDZ >= max(1,N).
// *
// *  WORK    (workspace) DOUBLE PRECISION array, dimension (max(1,2*N-2))
// *          If JOBZ = 'N', WORK is not referenced.
// *
// *  INFO    (output) INTEGER
// *          = 0:  successful exit
// *          < 0:  if INFO = -i, the i-th argument had an illegal value
// *          > 0:  if INFO = i, the algorithm failed to converge; i
// *                off-diagonal elements of E did not converge to zero.
// *
// *  =====================================================================
// *
// *     .. Parameters ..
static double zero= 0.0e0;
static double one= 1.0e0;
// *     ..
// *     .. Local Scalars ..
static boolean wantz= false;
static int imax= 0;
static int iscale= 0;
static double bignum= 0.0;
static double eps= 0.0;
static double rmax= 0.0;
static double rmin= 0.0;
static double safmin= 0.0;
static double sigma= 0.0;
static double smlnum= 0.0;
static double tnrm= 0.0;
// *     ..
// *     .. External Functions ..
// *     ..
// *     .. External Subroutines ..
// *     ..
// *     .. Intrinsic Functions ..
// *     ..
// *     .. Executable Statements ..
// *
// *     Test the input parameters.
// *

public static void dstev (String jobz,
int n,
double [] d, int _d_offset,
double [] e, int _e_offset,
double [] z, int _z_offset,
int ldz,
double [] work, int _work_offset,
intW info)  {

wantz = (jobz.toLowerCase().charAt(0) == "V".toLowerCase().charAt(0));
// *
info.val = 0;
if (!(wantz || (jobz.toLowerCase().charAt(0) == "N".toLowerCase().charAt(0))))  {
    info.val = -1;
}              // Close if()
else if (n < 0)  {
    info.val = -2;
}              // Close else if()
else if (ldz < 1 || (wantz && ldz < n))  {
    info.val = -6;
}              // Close else if()
// *
if (info.val != 0)  {
    Xerbla.xerbla("DSTEV ",-info.val);
Dummy.go_to("Dstev",999999);
}              // Close if()
// *
// *     Quick return if possible
// *
if (n == 0)  
    Dummy.go_to("Dstev",999999);
// *
if (n == 1)  {
    if (wantz)  
    z[(1)- 1+(1- 1)*ldz+ _z_offset] = one;
Dummy.go_to("Dstev",999999);
}              // Close if()
// *
// *     Get machine constants.
// *
safmin = Dlamch.dlamch("Safe minimum");
eps = Dlamch.dlamch("Precision");
smlnum = safmin/eps;
bignum = one/smlnum;
rmin = Math.sqrt(smlnum);
rmax = Math.sqrt(bignum);
// *
// *     Scale matrix to allowable range, if necessary.
// *
iscale = 0;
tnrm = Dlanst.dlanst("M",n,d,_d_offset,e,_e_offset);
if (tnrm > zero && tnrm < rmin)  {
    iscale = 1;
sigma = rmin/tnrm;
}              // Close if()
else if (tnrm > rmax)  {
    iscale = 1;
sigma = rmax/tnrm;
}              // Close else if()
if (iscale == 1)  {
    Dscal.dscal(n,sigma,d,_d_offset,1);
Dscal.dscal(n-1,sigma,e,(1)- 1+ _e_offset,1);
}              // Close if()
// *
// *     For eigenvalues only, call DSTERF.  For eigenvalues and
// *     eigenvectors, call DSTEQR.
// *
if (!wantz)  {
    Dsterf.dsterf(n,d,_d_offset,e,_e_offset,info);
}              // Close if()
else  {
  Dsteqr.dsteqr("I",n,d,_d_offset,e,_e_offset,z,_z_offset,ldz,work,_work_offset,info);
}              //  Close else.
// *
// *     If matrix was scaled, then rescale eigenvalues appropriately.
// *
if (iscale == 1)  {
    if (info.val == 0)  {
    imax = n;
}              // Close if()
else  {
  imax = info.val-1;
}              //  Close else.
Dscal.dscal(imax,one/sigma,d,_d_offset,1);
}              // Close if()
// *
Dummy.go_to("Dstev",999999);
// *
// *     End of DSTEV
// *
Dummy.label("Dstev",999999);
return;
   }
} // End class.
