/*
 *  Produced by f2java.  f2java is part of the Fortran-
 *  -to-Java project at the University of Tennessee Netlib
 *  numerical software repository.
 *
 *  Original authorship for the BLAS and LAPACK numerical
 *  routines may be found in the Fortran source, available at
 *  www.netlib.org.
 *
 *  Fortran input file: jlapack.f
 *
 *  The f2j compiler code was written by
 *  David M. Doolin (doolin@cs.utk.edu) and
 *  Keith  Seymour (seymour@cs.utk.edu)
 */

package org.netlib.lapack;
import java.lang.*;
import org.netlib.util.*;

import org.netlib.blas.*;


public class Dstevd {

// *
// *  -- LAPACK driver routine (version 2.0) --
// *     Univ. of Tennessee, Univ. of California Berkeley, NAG Ltd.,
// *     Courant Institute, Argonne National Lab, and Rice University
// *     September 30, 1994
// *
// *     .. Scalar Arguments ..
// *     ..
// *     .. Array Arguments ..
// *     ..
// *
// *  Purpose
// *  =======
// *
// *  DSTEVD computes all eigenvalues and, optionally, eigenvectors of a
// *  real symmetric tridiagonal matrix. If eigenvectors are desired, it
// *  uses a divide and conquer algorithm.
// *
// *  The divide and conquer algorithm makes very mild assumptions about
// *  floating point arithmetic. It will work on machines with a guard
// *  digit in add/subtract, or on those binary machines without guard
// *  digits which subtract like the Cray X-MP, Cray Y-MP, Cray C-90, or
// *  Cray-2. It could conceivably fail on hexadecimal or decimal machines
// *  without guard digits, but we know of none.
// *
// *  Arguments
// *  =========
// *
// *  JOBZ    (input) CHARACTER*1
// *          = 'N':  Compute eigenvalues only;
// *          = 'V':  Compute eigenvalues and eigenvectors.
// *
// *  N       (input) INTEGER
// *          The order of the matrix.  N >= 0.
// *
// *  D       (input/output) DOUBLE PRECISION array, dimension (N)
// *          On entry, the n diagonal elements of the tridiagonal matrix
// *          A.
// *          On exit, if INFO = 0, the eigenvalues in ascending order.
// *
// *  E       (input/output) DOUBLE PRECISION array, dimension (N)
// *          On entry, the (n-1) subdiagonal elements of the tridiagonal
// *          matrix A, stored in elements 1 to N-1 of E; E(N) need not
// *          be set, but is used by the routine.
// *          On exit, the contents of E are destroyed.
// *
// *  Z       (output) DOUBLE PRECISION array, dimension (LDZ, N)
// *          If JOBZ = 'V', then if INFO = 0, Z contains the orthonormal
// *          eigenvectors of the matrix A, with the i-th column of Z
// *          holding the eigenvector associated with D(i).
// *          If JOBZ = 'N', then Z is not referenced.
// *
// *  LDZ     (input) INTEGER
// *          The leading dimension of the array Z.  LDZ >= 1, and if
// *          JOBZ = 'V', LDZ >= max(1,N).
// *
// *  WORK    (workspace/output) DOUBLE PRECISION array,
// *                                         dimension (LWORK)
// *          On exit, if LWORK > 0, WORK(1) returns the optimal LWORK.
// *
// *  LWORK   (input) INTEGER
// *          The dimension of the array WORK.
// *          If JOBZ  = 'N' or N <= 1 then LWORK must be at least 1.
// *          If JOBZ  = 'V' and N > 1 then LWORK must be at least
// *                         ( 1 + 3*N + 2*N*lg N + 2*N**2 ),
// *                         where lg( N ) = smallest integer k such
// *                                         that 2**k >= N.
// *
// *  IWORK   (workspace/output) INTEGER array, dimension (LIWORK)
// *          On exit, if LIWORK > 0, IWORK(1) returns the optimal LIWORK.
// *
// *  LIWORK  (input) INTEGER
// *          The dimension of the array IWORK.
// *          If JOBZ  = 'N' or N <= 1 then LIWORK must be at least 1.
// *          If JOBZ  = 'V' and N > 1 then LIWORK must be at least 2+5*N.
// *
// *  INFO    (output) INTEGER
// *          = 0:  successful exit
// *          < 0:  if INFO = -i, the i-th argument had an illegal value
// *          > 0:  if INFO = i, the algorithm failed to converge; i
// *                off-diagonal elements of E did not converge to zero.
// *
// *  =====================================================================
// *
// *     .. Parameters ..
static double zero= 0.0e0;
static double one= 1.0e0;
static double two= 2.0e0;
// *     ..
// *     .. Local Scalars ..
static boolean wantz= false;
static int iscale= 0;
static int lgn= 0;
static int liwmin= 0;
static int lwmin= 0;
static double bignum= 0.0;
static double eps= 0.0;
static double rmax= 0.0;
static double rmin= 0.0;
static double safmin= 0.0;
static double sigma= 0.0;
static double smlnum= 0.0;
static double tnrm= 0.0;
// *     ..
// *     .. External Functions ..
// *     ..
// *     .. External Subroutines ..
// *     ..
// *     .. Intrinsic Functions ..
// *     ..
// *     .. Executable Statements ..
// *
// *     Test the input parameters.
// *

public static void dstevd (String jobz,
int n,
double [] d, int _d_offset,
double [] e, int _e_offset,
double [] z, int _z_offset,
int ldz,
double [] work, int _work_offset,
int lwork,
int [] iwork, int _iwork_offset,
int liwork,
intW info)  {

wantz = (jobz.toLowerCase().charAt(0) == "V".toLowerCase().charAt(0));
// *
info.val = 0;
liwmin = 1;
lwmin = 1;
if (!(wantz || (jobz.toLowerCase().charAt(0) == "N".toLowerCase().charAt(0))))  {
    info.val = -1;
}              // Close if()
else if (n < 0)  {
    info.val = -2;
}              // Close else if()
else if (ldz < 1 || (wantz && ldz < n))  {
    info.val = -6;
}              // Close else if()
else if (n > 1 && wantz)  {
    lgn = (int)(Math.log((double)(n))/Math.log(two));
if (Math.pow(2, lgn) < n)  
    lgn = lgn+1;
if (Math.pow(2, lgn) < n)  
    lgn = lgn+1;
lwmin = (int)(1+3*n+2*n*lgn+2*Math.pow(n, 2));
liwmin = 2+5*n;
if (lwork < lwmin)  {
    info.val = -8;
}              // Close if()
else if (liwork < liwmin)  {
    info.val = -10;
}              // Close else if()
}              // Close else if()
else if (lwork < 1)  {
    info.val = -8;
}              // Close else if()
else if (liwork < 1)  {
    info.val = -10;
}              // Close else if()
// *
if (info.val != 0)  {
    Xerbla.xerbla("DSTEVD",-info.val);
Dummy.go_to("Dstevd",10);
}              // Close if()
// *
// *     Quick return if possible
// *
if (n == 0)  
    Dummy.go_to("Dstevd",10);
// *
if (n == 1)  {
    if (wantz)  
    z[(1)- 1+(1- 1)*ldz+ _z_offset] = one;
Dummy.go_to("Dstevd",10);
}              // Close if()
// *
// *     Get machine constants.
// *
safmin = Dlamch.dlamch("Safe minimum");
eps = Dlamch.dlamch("Precision");
smlnum = safmin/eps;
bignum = one/smlnum;
rmin = Math.sqrt(smlnum);
rmax = Math.sqrt(bignum);
// *
// *     Scale matrix to allowable range, if necessary.
// *
iscale = 0;
tnrm = Dlanst.dlanst("M",n,d,_d_offset,e,_e_offset);
if (tnrm > zero && tnrm < rmin)  {
    iscale = 1;
sigma = rmin/tnrm;
}              // Close if()
else if (tnrm > rmax)  {
    iscale = 1;
sigma = rmax/tnrm;
}              // Close else if()
if (iscale == 1)  {
    Dscal.dscal(n,sigma,d,_d_offset,1);
Dscal.dscal(n-1,sigma,e,(1)- 1+ _e_offset,1);
}              // Close if()
// *
// *     For eigenvalues only, call DSTERF.  For eigenvalues and
// *     eigenvectors, call DSTEDC.
// *
if (!wantz)  {
    Dsterf.dsterf(n,d,_d_offset,e,_e_offset,info);
}              // Close if()
else  {
  Dstedc.dstedc("I",n,d,_d_offset,e,_e_offset,z,_z_offset,ldz,work,_work_offset,lwork,iwork,_iwork_offset,liwork,info);
}              //  Close else.
// *
// *     If matrix was scaled, then rescale eigenvalues appropriately.
// *
if (iscale == 1)  
    Dscal.dscal(n,one/sigma,d,_d_offset,1);
// *
label10:
   Dummy.label("Dstevd",10);
if (lwork > 0)  
    work[(1)- 1+ _work_offset] = (double)(lwmin);
if (liwork > 0)  
    iwork[(1)- 1+ _iwork_offset] = liwmin;
Dummy.go_to("Dstevd",999999);
// *
// *     End of DSTEVD
// *
Dummy.label("Dstevd",999999);
return;
   }
} // End class.
