/*
 *  Produced by f2java.  f2java is part of the Fortran-
 *  -to-Java project at the University of Tennessee Netlib
 *  numerical software repository.
 *
 *  Original authorship for the BLAS and LAPACK numerical
 *  routines may be found in the Fortran source, available at
 *  www.netlib.org.
 *
 *  Fortran input file: jlapack.f
 *
 *  The f2j compiler code was written by
 *  David M. Doolin (doolin@cs.utk.edu) and
 *  Keith  Seymour (seymour@cs.utk.edu)
 */

package org.netlib.lapack;
import java.lang.*;
import org.netlib.util.*;

import org.netlib.blas.*;


public class Dsyev {

// *
// *  -- LAPACK driver routine (version 2.0) --
// *     Univ. of Tennessee, Univ. of California Berkeley, NAG Ltd.,
// *     Courant Institute, Argonne National Lab, and Rice University
// *     September 30, 1994
// *
// *     .. Scalar Arguments ..
// *     ..
// *     .. Array Arguments ..
// *     ..
// *
// *  Purpose
// *  =======
// *
// *  DSYEV computes all eigenvalues and, optionally, eigenvectors of a
// *  real symmetric matrix A.
// *
// *  Arguments
// *  =========
// *
// *  JOBZ    (input) CHARACTER*1
// *          = 'N':  Compute eigenvalues only;
// *          = 'V':  Compute eigenvalues and eigenvectors.
// *
// *  UPLO    (input) CHARACTER*1
// *          = 'U':  Upper triangle of A is stored;
// *          = 'L':  Lower triangle of A is stored.
// *
// *  N       (input) INTEGER
// *          The order of the matrix A.  N >= 0.
// *
// *  A       (input/output) DOUBLE PRECISION array, dimension (LDA, N)
// *          On entry, the symmetric matrix A.  If UPLO = 'U', the
// *          leading N-by-N upper triangular part of A contains the
// *          upper triangular part of the matrix A.  If UPLO = 'L',
// *          the leading N-by-N lower triangular part of A contains
// *          the lower triangular part of the matrix A.
// *          On exit, if JOBZ = 'V', then if INFO = 0, A contains the
// *          orthonormal eigenvectors of the matrix A.
// *          If JOBZ = 'N', then on exit the lower triangle (if UPLO='L')
// *          or the upper triangle (if UPLO='U') of A, including the
// *          diagonal, is destroyed.
// *
// *  LDA     (input) INTEGER
// *          The leading dimension of the array A.  LDA >= max(1,N).
// *
// *  W       (output) DOUBLE PRECISION array, dimension (N)
// *          If INFO = 0, the eigenvalues in ascending order.
// *
// *  WORK    (workspace/output) DOUBLE PRECISION array, dimension (LWORK)
// *          On exit, if INFO = 0, WORK(1) returns the optimal LWORK.
// *
// *  LWORK   (input) INTEGER
// *          The length of the array WORK.  LWORK >= max(1,3*N-1).
// *          For optimal efficiency, LWORK >= (NB+2)*N,
// *          where NB is the blocksize for DSYTRD returned by ILAENV.
// *
// *  INFO    (output) INTEGER
// *          = 0:  successful exit
// *          < 0:  if INFO = -i, the i-th argument had an illegal value
// *          > 0:  if INFO = i, the algorithm failed to converge; i
// *                off-diagonal elements of an intermediate tridiagonal
// *                form did not converge to zero.
// *
// *  =====================================================================
// *
// *     .. Parameters ..
static double zero= 0.0e0;
static double one= 1.0e0;
// *     ..
// *     .. Local Scalars ..
static boolean lower= false;
static boolean wantz= false;
static intW iinfo= new intW(0);
static int imax= 0;
static int inde= 0;
static int indtau= 0;
static int indwrk= 0;
static int iscale= 0;
static int llwork= 0;
static int lopt= 0;
static double anrm= 0.0;
static double bignum= 0.0;
static double eps= 0.0;
static double rmax= 0.0;
static double rmin= 0.0;
static double safmin= 0.0;
static double sigma= 0.0;
static double smlnum= 0.0;
// *     ..
// *     .. External Functions ..
// *     ..
// *     .. External Subroutines ..
// *     ..
// *     .. Intrinsic Functions ..
// *     ..
// *     .. Executable Statements ..
// *
// *     Test the input parameters.
// *

public static void dsyev (String jobz,
String uplo,
int n,
double [] a, int _a_offset,
int lda,
double [] w, int _w_offset,
double [] work, int _work_offset,
int lwork,
intW info)  {

wantz = (jobz.toLowerCase().charAt(0) == "V".toLowerCase().charAt(0));
lower = (uplo.toLowerCase().charAt(0) == "L".toLowerCase().charAt(0));
// *
info.val = 0;
if (!(wantz || (jobz.toLowerCase().charAt(0) == "N".toLowerCase().charAt(0))))  {
    info.val = -1;
}              // Close if()
else if (!(lower || (uplo.toLowerCase().charAt(0) == "U".toLowerCase().charAt(0))))  {
    info.val = -2;
}              // Close else if()
else if (n < 0)  {
    info.val = -3;
}              // Close else if()
else if (lda < Math.max(1, n) )  {
    info.val = -5;
}              // Close else if()
else if (lwork < Math.max(1, 3*n-1) )  {
    info.val = -8;
}              // Close else if()
// *
if (info.val != 0)  {
    Xerbla.xerbla("DSYEV ",-info.val);
Dummy.go_to("Dsyev",999999);
}              // Close if()
// *
// *     Quick return if possible
// *
if (n == 0)  {
    work[(1)- 1+ _work_offset] = (double)(1);
Dummy.go_to("Dsyev",999999);
}              // Close if()
// *
if (n == 1)  {
    w[(1)- 1+ _w_offset] = a[(1)- 1+(1- 1)*lda+ _a_offset];
work[(1)- 1+ _work_offset] = (double)(3);
if (wantz)  
    a[(1)- 1+(1- 1)*lda+ _a_offset] = one;
Dummy.go_to("Dsyev",999999);
}              // Close if()
// *
// *     Get machine constants.
// *
safmin = Dlamch.dlamch("Safe minimum");
eps = Dlamch.dlamch("Precision");
smlnum = safmin/eps;
bignum = one/smlnum;
rmin = Math.sqrt(smlnum);
rmax = Math.sqrt(bignum);
// *
// *     Scale matrix to allowable range, if necessary.
// *
anrm = Dlansy.dlansy("M",uplo,n,a,_a_offset,lda,work,_work_offset);
iscale = 0;
if (anrm > zero && anrm < rmin)  {
    iscale = 1;
sigma = rmin/anrm;
}              // Close if()
else if (anrm > rmax)  {
    iscale = 1;
sigma = rmax/anrm;
}              // Close else if()
if (iscale == 1)  
    Dlascl.dlascl(uplo,0,0,one,sigma,n,n,a,_a_offset,lda,info);
// *
// *     Call DSYTRD to reduce symmetric matrix to tridiagonal form.
// *
inde = 1;
indtau = inde+n;
indwrk = indtau+n;
llwork = lwork-indwrk+1;
Dsytrd.dsytrd(uplo,n,a,_a_offset,lda,w,_w_offset,work,(inde)- 1+ _work_offset,work,(indtau)- 1+ _work_offset,work,(indwrk)- 1+ _work_offset,llwork,iinfo);
lopt = (int)(2*n+work[(indwrk)- 1+ _work_offset]);
// *
// *     For eigenvalues only, call DSTERF.  For eigenvectors, first call
// *     DORGTR to generate the orthogonal matrix, then call DSTEQR.
// *
if (!wantz)  {
    Dsterf.dsterf(n,w,_w_offset,work,(inde)- 1+ _work_offset,info);
}              // Close if()
else  {
  Dorgtr.dorgtr(uplo,n,a,_a_offset,lda,work,(indtau)- 1+ _work_offset,work,(indwrk)- 1+ _work_offset,llwork,iinfo);
Dsteqr.dsteqr(jobz,n,w,_w_offset,work,(inde)- 1+ _work_offset,a,_a_offset,lda,work,(indtau)- 1+ _work_offset,info);
}              //  Close else.
// *
// *     If matrix was scaled, then rescale eigenvalues appropriately.
// *
if (iscale == 1)  {
    if (info.val == 0)  {
    imax = n;
}              // Close if()
else  {
  imax = info.val-1;
}              //  Close else.
Dscal.dscal(imax,one/sigma,w,_w_offset,1);
}              // Close if()
// *
// *     Set WORK(1) to optimal workspace size.
// *
work[(1)- 1+ _work_offset] = Math.max(3*n-1, lopt) ;
// *
Dummy.go_to("Dsyev",999999);
// *
// *     End of DSYEV
// *
Dummy.label("Dsyev",999999);
return;
   }
} // End class.
