/*
 *  Produced by f2java.  f2java is part of the Fortran-
 *  -to-Java project at the University of Tennessee Netlib
 *  numerical software repository.
 *
 *  Original authorship for the BLAS and LAPACK numerical
 *  routines may be found in the Fortran source, available at
 *  www.netlib.org.
 *
 *  Fortran input file: jlapack.f
 *
 *  The f2j compiler code was written by
 *  David M. Doolin (doolin@cs.utk.edu) and
 *  Keith  Seymour (seymour@cs.utk.edu)
 */

package org.netlib.lapack;
import java.lang.*;
import org.netlib.util.*;

import org.netlib.blas.*;


public class Dsyevd {

// *
// *  -- LAPACK driver routine (version 2.0) --
// *     Univ. of Tennessee, Univ. of California Berkeley, NAG Ltd.,
// *     Courant Institute, Argonne National Lab, and Rice University
// *     September 30, 1994
// *
// *     .. Scalar Arguments ..
// *     ..
// *     .. Array Arguments ..
// *     ..
// *
// *  Purpose
// *  =======
// *
// *  DSYEVD computes all eigenvalues and, optionally, eigenvectors of a
// *  real symmetric matrix A. If eigenvectors are desired, it uses a
// *  divide and conquer algorithm.
// *
// *  The divide and conquer algorithm makes very mild assumptions about
// *  floating point arithmetic. It will work on machines with a guard
// *  digit in add/subtract, or on those binary machines without guard
// *  digits which subtract like the Cray X-MP, Cray Y-MP, Cray C-90, or
// *  Cray-2. It could conceivably fail on hexadecimal or decimal machines
// *  without guard digits, but we know of none.
// *
// *  Arguments
// *  =========
// *
// *  JOBZ    (input) CHARACTER*1
// *          = 'N':  Compute eigenvalues only;
// *          = 'V':  Compute eigenvalues and eigenvectors.
// *
// *  UPLO    (input) CHARACTER*1
// *          = 'U':  Upper triangle of A is stored;
// *          = 'L':  Lower triangle of A is stored.
// *
// *  N       (input) INTEGER
// *          The order of the matrix A.  N >= 0.
// *
// *  A       (input/output) DOUBLE PRECISION array, dimension (LDA, N)
// *          On entry, the symmetric matrix A.  If UPLO = 'U', the
// *          leading N-by-N upper triangular part of A contains the
// *          upper triangular part of the matrix A.  If UPLO = 'L',
// *          the leading N-by-N lower triangular part of A contains
// *          the lower triangular part of the matrix A.
// *          On exit, if JOBZ = 'V', then if INFO = 0, A contains the
// *          orthonormal eigenvectors of the matrix A.
// *          If JOBZ = 'N', then on exit the lower triangle (if UPLO='L')
// *          or the upper triangle (if UPLO='U') of A, including the
// *          diagonal, is destroyed.
// *
// *  LDA     (input) INTEGER
// *          The leading dimension of the array A.  LDA >= max(1,N).
// *
// *  W       (output) DOUBLE PRECISION array, dimension (N)
// *          If INFO = 0, the eigenvalues in ascending order.
// *
// *  WORK    (workspace/output) DOUBLE PRECISION array,
// *                                         dimension (LWORK)
// *          On exit, if LWORK > 0, WORK(1) returns the optimal LWORK.
// *
// *  LWORK   (input) INTEGER
// *          The dimension of the array WORK.
// *          If N <= 1,               LWORK must be at least 1.
// *          If JOBZ = 'N' and N > 1, LWORK must be at least 2*N+1.
// *          If JOBZ = 'V' and N > 1, LWORK must be at least
// *                         1 + 5*N + 2*N*lg N + 3*N**2,
// *                         where lg( N ) = smallest integer k such
// *                                         that 2**k >= N.
// *
// *  IWORK   (workspace/output) INTEGER array, dimension (LIWORK)
// *          On exit, if LIWORK > 0, IWORK(1) returns the optimal LIWORK.
// *
// *  LIWORK  (input) INTEGER
// *          The dimension of the array IWORK.
// *          If N <= 1,                LIWORK must be at least 1.
// *          If JOBZ  = 'N' and N > 1, LIWORK must be at least 1.
// *          If JOBZ  = 'V' and N > 1, LIWORK must be at least 2 + 5*N.
// *
// *  INFO    (output) INTEGER
// *          = 0:  successful exit
// *          < 0:  if INFO = -i, the i-th argument had an illegal value
// *          > 0:  if INFO = i, the algorithm failed to converge; i
// *                off-diagonal elements of an intermediate tridiagonal
// *                form did not converge to zero.
// *
// *  =====================================================================
// *
// *     .. Parameters ..
static double zero= 0.0e0;
static double one= 1.0e0;
static double two= 2.0e0;
// *     ..
// *     .. Local Scalars ..
// *
static boolean lower= false;
static boolean wantz= false;
static intW iinfo= new intW(0);
static int inde= 0;
static int indtau= 0;
static int indwk2= 0;
static int indwrk= 0;
static int iscale= 0;
static int lgn= 0;
static int liopt= 0;
static int liwmin= 0;
static int llwork= 0;
static int llwrk2= 0;
static int lopt= 0;
static int lwmin= 0;
static double anrm= 0.0;
static double bignum= 0.0;
static double eps= 0.0;
static double rmax= 0.0;
static double rmin= 0.0;
static double safmin= 0.0;
static double sigma= 0.0;
static double smlnum= 0.0;
// *     ..
// *     .. External Functions ..
// *     ..
// *     .. External Subroutines ..
// *     ..
// *     .. Intrinsic Functions ..
// *     ..
// *     .. Executable Statements ..
// *
// *     Test the input parameters.
// *

public static void dsyevd (String jobz,
String uplo,
int n,
double [] a, int _a_offset,
int lda,
double [] w, int _w_offset,
double [] work, int _work_offset,
int lwork,
int [] iwork, int _iwork_offset,
int liwork,
intW info)  {

wantz = (jobz.toLowerCase().charAt(0) == "V".toLowerCase().charAt(0));
lower = (uplo.toLowerCase().charAt(0) == "L".toLowerCase().charAt(0));
// *
info.val = 0;
if (n <= 1)  {
    lgn = 0;
liwmin = 1;
lwmin = 1;
lopt = lwmin;
liopt = liwmin;
}              // Close if()
else  {
  lgn = (int)(Math.log((double)(n))/Math.log(two));
if (Math.pow(2, lgn) < n)  
    lgn = lgn+1;
if (Math.pow(2, lgn) < n)  
    lgn = lgn+1;
if (wantz)  {
    liwmin = 2+5*n;
lwmin = (int)(1+5*n+2*n*lgn+3*Math.pow(n, 2));
}              // Close if()
else  {
  liwmin = 1;
lwmin = 2*n+1;
}              //  Close else.
lopt = lwmin;
liopt = liwmin;
}              //  Close else.
if (!(wantz || (jobz.toLowerCase().charAt(0) == "N".toLowerCase().charAt(0))))  {
    info.val = -1;
}              // Close if()
else if (!(lower || (uplo.toLowerCase().charAt(0) == "U".toLowerCase().charAt(0))))  {
    info.val = -2;
}              // Close else if()
else if (n < 0)  {
    info.val = -3;
}              // Close else if()
else if (lda < Math.max(1, n) )  {
    info.val = -5;
}              // Close else if()
else if (lwork < lwmin)  {
    info.val = -8;
}              // Close else if()
else if (liwork < liwmin)  {
    info.val = -10;
}              // Close else if()
// *
if (info.val != 0)  {
    Xerbla.xerbla("DSYEVD ",-info.val);
Dummy.go_to("Dsyevd",10);
}              // Close if()
// *
// *     Quick return if possible
// *
if (n == 0)  
    Dummy.go_to("Dsyevd",10);
// *
if (n == 1)  {
    w[(1)- 1+ _w_offset] = a[(1)- 1+(1- 1)*lda+ _a_offset];
if (wantz)  
    a[(1)- 1+(1- 1)*lda+ _a_offset] = one;
Dummy.go_to("Dsyevd",10);
}              // Close if()
// *
// *     Get machine constants.
// *
safmin = Dlamch.dlamch("Safe minimum");
eps = Dlamch.dlamch("Precision");
smlnum = safmin/eps;
bignum = one/smlnum;
rmin = Math.sqrt(smlnum);
rmax = Math.sqrt(bignum);
// *
// *     Scale matrix to allowable range, if necessary.
// *
anrm = Dlansy.dlansy("M",uplo,n,a,_a_offset,lda,work,_work_offset);
iscale = 0;
if (anrm > zero && anrm < rmin)  {
    iscale = 1;
sigma = rmin/anrm;
}              // Close if()
else if (anrm > rmax)  {
    iscale = 1;
sigma = rmax/anrm;
}              // Close else if()
if (iscale == 1)  
    Dlascl.dlascl(uplo,0,0,one,sigma,n,n,a,_a_offset,lda,info);
// *
// *     Call DSYTRD to reduce symmetric matrix to tridiagonal form.
// *
inde = 1;
indtau = inde+n;
indwrk = indtau+n;
llwork = lwork-indwrk+1;
indwk2 = indwrk+n*n;
llwrk2 = lwork-indwk2+1;
// *
Dsytrd.dsytrd(uplo,n,a,_a_offset,lda,w,_w_offset,work,(inde)- 1+ _work_offset,work,(indtau)- 1+ _work_offset,work,(indwrk)- 1+ _work_offset,llwork,iinfo);
lopt = (int)(2*n+work[(indwrk)- 1+ _work_offset]);
// *
// *     For eigenvalues only, call DSTERF.  For eigenvectors, first call
// *     DSTEDC to generate the eigenvector matrix, WORK(INDWRK), of the
// *     tridiagonal matrix, then call DORMTR to multiply it by the
// *     Householder transformations stored in A.
// *
if (!wantz)  {
    Dsterf.dsterf(n,w,_w_offset,work,(inde)- 1+ _work_offset,info);
}              // Close if()
else  {
  Dstedc.dstedc("I",n,w,_w_offset,work,(inde)- 1+ _work_offset,work,(indwrk)- 1+ _work_offset,n,work,(indwk2)- 1+ _work_offset,llwrk2,iwork,_iwork_offset,liwork,info);
Dormtr.dormtr("L",uplo,"N",n,n,a,_a_offset,lda,work,(indtau)- 1+ _work_offset,work,(indwrk)- 1+ _work_offset,n,work,(indwk2)- 1+ _work_offset,llwrk2,iinfo);
Dlacpy.dlacpy("A",n,n,work,(indwrk)- 1+ _work_offset,n,a,_a_offset,lda);
lopt = (int)(Math.max(lopt, 1+5*n+2*n*lgn+3*Math.pow(n, 2)) );
}              //  Close else.
// *
// *     If matrix was scaled, then rescale eigenvalues appropriately.
// *
if (iscale == 1)  
    Dscal.dscal(n,one/sigma,w,_w_offset,1);
label10:
   Dummy.label("Dsyevd",10);
if (lwork > 0)  
    work[(1)- 1+ _work_offset] = (double)(lopt);
if (liwork > 0)  
    iwork[(1)- 1+ _iwork_offset] = liopt;
Dummy.go_to("Dsyevd",999999);
// *
// *     End of DSYEVD
// *
Dummy.label("Dsyevd",999999);
return;
   }
} // End class.
