/*
 *  Produced by f2java.  f2java is part of the Fortran-
 *  -to-Java project at the University of Tennessee Netlib
 *  numerical software repository.
 *
 *  Original authorship for the BLAS and LAPACK numerical
 *  routines may be found in the Fortran source, available at
 *  www.netlib.org.
 *
 *  Fortran input file: jlapack.f
 *
 *  The f2j compiler code was written by
 *  David M. Doolin (doolin@cs.utk.edu) and
 *  Keith  Seymour (seymour@cs.utk.edu)
 */

package org.netlib.lapack;
import java.lang.*;
import org.netlib.util.*;

import org.netlib.blas.*;


public class Dsygs2 {

// *
// *  -- LAPACK routine (version 2.0) --
// *     Univ. of Tennessee, Univ. of California Berkeley, NAG Ltd.,
// *     Courant Institute, Argonne National Lab, and Rice University
// *     February 29, 1992
// *
// *     .. Scalar Arguments ..
// *     ..
// *     .. Array Arguments ..
// *     ..
// *
// *  Purpose
// *  =======
// *
// *  DSYGS2 reduces a real symmetric-definite generalized eigenproblem
// *  to standard form.
// *
// *  If ITYPE = 1, the problem is A*x = lambda*B*x,
// *  and A is overwritten by inv(U')*A*inv(U) or inv(L)*A*inv(L')
// *
// *  If ITYPE = 2 or 3, the problem is A*B*x = lambda*x or
// *  B*A*x = lambda*x, and A is overwritten by U*A*U` or L'*A*L.
// *
// *  B must have been previously factorized as U'*U or L*L' by DPOTRF.
// *
// *  Arguments
// *  =========
// *
// *  ITYPE   (input) INTEGER
// *          = 1: compute inv(U')*A*inv(U) or inv(L)*A*inv(L');
// *          = 2 or 3: compute U*A*U' or L'*A*L.
// *
// *  UPLO    (input) CHARACTER
// *          Specifies whether the upper or lower triangular part of the
// *          symmetric matrix A is stored, and how B has been factorized.
// *          = 'U':  Upper triangular
// *          = 'L':  Lower triangular
// *
// *  N       (input) INTEGER
// *          The order of the matrices A and B.  N >= 0.
// *
// *  A       (input/output) DOUBLE PRECISION array, dimension (LDA,N)
// *          On entry, the symmetric matrix A.  If UPLO = 'U', the leading
// *          n by n upper triangular part of A contains the upper
// *          triangular part of the matrix A, and the strictly lower
// *          triangular part of A is not referenced.  If UPLO = 'L', the
// *          leading n by n lower triangular part of A contains the lower
// *          triangular part of the matrix A, and the strictly upper
// *          triangular part of A is not referenced.
// *
// *          On exit, if INFO = 0, the transformed matrix, stored in the
// *          same format as A.
// *
// *  LDA     (input) INTEGER
// *          The leading dimension of the array A.  LDA >= max(1,N).
// *
// *  B       (input) DOUBLE PRECISION array, dimension (LDB,N)
// *          The triangular factor from the Cholesky factorization of B,
// *          as returned by DPOTRF.
// *
// *  LDB     (input) INTEGER
// *          The leading dimension of the array B.  LDB >= max(1,N).
// *
// *  INFO    (output) INTEGER
// *          = 0:  successful exit.
// *          < 0:  if INFO = -i, the i-th argument had an illegal value.
// *
// *  =====================================================================
// *
// *     .. Parameters ..
static double one= 1.0e0;
static double half= 0.5e0;
// *     ..
// *     .. Local Scalars ..
static boolean upper= false;
static int k= 0;
static double akk= 0.0;
static double bkk= 0.0;
static double ct= 0.0;
// *     ..
// *     .. External Subroutines ..
// *     ..
// *     .. Intrinsic Functions ..
// *     ..
// *     .. External Functions ..
// *     ..
// *     .. Executable Statements ..
// *
// *     Test the input parameters.
// *

public static void dsygs2 (int itype,
String uplo,
int n,
double [] a, int _a_offset,
int lda,
double [] b, int _b_offset,
int ldb,
intW info)  {

info.val = 0;
upper = (uplo.toLowerCase().charAt(0) == "U".toLowerCase().charAt(0));
if (itype < 1 || itype > 3)  {
    info.val = -1;
}              // Close if()
else if (!upper && !(uplo.toLowerCase().charAt(0) == "L".toLowerCase().charAt(0)))  {
    info.val = -2;
}              // Close else if()
else if (n < 0)  {
    info.val = -3;
}              // Close else if()
else if (lda < Math.max(1, n) )  {
    info.val = -5;
}              // Close else if()
else if (ldb < Math.max(1, n) )  {
    info.val = -7;
}              // Close else if()
if (info.val != 0)  {
    Xerbla.xerbla("DSYGS2",-info.val);
Dummy.go_to("Dsygs2",999999);
}              // Close if()
// *
if (itype == 1)  {
    if (upper)  {
    // *
// *           Compute inv(U')*A*inv(U)
// *
{
forloop10:
for (k = 1; k <= n; k++) {
// *
// *              Update the upper triangle of A(k:n,k:n)
// *
akk = a[(k)- 1+(k- 1)*lda+ _a_offset];
bkk = b[(k)- 1+(k- 1)*ldb+ _b_offset];
akk = akk/Math.pow(bkk, 2);
a[(k)- 1+(k- 1)*lda+ _a_offset] = akk;
if (k < n)  {
    Dscal.dscal(n-k,one/bkk,a,(k)- 1+(k+1- 1)*lda+ _a_offset,lda);
ct = -half*akk;
Daxpy.daxpy(n-k,ct,b,(k)- 1+(k+1- 1)*ldb+ _b_offset,ldb,a,(k)- 1+(k+1- 1)*lda+ _a_offset,lda);
Dsyr2.dsyr2(uplo,n-k,-one,a,(k)- 1+(k+1- 1)*lda+ _a_offset,lda,b,(k)- 1+(k+1- 1)*ldb+ _b_offset,ldb,a,(k+1)- 1+(k+1- 1)*lda+ _a_offset,lda);
Daxpy.daxpy(n-k,ct,b,(k)- 1+(k+1- 1)*ldb+ _b_offset,ldb,a,(k)- 1+(k+1- 1)*lda+ _a_offset,lda);
Dtrsv.dtrsv(uplo,"Transpose","Non-unit",n-k,b,(k+1)- 1+(k+1- 1)*ldb+ _b_offset,ldb,a,(k)- 1+(k+1- 1)*lda+ _a_offset,lda);
}              // Close if()
Dummy.label("Dsygs2",10);
}              //  Close for() loop. 
}
}              // Close if()
else  {
  // *
// *           Compute inv(L)*A*inv(L')
// *
{
forloop20:
for (k = 1; k <= n; k++) {
// *
// *              Update the lower triangle of A(k:n,k:n)
// *
akk = a[(k)- 1+(k- 1)*lda+ _a_offset];
bkk = b[(k)- 1+(k- 1)*ldb+ _b_offset];
akk = akk/Math.pow(bkk, 2);
a[(k)- 1+(k- 1)*lda+ _a_offset] = akk;
if (k < n)  {
    Dscal.dscal(n-k,one/bkk,a,(k+1)- 1+(k- 1)*lda+ _a_offset,1);
ct = -half*akk;
Daxpy.daxpy(n-k,ct,b,(k+1)- 1+(k- 1)*ldb+ _b_offset,1,a,(k+1)- 1+(k- 1)*lda+ _a_offset,1);
Dsyr2.dsyr2(uplo,n-k,-one,a,(k+1)- 1+(k- 1)*lda+ _a_offset,1,b,(k+1)- 1+(k- 1)*ldb+ _b_offset,1,a,(k+1)- 1+(k+1- 1)*lda+ _a_offset,lda);
Daxpy.daxpy(n-k,ct,b,(k+1)- 1+(k- 1)*ldb+ _b_offset,1,a,(k+1)- 1+(k- 1)*lda+ _a_offset,1);
Dtrsv.dtrsv(uplo,"No transpose","Non-unit",n-k,b,(k+1)- 1+(k+1- 1)*ldb+ _b_offset,ldb,a,(k+1)- 1+(k- 1)*lda+ _a_offset,1);
}              // Close if()
Dummy.label("Dsygs2",20);
}              //  Close for() loop. 
}
}              //  Close else.
}              // Close if()
else  {
  if (upper)  {
    // *
// *           Compute U*A*U'
// *
{
forloop30:
for (k = 1; k <= n; k++) {
// *
// *              Update the upper triangle of A(1:k,1:k)
// *
akk = a[(k)- 1+(k- 1)*lda+ _a_offset];
bkk = b[(k)- 1+(k- 1)*ldb+ _b_offset];
Dtrmv.dtrmv(uplo,"No transpose","Non-unit",k-1,b,_b_offset,ldb,a,(1)- 1+(k- 1)*lda+ _a_offset,1);
ct = half*akk;
Daxpy.daxpy(k-1,ct,b,(1)- 1+(k- 1)*ldb+ _b_offset,1,a,(1)- 1+(k- 1)*lda+ _a_offset,1);
Dsyr2.dsyr2(uplo,k-1,one,a,(1)- 1+(k- 1)*lda+ _a_offset,1,b,(1)- 1+(k- 1)*ldb+ _b_offset,1,a,_a_offset,lda);
Daxpy.daxpy(k-1,ct,b,(1)- 1+(k- 1)*ldb+ _b_offset,1,a,(1)- 1+(k- 1)*lda+ _a_offset,1);
Dscal.dscal(k-1,bkk,a,(1)- 1+(k- 1)*lda+ _a_offset,1);
a[(k)- 1+(k- 1)*lda+ _a_offset] = akk*Math.pow(bkk, 2);
Dummy.label("Dsygs2",30);
}              //  Close for() loop. 
}
}              // Close if()
else  {
  // *
// *           Compute L'*A*L
// *
{
forloop40:
for (k = 1; k <= n; k++) {
// *
// *              Update the lower triangle of A(1:k,1:k)
// *
akk = a[(k)- 1+(k- 1)*lda+ _a_offset];
bkk = b[(k)- 1+(k- 1)*ldb+ _b_offset];
Dtrmv.dtrmv(uplo,"Transpose","Non-unit",k-1,b,_b_offset,ldb,a,(k)- 1+(1- 1)*lda+ _a_offset,lda);
ct = half*akk;
Daxpy.daxpy(k-1,ct,b,(k)- 1+(1- 1)*ldb+ _b_offset,ldb,a,(k)- 1+(1- 1)*lda+ _a_offset,lda);
Dsyr2.dsyr2(uplo,k-1,one,a,(k)- 1+(1- 1)*lda+ _a_offset,lda,b,(k)- 1+(1- 1)*ldb+ _b_offset,ldb,a,_a_offset,lda);
Daxpy.daxpy(k-1,ct,b,(k)- 1+(1- 1)*ldb+ _b_offset,ldb,a,(k)- 1+(1- 1)*lda+ _a_offset,lda);
Dscal.dscal(k-1,bkk,a,(k)- 1+(1- 1)*lda+ _a_offset,lda);
a[(k)- 1+(k- 1)*lda+ _a_offset] = akk*Math.pow(bkk, 2);
Dummy.label("Dsygs2",40);
}              //  Close for() loop. 
}
}              //  Close else.
}              //  Close else.
Dummy.go_to("Dsygs2",999999);
// *
// *     End of DSYGS2
// *
Dummy.label("Dsygs2",999999);
return;
   }
} // End class.
