/*
 *  Produced by f2java.  f2java is part of the Fortran-
 *  -to-Java project at the University of Tennessee Netlib
 *  numerical software repository.
 *
 *  Original authorship for the BLAS and LAPACK numerical
 *  routines may be found in the Fortran source, available at
 *  www.netlib.org.
 *
 *  Fortran input file: jlapack.f
 *
 *  The f2j compiler code was written by
 *  David M. Doolin (doolin@cs.utk.edu) and
 *  Keith  Seymour (seymour@cs.utk.edu)
 */

package org.netlib.lapack;
import java.lang.*;
import org.netlib.util.*;

import org.netlib.blas.*;


public class Dsygst {

// *
// *  -- LAPACK routine (version 2.0) --
// *     Univ. of Tennessee, Univ. of California Berkeley, NAG Ltd.,
// *     Courant Institute, Argonne National Lab, and Rice University
// *     September 30, 1994
// *
// *     .. Scalar Arguments ..
// *     ..
// *     .. Array Arguments ..
// *     ..
// *
// *  Purpose
// *  =======
// *
// *  DSYGST reduces a real symmetric-definite generalized eigenproblem
// *  to standard form.
// *
// *  If ITYPE = 1, the problem is A*x = lambda*B*x,
// *  and A is overwritten by inv(U**T)*A*inv(U) or inv(L)*A*inv(L**T)
// *
// *  If ITYPE = 2 or 3, the problem is A*B*x = lambda*x or
// *  B*A*x = lambda*x, and A is overwritten by U*A*U**T or L**T*A*L.
// *
// *  B must have been previously factorized as U**T*U or L*L**T by DPOTRF.
// *
// *  Arguments
// *  =========
// *
// *  ITYPE   (input) INTEGER
// *          = 1: compute inv(U**T)*A*inv(U) or inv(L)*A*inv(L**T);
// *          = 2 or 3: compute U*A*U**T or L**T*A*L.
// *
// *  UPLO    (input) CHARACTER
// *          = 'U':  Upper triangle of A is stored and B is factored as
// *                  U**T*U;
// *          = 'L':  Lower triangle of A is stored and B is factored as
// *                  L*L**T.
// *
// *  N       (input) INTEGER
// *          The order of the matrices A and B.  N >= 0.
// *
// *  A       (input/output) DOUBLE PRECISION array, dimension (LDA,N)
// *          On entry, the symmetric matrix A.  If UPLO = 'U', the leading
// *          N-by-N upper triangular part of A contains the upper
// *          triangular part of the matrix A, and the strictly lower
// *          triangular part of A is not referenced.  If UPLO = 'L', the
// *          leading N-by-N lower triangular part of A contains the lower
// *          triangular part of the matrix A, and the strictly upper
// *          triangular part of A is not referenced.
// *
// *          On exit, if INFO = 0, the transformed matrix, stored in the
// *          same format as A.
// *
// *  LDA     (input) INTEGER
// *          The leading dimension of the array A.  LDA >= max(1,N).
// *
// *  B       (input) DOUBLE PRECISION array, dimension (LDB,N)
// *          The triangular factor from the Cholesky factorization of B,
// *          as returned by DPOTRF.
// *
// *  LDB     (input) INTEGER
// *          The leading dimension of the array B.  LDB >= max(1,N).
// *
// *  INFO    (output) INTEGER
// *          = 0:  successful exit
// *          < 0:  if INFO = -i, the i-th argument had an illegal value
// *
// *  =====================================================================
// *
// *     .. Parameters ..
static double one= 1.0e0;
static double half= 0.5e0;
// *     ..
// *     .. Local Scalars ..
static boolean upper= false;
static int k= 0;
static int kb= 0;
static int nb= 0;
// *     ..
// *     .. External Subroutines ..
// *     ..
// *     .. Intrinsic Functions ..
// *     ..
// *     .. External Functions ..
// *     ..
// *     .. Executable Statements ..
// *
// *     Test the input parameters.
// *

public static void dsygst (int itype,
String uplo,
int n,
double [] a, int _a_offset,
int lda,
double [] b, int _b_offset,
int ldb,
intW info)  {

info.val = 0;
upper = (uplo.toLowerCase().charAt(0) == "U".toLowerCase().charAt(0));
if (itype < 1 || itype > 3)  {
    info.val = -1;
}              // Close if()
else if (!upper && !(uplo.toLowerCase().charAt(0) == "L".toLowerCase().charAt(0)))  {
    info.val = -2;
}              // Close else if()
else if (n < 0)  {
    info.val = -3;
}              // Close else if()
else if (lda < Math.max(1, n) )  {
    info.val = -5;
}              // Close else if()
else if (ldb < Math.max(1, n) )  {
    info.val = -7;
}              // Close else if()
if (info.val != 0)  {
    Xerbla.xerbla("DSYGST",-info.val);
Dummy.go_to("Dsygst",999999);
}              // Close if()
// *
// *     Quick return if possible
// *
if (n == 0)  
    Dummy.go_to("Dsygst",999999);
// *
// *     Determine the block size for this environment.
// *
nb = Ilaenv.ilaenv(1,"DSYGST",uplo,n,-1,-1,-1);
// *
if (nb <= 1 || nb >= n)  {
    // *
// *        Use unblocked code
// *
Dsygs2.dsygs2(itype,uplo,n,a,_a_offset,lda,b,_b_offset,ldb,info);
}              // Close if()
else  {
  // *
// *        Use blocked code
// *
if (itype == 1)  {
    if (upper)  {
    // *
// *              Compute inv(U')*A*inv(U)
// *
{
int _k_inc = nb;
forloop10:
for (k = 1; (_k_inc < 0) ? k >= n : k <= n; k += _k_inc) {
kb = (int)(Math.min(n-k+1, nb) );
// *
// *                 Update the upper triangle of A(k:n,k:n)
// *
Dsygs2.dsygs2(itype,uplo,kb,a,(k)- 1+(k- 1)*lda+ _a_offset,lda,b,(k)- 1+(k- 1)*ldb+ _b_offset,ldb,info);
if (k+kb <= n)  {
    Dtrsm.dtrsm("Left",uplo,"Transpose","Non-unit",kb,n-k-kb+1,one,b,(k)- 1+(k- 1)*ldb+ _b_offset,ldb,a,(k)- 1+(k+kb- 1)*lda+ _a_offset,lda);
Dsymm.dsymm("Left",uplo,kb,n-k-kb+1,-half,a,(k)- 1+(k- 1)*lda+ _a_offset,lda,b,(k)- 1+(k+kb- 1)*ldb+ _b_offset,ldb,one,a,(k)- 1+(k+kb- 1)*lda+ _a_offset,lda);
Dsyr2k.dsyr2k(uplo,"Transpose",n-k-kb+1,kb,-one,a,(k)- 1+(k+kb- 1)*lda+ _a_offset,lda,b,(k)- 1+(k+kb- 1)*ldb+ _b_offset,ldb,one,a,(k+kb)- 1+(k+kb- 1)*lda+ _a_offset,lda);
Dsymm.dsymm("Left",uplo,kb,n-k-kb+1,-half,a,(k)- 1+(k- 1)*lda+ _a_offset,lda,b,(k)- 1+(k+kb- 1)*ldb+ _b_offset,ldb,one,a,(k)- 1+(k+kb- 1)*lda+ _a_offset,lda);
Dtrsm.dtrsm("Right",uplo,"No transpose","Non-unit",kb,n-k-kb+1,one,b,(k+kb)- 1+(k+kb- 1)*ldb+ _b_offset,ldb,a,(k)- 1+(k+kb- 1)*lda+ _a_offset,lda);
}              // Close if()
Dummy.label("Dsygst",10);
}              //  Close for() loop. 
}
}              // Close if()
else  {
  // *
// *              Compute inv(L)*A*inv(L')
// *
{
int _k_inc = nb;
forloop20:
for (k = 1; (_k_inc < 0) ? k >= n : k <= n; k += _k_inc) {
kb = (int)(Math.min(n-k+1, nb) );
// *
// *                 Update the lower triangle of A(k:n,k:n)
// *
Dsygs2.dsygs2(itype,uplo,kb,a,(k)- 1+(k- 1)*lda+ _a_offset,lda,b,(k)- 1+(k- 1)*ldb+ _b_offset,ldb,info);
if (k+kb <= n)  {
    Dtrsm.dtrsm("Right",uplo,"Transpose","Non-unit",n-k-kb+1,kb,one,b,(k)- 1+(k- 1)*ldb+ _b_offset,ldb,a,(k+kb)- 1+(k- 1)*lda+ _a_offset,lda);
Dsymm.dsymm("Right",uplo,n-k-kb+1,kb,-half,a,(k)- 1+(k- 1)*lda+ _a_offset,lda,b,(k+kb)- 1+(k- 1)*ldb+ _b_offset,ldb,one,a,(k+kb)- 1+(k- 1)*lda+ _a_offset,lda);
Dsyr2k.dsyr2k(uplo,"No transpose",n-k-kb+1,kb,-one,a,(k+kb)- 1+(k- 1)*lda+ _a_offset,lda,b,(k+kb)- 1+(k- 1)*ldb+ _b_offset,ldb,one,a,(k+kb)- 1+(k+kb- 1)*lda+ _a_offset,lda);
Dsymm.dsymm("Right",uplo,n-k-kb+1,kb,-half,a,(k)- 1+(k- 1)*lda+ _a_offset,lda,b,(k+kb)- 1+(k- 1)*ldb+ _b_offset,ldb,one,a,(k+kb)- 1+(k- 1)*lda+ _a_offset,lda);
Dtrsm.dtrsm("Left",uplo,"No transpose","Non-unit",n-k-kb+1,kb,one,b,(k+kb)- 1+(k+kb- 1)*ldb+ _b_offset,ldb,a,(k+kb)- 1+(k- 1)*lda+ _a_offset,lda);
}              // Close if()
Dummy.label("Dsygst",20);
}              //  Close for() loop. 
}
}              //  Close else.
}              // Close if()
else  {
  if (upper)  {
    // *
// *              Compute U*A*U'
// *
{
int _k_inc = nb;
forloop30:
for (k = 1; (_k_inc < 0) ? k >= n : k <= n; k += _k_inc) {
kb = (int)(Math.min(n-k+1, nb) );
// *
// *                 Update the upper triangle of A(1:k+kb-1,1:k+kb-1)
// *
Dtrmm.dtrmm("Left",uplo,"No transpose","Non-unit",k-1,kb,one,b,_b_offset,ldb,a,(1)- 1+(k- 1)*lda+ _a_offset,lda);
Dsymm.dsymm("Right",uplo,k-1,kb,half,a,(k)- 1+(k- 1)*lda+ _a_offset,lda,b,(1)- 1+(k- 1)*ldb+ _b_offset,ldb,one,a,(1)- 1+(k- 1)*lda+ _a_offset,lda);
Dsyr2k.dsyr2k(uplo,"No transpose",k-1,kb,one,a,(1)- 1+(k- 1)*lda+ _a_offset,lda,b,(1)- 1+(k- 1)*ldb+ _b_offset,ldb,one,a,_a_offset,lda);
Dsymm.dsymm("Right",uplo,k-1,kb,half,a,(k)- 1+(k- 1)*lda+ _a_offset,lda,b,(1)- 1+(k- 1)*ldb+ _b_offset,ldb,one,a,(1)- 1+(k- 1)*lda+ _a_offset,lda);
Dtrmm.dtrmm("Right",uplo,"Transpose","Non-unit",k-1,kb,one,b,(k)- 1+(k- 1)*ldb+ _b_offset,ldb,a,(1)- 1+(k- 1)*lda+ _a_offset,lda);
Dsygs2.dsygs2(itype,uplo,kb,a,(k)- 1+(k- 1)*lda+ _a_offset,lda,b,(k)- 1+(k- 1)*ldb+ _b_offset,ldb,info);
Dummy.label("Dsygst",30);
}              //  Close for() loop. 
}
}              // Close if()
else  {
  // *
// *              Compute L'*A*L
// *
{
int _k_inc = nb;
forloop40:
for (k = 1; (_k_inc < 0) ? k >= n : k <= n; k += _k_inc) {
kb = (int)(Math.min(n-k+1, nb) );
// *
// *                 Update the lower triangle of A(1:k+kb-1,1:k+kb-1)
// *
Dtrmm.dtrmm("Right",uplo,"No transpose","Non-unit",kb,k-1,one,b,_b_offset,ldb,a,(k)- 1+(1- 1)*lda+ _a_offset,lda);
Dsymm.dsymm("Left",uplo,kb,k-1,half,a,(k)- 1+(k- 1)*lda+ _a_offset,lda,b,(k)- 1+(1- 1)*ldb+ _b_offset,ldb,one,a,(k)- 1+(1- 1)*lda+ _a_offset,lda);
Dsyr2k.dsyr2k(uplo,"Transpose",k-1,kb,one,a,(k)- 1+(1- 1)*lda+ _a_offset,lda,b,(k)- 1+(1- 1)*ldb+ _b_offset,ldb,one,a,_a_offset,lda);
Dsymm.dsymm("Left",uplo,kb,k-1,half,a,(k)- 1+(k- 1)*lda+ _a_offset,lda,b,(k)- 1+(1- 1)*ldb+ _b_offset,ldb,one,a,(k)- 1+(1- 1)*lda+ _a_offset,lda);
Dtrmm.dtrmm("Left",uplo,"Transpose","Non-unit",kb,k-1,one,b,(k)- 1+(k- 1)*ldb+ _b_offset,ldb,a,(k)- 1+(1- 1)*lda+ _a_offset,lda);
Dsygs2.dsygs2(itype,uplo,kb,a,(k)- 1+(k- 1)*lda+ _a_offset,lda,b,(k)- 1+(k- 1)*ldb+ _b_offset,ldb,info);
Dummy.label("Dsygst",40);
}              //  Close for() loop. 
}
}              //  Close else.
}              //  Close else.
}              //  Close else.
Dummy.go_to("Dsygst",999999);
// *
// *     End of DSYGST
// *
Dummy.label("Dsygst",999999);
return;
   }
} // End class.
