/*
 *  Produced by f2java.  f2java is part of the Fortran-
 *  -to-Java project at the University of Tennessee Netlib
 *  numerical software repository.
 *
 *  Original authorship for the BLAS and LAPACK numerical
 *  routines may be found in the Fortran source, available at
 *  www.netlib.org.
 *
 *  Fortran input file: jlapack.f
 *
 *  The f2j compiler code was written by
 *  David M. Doolin (doolin@cs.utk.edu) and
 *  Keith  Seymour (seymour@cs.utk.edu)
 */

package org.netlib.lapack;
import java.lang.*;
import org.netlib.util.*;

import org.netlib.blas.*;


public class Dsygv {

// *
// *  -- LAPACK driver routine (version 2.0) --
// *     Univ. of Tennessee, Univ. of California Berkeley, NAG Ltd.,
// *     Courant Institute, Argonne National Lab, and Rice University
// *     September 30, 1994
// *
// *     .. Scalar Arguments ..
// *     ..
// *     .. Array Arguments ..
// *     ..
// *
// *  Purpose
// *  =======
// *
// *  DSYGV computes all the eigenvalues, and optionally, the eigenvectors
// *  of a real generalized symmetric-definite eigenproblem, of the form
// *  A*x=(lambda)*B*x,  A*Bx=(lambda)*x,  or B*A*x=(lambda)*x.
// *  Here A and B are assumed to be symmetric and B is also
// *  positive definite.
// *
// *  Arguments
// *  =========
// *
// *  ITYPE   (input) INTEGER
// *          Specifies the problem type to be solved:
// *          = 1:  A*x = (lambda)*B*x
// *          = 2:  A*B*x = (lambda)*x
// *          = 3:  B*A*x = (lambda)*x
// *
// *  JOBZ    (input) CHARACTER*1
// *          = 'N':  Compute eigenvalues only;
// *          = 'V':  Compute eigenvalues and eigenvectors.
// *
// *  UPLO    (input) CHARACTER*1
// *          = 'U':  Upper triangles of A and B are stored;
// *          = 'L':  Lower triangles of A and B are stored.
// *
// *  N       (input) INTEGER
// *          The order of the matrices A and B.  N >= 0.
// *
// *  A       (input/output) DOUBLE PRECISION array, dimension (LDA, N)
// *          On entry, the symmetric matrix A.  If UPLO = 'U', the
// *          leading N-by-N upper triangular part of A contains the
// *          upper triangular part of the matrix A.  If UPLO = 'L',
// *          the leading N-by-N lower triangular part of A contains
// *          the lower triangular part of the matrix A.
// *
// *          On exit, if JOBZ = 'V', then if INFO = 0, A contains the
// *          matrix Z of eigenvectors.  The eigenvectors are normalized
// *          as follows:
// *          if ITYPE = 1 or 2, Z**T*B*Z = I;
// *          if ITYPE = 3, Z**T*inv(B)*Z = I.
// *          If JOBZ = 'N', then on exit the upper triangle (if UPLO='U')
// *          or the lower triangle (if UPLO='L') of A, including the
// *          diagonal, is destroyed.
// *
// *  LDA     (input) INTEGER
// *          The leading dimension of the array A.  LDA >= max(1,N).
// *
// *  B       (input/output) DOUBLE PRECISION array, dimension (LDB, N)
// *          On entry, the symmetric matrix B.  If UPLO = 'U', the
// *          leading N-by-N upper triangular part of B contains the
// *          upper triangular part of the matrix B.  If UPLO = 'L',
// *          the leading N-by-N lower triangular part of B contains
// *          the lower triangular part of the matrix B.
// *
// *          On exit, if INFO <= N, the part of B containing the matrix is
// *          overwritten by the triangular factor U or L from the Cholesky
// *          factorization B = U**T*U or B = L*L**T.
// *
// *  LDB     (input) INTEGER
// *          The leading dimension of the array B.  LDB >= max(1,N).
// *
// *  W       (output) DOUBLE PRECISION array, dimension (N)
// *          If INFO = 0, the eigenvalues in ascending order.
// *
// *  WORK    (workspace/output) DOUBLE PRECISION array, dimension (LWORK)
// *          On exit, if INFO = 0, WORK(1) returns the optimal LWORK.
// *
// *  LWORK   (input) INTEGER
// *          The length of the array WORK.  LWORK >= max(1,3*N-1).
// *          For optimal efficiency, LWORK >= (NB+2)*N,
// *          where NB is the blocksize for DSYTRD returned by ILAENV.
// *
// *  INFO    (output) INTEGER
// *          = 0:  successful exit
// *          < 0:  if INFO = -i, the i-th argument had an illegal value
// *          > 0:  DPOTRF or DSYEV returned an error code:
// *             <= N:  if INFO = i, DSYEV failed to converge;
// *                    i off-diagonal elements of an intermediate
// *                    tridiagonal form did not converge to zero;
// *             > N:   if INFO = N + i, for 1 <= i <= N, then the leading
// *                    minor of order i of B is not positive definite.
// *                    The factorization of B could not be completed and
// *                    no eigenvalues or eigenvectors were computed.
// *
// *  =====================================================================
// *
// *     .. Parameters ..
static double one= 1.0e+0;
// *     ..
// *     .. Local Scalars ..
static boolean upper= false;
static boolean wantz= false;
static String trans= new String(" ");
static int neig= 0;
// *     ..
// *     .. External Functions ..
// *     ..
// *     .. External Subroutines ..
// *     ..
// *     .. Intrinsic Functions ..
// *     ..
// *     .. Executable Statements ..
// *
// *     Test the input parameters.
// *

public static void dsygv (int itype,
String jobz,
String uplo,
int n,
double [] a, int _a_offset,
int lda,
double [] b, int _b_offset,
int ldb,
double [] w, int _w_offset,
double [] work, int _work_offset,
int lwork,
intW info)  {

wantz = (jobz.toLowerCase().charAt(0) == "V".toLowerCase().charAt(0));
upper = (uplo.toLowerCase().charAt(0) == "U".toLowerCase().charAt(0));
// *
info.val = 0;
if (itype < 0 || itype > 3)  {
    info.val = -1;
}              // Close if()
else if (!(wantz || (jobz.toLowerCase().charAt(0) == "N".toLowerCase().charAt(0))))  {
    info.val = -2;
}              // Close else if()
else if (!(upper || (uplo.toLowerCase().charAt(0) == "L".toLowerCase().charAt(0))))  {
    info.val = -3;
}              // Close else if()
else if (n < 0)  {
    info.val = -4;
}              // Close else if()
else if (lda < Math.max(1, n) )  {
    info.val = -6;
}              // Close else if()
else if (ldb < Math.max(1, n) )  {
    info.val = -8;
}              // Close else if()
else if (lwork < Math.max(1, 3*n-1) )  {
    info.val = -11;
}              // Close else if()
if (info.val != 0)  {
    Xerbla.xerbla("DSYGV ",-info.val);
Dummy.go_to("Dsygv",999999);
}              // Close if()
// *
// *     Quick return if possible
// *
if (n == 0)  
    Dummy.go_to("Dsygv",999999);
// *
// *     Form a Cholesky factorization of B.
// *
Dpotrf.dpotrf(uplo,n,b,_b_offset,ldb,info);
if (info.val != 0)  {
    info.val = n+info.val;
Dummy.go_to("Dsygv",999999);
}              // Close if()
// *
// *     Transform problem to standard eigenvalue problem and solve.
// *
Dsygst.dsygst(itype,uplo,n,a,_a_offset,lda,b,_b_offset,ldb,info);
Dsyev.dsyev(jobz,uplo,n,a,_a_offset,lda,w,_w_offset,work,_work_offset,lwork,info);
// *
if (wantz)  {
    // *
// *        Backtransform eigenvectors to the original problem.
// *
neig = n;
if (info.val > 0)  
    neig = info.val-1;
if (itype == 1 || itype == 2)  {
    // *
// *           For A*x=(lambda)*B*x and A*B*x=(lambda)*x;
// *           backtransform eigenvectors: x = inv(L)'*y or inv(U)*y
// *
if (upper)  {
    trans = "N";
}              // Close if()
else  {
  trans = "T";
}              //  Close else.
// *
Dtrsm.dtrsm("Left",uplo,trans,"Non-unit",n,neig,one,b,_b_offset,ldb,a,_a_offset,lda);
// *
}              // Close if()
else if (itype == 3)  {
    // *
// *           For B*A*x=(lambda)*x;
// *           backtransform eigenvectors: x = L*y or U'*y
// *
if (upper)  {
    trans = "T";
}              // Close if()
else  {
  trans = "N";
}              //  Close else.
// *
Dtrmm.dtrmm("Left",uplo,trans,"Non-unit",n,neig,one,b,_b_offset,ldb,a,_a_offset,lda);
}              // Close else if()
}              // Close if()
Dummy.go_to("Dsygv",999999);
// *
// *     End of DSYGV
// *
Dummy.label("Dsygv",999999);
return;
   }
} // End class.
