/*
 *  Produced by f2java.  f2java is part of the Fortran-
 *  -to-Java project at the University of Tennessee Netlib
 *  numerical software repository.
 *
 *  Original authorship for the BLAS and LAPACK numerical
 *  routines may be found in the Fortran source, available at
 *  www.netlib.org.
 *
 *  Fortran input file: jlapack.f
 *
 *  The f2j compiler code was written by
 *  David M. Doolin (doolin@cs.utk.edu) and
 *  Keith  Seymour (seymour@cs.utk.edu)
 */

package org.netlib.lapack;
import java.lang.*;
import org.netlib.util.*;



public class Dsysv {

// *
// *  -- LAPACK driver routine (version 2.0) --
// *     Univ. of Tennessee, Univ. of California Berkeley, NAG Ltd.,
// *     Courant Institute, Argonne National Lab, and Rice University
// *     September 30, 1994
// *
// *     .. Scalar Arguments ..
// *     ..
// *     .. Array Arguments ..
// *     ..
// *
// *  Purpose
// *  =======
// *
// *  DSYSV computes the solution to a real system of linear equations
// *     A * X = B,
// *  where A is an N-by-N symmetric matrix and X and B are N-by-NRHS
// *  matrices.
// *
// *  The diagonal pivoting method is used to factor A as
// *     A = U * D * U**T,  if UPLO = 'U', or
// *     A = L * D * L**T,  if UPLO = 'L',
// *  where U (or L) is a product of permutation and unit upper (lower)
// *  triangular matrices, and D is symmetric and block diagonal with
// *  1-by-1 and 2-by-2 diagonal blocks.  The factored form of A is then
// *  used to solve the system of equations A * X = B.
// *
// *  Arguments
// *  =========
// *
// *  UPLO    (input) CHARACTER*1
// *          = 'U':  Upper triangle of A is stored;
// *          = 'L':  Lower triangle of A is stored.
// *
// *  N       (input) INTEGER
// *          The number of linear equations, i.e., the order of the
// *          matrix A.  N >= 0.
// *
// *  NRHS    (input) INTEGER
// *          The number of right hand sides, i.e., the number of columns
// *          of the matrix B.  NRHS >= 0.
// *
// *  A       (input/output) DOUBLE PRECISION array, dimension (LDA,N)
// *          On entry, the symmetric matrix A.  If UPLO = 'U', the leading
// *          N-by-N upper triangular part of A contains the upper
// *          triangular part of the matrix A, and the strictly lower
// *          triangular part of A is not referenced.  If UPLO = 'L', the
// *          leading N-by-N lower triangular part of A contains the lower
// *          triangular part of the matrix A, and the strictly upper
// *          triangular part of A is not referenced.
// *
// *          On exit, if INFO = 0, the block diagonal matrix D and the
// *          multipliers used to obtain the factor U or L from the
// *          factorization A = U*D*U**T or A = L*D*L**T as computed by
// *          DSYTRF.
// *
// *  LDA     (input) INTEGER
// *          The leading dimension of the array A.  LDA >= max(1,N).
// *
// *  IPIV    (output) INTEGER array, dimension (N)
// *          Details of the interchanges and the block structure of D, as
// *          determined by DSYTRF.  If IPIV(k) > 0, then rows and columns
// *          k and IPIV(k) were interchanged, and D(k,k) is a 1-by-1
// *          diagonal block.  If UPLO = 'U' and IPIV(k) = IPIV(k-1) < 0,
// *          then rows and columns k-1 and -IPIV(k) were interchanged and
// *          D(k-1:k,k-1:k) is a 2-by-2 diagonal block.  If UPLO = 'L' and
// *          IPIV(k) = IPIV(k+1) < 0, then rows and columns k+1 and
// *          -IPIV(k) were interchanged and D(k:k+1,k:k+1) is a 2-by-2
// *          diagonal block.
// *
// *  B       (input/output) DOUBLE PRECISION array, dimension (LDB,NRHS)
// *          On entry, the N-by-NRHS right hand side matrix B.
// *          On exit, if INFO = 0, the N-by-NRHS solution matrix X.
// *
// *  LDB     (input) INTEGER
// *          The leading dimension of the array B.  LDB >= max(1,N).
// *
// *  WORK    (workspace/output) DOUBLE PRECISION array, dimension (LWORK)
// *          On exit, if INFO = 0, WORK(1) returns the optimal LWORK.
// *
// *  LWORK   (input) INTEGER
// *          The length of WORK.  LWORK >= 1, and for best performance
// *          LWORK >= N*NB, where NB is the optimal blocksize for
// *          DSYTRF.
// *
// *  INFO    (output) INTEGER
// *          = 0: successful exit
// *          < 0: if INFO = -i, the i-th argument had an illegal value
// *          > 0: if INFO = i, D(i,i) is exactly zero.  The factorization
// *               has been completed, but the block diagonal matrix D is
// *               exactly singular, so the solution could not be computed.
// *
// *  =====================================================================
// *
// *     .. External Functions ..
// *     ..
// *     .. External Subroutines ..
// *     ..
// *     .. Intrinsic Functions ..
// *     ..
// *     .. Executable Statements ..
// *
// *     Test the input parameters.
// *

public static void dsysv (String uplo,
int n,
int nrhs,
double [] a, int _a_offset,
int lda,
int [] ipiv, int _ipiv_offset,
double [] b, int _b_offset,
int ldb,
double [] work, int _work_offset,
int lwork,
intW info)  {

info.val = 0;
if (!(uplo.toLowerCase().charAt(0) == "U".toLowerCase().charAt(0)) && !(uplo.toLowerCase().charAt(0) == "L".toLowerCase().charAt(0)))  {
    info.val = -1;
}              // Close if()
else if (n < 0)  {
    info.val = -2;
}              // Close else if()
else if (nrhs < 0)  {
    info.val = -3;
}              // Close else if()
else if (lda < Math.max(1, n) )  {
    info.val = -5;
}              // Close else if()
else if (ldb < Math.max(1, n) )  {
    info.val = -8;
}              // Close else if()
else if (lwork < 1)  {
    info.val = -10;
}              // Close else if()
if (info.val != 0)  {
    Xerbla.xerbla("DSYSV ",-info.val);
Dummy.go_to("Dsysv",999999);
}              // Close if()
// *
// *     Compute the factorization A = U*D*U' or A = L*D*L'.
// *
Dsytrf.dsytrf(uplo,n,a,_a_offset,lda,ipiv,_ipiv_offset,work,_work_offset,lwork,info);
if (info.val == 0)  {
    // *
// *        Solve the system A*X = B, overwriting B with X.
// *
Dsytrs.dsytrs(uplo,n,nrhs,a,_a_offset,lda,ipiv,_ipiv_offset,b,_b_offset,ldb,info);
// *
}              // Close if()
Dummy.go_to("Dsysv",999999);
// *
// *     End of DSYSV
// *
Dummy.label("Dsysv",999999);
return;
   }
} // End class.
