/*
 *  Produced by f2java.  f2java is part of the Fortran-
 *  -to-Java project at the University of Tennessee Netlib
 *  numerical software repository.
 *
 *  Original authorship for the BLAS and LAPACK numerical
 *  routines may be found in the Fortran source, available at
 *  www.netlib.org.
 *
 *  Fortran input file: jlapack.f
 *
 *  The f2j compiler code was written by
 *  David M. Doolin (doolin@cs.utk.edu) and
 *  Keith  Seymour (seymour@cs.utk.edu)
 */

package org.netlib.lapack;
import java.lang.*;
import org.netlib.util.*;



public class Dsysvx {

// *
// *  -- LAPACK driver routine (version 2.0) --
// *     Univ. of Tennessee, Univ. of California Berkeley, NAG Ltd.,
// *     Courant Institute, Argonne National Lab, and Rice University
// *     September 30, 1994
// *
// *     .. Scalar Arguments ..
// *     ..
// *     .. Array Arguments ..
// *     ..
// *
// *  Purpose
// *  =======
// *
// *  DSYSVX uses the diagonal pivoting factorization to compute the
// *  solution to a real system of linear equations A * X = B,
// *  where A is an N-by-N symmetric matrix and X and B are N-by-NRHS
// *  matrices.
// *
// *  Error bounds on the solution and a condition estimate are also
// *  provided.
// *
// *  Description
// *  ===========
// *
// *  The following steps are performed:
// *
// *  1. If FACT = 'N', the diagonal pivoting method is used to factor A.
// *     The form of the factorization is
// *        A = U * D * U**T,  if UPLO = 'U', or
// *        A = L * D * L**T,  if UPLO = 'L',
// *     where U (or L) is a product of permutation and unit upper (lower)
// *     triangular matrices, and D is symmetric and block diagonal with
// *     1-by-1 and 2-by-2 diagonal blocks.
// *
// *  2. The factored form of A is used to estimate the condition number
// *     of the matrix A.  If the reciprocal of the condition number is
// *     less than machine precision, steps 3 and 4 are skipped.
// *
// *  3. The system of equations is solved for X using the factored form
// *     of A.
// *
// *  4. Iterative refinement is applied to improve the computed solution
// *     matrix and calculate error bounds and backward error estimates
// *     for it.
// *
// *  Arguments
// *  =========
// *
// *  FACT    (input) CHARACTER*1
// *          Specifies whether or not the factored form of A has been
// *          supplied on entry.
// *          = 'F':  On entry, AF and IPIV contain the factored form of
// *                  A.  AF and IPIV will not be modified.
// *          = 'N':  The matrix A will be copied to AF and factored.
// *
// *  UPLO    (input) CHARACTER*1
// *          = 'U':  Upper triangle of A is stored;
// *          = 'L':  Lower triangle of A is stored.
// *
// *  N       (input) INTEGER
// *          The number of linear equations, i.e., the order of the
// *          matrix A.  N >= 0.
// *
// *  NRHS    (input) INTEGER
// *          The number of right hand sides, i.e., the number of columns
// *          of the matrices B and X.  NRHS >= 0.
// *
// *  A       (input) DOUBLE PRECISION array, dimension (LDA,N)
// *          The symmetric matrix A.  If UPLO = 'U', the leading N-by-N
// *          upper triangular part of A contains the upper triangular part
// *          of the matrix A, and the strictly lower triangular part of A
// *          is not referenced.  If UPLO = 'L', the leading N-by-N lower
// *          triangular part of A contains the lower triangular part of
// *          the matrix A, and the strictly upper triangular part of A is
// *          not referenced.
// *
// *  LDA     (input) INTEGER
// *          The leading dimension of the array A.  LDA >= max(1,N).
// *
// *  AF      (input or output) DOUBLE PRECISION array, dimension (LDAF,N)
// *          If FACT = 'F', then AF is an input argument and on entry
// *          contains the block diagonal matrix D and the multipliers used
// *          to obtain the factor U or L from the factorization
// *          A = U*D*U**T or A = L*D*L**T as computed by DSYTRF.
// *
// *          If FACT = 'N', then AF is an output argument and on exit
// *          returns the block diagonal matrix D and the multipliers used
// *          to obtain the factor U or L from the factorization
// *          A = U*D*U**T or A = L*D*L**T.
// *
// *  LDAF    (input) INTEGER
// *          The leading dimension of the array AF.  LDAF >= max(1,N).
// *
// *  IPIV    (input or output) INTEGER array, dimension (N)
// *          If FACT = 'F', then IPIV is an input argument and on entry
// *          contains details of the interchanges and the block structure
// *          of D, as determined by DSYTRF.
// *          If IPIV(k) > 0, then rows and columns k and IPIV(k) were
// *          interchanged and D(k,k) is a 1-by-1 diagonal block.
// *          If UPLO = 'U' and IPIV(k) = IPIV(k-1) < 0, then rows and
// *          columns k-1 and -IPIV(k) were interchanged and D(k-1:k,k-1:k)
// *          is a 2-by-2 diagonal block.  If UPLO = 'L' and IPIV(k) =
// *          IPIV(k+1) < 0, then rows and columns k+1 and -IPIV(k) were
// *          interchanged and D(k:k+1,k:k+1) is a 2-by-2 diagonal block.
// *
// *          If FACT = 'N', then IPIV is an output argument and on exit
// *          contains details of the interchanges and the block structure
// *          of D, as determined by DSYTRF.
// *
// *  B       (input) DOUBLE PRECISION array, dimension (LDB,NRHS)
// *          The N-by-NRHS right hand side matrix B.
// *
// *  LDB     (input) INTEGER
// *          The leading dimension of the array B.  LDB >= max(1,N).
// *
// *  X       (output) DOUBLE PRECISION array, dimension (LDX,NRHS)
// *          If INFO = 0, the N-by-NRHS solution matrix X.
// *
// *  LDX     (input) INTEGER
// *          The leading dimension of the array X.  LDX >= max(1,N).
// *
// *  RCOND   (output) DOUBLE PRECISION
// *          The estimate of the reciprocal condition number of the matrix
// *          A.  If RCOND is less than the machine precision (in
// *          particular, if RCOND = 0), the matrix is singular to working
// *          precision.  This condition is indicated by a return code of
// *          INFO > 0, and the solution and error bounds are not computed.
// *
// *  FERR    (output) DOUBLE PRECISION array, dimension (NRHS)
// *          The estimated forward error bound for each solution vector
// *          X(j) (the j-th column of the solution matrix X).
// *          If XTRUE is the true solution corresponding to X(j), FERR(j)
// *          is an estimated upper bound for the magnitude of the largest
// *          element in (X(j) - XTRUE) divided by the magnitude of the
// *          largest element in X(j).  The estimate is as reliable as
// *          the estimate for RCOND, and is almost always a slight
// *          overestimate of the true error.
// *
// *  BERR    (output) DOUBLE PRECISION array, dimension (NRHS)
// *          The componentwise relative backward error of each solution
// *          vector X(j) (i.e., the smallest relative change in
// *          any element of A or B that makes X(j) an exact solution).
// *
// *  WORK    (workspace/output) DOUBLE PRECISION array, dimension (LWORK)
// *          On exit, if INFO = 0, WORK(1) returns the optimal LWORK.
// *
// *  LWORK   (input) INTEGER
// *          The length of WORK.  LWORK >= 3*N, and for best performance
// *          LWORK >= N*NB, where NB is the optimal blocksize for
// *          DSYTRF.
// *
// *  IWORK   (workspace) INTEGER array, dimension (N)
// *
// *  INFO    (output) INTEGER
// *          = 0: successful exit
// *          < 0: if INFO = -i, the i-th argument had an illegal value
// *          > 0: if INFO = i, and i is
// *               <= N: D(i,i) is exactly zero.  The factorization has
// *                     has been completed, but the block diagonal
// *                     matrix D is exactly singular, so the solution and
// *                     error bounds could not be computed.
// *               = N+1: the block diagonal matrix D is nonsingular, but
// *                     RCOND is less than machine precision.  The
// *                     factorization has been completed, but the matrix
// *                     is singular to working precision, so the solution
// *                     and error bounds have not been computed.
// *
// *  =====================================================================
// *
// *     .. Parameters ..
static double zero= 0.0e+0;
// *     ..
// *     .. Local Scalars ..
static boolean nofact= false;
static double anorm= 0.0;
// *     ..
// *     .. External Functions ..
// *     ..
// *     .. External Subroutines ..
// *     ..
// *     .. Intrinsic Functions ..
// *     ..
// *     .. Executable Statements ..
// *
// *     Test the input parameters.
// *

public static void dsysvx (String fact,
String uplo,
int n,
int nrhs,
double [] a, int _a_offset,
int lda,
double [] af, int _af_offset,
int ldaf,
int [] ipiv, int _ipiv_offset,
double [] b, int _b_offset,
int ldb,
double [] x, int _x_offset,
int ldx,
doubleW rcond,
double [] ferr, int _ferr_offset,
double [] berr, int _berr_offset,
double [] work, int _work_offset,
int lwork,
int [] iwork, int _iwork_offset,
intW info)  {

info.val = 0;
nofact = (fact.toLowerCase().charAt(0) == "N".toLowerCase().charAt(0));
if (!nofact && !(fact.toLowerCase().charAt(0) == "F".toLowerCase().charAt(0)))  {
    info.val = -1;
}              // Close if()
else if (!(uplo.toLowerCase().charAt(0) == "U".toLowerCase().charAt(0)) && !(uplo.toLowerCase().charAt(0) == "L".toLowerCase().charAt(0)))  {
    info.val = -2;
}              // Close else if()
else if (n < 0)  {
    info.val = -3;
}              // Close else if()
else if (nrhs < 0)  {
    info.val = -4;
}              // Close else if()
else if (lda < Math.max(1, n) )  {
    info.val = -6;
}              // Close else if()
else if (ldaf < Math.max(1, n) )  {
    info.val = -8;
}              // Close else if()
else if (ldb < Math.max(1, n) )  {
    info.val = -11;
}              // Close else if()
else if (ldx < Math.max(1, n) )  {
    info.val = -13;
}              // Close else if()
else if (lwork < 2*n)  {
    info.val = -18;
}              // Close else if()
if (info.val != 0)  {
    Xerbla.xerbla("DSYSVX",-info.val);
Dummy.go_to("Dsysvx",999999);
}              // Close if()
// *
if (nofact)  {
    // *
// *        Compute the factorization A = U*D*U' or A = L*D*L'.
// *
Dlacpy.dlacpy(uplo,n,n,a,_a_offset,lda,af,_af_offset,ldaf);
Dsytrf.dsytrf(uplo,n,af,_af_offset,ldaf,ipiv,_ipiv_offset,work,_work_offset,lwork,info);
// *
// *        Return if INFO is non-zero.
// *
if (info.val != 0)  {
    if (info.val > 0)  
    rcond.val = zero;
Dummy.go_to("Dsysvx",999999);
}              // Close if()
}              // Close if()
// *
// *     Compute the norm of the matrix A.
// *
anorm = Dlansy.dlansy("I",uplo,n,a,_a_offset,lda,work,_work_offset);
// *
// *     Compute the reciprocal of the condition number of A.
// *
Dsycon.dsycon(uplo,n,af,_af_offset,ldaf,ipiv,_ipiv_offset,anorm,rcond,work,_work_offset,iwork,_iwork_offset,info);
// *
// *     Return if the matrix is singular to working precision.
// *
if (rcond.val < Dlamch.dlamch("Epsilon"))  {
    info.val = n+1;
Dummy.go_to("Dsysvx",999999);
}              // Close if()
// *
// *     Compute the solution vectors X.
// *
Dlacpy.dlacpy("Full",n,nrhs,b,_b_offset,ldb,x,_x_offset,ldx);
Dsytrs.dsytrs(uplo,n,nrhs,af,_af_offset,ldaf,ipiv,_ipiv_offset,x,_x_offset,ldx,info);
// *
// *     Use iterative refinement to improve the computed solutions and
// *     compute error bounds and backward error estimates for them.
// *
Dsyrfs.dsyrfs(uplo,n,nrhs,a,_a_offset,lda,af,_af_offset,ldaf,ipiv,_ipiv_offset,b,_b_offset,ldb,x,_x_offset,ldx,ferr,_ferr_offset,berr,_berr_offset,work,_work_offset,iwork,_iwork_offset,info);
// *
Dummy.go_to("Dsysvx",999999);
// *
// *     End of DSYSVX
// *
Dummy.label("Dsysvx",999999);
return;
   }
} // End class.
