/*
 *  Produced by f2java.  f2java is part of the Fortran-
 *  -to-Java project at the University of Tennessee Netlib
 *  numerical software repository.
 *
 *  Original authorship for the BLAS and LAPACK numerical
 *  routines may be found in the Fortran source, available at
 *  www.netlib.org.
 *
 *  Fortran input file: jlapack.f
 *
 *  The f2j compiler code was written by
 *  David M. Doolin (doolin@cs.utk.edu) and
 *  Keith  Seymour (seymour@cs.utk.edu)
 */

package org.netlib.lapack;
import java.lang.*;
import org.netlib.util.*;

import org.netlib.blas.*;


public class Dsytri {

// *
// *  -- LAPACK routine (version 2.0) --
// *     Univ. of Tennessee, Univ. of California Berkeley, NAG Ltd.,
// *     Courant Institute, Argonne National Lab, and Rice University
// *     March 31, 1993
// *
// *     .. Scalar Arguments ..
// *     ..
// *     .. Array Arguments ..
// *     ..
// *
// *  Purpose
// *  =======
// *
// *  DSYTRI computes the inverse of a real symmetric indefinite matrix
// *  A using the factorization A = U*D*U**T or A = L*D*L**T computed by
// *  DSYTRF.
// *
// *  Arguments
// *  =========
// *
// *  UPLO    (input) CHARACTER*1
// *          Specifies whether the details of the factorization are stored
// *          as an upper or lower triangular matrix.
// *          = 'U':  Upper triangular, form is A = U*D*U**T;
// *          = 'L':  Lower triangular, form is A = L*D*L**T.
// *
// *  N       (input) INTEGER
// *          The order of the matrix A.  N >= 0.
// *
// *  A       (input/output) DOUBLE PRECISION array, dimension (LDA,N)
// *          On entry, the block diagonal matrix D and the multipliers
// *          used to obtain the factor U or L as computed by DSYTRF.
// *
// *          On exit, if INFO = 0, the (symmetric) inverse of the original
// *          matrix.  If UPLO = 'U', the upper triangular part of the
// *          inverse is formed and the part of A below the diagonal is not
// *          referenced; if UPLO = 'L' the lower triangular part of the
// *          inverse is formed and the part of A above the diagonal is
// *          not referenced.
// *
// *  LDA     (input) INTEGER
// *          The leading dimension of the array A.  LDA >= max(1,N).
// *
// *  IPIV    (input) INTEGER array, dimension (N)
// *          Details of the interchanges and the block structure of D
// *          as determined by DSYTRF.
// *
// *  WORK    (workspace) DOUBLE PRECISION array, dimension (N)
// *
// *  INFO    (output) INTEGER
// *          = 0: successful exit
// *          < 0: if INFO = -i, the i-th argument had an illegal value
// *          > 0: if INFO = i, D(i,i) = 0; the matrix is singular and its
// *               inverse could not be computed.
// *
// *  =====================================================================
// *
// *     .. Parameters ..
static double one= 1.0e+0;
static double zero= 0.0e+0;
// *     ..
// *     .. Local Scalars ..
static boolean upper= false;
static int k= 0;
static int kp= 0;
static int kstep= 0;
static double ak= 0.0;
static double akkp1= 0.0;
static double akp1= 0.0;
static double d= 0.0;
static double t= 0.0;
static double temp= 0.0;
// *     ..
// *     .. External Functions ..
// *     ..
// *     .. External Subroutines ..
// *     ..
// *     .. Intrinsic Functions ..
// *     ..
// *     .. Executable Statements ..
// *
// *     Test the input parameters.
// *

public static void dsytri (String uplo,
int n,
double [] a, int _a_offset,
int lda,
int [] ipiv, int _ipiv_offset,
double [] work, int _work_offset,
intW info)  {

info.val = 0;
upper = (uplo.toLowerCase().charAt(0) == "U".toLowerCase().charAt(0));
if (!upper && !(uplo.toLowerCase().charAt(0) == "L".toLowerCase().charAt(0)))  {
    info.val = -1;
}              // Close if()
else if (n < 0)  {
    info.val = -2;
}              // Close else if()
else if (lda < Math.max(1, n) )  {
    info.val = -4;
}              // Close else if()
if (info.val != 0)  {
    Xerbla.xerbla("DSYTRI",-info.val);
Dummy.go_to("Dsytri",999999);
}              // Close if()
// *
// *     Quick return if possible
// *
if (n == 0)  
    Dummy.go_to("Dsytri",999999);
// *
// *     Check that the diagonal matrix D is nonsingular.
// *
if (upper)  {
    // *
// *        Upper triangular storage: examine D from bottom to top
// *
{
int _info_inc = -1;
forloop10:
for (info.val = n; info.val >= 1; info.val += _info_inc) {
if (ipiv[(info.val)- 1+ _ipiv_offset] > 0 && a[(info.val)- 1+(info.val- 1)*lda+ _a_offset] == zero)  
    Dummy.go_to("Dsytri",999999);
Dummy.label("Dsytri",10);
}              //  Close for() loop. 
}
}              // Close if()
else  {
  // *
// *        Lower triangular storage: examine D from top to bottom.
// *
{
forloop20:
for (info.val = 1; info.val <= n; info.val++) {
if (ipiv[(info.val)- 1+ _ipiv_offset] > 0 && a[(info.val)- 1+(info.val- 1)*lda+ _a_offset] == zero)  
    Dummy.go_to("Dsytri",999999);
Dummy.label("Dsytri",20);
}              //  Close for() loop. 
}
}              //  Close else.
info.val = 0;
// *
if (upper)  {
    // *
// *        Compute inv(A) from the factorization A = U*D*U'.
// *
// *        K is the main loop index, increasing from 1 to N in steps of
// *        1 or 2, depending on the size of the diagonal blocks.
// *
k = 1;
label30:
   Dummy.label("Dsytri",30);
// *
// *        If K > N, exit from loop.
// *
if (k > n)  
    Dummy.go_to("Dsytri",40);
// *
if (ipiv[(k)- 1+ _ipiv_offset] > 0)  {
    // *
// *           1 x 1 diagonal block
// *
// *           Invert the diagonal block.
// *
a[(k)- 1+(k- 1)*lda+ _a_offset] = one/a[(k)- 1+(k- 1)*lda+ _a_offset];
// *
// *           Compute column K of the inverse.
// *
if (k > 1)  {
    Dcopy.dcopy(k-1,a,(1)- 1+(k- 1)*lda+ _a_offset,1,work,_work_offset,1);
Dsymv.dsymv(uplo,k-1,-one,a,_a_offset,lda,work,_work_offset,1,zero,a,(1)- 1+(k- 1)*lda+ _a_offset,1);
a[(k)- 1+(k- 1)*lda+ _a_offset] = a[(k)- 1+(k- 1)*lda+ _a_offset]-Ddot.ddot(k-1,work,_work_offset,1,a,(1)- 1+(k- 1)*lda+ _a_offset,1);
}              // Close if()
kstep = 1;
}              // Close if()
else  {
  // *
// *           2 x 2 diagonal block
// *
// *           Invert the diagonal block.
// *
t = Math.abs(a[(k)- 1+(k+1- 1)*lda+ _a_offset]);
ak = a[(k)- 1+(k- 1)*lda+ _a_offset]/t;
akp1 = a[(k+1)- 1+(k+1- 1)*lda+ _a_offset]/t;
akkp1 = a[(k)- 1+(k+1- 1)*lda+ _a_offset]/t;
d = t*(ak*akp1-one);
a[(k)- 1+(k- 1)*lda+ _a_offset] = akp1/d;
a[(k+1)- 1+(k+1- 1)*lda+ _a_offset] = ak/d;
a[(k)- 1+(k+1- 1)*lda+ _a_offset] = -akkp1/d;
// *
// *           Compute columns K and K+1 of the inverse.
// *
if (k > 1)  {
    Dcopy.dcopy(k-1,a,(1)- 1+(k- 1)*lda+ _a_offset,1,work,_work_offset,1);
Dsymv.dsymv(uplo,k-1,-one,a,_a_offset,lda,work,_work_offset,1,zero,a,(1)- 1+(k- 1)*lda+ _a_offset,1);
a[(k)- 1+(k- 1)*lda+ _a_offset] = a[(k)- 1+(k- 1)*lda+ _a_offset]-Ddot.ddot(k-1,work,_work_offset,1,a,(1)- 1+(k- 1)*lda+ _a_offset,1);
a[(k)- 1+(k+1- 1)*lda+ _a_offset] = a[(k)- 1+(k+1- 1)*lda+ _a_offset]-Ddot.ddot(k-1,a,(1)- 1+(k- 1)*lda+ _a_offset,1,a,(1)- 1+(k+1- 1)*lda+ _a_offset,1);
Dcopy.dcopy(k-1,a,(1)- 1+(k+1- 1)*lda+ _a_offset,1,work,_work_offset,1);
Dsymv.dsymv(uplo,k-1,-one,a,_a_offset,lda,work,_work_offset,1,zero,a,(1)- 1+(k+1- 1)*lda+ _a_offset,1);
a[(k+1)- 1+(k+1- 1)*lda+ _a_offset] = a[(k+1)- 1+(k+1- 1)*lda+ _a_offset]-Ddot.ddot(k-1,work,_work_offset,1,a,(1)- 1+(k+1- 1)*lda+ _a_offset,1);
}              // Close if()
kstep = 2;
}              //  Close else.
// *
kp = (int)(Math.abs(ipiv[(k)- 1+ _ipiv_offset]));
if (kp != k)  {
    // *
// *           Interchange rows and columns K and KP in the leading
// *           submatrix A(1:k+1,1:k+1)
// *
Dswap.dswap(kp-1,a,(1)- 1+(k- 1)*lda+ _a_offset,1,a,(1)- 1+(kp- 1)*lda+ _a_offset,1);
Dswap.dswap(k-kp-1,a,(kp+1)- 1+(k- 1)*lda+ _a_offset,1,a,(kp)- 1+(kp+1- 1)*lda+ _a_offset,lda);
temp = a[(k)- 1+(k- 1)*lda+ _a_offset];
a[(k)- 1+(k- 1)*lda+ _a_offset] = a[(kp)- 1+(kp- 1)*lda+ _a_offset];
a[(kp)- 1+(kp- 1)*lda+ _a_offset] = temp;
if (kstep == 2)  {
    temp = a[(k)- 1+(k+1- 1)*lda+ _a_offset];
a[(k)- 1+(k+1- 1)*lda+ _a_offset] = a[(kp)- 1+(k+1- 1)*lda+ _a_offset];
a[(kp)- 1+(k+1- 1)*lda+ _a_offset] = temp;
}              // Close if()
}              // Close if()
// *
k = k+kstep;
Dummy.go_to("Dsytri",30);
label40:
   Dummy.label("Dsytri",40);
// *
}              // Close if()
else  {
  // *
// *        Compute inv(A) from the factorization A = L*D*L'.
// *
// *        K is the main loop index, increasing from 1 to N in steps of
// *        1 or 2, depending on the size of the diagonal blocks.
// *
k = n;
label50:
   Dummy.label("Dsytri",50);
// *
// *        If K < 1, exit from loop.
// *
if (k < 1)  
    Dummy.go_to("Dsytri",60);
// *
if (ipiv[(k)- 1+ _ipiv_offset] > 0)  {
    // *
// *           1 x 1 diagonal block
// *
// *           Invert the diagonal block.
// *
a[(k)- 1+(k- 1)*lda+ _a_offset] = one/a[(k)- 1+(k- 1)*lda+ _a_offset];
// *
// *           Compute column K of the inverse.
// *
if (k < n)  {
    Dcopy.dcopy(n-k,a,(k+1)- 1+(k- 1)*lda+ _a_offset,1,work,_work_offset,1);
Dsymv.dsymv(uplo,n-k,-one,a,(k+1)- 1+(k+1- 1)*lda+ _a_offset,lda,work,_work_offset,1,zero,a,(k+1)- 1+(k- 1)*lda+ _a_offset,1);
a[(k)- 1+(k- 1)*lda+ _a_offset] = a[(k)- 1+(k- 1)*lda+ _a_offset]-Ddot.ddot(n-k,work,_work_offset,1,a,(k+1)- 1+(k- 1)*lda+ _a_offset,1);
}              // Close if()
kstep = 1;
}              // Close if()
else  {
  // *
// *           2 x 2 diagonal block
// *
// *           Invert the diagonal block.
// *
t = Math.abs(a[(k)- 1+(k-1- 1)*lda+ _a_offset]);
ak = a[(k-1)- 1+(k-1- 1)*lda+ _a_offset]/t;
akp1 = a[(k)- 1+(k- 1)*lda+ _a_offset]/t;
akkp1 = a[(k)- 1+(k-1- 1)*lda+ _a_offset]/t;
d = t*(ak*akp1-one);
a[(k-1)- 1+(k-1- 1)*lda+ _a_offset] = akp1/d;
a[(k)- 1+(k- 1)*lda+ _a_offset] = ak/d;
a[(k)- 1+(k-1- 1)*lda+ _a_offset] = -akkp1/d;
// *
// *           Compute columns K-1 and K of the inverse.
// *
if (k < n)  {
    Dcopy.dcopy(n-k,a,(k+1)- 1+(k- 1)*lda+ _a_offset,1,work,_work_offset,1);
Dsymv.dsymv(uplo,n-k,-one,a,(k+1)- 1+(k+1- 1)*lda+ _a_offset,lda,work,_work_offset,1,zero,a,(k+1)- 1+(k- 1)*lda+ _a_offset,1);
a[(k)- 1+(k- 1)*lda+ _a_offset] = a[(k)- 1+(k- 1)*lda+ _a_offset]-Ddot.ddot(n-k,work,_work_offset,1,a,(k+1)- 1+(k- 1)*lda+ _a_offset,1);
a[(k)- 1+(k-1- 1)*lda+ _a_offset] = a[(k)- 1+(k-1- 1)*lda+ _a_offset]-Ddot.ddot(n-k,a,(k+1)- 1+(k- 1)*lda+ _a_offset,1,a,(k+1)- 1+(k-1- 1)*lda+ _a_offset,1);
Dcopy.dcopy(n-k,a,(k+1)- 1+(k-1- 1)*lda+ _a_offset,1,work,_work_offset,1);
Dsymv.dsymv(uplo,n-k,-one,a,(k+1)- 1+(k+1- 1)*lda+ _a_offset,lda,work,_work_offset,1,zero,a,(k+1)- 1+(k-1- 1)*lda+ _a_offset,1);
a[(k-1)- 1+(k-1- 1)*lda+ _a_offset] = a[(k-1)- 1+(k-1- 1)*lda+ _a_offset]-Ddot.ddot(n-k,work,_work_offset,1,a,(k+1)- 1+(k-1- 1)*lda+ _a_offset,1);
}              // Close if()
kstep = 2;
}              //  Close else.
// *
kp = (int)(Math.abs(ipiv[(k)- 1+ _ipiv_offset]));
if (kp != k)  {
    // *
// *           Interchange rows and columns K and KP in the trailing
// *           submatrix A(k-1:n,k-1:n)
// *
if (kp < n)  
    Dswap.dswap(n-kp,a,(kp+1)- 1+(k- 1)*lda+ _a_offset,1,a,(kp+1)- 1+(kp- 1)*lda+ _a_offset,1);
Dswap.dswap(kp-k-1,a,(k+1)- 1+(k- 1)*lda+ _a_offset,1,a,(kp)- 1+(k+1- 1)*lda+ _a_offset,lda);
temp = a[(k)- 1+(k- 1)*lda+ _a_offset];
a[(k)- 1+(k- 1)*lda+ _a_offset] = a[(kp)- 1+(kp- 1)*lda+ _a_offset];
a[(kp)- 1+(kp- 1)*lda+ _a_offset] = temp;
if (kstep == 2)  {
    temp = a[(k)- 1+(k-1- 1)*lda+ _a_offset];
a[(k)- 1+(k-1- 1)*lda+ _a_offset] = a[(kp)- 1+(k-1- 1)*lda+ _a_offset];
a[(kp)- 1+(k-1- 1)*lda+ _a_offset] = temp;
}              // Close if()
}              // Close if()
// *
k = k-kstep;
Dummy.go_to("Dsytri",50);
label60:
   Dummy.label("Dsytri",60);
}              //  Close else.
// *
Dummy.go_to("Dsytri",999999);
// *
// *     End of DSYTRI
// *
Dummy.label("Dsytri",999999);
return;
   }
} // End class.
