/*
 *  Produced by f2java.  f2java is part of the Fortran-
 *  -to-Java project at the University of Tennessee Netlib
 *  numerical software repository.
 *
 *  Original authorship for the BLAS and LAPACK numerical
 *  routines may be found in the Fortran source, available at
 *  www.netlib.org.
 *
 *  Fortran input file: jlapack.f
 *
 *  The f2j compiler code was written by
 *  David M. Doolin (doolin@cs.utk.edu) and
 *  Keith  Seymour (seymour@cs.utk.edu)
 */

package org.netlib.lapack;
import java.lang.*;
import org.netlib.util.*;

import org.netlib.blas.*;


public class Dsytrs {

// *
// *  -- LAPACK routine (version 2.0) --
// *     Univ. of Tennessee, Univ. of California Berkeley, NAG Ltd.,
// *     Courant Institute, Argonne National Lab, and Rice University
// *     March 31, 1993
// *
// *     .. Scalar Arguments ..
// *     ..
// *     .. Array Arguments ..
// *     ..
// *
// *  Purpose
// *  =======
// *
// *  DSYTRS solves a system of linear equations A*X = B with a real
// *  symmetric matrix A using the factorization A = U*D*U**T or
// *  A = L*D*L**T computed by DSYTRF.
// *
// *  Arguments
// *  =========
// *
// *  UPLO    (input) CHARACTER*1
// *          Specifies whether the details of the factorization are stored
// *          as an upper or lower triangular matrix.
// *          = 'U':  Upper triangular, form is A = U*D*U**T;
// *          = 'L':  Lower triangular, form is A = L*D*L**T.
// *
// *  N       (input) INTEGER
// *          The order of the matrix A.  N >= 0.
// *
// *  NRHS    (input) INTEGER
// *          The number of right hand sides, i.e., the number of columns
// *          of the matrix B.  NRHS >= 0.
// *
// *  A       (input) DOUBLE PRECISION array, dimension (LDA,N)
// *          The block diagonal matrix D and the multipliers used to
// *          obtain the factor U or L as computed by DSYTRF.
// *
// *  LDA     (input) INTEGER
// *          The leading dimension of the array A.  LDA >= max(1,N).
// *
// *  IPIV    (input) INTEGER array, dimension (N)
// *          Details of the interchanges and the block structure of D
// *          as determined by DSYTRF.
// *
// *  B       (input/output) DOUBLE PRECISION array, dimension (LDB,NRHS)
// *          On entry, the right hand side matrix B.
// *          On exit, the solution matrix X.
// *
// *  LDB     (input) INTEGER
// *          The leading dimension of the array B.  LDB >= max(1,N).
// *
// *  INFO    (output) INTEGER
// *          = 0:  successful exit
// *          < 0:  if INFO = -i, the i-th argument had an illegal value
// *
// *  =====================================================================
// *
// *     .. Parameters ..
static double one= 1.0e+0;
// *     ..
// *     .. Local Scalars ..
static boolean upper= false;
static int j= 0;
static int k= 0;
static int kp= 0;
static double ak= 0.0;
static double akm1= 0.0;
static double akm1k= 0.0;
static double bk= 0.0;
static double bkm1= 0.0;
static double denom= 0.0;
// *     ..
// *     .. External Functions ..
// *     ..
// *     .. External Subroutines ..
// *     ..
// *     .. Intrinsic Functions ..
// *     ..
// *     .. Executable Statements ..
// *

public static void dsytrs (String uplo,
int n,
int nrhs,
double [] a, int _a_offset,
int lda,
int [] ipiv, int _ipiv_offset,
double [] b, int _b_offset,
int ldb,
intW info)  {

info.val = 0;
upper = (uplo.toLowerCase().charAt(0) == "U".toLowerCase().charAt(0));
if (!upper && !(uplo.toLowerCase().charAt(0) == "L".toLowerCase().charAt(0)))  {
    info.val = -1;
}              // Close if()
else if (n < 0)  {
    info.val = -2;
}              // Close else if()
else if (nrhs < 0)  {
    info.val = -3;
}              // Close else if()
else if (lda < Math.max(1, n) )  {
    info.val = -5;
}              // Close else if()
else if (ldb < Math.max(1, n) )  {
    info.val = -8;
}              // Close else if()
if (info.val != 0)  {
    Xerbla.xerbla("DSYTRS",-info.val);
Dummy.go_to("Dsytrs",999999);
}              // Close if()
// *
// *     Quick return if possible
// *
if (n == 0 || nrhs == 0)  
    Dummy.go_to("Dsytrs",999999);
// *
if (upper)  {
    // *
// *        Solve A*X = B, where A = U*D*U'.
// *
// *        First solve U*D*X = B, overwriting B with X.
// *
// *        K is the main loop index, decreasing from N to 1 in steps of
// *        1 or 2, depending on the size of the diagonal blocks.
// *
k = n;
label10:
   Dummy.label("Dsytrs",10);
// *
// *        If K < 1, exit from loop.
// *
if (k < 1)  
    Dummy.go_to("Dsytrs",30);
// *
if (ipiv[(k)- 1+ _ipiv_offset] > 0)  {
    // *
// *           1 x 1 diagonal block
// *
// *           Interchange rows K and IPIV(K).
// *
kp = ipiv[(k)- 1+ _ipiv_offset];
if (kp != k)  
    Dswap.dswap(nrhs,b,(k)- 1+(1- 1)*ldb+ _b_offset,ldb,b,(kp)- 1+(1- 1)*ldb+ _b_offset,ldb);
// *
// *           Multiply by inv(U(K)), where U(K) is the transformation
// *           stored in column K of A.
// *
Dger.dger(k-1,nrhs,-one,a,(1)- 1+(k- 1)*lda+ _a_offset,1,b,(k)- 1+(1- 1)*ldb+ _b_offset,ldb,b,(1)- 1+(1- 1)*ldb+ _b_offset,ldb);
// *
// *           Multiply by the inverse of the diagonal block.
// *
Dscal.dscal(nrhs,one/a[(k)- 1+(k- 1)*lda+ _a_offset],b,(k)- 1+(1- 1)*ldb+ _b_offset,ldb);
k = k-1;
}              // Close if()
else  {
  // *
// *           2 x 2 diagonal block
// *
// *           Interchange rows K-1 and -IPIV(K).
// *
kp = -ipiv[(k)- 1+ _ipiv_offset];
if (kp != k-1)  
    Dswap.dswap(nrhs,b,(k-1)- 1+(1- 1)*ldb+ _b_offset,ldb,b,(kp)- 1+(1- 1)*ldb+ _b_offset,ldb);
// *
// *           Multiply by inv(U(K)), where U(K) is the transformation
// *           stored in columns K-1 and K of A.
// *
Dger.dger(k-2,nrhs,-one,a,(1)- 1+(k- 1)*lda+ _a_offset,1,b,(k)- 1+(1- 1)*ldb+ _b_offset,ldb,b,(1)- 1+(1- 1)*ldb+ _b_offset,ldb);
Dger.dger(k-2,nrhs,-one,a,(1)- 1+(k-1- 1)*lda+ _a_offset,1,b,(k-1)- 1+(1- 1)*ldb+ _b_offset,ldb,b,(1)- 1+(1- 1)*ldb+ _b_offset,ldb);
// *
// *           Multiply by the inverse of the diagonal block.
// *
akm1k = a[(k-1)- 1+(k- 1)*lda+ _a_offset];
akm1 = a[(k-1)- 1+(k-1- 1)*lda+ _a_offset]/akm1k;
ak = a[(k)- 1+(k- 1)*lda+ _a_offset]/akm1k;
denom = akm1*ak-one;
{
forloop20:
for (j = 1; j <= nrhs; j++) {
bkm1 = b[(k-1)- 1+(j- 1)*ldb+ _b_offset]/akm1k;
bk = b[(k)- 1+(j- 1)*ldb+ _b_offset]/akm1k;
b[(k-1)- 1+(j- 1)*ldb+ _b_offset] = (ak*bkm1-bk)/denom;
b[(k)- 1+(j- 1)*ldb+ _b_offset] = (akm1*bk-bkm1)/denom;
Dummy.label("Dsytrs",20);
}              //  Close for() loop. 
}
k = k-2;
}              //  Close else.
// *
Dummy.go_to("Dsytrs",10);
label30:
   Dummy.label("Dsytrs",30);
// *
// *        Next solve U'*X = B, overwriting B with X.
// *
// *        K is the main loop index, increasing from 1 to N in steps of
// *        1 or 2, depending on the size of the diagonal blocks.
// *
k = 1;
label40:
   Dummy.label("Dsytrs",40);
// *
// *        If K > N, exit from loop.
// *
if (k > n)  
    Dummy.go_to("Dsytrs",50);
// *
if (ipiv[(k)- 1+ _ipiv_offset] > 0)  {
    // *
// *           1 x 1 diagonal block
// *
// *           Multiply by inv(U'(K)), where U(K) is the transformation
// *           stored in column K of A.
// *
Dgemv.dgemv("Transpose",k-1,nrhs,-one,b,_b_offset,ldb,a,(1)- 1+(k- 1)*lda+ _a_offset,1,one,b,(k)- 1+(1- 1)*ldb+ _b_offset,ldb);
// *
// *           Interchange rows K and IPIV(K).
// *
kp = ipiv[(k)- 1+ _ipiv_offset];
if (kp != k)  
    Dswap.dswap(nrhs,b,(k)- 1+(1- 1)*ldb+ _b_offset,ldb,b,(kp)- 1+(1- 1)*ldb+ _b_offset,ldb);
k = k+1;
}              // Close if()
else  {
  // *
// *           2 x 2 diagonal block
// *
// *           Multiply by inv(U'(K+1)), where U(K+1) is the transformation
// *           stored in columns K and K+1 of A.
// *
Dgemv.dgemv("Transpose",k-1,nrhs,-one,b,_b_offset,ldb,a,(1)- 1+(k- 1)*lda+ _a_offset,1,one,b,(k)- 1+(1- 1)*ldb+ _b_offset,ldb);
Dgemv.dgemv("Transpose",k-1,nrhs,-one,b,_b_offset,ldb,a,(1)- 1+(k+1- 1)*lda+ _a_offset,1,one,b,(k+1)- 1+(1- 1)*ldb+ _b_offset,ldb);
// *
// *           Interchange rows K and -IPIV(K).
// *
kp = -ipiv[(k)- 1+ _ipiv_offset];
if (kp != k)  
    Dswap.dswap(nrhs,b,(k)- 1+(1- 1)*ldb+ _b_offset,ldb,b,(kp)- 1+(1- 1)*ldb+ _b_offset,ldb);
k = k+2;
}              //  Close else.
// *
Dummy.go_to("Dsytrs",40);
label50:
   Dummy.label("Dsytrs",50);
// *
}              // Close if()
else  {
  // *
// *        Solve A*X = B, where A = L*D*L'.
// *
// *        First solve L*D*X = B, overwriting B with X.
// *
// *        K is the main loop index, increasing from 1 to N in steps of
// *        1 or 2, depending on the size of the diagonal blocks.
// *
k = 1;
label60:
   Dummy.label("Dsytrs",60);
// *
// *        If K > N, exit from loop.
// *
if (k > n)  
    Dummy.go_to("Dsytrs",80);
// *
if (ipiv[(k)- 1+ _ipiv_offset] > 0)  {
    // *
// *           1 x 1 diagonal block
// *
// *           Interchange rows K and IPIV(K).
// *
kp = ipiv[(k)- 1+ _ipiv_offset];
if (kp != k)  
    Dswap.dswap(nrhs,b,(k)- 1+(1- 1)*ldb+ _b_offset,ldb,b,(kp)- 1+(1- 1)*ldb+ _b_offset,ldb);
// *
// *           Multiply by inv(L(K)), where L(K) is the transformation
// *           stored in column K of A.
// *
if (k < n)  
    Dger.dger(n-k,nrhs,-one,a,(k+1)- 1+(k- 1)*lda+ _a_offset,1,b,(k)- 1+(1- 1)*ldb+ _b_offset,ldb,b,(k+1)- 1+(1- 1)*ldb+ _b_offset,ldb);
// *
// *           Multiply by the inverse of the diagonal block.
// *
Dscal.dscal(nrhs,one/a[(k)- 1+(k- 1)*lda+ _a_offset],b,(k)- 1+(1- 1)*ldb+ _b_offset,ldb);
k = k+1;
}              // Close if()
else  {
  // *
// *           2 x 2 diagonal block
// *
// *           Interchange rows K+1 and -IPIV(K).
// *
kp = -ipiv[(k)- 1+ _ipiv_offset];
if (kp != k+1)  
    Dswap.dswap(nrhs,b,(k+1)- 1+(1- 1)*ldb+ _b_offset,ldb,b,(kp)- 1+(1- 1)*ldb+ _b_offset,ldb);
// *
// *           Multiply by inv(L(K)), where L(K) is the transformation
// *           stored in columns K and K+1 of A.
// *
if (k < n-1)  {
    Dger.dger(n-k-1,nrhs,-one,a,(k+2)- 1+(k- 1)*lda+ _a_offset,1,b,(k)- 1+(1- 1)*ldb+ _b_offset,ldb,b,(k+2)- 1+(1- 1)*ldb+ _b_offset,ldb);
Dger.dger(n-k-1,nrhs,-one,a,(k+2)- 1+(k+1- 1)*lda+ _a_offset,1,b,(k+1)- 1+(1- 1)*ldb+ _b_offset,ldb,b,(k+2)- 1+(1- 1)*ldb+ _b_offset,ldb);
}              // Close if()
// *
// *           Multiply by the inverse of the diagonal block.
// *
akm1k = a[(k+1)- 1+(k- 1)*lda+ _a_offset];
akm1 = a[(k)- 1+(k- 1)*lda+ _a_offset]/akm1k;
ak = a[(k+1)- 1+(k+1- 1)*lda+ _a_offset]/akm1k;
denom = akm1*ak-one;
{
forloop70:
for (j = 1; j <= nrhs; j++) {
bkm1 = b[(k)- 1+(j- 1)*ldb+ _b_offset]/akm1k;
bk = b[(k+1)- 1+(j- 1)*ldb+ _b_offset]/akm1k;
b[(k)- 1+(j- 1)*ldb+ _b_offset] = (ak*bkm1-bk)/denom;
b[(k+1)- 1+(j- 1)*ldb+ _b_offset] = (akm1*bk-bkm1)/denom;
Dummy.label("Dsytrs",70);
}              //  Close for() loop. 
}
k = k+2;
}              //  Close else.
// *
Dummy.go_to("Dsytrs",60);
label80:
   Dummy.label("Dsytrs",80);
// *
// *        Next solve L'*X = B, overwriting B with X.
// *
// *        K is the main loop index, decreasing from N to 1 in steps of
// *        1 or 2, depending on the size of the diagonal blocks.
// *
k = n;
label90:
   Dummy.label("Dsytrs",90);
// *
// *        If K < 1, exit from loop.
// *
if (k < 1)  
    Dummy.go_to("Dsytrs",100);
// *
if (ipiv[(k)- 1+ _ipiv_offset] > 0)  {
    // *
// *           1 x 1 diagonal block
// *
// *           Multiply by inv(L'(K)), where L(K) is the transformation
// *           stored in column K of A.
// *
if (k < n)  
    Dgemv.dgemv("Transpose",n-k,nrhs,-one,b,(k+1)- 1+(1- 1)*ldb+ _b_offset,ldb,a,(k+1)- 1+(k- 1)*lda+ _a_offset,1,one,b,(k)- 1+(1- 1)*ldb+ _b_offset,ldb);
// *
// *           Interchange rows K and IPIV(K).
// *
kp = ipiv[(k)- 1+ _ipiv_offset];
if (kp != k)  
    Dswap.dswap(nrhs,b,(k)- 1+(1- 1)*ldb+ _b_offset,ldb,b,(kp)- 1+(1- 1)*ldb+ _b_offset,ldb);
k = k-1;
}              // Close if()
else  {
  // *
// *           2 x 2 diagonal block
// *
// *           Multiply by inv(L'(K-1)), where L(K-1) is the transformation
// *           stored in columns K-1 and K of A.
// *
if (k < n)  {
    Dgemv.dgemv("Transpose",n-k,nrhs,-one,b,(k+1)- 1+(1- 1)*ldb+ _b_offset,ldb,a,(k+1)- 1+(k- 1)*lda+ _a_offset,1,one,b,(k)- 1+(1- 1)*ldb+ _b_offset,ldb);
Dgemv.dgemv("Transpose",n-k,nrhs,-one,b,(k+1)- 1+(1- 1)*ldb+ _b_offset,ldb,a,(k+1)- 1+(k-1- 1)*lda+ _a_offset,1,one,b,(k-1)- 1+(1- 1)*ldb+ _b_offset,ldb);
}              // Close if()
// *
// *           Interchange rows K and -IPIV(K).
// *
kp = -ipiv[(k)- 1+ _ipiv_offset];
if (kp != k)  
    Dswap.dswap(nrhs,b,(k)- 1+(1- 1)*ldb+ _b_offset,ldb,b,(kp)- 1+(1- 1)*ldb+ _b_offset,ldb);
k = k-2;
}              //  Close else.
// *
Dummy.go_to("Dsytrs",90);
label100:
   Dummy.label("Dsytrs",100);
}              //  Close else.
// *
Dummy.go_to("Dsytrs",999999);
// *
// *     End of DSYTRS
// *
Dummy.label("Dsytrs",999999);
return;
   }
} // End class.
