/*
 *  Produced by f2java.  f2java is part of the Fortran-
 *  -to-Java project at the University of Tennessee Netlib
 *  numerical software repository.
 *
 *  Original authorship for the BLAS and LAPACK numerical
 *  routines may be found in the Fortran source, available at
 *  www.netlib.org.
 *
 *  Fortran input file: jlapack.f
 *
 *  The f2j compiler code was written by
 *  David M. Doolin (doolin@cs.utk.edu) and
 *  Keith  Seymour (seymour@cs.utk.edu)
 */

package org.netlib.lapack;
import java.lang.*;
import org.netlib.util.*;

import org.netlib.blas.*;


public class Dtbcon {

// *
// *  -- LAPACK routine (version 2.0) --
// *     Univ. of Tennessee, Univ. of California Berkeley, NAG Ltd.,
// *     Courant Institute, Argonne National Lab, and Rice University
// *     March 31, 1993
// *
// *     .. Scalar Arguments ..
// *     ..
// *     .. Array Arguments ..
// *     ..
// *
// *  Purpose
// *  =======
// *
// *  DTBCON estimates the reciprocal of the condition number of a
// *  triangular band matrix A, in either the 1-norm or the infinity-norm.
// *
// *  The norm of A is computed and an estimate is obtained for
// *  norm(inv(A)), then the reciprocal of the condition number is
// *  computed as
// *     RCOND = 1 / ( norm(A) * norm(inv(A)) ).
// *
// *  Arguments
// *  =========
// *
// *  NORM    (input) CHARACTER*1
// *          Specifies whether the 1-norm condition number or the
// *          infinity-norm condition number is required:
// *          = '1' or 'O':  1-norm;
// *          = 'I':         Infinity-norm.
// *
// *  UPLO    (input) CHARACTER*1
// *          = 'U':  A is upper triangular;
// *          = 'L':  A is lower triangular.
// *
// *  DIAG    (input) CHARACTER*1
// *          = 'N':  A is non-unit triangular;
// *          = 'U':  A is unit triangular.
// *
// *  N       (input) INTEGER
// *          The order of the matrix A.  N >= 0.
// *
// *  KD      (input) INTEGER
// *          The number of superdiagonals or subdiagonals of the
// *          triangular band matrix A.  KD >= 0.
// *
// *  AB      (input) DOUBLE PRECISION array, dimension (LDAB,N)
// *          The upper or lower triangular band matrix A, stored in the
// *          first kd+1 rows of the array. The j-th column of A is stored
// *          in the j-th column of the array AB as follows:
// *          if UPLO = 'U', AB(kd+1+i-j,j) = A(i,j) for max(1,j-kd)<=i<=j;
// *          if UPLO = 'L', AB(1+i-j,j)    = A(i,j) for j<=i<=min(n,j+kd).
// *          If DIAG = 'U', the diagonal elements of A are not referenced
// *          and are assumed to be 1.
// *
// *  LDAB    (input) INTEGER
// *          The leading dimension of the array AB.  LDAB >= KD+1.
// *
// *  RCOND   (output) DOUBLE PRECISION
// *          The reciprocal of the condition number of the matrix A,
// *          computed as RCOND = 1/(norm(A) * norm(inv(A))).
// *
// *  WORK    (workspace) DOUBLE PRECISION array, dimension (3*N)
// *
// *  IWORK   (workspace) INTEGER array, dimension (N)
// *
// *  INFO    (output) INTEGER
// *          = 0:  successful exit
// *          < 0:  if INFO = -i, the i-th argument had an illegal value
// *
// *  =====================================================================
// *
// *     .. Parameters ..
static double one= 1.0e+0;
static double zero= 0.0e+0;
// *     ..
// *     .. Local Scalars ..
static boolean nounit= false;
static boolean onenrm= false;
static boolean upper= false;
static String normin= new String(" ");
static int ix= 0;
static intW kase= new intW(0);
static int kase1= 0;
static doubleW ainvnm= new doubleW(0.0);
static double anorm= 0.0;
static doubleW scale= new doubleW(0.0);
static double smlnum= 0.0;
static double xnorm= 0.0;
// *     ..
// *     .. External Functions ..
// *     ..
// *     .. External Subroutines ..
// *     ..
// *     .. Intrinsic Functions ..
// *     ..
// *     .. Executable Statements ..
// *
// *     Test the input parameters.
// *

public static void dtbcon (String norm,
String uplo,
String diag,
int n,
int kd,
double [] ab, int _ab_offset,
int ldab,
doubleW rcond,
double [] work, int _work_offset,
int [] iwork, int _iwork_offset,
intW info)  {

info.val = 0;
upper = (uplo.toLowerCase().charAt(0) == "U".toLowerCase().charAt(0));
onenrm = norm.trim().equalsIgnoreCase("1".trim()) || (norm.toLowerCase().charAt(0) == "O".toLowerCase().charAt(0));
nounit = (diag.toLowerCase().charAt(0) == "N".toLowerCase().charAt(0));
// *
if (!onenrm && !(norm.toLowerCase().charAt(0) == "I".toLowerCase().charAt(0)))  {
    info.val = -1;
}              // Close if()
else if (!upper && !(uplo.toLowerCase().charAt(0) == "L".toLowerCase().charAt(0)))  {
    info.val = -2;
}              // Close else if()
else if (!nounit && !(diag.toLowerCase().charAt(0) == "U".toLowerCase().charAt(0)))  {
    info.val = -3;
}              // Close else if()
else if (n < 0)  {
    info.val = -4;
}              // Close else if()
else if (kd < 0)  {
    info.val = -5;
}              // Close else if()
else if (ldab < kd+1)  {
    info.val = -7;
}              // Close else if()
if (info.val != 0)  {
    Xerbla.xerbla("DTBCON",-info.val);
Dummy.go_to("Dtbcon",999999);
}              // Close if()
// *
// *     Quick return if possible
// *
if (n == 0)  {
    rcond.val = one;
Dummy.go_to("Dtbcon",999999);
}              // Close if()
// *
rcond.val = zero;
smlnum = Dlamch.dlamch("Safe minimum")*(double)(Math.max(1, n) );
// *
// *     Compute the norm of the triangular matrix A.
// *
anorm = Dlantb.dlantb(norm,uplo,diag,n,kd,ab,_ab_offset,ldab,work,_work_offset);
// *
// *     Continue only if ANORM > 0.
// *
if (anorm > zero)  {
    // *
// *        Estimate the norm of the inverse of A.
// *
ainvnm.val = zero;
normin = "N";
if (onenrm)  {
    kase1 = 1;
}              // Close if()
else  {
  kase1 = 2;
}              //  Close else.
kase.val = 0;
label10:
   Dummy.label("Dtbcon",10);
Dlacon.dlacon(n,work,(n+1)- 1+ _work_offset,work,_work_offset,iwork,_iwork_offset,ainvnm,kase);
if (kase.val != 0)  {
    if (kase.val == kase1)  {
    // *
// *              Multiply by inv(A).
// *
Dlatbs.dlatbs(uplo,"No transpose",diag,normin,n,kd,ab,_ab_offset,ldab,work,_work_offset,scale,work,(2*n+1)- 1+ _work_offset,info);
}              // Close if()
else  {
  // *
// *              Multiply by inv(A').
// *
Dlatbs.dlatbs(uplo,"Transpose",diag,normin,n,kd,ab,_ab_offset,ldab,work,_work_offset,scale,work,(2*n+1)- 1+ _work_offset,info);
}              //  Close else.
normin = "Y";
// *
// *           Multiply by 1/SCALE if doing so will not cause overflow.
// *
if (scale.val != one)  {
    ix = Idamax.idamax(n,work,_work_offset,1);
xnorm = Math.abs(work[(ix)- 1+ _work_offset]);
if (scale.val < xnorm*smlnum || scale.val == zero)  
    Dummy.go_to("Dtbcon",20);
Drscl.drscl(n,scale.val,work,_work_offset,1);
}              // Close if()
Dummy.go_to("Dtbcon",10);
}              // Close if()
// *
// *        Compute the estimate of the reciprocal condition number.
// *
if (ainvnm.val != zero)  
    rcond.val = (one/anorm)/ainvnm.val;
}              // Close if()
// *
label20:
   Dummy.label("Dtbcon",20);
Dummy.go_to("Dtbcon",999999);
// *
// *     End of DTBCON
// *
Dummy.label("Dtbcon",999999);
return;
   }
} // End class.
