/*
 *  Produced by f2java.  f2java is part of the Fortran-
 *  -to-Java project at the University of Tennessee Netlib
 *  numerical software repository.
 *
 *  Original authorship for the BLAS and LAPACK numerical
 *  routines may be found in the Fortran source, available at
 *  www.netlib.org.
 *
 *  Fortran input file: jlapack.f
 *
 *  The f2j compiler code was written by
 *  David M. Doolin (doolin@cs.utk.edu) and
 *  Keith  Seymour (seymour@cs.utk.edu)
 */

package org.netlib.lapack;
import java.lang.*;
import org.netlib.util.*;

import org.netlib.blas.*;


public class Dtbtrs {

// *
// *  -- LAPACK routine (version 2.0) --
// *     Univ. of Tennessee, Univ. of California Berkeley, NAG Ltd.,
// *     Courant Institute, Argonne National Lab, and Rice University
// *     March 31, 1993
// *
// *     .. Scalar Arguments ..
// *     ..
// *     .. Array Arguments ..
// *     ..
// *
// *  Purpose
// *  =======
// *
// *  DTBTRS solves a triangular system of the form
// *
// *     A * X = B  or  A**T * X = B,
// *
// *  where A is a triangular band matrix of order N, and B is an
// *  N-by NRHS matrix.  A check is made to verify that A is nonsingular.
// *
// *  Arguments
// *  =========
// *
// *  UPLO    (input) CHARACTER*1
// *          = 'U':  A is upper triangular;
// *          = 'L':  A is lower triangular.
// *
// *  TRANS   (input) CHARACTER*1
// *          Specifies the form the system of equations:
// *          = 'N':  A * X = B  (No transpose)
// *          = 'T':  A**T * X = B  (Transpose)
// *          = 'C':  A**H * X = B  (Conjugate transpose = Transpose)
// *
// *  DIAG    (input) CHARACTER*1
// *          = 'N':  A is non-unit triangular;
// *          = 'U':  A is unit triangular.
// *
// *  N       (input) INTEGER
// *          The order of the matrix A.  N >= 0.
// *
// *  KD      (input) INTEGER
// *          The number of superdiagonals or subdiagonals of the
// *          triangular band matrix A.  KD >= 0.
// *
// *  NRHS    (input) INTEGER
// *          The number of right hand sides, i.e., the number of columns
// *          of the matrix B.  NRHS >= 0.
// *
// *  AB      (input) DOUBLE PRECISION array, dimension (LDAB,N)
// *          The upper or lower triangular band matrix A, stored in the
// *          first kd+1 rows of AB.  The j-th column of A is stored
// *          in the j-th column of the array AB as follows:
// *          if UPLO = 'U', AB(kd+1+i-j,j) = A(i,j) for max(1,j-kd)<=i<=j;
// *          if UPLO = 'L', AB(1+i-j,j)    = A(i,j) for j<=i<=min(n,j+kd).
// *          If DIAG = 'U', the diagonal elements of A are not referenced
// *          and are assumed to be 1.
// *
// *  LDAB    (input) INTEGER
// *          The leading dimension of the array AB.  LDAB >= KD+1.
// *
// *  B       (input/output) DOUBLE PRECISION array, dimension (LDB,NRHS)
// *          On entry, the right hand side matrix B.
// *          On exit, if INFO = 0, the solution matrix X.
// *
// *  LDB     (input) INTEGER
// *          The leading dimension of the array B.  LDB >= max(1,N).
// *
// *  INFO    (output) INTEGER
// *          = 0:  successful exit
// *          < 0:  if INFO = -i, the i-th argument had an illegal value
// *          > 0:  if INFO = i, the i-th diagonal element of A is zero,
// *                indicating that the matrix is singular and the
// *                solutions X have not been computed.
// *
// *  =====================================================================
// *
// *     .. Parameters ..
static double zero= 0.0e+0;
// *     ..
// *     .. Local Scalars ..
static boolean nounit= false;
static boolean upper= false;
static int j= 0;
// *     ..
// *     .. External Functions ..
// *     ..
// *     .. External Subroutines ..
// *     ..
// *     .. Intrinsic Functions ..
// *     ..
// *     .. Executable Statements ..
// *
// *     Test the input parameters.
// *

public static void dtbtrs (String uplo,
String trans,
String diag,
int n,
int kd,
int nrhs,
double [] ab, int _ab_offset,
int ldab,
double [] b, int _b_offset,
int ldb,
intW info)  {

info.val = 0;
nounit = (diag.toLowerCase().charAt(0) == "N".toLowerCase().charAt(0));
upper = (uplo.toLowerCase().charAt(0) == "U".toLowerCase().charAt(0));
if (!upper && !(uplo.toLowerCase().charAt(0) == "L".toLowerCase().charAt(0)))  {
    info.val = -1;
}              // Close if()
else if (!(trans.toLowerCase().charAt(0) == "N".toLowerCase().charAt(0)) && !(trans.toLowerCase().charAt(0) == "T".toLowerCase().charAt(0)) && !(trans.toLowerCase().charAt(0) == "C".toLowerCase().charAt(0)))  {
    info.val = -2;
}              // Close else if()
else if (!nounit && !(diag.toLowerCase().charAt(0) == "U".toLowerCase().charAt(0)))  {
    info.val = -3;
}              // Close else if()
else if (n < 0)  {
    info.val = -4;
}              // Close else if()
else if (kd < 0)  {
    info.val = -5;
}              // Close else if()
else if (nrhs < 0)  {
    info.val = -6;
}              // Close else if()
else if (ldab < kd+1)  {
    info.val = -8;
}              // Close else if()
else if (ldb < Math.max(1, n) )  {
    info.val = -10;
}              // Close else if()
if (info.val != 0)  {
    Xerbla.xerbla("DTBTRS",-info.val);
Dummy.go_to("Dtbtrs",999999);
}              // Close if()
// *
// *     Quick return if possible
// *
if (n == 0)  
    Dummy.go_to("Dtbtrs",999999);
// *
// *     Check for singularity.
// *
if (nounit)  {
    if (upper)  {
    {
forloop10:
for (info.val = 1; info.val <= n; info.val++) {
if (ab[(kd+1)- 1+(info.val- 1)*ldab+ _ab_offset] == zero)  
    Dummy.go_to("Dtbtrs",999999);
Dummy.label("Dtbtrs",10);
}              //  Close for() loop. 
}
}              // Close if()
else  {
  {
forloop20:
for (info.val = 1; info.val <= n; info.val++) {
if (ab[(1)- 1+(info.val- 1)*ldab+ _ab_offset] == zero)  
    Dummy.go_to("Dtbtrs",999999);
Dummy.label("Dtbtrs",20);
}              //  Close for() loop. 
}
}              //  Close else.
}              // Close if()
info.val = 0;
// *
// *     Solve A * X = B  or  A' * X = B.
// *
{
forloop30:
for (j = 1; j <= nrhs; j++) {
Dtbsv.dtbsv(uplo,trans,diag,n,kd,ab,_ab_offset,ldab,b,(1)- 1+(j- 1)*ldb+ _b_offset,1);
Dummy.label("Dtbtrs",30);
}              //  Close for() loop. 
}
// *
Dummy.go_to("Dtbtrs",999999);
// *
// *     End of DTBTRS
// *
Dummy.label("Dtbtrs",999999);
return;
   }
} // End class.
