/*
 *  Produced by f2java.  f2java is part of the Fortran-
 *  -to-Java project at the University of Tennessee Netlib
 *  numerical software repository.
 *
 *  Original authorship for the BLAS and LAPACK numerical
 *  routines may be found in the Fortran source, available at
 *  www.netlib.org.
 *
 *  Fortran input file: jlapack.f
 *
 *  The f2j compiler code was written by
 *  David M. Doolin (doolin@cs.utk.edu) and
 *  Keith  Seymour (seymour@cs.utk.edu)
 */

package org.netlib.lapack;
import java.lang.*;
import org.netlib.util.*;

import org.netlib.blas.*;


public class Dtptri {

// *
// *  -- LAPACK routine (version 2.0) --
// *     Univ. of Tennessee, Univ. of California Berkeley, NAG Ltd.,
// *     Courant Institute, Argonne National Lab, and Rice University
// *     September 30, 1994
// *
// *     .. Scalar Arguments ..
// *     ..
// *     .. Array Arguments ..
// *     ..
// *
// *  Purpose
// *  =======
// *
// *  DTPTRI computes the inverse of a real upper or lower triangular
// *  matrix A stored in packed format.
// *
// *  Arguments
// *  =========
// *
// *  UPLO    (input) CHARACTER*1
// *          = 'U':  A is upper triangular;
// *          = 'L':  A is lower triangular.
// *
// *  DIAG    (input) CHARACTER*1
// *          = 'N':  A is non-unit triangular;
// *          = 'U':  A is unit triangular.
// *
// *  N       (input) INTEGER
// *          The order of the matrix A.  N >= 0.
// *
// *  AP      (input/output) DOUBLE PRECISION array, dimension (N*(N+1)/2)
// *          On entry, the upper or lower triangular matrix A, stored
// *          columnwise in a linear array.  The j-th column of A is stored
// *          in the array AP as follows:
// *          if UPLO = 'U', AP(i + (j-1)*j/2) = A(i,j) for 1<=i<=j;
// *          if UPLO = 'L', AP(i + (j-1)*((2*n-j)/2) = A(i,j) for j<=i<=n.
// *          See below for further details.
// *          On exit, the (triangular) inverse of the original matrix, in
// *          the same packed storage format.
// *
// *  INFO    (output) INTEGER
// *          = 0:  successful exit
// *          < 0:  if INFO = -i, the i-th argument had an illegal value
// *          > 0:  if INFO = i, A(i,i) is exactly zero.  The triangular
// *                matrix is singular and its inverse can not be computed.
// *
// *  Further Details
// *  ===============
// *
// *  A triangular matrix A can be transferred to packed storage using one
// *  of the following program segments:
// *
// *  UPLO = 'U':                      UPLO = 'L':
// *
// *        JC = 1                           JC = 1
// *        DO 2 J = 1, N                    DO 2 J = 1, N
// *           DO 1 I = 1, J                    DO 1 I = J, N
// *              AP(JC+I-1) = A(I,J)              AP(JC+I-J) = A(I,J)
// *      1    CONTINUE                    1    CONTINUE
// *           JC = JC + J                      JC = JC + N - J + 1
// *      2 CONTINUE                       2 CONTINUE
// *
// *  =====================================================================
// *
// *     .. Parameters ..
static double one= 1.0e+0;
static double zero= 0.0e+0;
// *     ..
// *     .. Local Scalars ..
static boolean nounit= false;
static boolean upper= false;
static int j= 0;
static int jc= 0;
static int jclast= 0;
static int jj= 0;
static double ajj= 0.0;
// *     ..
// *     .. External Functions ..
// *     ..
// *     .. External Subroutines ..
// *     ..
// *     .. Executable Statements ..
// *
// *     Test the input parameters.
// *

public static void dtptri (String uplo,
String diag,
int n,
double [] ap, int _ap_offset,
intW info)  {

info.val = 0;
upper = (uplo.toLowerCase().charAt(0) == "U".toLowerCase().charAt(0));
nounit = (diag.toLowerCase().charAt(0) == "N".toLowerCase().charAt(0));
if (!upper && !(uplo.toLowerCase().charAt(0) == "L".toLowerCase().charAt(0)))  {
    info.val = -1;
}              // Close if()
else if (!nounit && !(diag.toLowerCase().charAt(0) == "U".toLowerCase().charAt(0)))  {
    info.val = -2;
}              // Close else if()
else if (n < 0)  {
    info.val = -3;
}              // Close else if()
if (info.val != 0)  {
    Xerbla.xerbla("DTPTRI",-info.val);
Dummy.go_to("Dtptri",999999);
}              // Close if()
// *
// *     Check for singularity if non-unit.
// *
if (nounit)  {
    if (upper)  {
    jj = 0;
{
forloop10:
for (info.val = 1; info.val <= n; info.val++) {
jj = jj+info.val;
if (ap[(jj)- 1+ _ap_offset] == zero)  
    Dummy.go_to("Dtptri",999999);
Dummy.label("Dtptri",10);
}              //  Close for() loop. 
}
}              // Close if()
else  {
  jj = 1;
{
forloop20:
for (info.val = 1; info.val <= n; info.val++) {
if (ap[(jj)- 1+ _ap_offset] == zero)  
    Dummy.go_to("Dtptri",999999);
jj = jj+n-info.val+1;
Dummy.label("Dtptri",20);
}              //  Close for() loop. 
}
}              //  Close else.
info.val = 0;
}              // Close if()
// *
if (upper)  {
    // *
// *        Compute inverse of upper triangular matrix.
// *
jc = 1;
{
forloop30:
for (j = 1; j <= n; j++) {
if (nounit)  {
    ap[(jc+j-1)- 1+ _ap_offset] = one/ap[(jc+j-1)- 1+ _ap_offset];
ajj = -ap[(jc+j-1)- 1+ _ap_offset];
}              // Close if()
else  {
  ajj = -one;
}              //  Close else.
// *
// *           Compute elements 1:j-1 of j-th column.
// *
Dtpmv.dtpmv("Upper","No transpose",diag,j-1,ap,_ap_offset,ap,(jc)- 1+ _ap_offset,1);
Dscal.dscal(j-1,ajj,ap,(jc)- 1+ _ap_offset,1);
jc = jc+j;
Dummy.label("Dtptri",30);
}              //  Close for() loop. 
}
// *
}              // Close if()
else  {
  // *
// *        Compute inverse of lower triangular matrix.
// *
jc = n*(n+1)/2;
{
int _j_inc = -1;
forloop40:
for (j = n; j >= 1; j += _j_inc) {
if (nounit)  {
    ap[(jc)- 1+ _ap_offset] = one/ap[(jc)- 1+ _ap_offset];
ajj = -ap[(jc)- 1+ _ap_offset];
}              // Close if()
else  {
  ajj = -one;
}              //  Close else.
if (j < n)  {
    // *
// *              Compute elements j+1:n of j-th column.
// *
Dtpmv.dtpmv("Lower","No transpose",diag,n-j,ap,(jclast)- 1+ _ap_offset,ap,(jc+1)- 1+ _ap_offset,1);
Dscal.dscal(n-j,ajj,ap,(jc+1)- 1+ _ap_offset,1);
}              // Close if()
jclast = jc;
jc = jc-n+j-2;
Dummy.label("Dtptri",40);
}              //  Close for() loop. 
}
}              //  Close else.
// *
Dummy.go_to("Dtptri",999999);
// *
// *     End of DTPTRI
// *
Dummy.label("Dtptri",999999);
return;
   }
} // End class.
