/*
 *  Produced by f2java.  f2java is part of the Fortran-
 *  -to-Java project at the University of Tennessee Netlib
 *  numerical software repository.
 *
 *  Original authorship for the BLAS and LAPACK numerical
 *  routines may be found in the Fortran source, available at
 *  www.netlib.org.
 *
 *  Fortran input file: jlapack.f
 *
 *  The f2j compiler code was written by
 *  David M. Doolin (doolin@cs.utk.edu) and
 *  Keith  Seymour (seymour@cs.utk.edu)
 */

package org.netlib.lapack;
import java.lang.*;
import org.netlib.util.*;



public class Dtrexc {

// *
// *  -- LAPACK routine (version 2.0) --
// *     Univ. of Tennessee, Univ. of California Berkeley, NAG Ltd.,
// *     Courant Institute, Argonne National Lab, and Rice University
// *     March 31, 1993
// *
// *     .. Scalar Arguments ..
// *     ..
// *     .. Array Arguments ..
// *     ..
// *
// *  Purpose
// *  =======
// *
// *  DTREXC reorders the real Schur factorization of a real matrix
// *  A = Q*T*Q**T, so that the diagonal block of T with row index IFST is
// *  moved to row ILST.
// *
// *  The real Schur form T is reordered by an orthogonal similarity
// *  transformation Z**T*T*Z, and optionally the matrix Q of Schur vectors
// *  is updated by postmultiplying it with Z.
// *
// *  T must be in Schur canonical form (as returned by DHSEQR), that is,
// *  block upper triangular with 1-by-1 and 2-by-2 diagonal blocks; each
// *  2-by-2 diagonal block has its diagonal elements equal and its
// *  off-diagonal elements of opposite sign.
// *
// *  Arguments
// *  =========
// *
// *  COMPQ   (input) CHARACTER*1
// *          = 'V':  update the matrix Q of Schur vectors;
// *          = 'N':  do not update Q.
// *
// *  N       (input) INTEGER
// *          The order of the matrix T. N >= 0.
// *
// *  T       (input/output) DOUBLE PRECISION array, dimension (LDT,N)
// *          On entry, the upper quasi-triangular matrix T, in Schur
// *          Schur canonical form.
// *          On exit, the reordered upper quasi-triangular matrix, again
// *          in Schur canonical form.
// *
// *  LDT     (input) INTEGER
// *          The leading dimension of the array T. LDT >= max(1,N).
// *
// *  Q       (input/output) DOUBLE PRECISION array, dimension (LDQ,N)
// *          On entry, if COMPQ = 'V', the matrix Q of Schur vectors.
// *          On exit, if COMPQ = 'V', Q has been postmultiplied by the
// *          orthogonal transformation matrix Z which reorders T.
// *          If COMPQ = 'N', Q is not referenced.
// *
// *  LDQ     (input) INTEGER
// *          The leading dimension of the array Q.  LDQ >= max(1,N).
// *
// *  IFST    (input/output) INTEGER
// *  ILST    (input/output) INTEGER
// *          Specify the reordering of the diagonal blocks of T.
// *          The block with row index IFST is moved to row ILST, by a
// *          sequence of transpositions between adjacent blocks.
// *          On exit, if IFST pointed on entry to the second row of a
// *          2-by-2 block, it is changed to point to the first row; ILST
// *          always points to the first row of the block in its final
// *          position (which may differ from its input value by +1 or -1).
// *          1 <= IFST <= N; 1 <= ILST <= N.
// *
// *  WORK    (workspace) DOUBLE PRECISION array, dimension (N)
// *
// *  INFO    (output) INTEGER
// *          = 0:  successful exit
// *          < 0:  if INFO = -i, the i-th argument had an illegal value
// *          = 1:  two adjacent blocks were too close to swap (the problem
// *                is very ill-conditioned); T may have been partially
// *                reordered, and ILST points to the first row of the
// *                current position of the block being moved.
// *
// *  =====================================================================
// *
// *     .. Parameters ..
static double zero= 0.0e+0;
// *     ..
// *     .. Local Scalars ..
static boolean wantq= false;
static int here= 0;
static int nbf= 0;
static int nbl= 0;
static int nbnext= 0;
// *     ..
// *     .. External Functions ..
// *     ..
// *     .. External Subroutines ..
// *     ..
// *     .. Intrinsic Functions ..
// *     ..
// *     .. Executable Statements ..
// *
// *     Decode and test the input arguments.
// *

public static void dtrexc (String compq,
int n,
double [] t, int _t_offset,
int ldt,
double [] q, int _q_offset,
int ldq,
intW ifst,
intW ilst,
double [] work, int _work_offset,
intW info)  {

info.val = 0;
wantq = (compq.toLowerCase().charAt(0) == "V".toLowerCase().charAt(0));
if (!wantq && !(compq.toLowerCase().charAt(0) == "N".toLowerCase().charAt(0)))  {
    info.val = -1;
}              // Close if()
else if (n < 0)  {
    info.val = -2;
}              // Close else if()
else if (ldt < Math.max(1, n) )  {
    info.val = -4;
}              // Close else if()
else if (ldq < 1 || (wantq && ldq < Math.max(1, n) ))  {
    info.val = -6;
}              // Close else if()
else if (ifst.val < 1 || ifst.val > n)  {
    info.val = -7;
}              // Close else if()
else if (ilst.val < 1 || ilst.val > n)  {
    info.val = -8;
}              // Close else if()
if (info.val != 0)  {
    Xerbla.xerbla("DTREXC",-info.val);
Dummy.go_to("Dtrexc",999999);
}              // Close if()
// *
// *     Quick return if possible
// *
if (n <= 1)  
    Dummy.go_to("Dtrexc",999999);
// *
// *     Determine the first row of specified block
// *     and find out it is 1 by 1 or 2 by 2.
// *
if (ifst.val > 1)  {
    if (t[(ifst.val)- 1+(ifst.val-1- 1)*ldt+ _t_offset] != zero)  
    ifst.val = ifst.val-1;
}              // Close if()
nbf = 1;
if (ifst.val < n)  {
    if (t[(ifst.val+1)- 1+(ifst.val- 1)*ldt+ _t_offset] != zero)  
    nbf = 2;
}              // Close if()
// *
// *     Determine the first row of the final block
// *     and find out it is 1 by 1 or 2 by 2.
// *
if (ilst.val > 1)  {
    if (t[(ilst.val)- 1+(ilst.val-1- 1)*ldt+ _t_offset] != zero)  
    ilst.val = ilst.val-1;
}              // Close if()
nbl = 1;
if (ilst.val < n)  {
    if (t[(ilst.val+1)- 1+(ilst.val- 1)*ldt+ _t_offset] != zero)  
    nbl = 2;
}              // Close if()
// *
if (ifst.val == ilst.val)  
    Dummy.go_to("Dtrexc",999999);
// *
if (ifst.val < ilst.val)  {
    // *
// *        Update ILST
// *
if (nbf == 2 && nbl == 1)  
    ilst.val = ilst.val-1;
if (nbf == 1 && nbl == 2)  
    ilst.val = ilst.val+1;
// *
here = ifst.val;
// *
label10:
   Dummy.label("Dtrexc",10);
// *
// *        Swap block with next one below
// *
if (nbf == 1 || nbf == 2)  {
    // *
// *           Current block either 1 by 1 or 2 by 2
// *
nbnext = 1;
if (here+nbf+1 <= n)  {
    if (t[(here+nbf+1)- 1+(here+nbf- 1)*ldt+ _t_offset] != zero)  
    nbnext = 2;
}              // Close if()
Dlaexc.dlaexc(wantq,n,t,_t_offset,ldt,q,_q_offset,ldq,here,nbf,nbnext,work,_work_offset,info);
if (info.val != 0)  {
    ilst.val = here;
Dummy.go_to("Dtrexc",999999);
}              // Close if()
here = here+nbnext;
// *
// *           Test if 2 by 2 block breaks into two 1 by 1 blocks
// *
if (nbf == 2)  {
    if (t[(here+1)- 1+(here- 1)*ldt+ _t_offset] == zero)  
    nbf = 3;
}              // Close if()
// *
}              // Close if()
else  {
  // *
// *           Current block consists of two 1 by 1 blocks each of which
// *           must be swapped individually
// *
nbnext = 1;
if (here+3 <= n)  {
    if (t[(here+3)- 1+(here+2- 1)*ldt+ _t_offset] != zero)  
    nbnext = 2;
}              // Close if()
Dlaexc.dlaexc(wantq,n,t,_t_offset,ldt,q,_q_offset,ldq,here+1,1,nbnext,work,_work_offset,info);
if (info.val != 0)  {
    ilst.val = here;
Dummy.go_to("Dtrexc",999999);
}              // Close if()
if (nbnext == 1)  {
    // *
// *              Swap two 1 by 1 blocks, no problems possible
// *
Dlaexc.dlaexc(wantq,n,t,_t_offset,ldt,q,_q_offset,ldq,here,1,nbnext,work,_work_offset,info);
here = here+1;
}              // Close if()
else  {
  // *
// *              Recompute NBNEXT in case 2 by 2 split
// *
if (t[(here+2)- 1+(here+1- 1)*ldt+ _t_offset] == zero)  
    nbnext = 1;
if (nbnext == 2)  {
    // *
// *                 2 by 2 Block did not split
// *
Dlaexc.dlaexc(wantq,n,t,_t_offset,ldt,q,_q_offset,ldq,here,1,nbnext,work,_work_offset,info);
if (info.val != 0)  {
    ilst.val = here;
Dummy.go_to("Dtrexc",999999);
}              // Close if()
here = here+2;
}              // Close if()
else  {
  // *
// *                 2 by 2 Block did split
// *
Dlaexc.dlaexc(wantq,n,t,_t_offset,ldt,q,_q_offset,ldq,here,1,1,work,_work_offset,info);
Dlaexc.dlaexc(wantq,n,t,_t_offset,ldt,q,_q_offset,ldq,here+1,1,1,work,_work_offset,info);
here = here+2;
}              //  Close else.
}              //  Close else.
}              //  Close else.
if (here < ilst.val)  
    Dummy.go_to("Dtrexc",10);
// *
}              // Close if()
else  {
  // *
here = ifst.val;
label20:
   Dummy.label("Dtrexc",20);
// *
// *        Swap block with next one above
// *
if (nbf == 1 || nbf == 2)  {
    // *
// *           Current block either 1 by 1 or 2 by 2
// *
nbnext = 1;
if (here >= 3)  {
    if (t[(here-1)- 1+(here-2- 1)*ldt+ _t_offset] != zero)  
    nbnext = 2;
}              // Close if()
Dlaexc.dlaexc(wantq,n,t,_t_offset,ldt,q,_q_offset,ldq,here-nbnext,nbnext,nbf,work,_work_offset,info);
if (info.val != 0)  {
    ilst.val = here;
Dummy.go_to("Dtrexc",999999);
}              // Close if()
here = here-nbnext;
// *
// *           Test if 2 by 2 block breaks into two 1 by 1 blocks
// *
if (nbf == 2)  {
    if (t[(here+1)- 1+(here- 1)*ldt+ _t_offset] == zero)  
    nbf = 3;
}              // Close if()
// *
}              // Close if()
else  {
  // *
// *           Current block consists of two 1 by 1 blocks each of which
// *           must be swapped individually
// *
nbnext = 1;
if (here >= 3)  {
    if (t[(here-1)- 1+(here-2- 1)*ldt+ _t_offset] != zero)  
    nbnext = 2;
}              // Close if()
Dlaexc.dlaexc(wantq,n,t,_t_offset,ldt,q,_q_offset,ldq,here-nbnext,nbnext,1,work,_work_offset,info);
if (info.val != 0)  {
    ilst.val = here;
Dummy.go_to("Dtrexc",999999);
}              // Close if()
if (nbnext == 1)  {
    // *
// *              Swap two 1 by 1 blocks, no problems possible
// *
Dlaexc.dlaexc(wantq,n,t,_t_offset,ldt,q,_q_offset,ldq,here,nbnext,1,work,_work_offset,info);
here = here-1;
}              // Close if()
else  {
  // *
// *              Recompute NBNEXT in case 2 by 2 split
// *
if (t[(here)- 1+(here-1- 1)*ldt+ _t_offset] == zero)  
    nbnext = 1;
if (nbnext == 2)  {
    // *
// *                 2 by 2 Block did not split
// *
Dlaexc.dlaexc(wantq,n,t,_t_offset,ldt,q,_q_offset,ldq,here-1,2,1,work,_work_offset,info);
if (info.val != 0)  {
    ilst.val = here;
Dummy.go_to("Dtrexc",999999);
}              // Close if()
here = here-2;
}              // Close if()
else  {
  // *
// *                 2 by 2 Block did split
// *
Dlaexc.dlaexc(wantq,n,t,_t_offset,ldt,q,_q_offset,ldq,here,1,1,work,_work_offset,info);
Dlaexc.dlaexc(wantq,n,t,_t_offset,ldt,q,_q_offset,ldq,here-1,1,1,work,_work_offset,info);
here = here-2;
}              //  Close else.
}              //  Close else.
}              //  Close else.
if (here > ilst.val)  
    Dummy.go_to("Dtrexc",20);
}              //  Close else.
ilst.val = here;
// *
Dummy.go_to("Dtrexc",999999);
// *
// *     End of DTREXC
// *
Dummy.label("Dtrexc",999999);
return;
   }
} // End class.
