/*
 *  Produced by f2java.  f2java is part of the Fortran-
 *  -to-Java project at the University of Tennessee Netlib
 *  numerical software repository.
 *
 *  Original authorship for the BLAS and LAPACK numerical
 *  routines may be found in the Fortran source, available at
 *  www.netlib.org.
 *
 *  Fortran input file: jlapack.f
 *
 *  The f2j compiler code was written by
 *  David M. Doolin (doolin@cs.utk.edu) and
 *  Keith  Seymour (seymour@cs.utk.edu)
 */

package org.netlib.lapack;
import java.lang.*;
import org.netlib.util.*;

import org.netlib.blas.*;


public class Dtrrfs {

// *
// *  -- LAPACK routine (version 2.0) --
// *     Univ. of Tennessee, Univ. of California Berkeley, NAG Ltd.,
// *     Courant Institute, Argonne National Lab, and Rice University
// *     September 30, 1994
// *
// *     .. Scalar Arguments ..
// *     ..
// *     .. Array Arguments ..
// *     ..
// *
// *  Purpose
// *  =======
// *
// *  DTRRFS provides error bounds and backward error estimates for the
// *  solution to a system of linear equations with a triangular
// *  coefficient matrix.
// *
// *  The solution matrix X must be computed by DTRTRS or some other
// *  means before entering this routine.  DTRRFS does not do iterative
// *  refinement because doing so cannot improve the backward error.
// *
// *  Arguments
// *  =========
// *
// *  UPLO    (input) CHARACTER*1
// *          = 'U':  A is upper triangular;
// *          = 'L':  A is lower triangular.
// *
// *  TRANS   (input) CHARACTER*1
// *          Specifies the form of the system of equations:
// *          = 'N':  A * X = B  (No transpose)
// *          = 'T':  A**T * X = B  (Transpose)
// *          = 'C':  A**H * X = B  (Conjugate transpose = Transpose)
// *
// *  DIAG    (input) CHARACTER*1
// *          = 'N':  A is non-unit triangular;
// *          = 'U':  A is unit triangular.
// *
// *  N       (input) INTEGER
// *          The order of the matrix A.  N >= 0.
// *
// *  NRHS    (input) INTEGER
// *          The number of right hand sides, i.e., the number of columns
// *          of the matrices B and X.  NRHS >= 0.
// *
// *  A       (input) DOUBLE PRECISION array, dimension (LDA,N)
// *          The triangular matrix A.  If UPLO = 'U', the leading N-by-N
// *          upper triangular part of the array A contains the upper
// *          triangular matrix, and the strictly lower triangular part of
// *          A is not referenced.  If UPLO = 'L', the leading N-by-N lower
// *          triangular part of the array A contains the lower triangular
// *          matrix, and the strictly upper triangular part of A is not
// *          referenced.  If DIAG = 'U', the diagonal elements of A are
// *          also not referenced and are assumed to be 1.
// *
// *  LDA     (input) INTEGER
// *          The leading dimension of the array A.  LDA >= max(1,N).
// *
// *  B       (input) DOUBLE PRECISION array, dimension (LDB,NRHS)
// *          The right hand side matrix B.
// *
// *  LDB     (input) INTEGER
// *          The leading dimension of the array B.  LDB >= max(1,N).
// *
// *  X       (input) DOUBLE PRECISION array, dimension (LDX,NRHS)
// *          The solution matrix X.
// *
// *  LDX     (input) INTEGER
// *          The leading dimension of the array X.  LDX >= max(1,N).
// *
// *  FERR    (output) DOUBLE PRECISION array, dimension (NRHS)
// *          The estimated forward error bound for each solution vector
// *          X(j) (the j-th column of the solution matrix X).
// *          If XTRUE is the true solution corresponding to X(j), FERR(j)
// *          is an estimated upper bound for the magnitude of the largest
// *          element in (X(j) - XTRUE) divided by the magnitude of the
// *          largest element in X(j).  The estimate is as reliable as
// *          the estimate for RCOND, and is almost always a slight
// *          overestimate of the true error.
// *
// *  BERR    (output) DOUBLE PRECISION array, dimension (NRHS)
// *          The componentwise relative backward error of each solution
// *          vector X(j) (i.e., the smallest relative change in
// *          any element of A or B that makes X(j) an exact solution).
// *
// *  WORK    (workspace) DOUBLE PRECISION array, dimension (3*N)
// *
// *  IWORK   (workspace) INTEGER array, dimension (N)
// *
// *  INFO    (output) INTEGER
// *          = 0:  successful exit
// *          < 0:  if INFO = -i, the i-th argument had an illegal value
// *
// *  =====================================================================
// *
// *     .. Parameters ..
static double zero= 0.0e+0;
static double one= 1.0e+0;
// *     ..
// *     .. Local Scalars ..
static boolean notran= false;
static boolean nounit= false;
static boolean upper= false;
static String transt= new String(" ");
static int i= 0;
static int j= 0;
static int k= 0;
static intW kase= new intW(0);
static int nz= 0;
static double eps= 0.0;
static double lstres= 0.0;
static double s= 0.0;
static double safe1= 0.0;
static double safe2= 0.0;
static double safmin= 0.0;
static double xk= 0.0;
// *     ..
// *     .. External Subroutines ..
// *     ..
// *     .. Intrinsic Functions ..
// *     ..
// *     .. External Functions ..
// *     ..
// *     .. Executable Statements ..
// *
// *     Test the input parameters.
// *

public static void dtrrfs (String uplo,
String trans,
String diag,
int n,
int nrhs,
double [] a, int _a_offset,
int lda,
double [] b, int _b_offset,
int ldb,
double [] x, int _x_offset,
int ldx,
double [] ferr, int _ferr_offset,
double [] berr, int _berr_offset,
double [] work, int _work_offset,
int [] iwork, int _iwork_offset,
intW info)  {

info.val = 0;
upper = (uplo.toLowerCase().charAt(0) == "U".toLowerCase().charAt(0));
notran = (trans.toLowerCase().charAt(0) == "N".toLowerCase().charAt(0));
nounit = (diag.toLowerCase().charAt(0) == "N".toLowerCase().charAt(0));
// *
if (!upper && !(uplo.toLowerCase().charAt(0) == "L".toLowerCase().charAt(0)))  {
    info.val = -1;
}              // Close if()
else if (!notran && !(trans.toLowerCase().charAt(0) == "T".toLowerCase().charAt(0)) && !(trans.toLowerCase().charAt(0) == "C".toLowerCase().charAt(0)))  {
    info.val = -2;
}              // Close else if()
else if (!nounit && !(diag.toLowerCase().charAt(0) == "U".toLowerCase().charAt(0)))  {
    info.val = -3;
}              // Close else if()
else if (n < 0)  {
    info.val = -4;
}              // Close else if()
else if (nrhs < 0)  {
    info.val = -5;
}              // Close else if()
else if (lda < Math.max(1, n) )  {
    info.val = -7;
}              // Close else if()
else if (ldb < Math.max(1, n) )  {
    info.val = -9;
}              // Close else if()
else if (ldx < Math.max(1, n) )  {
    info.val = -11;
}              // Close else if()
if (info.val != 0)  {
    Xerbla.xerbla("DTRRFS",-info.val);
Dummy.go_to("Dtrrfs",999999);
}              // Close if()
// *
// *     Quick return if possible
// *
if (n == 0 || nrhs == 0)  {
    {
forloop10:
for (j = 1; j <= nrhs; j++) {
ferr[(j)- 1+ _ferr_offset] = zero;
berr[(j)- 1+ _berr_offset] = zero;
Dummy.label("Dtrrfs",10);
}              //  Close for() loop. 
}
Dummy.go_to("Dtrrfs",999999);
}              // Close if()
// *
if (notran)  {
    transt = "T";
}              // Close if()
else  {
  transt = "N";
}              //  Close else.
// *
// *     NZ = maximum number of nonzero elements in each row of A, plus 1
// *
nz = n+1;
eps = Dlamch.dlamch("Epsilon");
safmin = Dlamch.dlamch("Safe minimum");
safe1 = nz*safmin;
safe2 = safe1/eps;
// *
// *     Do for each right hand side
// *
{
forloop250:
for (j = 1; j <= nrhs; j++) {
// *
// *        Compute residual R = B - op(A) * X,
// *        where op(A) = A or A', depending on TRANS.
// *
Dcopy.dcopy(n,x,(1)- 1+(j- 1)*ldx+ _x_offset,1,work,(n+1)- 1+ _work_offset,1);
Dtrmv.dtrmv(uplo,trans,diag,n,a,_a_offset,lda,work,(n+1)- 1+ _work_offset,1);
Daxpy.daxpy(n,-one,b,(1)- 1+(j- 1)*ldb+ _b_offset,1,work,(n+1)- 1+ _work_offset,1);
// *
// *        Compute componentwise relative backward error from formula
// *
// *        max(i) ( abs(R(i)) / ( abs(op(A))*abs(X) + abs(B) )(i) )
// *
// *        where abs(Z) is the componentwise absolute value of the matrix
// *        or vector Z.  If the i-th component of the denominator is less
// *        than SAFE2, then SAFE1 is added to the i-th components of the
// *        numerator and denominator before dividing.
// *
{
forloop20:
for (i = 1; i <= n; i++) {
work[(i)- 1+ _work_offset] = Math.abs(b[(i)- 1+(j- 1)*ldb+ _b_offset]);
Dummy.label("Dtrrfs",20);
}              //  Close for() loop. 
}
// *
if (notran)  {
    // *
// *           Compute abs(A)*abs(X) + abs(B).
// *
if (upper)  {
    if (nounit)  {
    {
forloop40:
for (k = 1; k <= n; k++) {
xk = Math.abs(x[(k)- 1+(j- 1)*ldx+ _x_offset]);
{
forloop30:
for (i = 1; i <= k; i++) {
work[(i)- 1+ _work_offset] = work[(i)- 1+ _work_offset]+Math.abs(a[(i)- 1+(k- 1)*lda+ _a_offset])*xk;
Dummy.label("Dtrrfs",30);
}              //  Close for() loop. 
}
Dummy.label("Dtrrfs",40);
}              //  Close for() loop. 
}
}              // Close if()
else  {
  {
forloop60:
for (k = 1; k <= n; k++) {
xk = Math.abs(x[(k)- 1+(j- 1)*ldx+ _x_offset]);
{
forloop50:
for (i = 1; i <= k-1; i++) {
work[(i)- 1+ _work_offset] = work[(i)- 1+ _work_offset]+Math.abs(a[(i)- 1+(k- 1)*lda+ _a_offset])*xk;
Dummy.label("Dtrrfs",50);
}              //  Close for() loop. 
}
work[(k)- 1+ _work_offset] = work[(k)- 1+ _work_offset]+xk;
Dummy.label("Dtrrfs",60);
}              //  Close for() loop. 
}
}              //  Close else.
}              // Close if()
else  {
  if (nounit)  {
    {
forloop80:
for (k = 1; k <= n; k++) {
xk = Math.abs(x[(k)- 1+(j- 1)*ldx+ _x_offset]);
{
forloop70:
for (i = k; i <= n; i++) {
work[(i)- 1+ _work_offset] = work[(i)- 1+ _work_offset]+Math.abs(a[(i)- 1+(k- 1)*lda+ _a_offset])*xk;
Dummy.label("Dtrrfs",70);
}              //  Close for() loop. 
}
Dummy.label("Dtrrfs",80);
}              //  Close for() loop. 
}
}              // Close if()
else  {
  {
forloop100:
for (k = 1; k <= n; k++) {
xk = Math.abs(x[(k)- 1+(j- 1)*ldx+ _x_offset]);
{
forloop90:
for (i = k+1; i <= n; i++) {
work[(i)- 1+ _work_offset] = work[(i)- 1+ _work_offset]+Math.abs(a[(i)- 1+(k- 1)*lda+ _a_offset])*xk;
Dummy.label("Dtrrfs",90);
}              //  Close for() loop. 
}
work[(k)- 1+ _work_offset] = work[(k)- 1+ _work_offset]+xk;
Dummy.label("Dtrrfs",100);
}              //  Close for() loop. 
}
}              //  Close else.
}              //  Close else.
}              // Close if()
else  {
  // *
// *           Compute abs(A')*abs(X) + abs(B).
// *
if (upper)  {
    if (nounit)  {
    {
forloop120:
for (k = 1; k <= n; k++) {
s = zero;
{
forloop110:
for (i = 1; i <= k; i++) {
s = s+Math.abs(a[(i)- 1+(k- 1)*lda+ _a_offset])*Math.abs(x[(i)- 1+(j- 1)*ldx+ _x_offset]);
Dummy.label("Dtrrfs",110);
}              //  Close for() loop. 
}
work[(k)- 1+ _work_offset] = work[(k)- 1+ _work_offset]+s;
Dummy.label("Dtrrfs",120);
}              //  Close for() loop. 
}
}              // Close if()
else  {
  {
forloop140:
for (k = 1; k <= n; k++) {
s = Math.abs(x[(k)- 1+(j- 1)*ldx+ _x_offset]);
{
forloop130:
for (i = 1; i <= k-1; i++) {
s = s+Math.abs(a[(i)- 1+(k- 1)*lda+ _a_offset])*Math.abs(x[(i)- 1+(j- 1)*ldx+ _x_offset]);
Dummy.label("Dtrrfs",130);
}              //  Close for() loop. 
}
work[(k)- 1+ _work_offset] = work[(k)- 1+ _work_offset]+s;
Dummy.label("Dtrrfs",140);
}              //  Close for() loop. 
}
}              //  Close else.
}              // Close if()
else  {
  if (nounit)  {
    {
forloop160:
for (k = 1; k <= n; k++) {
s = zero;
{
forloop150:
for (i = k; i <= n; i++) {
s = s+Math.abs(a[(i)- 1+(k- 1)*lda+ _a_offset])*Math.abs(x[(i)- 1+(j- 1)*ldx+ _x_offset]);
Dummy.label("Dtrrfs",150);
}              //  Close for() loop. 
}
work[(k)- 1+ _work_offset] = work[(k)- 1+ _work_offset]+s;
Dummy.label("Dtrrfs",160);
}              //  Close for() loop. 
}
}              // Close if()
else  {
  {
forloop180:
for (k = 1; k <= n; k++) {
s = Math.abs(x[(k)- 1+(j- 1)*ldx+ _x_offset]);
{
forloop170:
for (i = k+1; i <= n; i++) {
s = s+Math.abs(a[(i)- 1+(k- 1)*lda+ _a_offset])*Math.abs(x[(i)- 1+(j- 1)*ldx+ _x_offset]);
Dummy.label("Dtrrfs",170);
}              //  Close for() loop. 
}
work[(k)- 1+ _work_offset] = work[(k)- 1+ _work_offset]+s;
Dummy.label("Dtrrfs",180);
}              //  Close for() loop. 
}
}              //  Close else.
}              //  Close else.
}              //  Close else.
s = zero;
{
forloop190:
for (i = 1; i <= n; i++) {
if (work[(i)- 1+ _work_offset] > safe2)  {
    s = Math.max(s, Math.abs(work[(n+i)- 1+ _work_offset])/work[(i)- 1+ _work_offset]) ;
}              // Close if()
else  {
  s = Math.max(s, (Math.abs(work[(n+i)- 1+ _work_offset])+safe1)/(work[(i)- 1+ _work_offset]+safe1)) ;
}              //  Close else.
Dummy.label("Dtrrfs",190);
}              //  Close for() loop. 
}
berr[(j)- 1+ _berr_offset] = s;
// *
// *        Bound error from formula
// *
// *        norm(X - XTRUE) / norm(X) .le. FERR =
// *        norm( abs(inv(op(A)))*
// *           ( abs(R) + NZ*EPS*( abs(op(A))*abs(X)+abs(B) ))) / norm(X)
// *
// *        where
// *          norm(Z) is the magnitude of the largest component of Z
// *          inv(op(A)) is the inverse of op(A)
// *          abs(Z) is the componentwise absolute value of the matrix or
// *             vector Z
// *          NZ is the maximum number of nonzeros in any row of A, plus 1
// *          EPS is machine epsilon
// *
// *        The i-th component of abs(R)+NZ*EPS*(abs(op(A))*abs(X)+abs(B))
// *        is incremented by SAFE1 if the i-th component of
// *        abs(op(A))*abs(X) + abs(B) is less than SAFE2.
// *
// *        Use DLACON to estimate the infinity-norm of the matrix
// *           inv(op(A)) * diag(W),
// *        where W = abs(R) + NZ*EPS*( abs(op(A))*abs(X)+abs(B) )))
// *
{
forloop200:
for (i = 1; i <= n; i++) {
if (work[(i)- 1+ _work_offset] > safe2)  {
    work[(i)- 1+ _work_offset] = Math.abs(work[(n+i)- 1+ _work_offset])+nz*eps*work[(i)- 1+ _work_offset];
}              // Close if()
else  {
  work[(i)- 1+ _work_offset] = Math.abs(work[(n+i)- 1+ _work_offset])+nz*eps*work[(i)- 1+ _work_offset]+safe1;
}              //  Close else.
Dummy.label("Dtrrfs",200);
}              //  Close for() loop. 
}
// *
kase.val = 0;
label210:
   Dummy.label("Dtrrfs",210);
dlacon_adapter(n,work,(2*n+1)- 1+ _work_offset,work,(n+1)- 1+ _work_offset,iwork,_iwork_offset,ferr,(j)- 1+ _ferr_offset,kase);
if (kase.val != 0)  {
    if (kase.val == 1)  {
    // *
// *              Multiply by diag(W)*inv(op(A)').
// *
Dtrsv.dtrsv(uplo,transt,diag,n,a,_a_offset,lda,work,(n+1)- 1+ _work_offset,1);
{
forloop220:
for (i = 1; i <= n; i++) {
work[(n+i)- 1+ _work_offset] = work[(i)- 1+ _work_offset]*work[(n+i)- 1+ _work_offset];
Dummy.label("Dtrrfs",220);
}              //  Close for() loop. 
}
}              // Close if()
else  {
  // *
// *              Multiply by inv(op(A))*diag(W).
// *
{
forloop230:
for (i = 1; i <= n; i++) {
work[(n+i)- 1+ _work_offset] = work[(i)- 1+ _work_offset]*work[(n+i)- 1+ _work_offset];
Dummy.label("Dtrrfs",230);
}              //  Close for() loop. 
}
Dtrsv.dtrsv(uplo,trans,diag,n,a,_a_offset,lda,work,(n+1)- 1+ _work_offset,1);
}              //  Close else.
Dummy.go_to("Dtrrfs",210);
}              // Close if()
// *
// *        Normalize error.
// *
lstres = zero;
{
forloop240:
for (i = 1; i <= n; i++) {
lstres = Math.max(lstres, Math.abs(x[(i)- 1+(j- 1)*ldx+ _x_offset])) ;
Dummy.label("Dtrrfs",240);
}              //  Close for() loop. 
}
if (lstres != zero)  
    ferr[(j)- 1+ _ferr_offset] = ferr[(j)- 1+ _ferr_offset]/lstres;
// *
Dummy.label("Dtrrfs",250);
}              //  Close for() loop. 
}
// *
Dummy.go_to("Dtrrfs",999999);
// *
// *     End of DTRRFS
// *
Dummy.label("Dtrrfs",999999);
return;
   }
// adapter for dlacon
private static void dlacon_adapter(int arg0 ,double [] arg1 , int arg1_offset ,double [] arg2 , int arg2_offset ,int [] arg3 , int arg3_offset ,double [] arg4 , int arg4_offset ,intW arg5 )
{
doubleW _f2j_tmp4 = new doubleW(arg4[arg4_offset]);

Dlacon.dlacon(arg0,arg1, arg1_offset,arg2, arg2_offset,arg3, arg3_offset,_f2j_tmp4,arg5);

arg4[arg4_offset] = _f2j_tmp4.val;
}

} // End class.
