/*
 *  Produced by f2java.  f2java is part of the Fortran-
 *  -to-Java project at the University of Tennessee Netlib
 *  numerical software repository.
 *
 *  Original authorship for the BLAS and LAPACK numerical
 *  routines may be found in the Fortran source, available at
 *  www.netlib.org.
 *
 *  Fortran input file: jlapack.f
 *
 *  The f2j compiler code was written by
 *  David M. Doolin (doolin@cs.utk.edu) and
 *  Keith  Seymour (seymour@cs.utk.edu)
 */

package org.netlib.lapack;
import java.lang.*;
import org.netlib.util.*;

import org.netlib.blas.*;


public class Dtrti2 {

// *
// *  -- LAPACK routine (version 2.0) --
// *     Univ. of Tennessee, Univ. of California Berkeley, NAG Ltd.,
// *     Courant Institute, Argonne National Lab, and Rice University
// *     February 29, 1992
// *
// *     .. Scalar Arguments ..
// *     ..
// *     .. Array Arguments ..
// *     ..
// *
// *  Purpose
// *  =======
// *
// *  DTRTI2 computes the inverse of a real upper or lower triangular
// *  matrix.
// *
// *  This is the Level 2 BLAS version of the algorithm.
// *
// *  Arguments
// *  =========
// *
// *  UPLO    (input) CHARACTER*1
// *          Specifies whether the matrix A is upper or lower triangular.
// *          = 'U':  Upper triangular
// *          = 'L':  Lower triangular
// *
// *  DIAG    (input) CHARACTER*1
// *          Specifies whether or not the matrix A is unit triangular.
// *          = 'N':  Non-unit triangular
// *          = 'U':  Unit triangular
// *
// *  N       (input) INTEGER
// *          The order of the matrix A.  N >= 0.
// *
// *  A       (input/output) DOUBLE PRECISION array, dimension (LDA,N)
// *          On entry, the triangular matrix A.  If UPLO = 'U', the
// *          leading n by n upper triangular part of the array A contains
// *          the upper triangular matrix, and the strictly lower
// *          triangular part of A is not referenced.  If UPLO = 'L', the
// *          leading n by n lower triangular part of the array A contains
// *          the lower triangular matrix, and the strictly upper
// *          triangular part of A is not referenced.  If DIAG = 'U', the
// *          diagonal elements of A are also not referenced and are
// *          assumed to be 1.
// *
// *          On exit, the (triangular) inverse of the original matrix, in
// *          the same storage format.
// *
// *  LDA     (input) INTEGER
// *          The leading dimension of the array A.  LDA >= max(1,N).
// *
// *  INFO    (output) INTEGER
// *          = 0: successful exit
// *          < 0: if INFO = -k, the k-th argument had an illegal value
// *
// *  =====================================================================
// *
// *     .. Parameters ..
static double one= 1.0e+0;
// *     ..
// *     .. Local Scalars ..
static boolean nounit= false;
static boolean upper= false;
static int j= 0;
static double ajj= 0.0;
// *     ..
// *     .. External Functions ..
// *     ..
// *     .. External Subroutines ..
// *     ..
// *     .. Intrinsic Functions ..
// *     ..
// *     .. Executable Statements ..
// *
// *     Test the input parameters.
// *

public static void dtrti2 (String uplo,
String diag,
int n,
double [] a, int _a_offset,
int lda,
intW info)  {

info.val = 0;
upper = (uplo.toLowerCase().charAt(0) == "U".toLowerCase().charAt(0));
nounit = (diag.toLowerCase().charAt(0) == "N".toLowerCase().charAt(0));
if (!upper && !(uplo.toLowerCase().charAt(0) == "L".toLowerCase().charAt(0)))  {
    info.val = -1;
}              // Close if()
else if (!nounit && !(diag.toLowerCase().charAt(0) == "U".toLowerCase().charAt(0)))  {
    info.val = -2;
}              // Close else if()
else if (n < 0)  {
    info.val = -3;
}              // Close else if()
else if (lda < Math.max(1, n) )  {
    info.val = -5;
}              // Close else if()
if (info.val != 0)  {
    Xerbla.xerbla("DTRTI2",-info.val);
Dummy.go_to("Dtrti2",999999);
}              // Close if()
// *
if (upper)  {
    // *
// *        Compute inverse of upper triangular matrix.
// *
{
forloop10:
for (j = 1; j <= n; j++) {
if (nounit)  {
    a[(j)- 1+(j- 1)*lda+ _a_offset] = one/a[(j)- 1+(j- 1)*lda+ _a_offset];
ajj = -a[(j)- 1+(j- 1)*lda+ _a_offset];
}              // Close if()
else  {
  ajj = -one;
}              //  Close else.
// *
// *           Compute elements 1:j-1 of j-th column.
// *
Dtrmv.dtrmv("Upper","No transpose",diag,j-1,a,_a_offset,lda,a,(1)- 1+(j- 1)*lda+ _a_offset,1);
Dscal.dscal(j-1,ajj,a,(1)- 1+(j- 1)*lda+ _a_offset,1);
Dummy.label("Dtrti2",10);
}              //  Close for() loop. 
}
}              // Close if()
else  {
  // *
// *        Compute inverse of lower triangular matrix.
// *
{
int _j_inc = -1;
forloop20:
for (j = n; j >= 1; j += _j_inc) {
if (nounit)  {
    a[(j)- 1+(j- 1)*lda+ _a_offset] = one/a[(j)- 1+(j- 1)*lda+ _a_offset];
ajj = -a[(j)- 1+(j- 1)*lda+ _a_offset];
}              // Close if()
else  {
  ajj = -one;
}              //  Close else.
if (j < n)  {
    // *
// *              Compute elements j+1:n of j-th column.
// *
Dtrmv.dtrmv("Lower","No transpose",diag,n-j,a,(j+1)- 1+(j+1- 1)*lda+ _a_offset,lda,a,(j+1)- 1+(j- 1)*lda+ _a_offset,1);
Dscal.dscal(n-j,ajj,a,(j+1)- 1+(j- 1)*lda+ _a_offset,1);
}              // Close if()
Dummy.label("Dtrti2",20);
}              //  Close for() loop. 
}
}              //  Close else.
// *
Dummy.go_to("Dtrti2",999999);
// *
// *     End of DTRTI2
// *
Dummy.label("Dtrti2",999999);
return;
   }
} // End class.
