/*
 *  Produced by f2java.  f2java is part of the Fortran-
 *  -to-Java project at the University of Tennessee Netlib
 *  numerical software repository.
 *
 *  Original authorship for the BLAS and LAPACK numerical
 *  routines may be found in the Fortran source, available at
 *  www.netlib.org.
 *
 *  Fortran input file: jlapack.f
 *
 *  The f2j compiler code was written by
 *  David M. Doolin (doolin@cs.utk.edu) and
 *  Keith  Seymour (seymour@cs.utk.edu)
 */

package org.netlib.lapack;
import java.lang.*;
import org.netlib.util.*;

import org.netlib.blas.*;


public class Dtrtri {

// *
// *  -- LAPACK routine (version 2.0) --
// *     Univ. of Tennessee, Univ. of California Berkeley, NAG Ltd.,
// *     Courant Institute, Argonne National Lab, and Rice University
// *     March 31, 1993
// *
// *     .. Scalar Arguments ..
// *     ..
// *     .. Array Arguments ..
// *     ..
// *
// *  Purpose
// *  =======
// *
// *  DTRTRI computes the inverse of a real upper or lower triangular
// *  matrix A.
// *
// *  This is the Level 3 BLAS version of the algorithm.
// *
// *  Arguments
// *  =========
// *
// *  UPLO    (input) CHARACTER*1
// *          = 'U':  A is upper triangular;
// *          = 'L':  A is lower triangular.
// *
// *  DIAG    (input) CHARACTER*1
// *          = 'N':  A is non-unit triangular;
// *          = 'U':  A is unit triangular.
// *
// *  N       (input) INTEGER
// *          The order of the matrix A.  N >= 0.
// *
// *  A       (input/output) DOUBLE PRECISION array, dimension (LDA,N)
// *          On entry, the triangular matrix A.  If UPLO = 'U', the
// *          leading N-by-N upper triangular part of the array A contains
// *          the upper triangular matrix, and the strictly lower
// *          triangular part of A is not referenced.  If UPLO = 'L', the
// *          leading N-by-N lower triangular part of the array A contains
// *          the lower triangular matrix, and the strictly upper
// *          triangular part of A is not referenced.  If DIAG = 'U', the
// *          diagonal elements of A are also not referenced and are
// *          assumed to be 1.
// *          On exit, the (triangular) inverse of the original matrix, in
// *          the same storage format.
// *
// *  LDA     (input) INTEGER
// *          The leading dimension of the array A.  LDA >= max(1,N).
// *
// *  INFO    (output) INTEGER
// *          = 0: successful exit
// *          < 0: if INFO = -i, the i-th argument had an illegal value
// *          > 0: if INFO = i, A(i,i) is exactly zero.  The triangular
// *               matrix is singular and its inverse can not be computed.
// *
// *  =====================================================================
// *
// *     .. Parameters ..
static double one= 1.0e+0;
static double zero= 0.0e+0;
// *     ..
// *     .. Local Scalars ..
static boolean nounit= false;
static boolean upper= false;
static int j= 0;
static int jb= 0;
static int nb= 0;
static int nn= 0;
// *     ..
// *     .. External Functions ..
// *     ..
// *     .. External Subroutines ..
// *     ..
// *     .. Intrinsic Functions ..
// *     ..
// *     .. Executable Statements ..
// *
// *     Test the input parameters.
// *

public static void dtrtri (String uplo,
String diag,
int n,
double [] a, int _a_offset,
int lda,
intW info)  {

info.val = 0;
upper = (uplo.toLowerCase().charAt(0) == "U".toLowerCase().charAt(0));
nounit = (diag.toLowerCase().charAt(0) == "N".toLowerCase().charAt(0));
if (!upper && !(uplo.toLowerCase().charAt(0) == "L".toLowerCase().charAt(0)))  {
    info.val = -1;
}              // Close if()
else if (!nounit && !(diag.toLowerCase().charAt(0) == "U".toLowerCase().charAt(0)))  {
    info.val = -2;
}              // Close else if()
else if (n < 0)  {
    info.val = -3;
}              // Close else if()
else if (lda < Math.max(1, n) )  {
    info.val = -5;
}              // Close else if()
if (info.val != 0)  {
    Xerbla.xerbla("DTRTRI",-info.val);
Dummy.go_to("Dtrtri",999999);
}              // Close if()
// *
// *     Quick return if possible
// *
if (n == 0)  
    Dummy.go_to("Dtrtri",999999);
// *
// *     Check for singularity if non-unit.
// *
if (nounit)  {
    {
forloop10:
for (info.val = 1; info.val <= n; info.val++) {
if (a[(info.val)- 1+(info.val- 1)*lda+ _a_offset] == zero)  
    Dummy.go_to("Dtrtri",999999);
Dummy.label("Dtrtri",10);
}              //  Close for() loop. 
}
info.val = 0;
}              // Close if()
// *
// *     Determine the block size for this environment.
// *
nb = Ilaenv.ilaenv(1,"DTRTRI",uplo+diag,n,-1,-1,-1);
if (nb <= 1 || nb >= n)  {
    // *
// *        Use unblocked code
// *
Dtrti2.dtrti2(uplo,diag,n,a,_a_offset,lda,info);
}              // Close if()
else  {
  // *
// *        Use blocked code
// *
if (upper)  {
    // *
// *           Compute inverse of upper triangular matrix
// *
{
int _j_inc = nb;
forloop20:
for (j = 1; (_j_inc < 0) ? j >= n : j <= n; j += _j_inc) {
jb = (int)(Math.min(nb, n-j+1) );
// *
// *              Compute rows 1:j-1 of current block column
// *
Dtrmm.dtrmm("Left","Upper","No transpose",diag,j-1,jb,one,a,_a_offset,lda,a,(1)- 1+(j- 1)*lda+ _a_offset,lda);
Dtrsm.dtrsm("Right","Upper","No transpose",diag,j-1,jb,-one,a,(j)- 1+(j- 1)*lda+ _a_offset,lda,a,(1)- 1+(j- 1)*lda+ _a_offset,lda);
// *
// *              Compute inverse of current diagonal block
// *
Dtrti2.dtrti2("Upper",diag,jb,a,(j)- 1+(j- 1)*lda+ _a_offset,lda,info);
Dummy.label("Dtrtri",20);
}              //  Close for() loop. 
}
}              // Close if()
else  {
  // *
// *           Compute inverse of lower triangular matrix
// *
nn = ((n-1)/nb)*nb+1;
{
int _j_inc = -nb;
forloop30:
for (j = nn; (_j_inc < 0) ? j >= 1 : j <= 1; j += _j_inc) {
jb = (int)(Math.min(nb, n-j+1) );
if (j+jb <= n)  {
    // *
// *                 Compute rows j+jb:n of current block column
// *
Dtrmm.dtrmm("Left","Lower","No transpose",diag,n-j-jb+1,jb,one,a,(j+jb)- 1+(j+jb- 1)*lda+ _a_offset,lda,a,(j+jb)- 1+(j- 1)*lda+ _a_offset,lda);
Dtrsm.dtrsm("Right","Lower","No transpose",diag,n-j-jb+1,jb,-one,a,(j)- 1+(j- 1)*lda+ _a_offset,lda,a,(j+jb)- 1+(j- 1)*lda+ _a_offset,lda);
}              // Close if()
// *
// *              Compute inverse of current diagonal block
// *
Dtrti2.dtrti2("Lower",diag,jb,a,(j)- 1+(j- 1)*lda+ _a_offset,lda,info);
Dummy.label("Dtrtri",30);
}              //  Close for() loop. 
}
}              //  Close else.
}              //  Close else.
// *
Dummy.go_to("Dtrtri",999999);
// *
// *     End of DTRTRI
// *
Dummy.label("Dtrtri",999999);
return;
   }
} // End class.
