/*
 *  Produced by f2java.  f2java is part of the Fortran-
 *  -to-Java project at the University of Tennessee Netlib
 *  numerical software repository.
 *
 *  Original authorship for the BLAS and LAPACK numerical
 *  routines may be found in the Fortran source, available at
 *  www.netlib.org.
 *
 *  Fortran input file: jlapack.f
 *
 *  The f2j compiler code was written by
 *  David M. Doolin (doolin@cs.utk.edu) and
 *  Keith  Seymour (seymour@cs.utk.edu)
 */

package org.netlib.lapack;
import java.lang.*;
import org.netlib.util.*;

import org.netlib.blas.*;


public class Dtrtrs {

// *
// *  -- LAPACK routine (version 2.0) --
// *     Univ. of Tennessee, Univ. of California Berkeley, NAG Ltd.,
// *     Courant Institute, Argonne National Lab, and Rice University
// *     March 31, 1993
// *
// *     .. Scalar Arguments ..
// *     ..
// *     .. Array Arguments ..
// *     ..
// *
// *  Purpose
// *  =======
// *
// *  DTRTRS solves a triangular system of the form
// *
// *     A * X = B  or  A**T * X = B,
// *
// *  where A is a triangular matrix of order N, and B is an N-by-NRHS
// *  matrix.  A check is made to verify that A is nonsingular.
// *
// *  Arguments
// *  =========
// *
// *  UPLO    (input) CHARACTER*1
// *          = 'U':  A is upper triangular;
// *          = 'L':  A is lower triangular.
// *
// *  TRANS   (input) CHARACTER*1
// *          Specifies the form of the system of equations:
// *          = 'N':  A * X = B  (No transpose)
// *          = 'T':  A**T * X = B  (Transpose)
// *          = 'C':  A**H * X = B  (Conjugate transpose = Transpose)
// *
// *  DIAG    (input) CHARACTER*1
// *          = 'N':  A is non-unit triangular;
// *          = 'U':  A is unit triangular.
// *
// *  N       (input) INTEGER
// *          The order of the matrix A.  N >= 0.
// *
// *  NRHS    (input) INTEGER
// *          The number of right hand sides, i.e., the number of columns
// *          of the matrix B.  NRHS >= 0.
// *
// *  A       (input) DOUBLE PRECISION array, dimension (LDA,N)
// *          The triangular matrix A.  If UPLO = 'U', the leading N-by-N
// *          upper triangular part of the array A contains the upper
// *          triangular matrix, and the strictly lower triangular part of
// *          A is not referenced.  If UPLO = 'L', the leading N-by-N lower
// *          triangular part of the array A contains the lower triangular
// *          matrix, and the strictly upper triangular part of A is not
// *          referenced.  If DIAG = 'U', the diagonal elements of A are
// *          also not referenced and are assumed to be 1.
// *
// *  LDA     (input) INTEGER
// *          The leading dimension of the array A.  LDA >= max(1,N).
// *
// *  B       (input/output) DOUBLE PRECISION array, dimension (LDB,NRHS)
// *          On entry, the right hand side matrix B.
// *          On exit, if INFO = 0, the solution matrix X.
// *
// *  LDB     (input) INTEGER
// *          The leading dimension of the array B.  LDB >= max(1,N).
// *
// *  INFO    (output) INTEGER
// *          = 0:  successful exit
// *          < 0: if INFO = -i, the i-th argument had an illegal value
// *          > 0: if INFO = i, the i-th diagonal element of A is zero,
// *               indicating that the matrix is singular and the solutions
// *               X have not been computed.
// *
// *  =====================================================================
// *
// *     .. Parameters ..
static double zero= 0.0e+0;
static double one= 1.0e+0;
// *     ..
// *     .. Local Scalars ..
static boolean nounit= false;
// *     ..
// *     .. External Functions ..
// *     ..
// *     .. External Subroutines ..
// *     ..
// *     .. Intrinsic Functions ..
// *     ..
// *     .. Executable Statements ..
// *
// *     Test the input parameters.
// *

public static void dtrtrs (String uplo,
String trans,
String diag,
int n,
int nrhs,
double [] a, int _a_offset,
int lda,
double [] b, int _b_offset,
int ldb,
intW info)  {

info.val = 0;
nounit = (diag.toLowerCase().charAt(0) == "N".toLowerCase().charAt(0));
if (!(uplo.toLowerCase().charAt(0) == "U".toLowerCase().charAt(0)) && !(uplo.toLowerCase().charAt(0) == "L".toLowerCase().charAt(0)))  {
    info.val = -1;
}              // Close if()
else if (!(trans.toLowerCase().charAt(0) == "N".toLowerCase().charAt(0)) && !(trans.toLowerCase().charAt(0) == "T".toLowerCase().charAt(0)) && !(trans.toLowerCase().charAt(0) == "C".toLowerCase().charAt(0)))  {
    info.val = -2;
}              // Close else if()
else if (!nounit && !(diag.toLowerCase().charAt(0) == "U".toLowerCase().charAt(0)))  {
    info.val = -3;
}              // Close else if()
else if (n < 0)  {
    info.val = -4;
}              // Close else if()
else if (nrhs < 0)  {
    info.val = -5;
}              // Close else if()
else if (lda < Math.max(1, n) )  {
    info.val = -7;
}              // Close else if()
else if (ldb < Math.max(1, n) )  {
    info.val = -9;
}              // Close else if()
if (info.val != 0)  {
    Xerbla.xerbla("DTRTRS",-info.val);
Dummy.go_to("Dtrtrs",999999);
}              // Close if()
// *
// *     Quick return if possible
// *
if (n == 0)  
    Dummy.go_to("Dtrtrs",999999);
// *
// *     Check for singularity.
// *
if (nounit)  {
    {
forloop10:
for (info.val = 1; info.val <= n; info.val++) {
if (a[(info.val)- 1+(info.val- 1)*lda+ _a_offset] == zero)  
    Dummy.go_to("Dtrtrs",999999);
Dummy.label("Dtrtrs",10);
}              //  Close for() loop. 
}
}              // Close if()
info.val = 0;
// *
// *     Solve A * x = b  or  A' * x = b.
// *
Dtrsm.dtrsm("Left",uplo,trans,diag,n,nrhs,one,a,_a_offset,lda,b,_b_offset,ldb);
// *
Dummy.go_to("Dtrtrs",999999);
// *
// *     End of DTRTRS
// *
Dummy.label("Dtrtrs",999999);
return;
   }
} // End class.
