/*
 *  Produced by f2java.  f2java is part of the Fortran-
 *  -to-Java project at the University of Tennessee Netlib
 *  numerical software repository.
 *
 *  Original authorship for the BLAS and LAPACK numerical
 *  routines may be found in the Fortran source, available at
 *  www.netlib.org.
 *
 *  Fortran input file: jlapack.f
 *
 *  The f2j compiler code was written by
 *  David M. Doolin (doolin@cs.utk.edu) and
 *  Keith  Seymour (seymour@cs.utk.edu)
 */

package org.netlib.lapack;
import java.lang.*;
import org.netlib.util.*;

import org.netlib.blas.*;


public class Dtzrqf {

// *
// *  -- LAPACK routine (version 2.0) --
// *     Univ. of Tennessee, Univ. of California Berkeley, NAG Ltd.,
// *     Courant Institute, Argonne National Lab, and Rice University
// *     September 30, 1994
// *
// *     .. Scalar Arguments ..
// *     ..
// *     .. Array Arguments ..
// *     ..
// *
// *  Purpose
// *  =======
// *
// *  DTZRQF reduces the M-by-N ( M<=N ) real upper trapezoidal matrix A
// *  to upper triangular form by means of orthogonal transformations.
// *
// *  The upper trapezoidal matrix A is factored as
// *
// *     A = ( R  0 ) * Z,
// *
// *  where Z is an N-by-N orthogonal matrix and R is an M-by-M upper
// *  triangular matrix.
// *
// *  Arguments
// *  =========
// *
// *  M       (input) INTEGER
// *          The number of rows of the matrix A.  M >= 0.
// *
// *  N       (input) INTEGER
// *          The number of columns of the matrix A.  N >= M.
// *
// *  A       (input/output) DOUBLE PRECISION array, dimension (LDA,N)
// *          On entry, the leading M-by-N upper trapezoidal part of the
// *          array A must contain the matrix to be factorized.
// *          On exit, the leading M-by-M upper triangular part of A
// *          contains the upper triangular matrix R, and elements M+1 to
// *          N of the first M rows of A, with the array TAU, represent the
// *          orthogonal matrix Z as a product of M elementary reflectors.
// *
// *  LDA     (input) INTEGER
// *          The leading dimension of the array A.  LDA >= max(1,M).
// *
// *  TAU     (output) DOUBLE PRECISION array, dimension (M)
// *          The scalar factors of the elementary reflectors.
// *
// *  INFO    (output) INTEGER
// *          = 0:  successful exit
// *          < 0:  if INFO = -i, the i-th argument had an illegal value
// *
// *  Further Details
// *  ===============
// *
// *  The factorization is obtained by Householder's method.  The kth
// *  transformation matrix, Z( k ), which is used to introduce zeros into
// *  the ( m - k + 1 )th row of A, is given in the form
// *
// *     Z( k ) = ( I     0   ),
// *              ( 0  T( k ) )
// *
// *  where
// *
// *     T( k ) = I - tau*u( k )*u( k )',   u( k ) = (   1    ),
// *                                                 (   0    )
// *                                                 ( z( k ) )
// *
// *  tau is a scalar and z( k ) is an ( n - m ) element vector.
// *  tau and z( k ) are chosen to annihilate the elements of the kth row
// *  of X.
// *
// *  The scalar tau is returned in the kth element of TAU and the vector
// *  u( k ) in the kth row of A, such that the elements of z( k ) are
// *  in  a( k, m + 1 ), ..., a( k, n ). The elements of R are returned in
// *  the upper triangular part of A.
// *
// *  Z is given by
// *
// *     Z =  Z( 1 ) * Z( 2 ) * ... * Z( m ).
// *
// *  =====================================================================
// *
// *     .. Parameters ..
static double one= 1.0e+0;
static double zero= 0.0e+0;
// *     ..
// *     .. Local Scalars ..
static int i= 0;
static int k= 0;
static int m1= 0;
// *     ..
// *     .. Intrinsic Functions ..
// *     ..
// *     .. External Subroutines ..
// *     ..
// *     .. Executable Statements ..
// *
// *     Test the input parameters.
// *

public static void dtzrqf (int m,
int n,
double [] a, int _a_offset,
int lda,
double [] tau, int _tau_offset,
intW info)  {

info.val = 0;
if (m < 0)  {
    info.val = -1;
}              // Close if()
else if (n < m)  {
    info.val = -2;
}              // Close else if()
else if (lda < Math.max(1, m) )  {
    info.val = -4;
}              // Close else if()
if (info.val != 0)  {
    Xerbla.xerbla("DTZRQF",-info.val);
Dummy.go_to("Dtzrqf",999999);
}              // Close if()
// *
// *     Perform the factorization.
// *
if (m == 0)  
    Dummy.go_to("Dtzrqf",999999);
if (m == n)  {
    {
forloop10:
for (i = 1; i <= n; i++) {
tau[(i)- 1+ _tau_offset] = zero;
Dummy.label("Dtzrqf",10);
}              //  Close for() loop. 
}
}              // Close if()
else  {
  m1 = (int)(Math.min(m+1, n) );
{
int _k_inc = -1;
forloop20:
for (k = m; k >= 1; k += _k_inc) {
// *
// *           Use a Householder reflection to zero the kth row of A.
// *           First set up the reflection.
// *
dlarfg_adapter(n-m+1,a,(k)- 1+(k- 1)*lda+ _a_offset,a,(k)- 1+(m1- 1)*lda+ _a_offset,lda,tau,(k)- 1+ _tau_offset);
// *
if ((tau[(k)- 1+ _tau_offset] != zero) && (k > 1))  {
    // *
// *              We now perform the operation  A := A*P( k ).
// *
// *              Use the first ( k - 1 ) elements of TAU to store  a( k ),
// *              where  a( k ) consists of the first ( k - 1 ) elements of
// *              the  kth column  of  A.  Also  let  B  denote  the  first
// *              ( k - 1 ) rows of the last ( n - m ) columns of A.
// *
Dcopy.dcopy(k-1,a,(1)- 1+(k- 1)*lda+ _a_offset,1,tau,_tau_offset,1);
// *
// *              Form   w = a( k ) + B*z( k )  in TAU.
// *
Dgemv.dgemv("No transpose",k-1,n-m,one,a,(1)- 1+(m1- 1)*lda+ _a_offset,lda,a,(k)- 1+(m1- 1)*lda+ _a_offset,lda,one,tau,_tau_offset,1);
// *
// *              Now form  a( k ) := a( k ) - tau*w
// *              and       B      := B      - tau*w*z( k )'.
// *
Daxpy.daxpy(k-1,-tau[(k)- 1+ _tau_offset],tau,_tau_offset,1,a,(1)- 1+(k- 1)*lda+ _a_offset,1);
Dger.dger(k-1,n-m,-tau[(k)- 1+ _tau_offset],tau,_tau_offset,1,a,(k)- 1+(m1- 1)*lda+ _a_offset,lda,a,(1)- 1+(m1- 1)*lda+ _a_offset,lda);
}              // Close if()
Dummy.label("Dtzrqf",20);
}              //  Close for() loop. 
}
}              //  Close else.
// *
Dummy.go_to("Dtzrqf",999999);
// *
// *     End of DTZRQF
// *
Dummy.label("Dtzrqf",999999);
return;
   }
// adapter for dlarfg
private static void dlarfg_adapter(int arg0 ,double [] arg1 , int arg1_offset ,double [] arg2 , int arg2_offset ,int arg3 ,double [] arg4 , int arg4_offset )
{
doubleW _f2j_tmp1 = new doubleW(arg1[arg1_offset]);
doubleW _f2j_tmp4 = new doubleW(arg4[arg4_offset]);

Dlarfg.dlarfg(arg0,_f2j_tmp1,arg2, arg2_offset,arg3,_f2j_tmp4);

arg1[arg1_offset] = _f2j_tmp1.val;
arg4[arg4_offset] = _f2j_tmp4.val;
}

} // End class.
